/*****************************************************************************
 *  Rohde&Schwarz EMI Test Receiver
 *  Original Release: August 2012
 *  By: Nabil Ouddane 
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/
#include "rsemi.h"
#include "time.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
    #define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
        #include <windows.h>
#endif
#if defined (_CVI_)
	#include <utility.h>
#endif
 

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
 static ViString switchArr[] = {"OFF","ON",VI_NULL};
 static ViString DetectorTypeArr[] = {"NEG","POS","RMS","AVER","QPE","CAV","CRMS", "ACV", "ACS", VI_NULL};
// static ViString TypeArr[]= {"Cont","Low","Upp", VI_NULL};
 static ViString limTypeArr[] = {"","LOW","UPP", VI_NULL};
 static ViString SANlimTypeArr[] = {"LOW","UPP", VI_NULL};
// static ViString ResultTypeArr[] = {"TRACE1","TRACE2","TRACE3","TRACE4","TRACE5","TRACE6","FINAL1","FINAL2","FINAL3","FINAL","LIST","PHOL","SCAN","SING","STAT",VI_NULL};
 static ViString PhaseArr[] = {"","L1","L2","L3","N",VI_NULL};
 static ViString statProbArr[] = {"P0_1","P1","P10"};
 static ViString listFilterTypeArr[] = {"NORM","CFIL","RRC","P5","PULS"};
 static ViString measTypeArr[] = {"MEAN","PEAK","CFAC"};
 static ViString resultsString = "PASSED,FAILED,NONE";
 static ViString timeStampModeArr[] = {"CURR","ALL", VI_NULL};
 static ViString channelTypeArr[] = {"RECEIVER","SANALYZER","IQ","ADEM",VI_NULL};
 static ViString statusGapArr[] = {"PASSED","FAIL","NONE"};
 static ViString gs_TestReportChannelTypeArr[] = {"CAPD","IQ","REC","RTIM","SAN",NULL};
/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/
ViStatus rsemi_CheckStatusCallback (ViSession io);
ViStatus rsemi_WaitForOPCCallback (ViSession io);

ViStatus _VI_FUNC rsemi_RsClose (ViSession vi);

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/// HIFN  This function performs the following initialization actions:
/// HIFN     
/// HIFN     - Creates a new instrument driver session.
/// HIFN     
/// HIFN     - Opens a session to the specified device using the interface and 
/// HIFN  address you specify for the Resource Name parameter.
/// HIFN     
/// HIFN     - If the ID Query parameter is set to VI_TRUE, this function queries 
/// HIFN  the instrument ID and checks that it is valid for this instrument 
/// HIFN  driver.
/// HIFN     
/// HIFN     - If the Reset parameter is set to VI_TRUE, this function resets the 
/// HIFN  instrument to a known state.
/// HIFN     
/// HIFN     - Sends initialization commands to set the instrument to the state 
/// HIFN  necessary for the operation of the instrument driver.
/// HIFN     
/// HIFN     - Returns a ViSession handle that you use to identify the instrument 
/// HIFN  in all subsequent instrument driver function calls.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function creates a new session each time you invoke it. Although 
/// HIFN  you can open more than one session for the same resource, it is best 
/// HIFN  not to do so. You can use the same session in multiple program threads.
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize.
/// HIPAR IDQuery/Specify whether you want the instrument driver to perform an ID Query.
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the 
/// HIPAR resetDevice/initialization procedure.
/// HIPAR newInstrSession/Returns a ViSession handle that you use to identify the instrument in 
/// HIPAR newInstrSession/all subsequent instrument driver function calls.
ViStatus _VI_FUNC rsemi_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rsemi_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/// HIFN  This function performs the following initialization actions:
/// HIFN     
/// HIFN     - Creates a new instrument driver session.
/// HIFN     
/// HIFN     - Opens a session to the specified device using the interface and 
/// HIFN  address you specify for the Resource Name parameter.
/// HIFN     
/// HIFN     - If the ID Query parameter is set to VI_TRUE, this function queries 
/// HIFN  the instrument ID and checks that it is valid for this instrument 
/// HIFN  driver.
/// HIFN     
/// HIFN     - If the Reset parameter is set to VI_TRUE, this function resets the 
/// HIFN  instrument to a known state.
/// HIFN     
/// HIFN     - Sends initialization commands to set the instrument to the state 
/// HIFN  necessary for the operation of the instrument driver.
/// HIFN     
/// HIFN     - Returns a ViSession handle that you use to identify the instrument 
/// HIFN  in all subsequent instrument driver function calls.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function creates a new session each time you invoke it. Although 
/// HIFN  you can open more than one session for the same resource, it is best 
/// HIFN  not to do so. You can use the same session in multiple program threads.
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize.
/// HIPAR IDQuery/Specify whether you want the instrument driver to perform an ID Query.
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the 
/// HIPAR resetDevice/initialization procedure.
/// HIPAR optionString/You can use this control to set the initial value of certain 
/// HIPAR optionString/attributes for the session.  The following table lists the attributes 
/// HIPAR optionString/and the name you use in this parameter to identify the attribute.
/// HIPAR newInstrSession/Returns a ViSession handle that you use to identify the instrument in 
/// HIPAR newInstrSession/all subsequent instrument driver function calls.
ViStatus _VI_FUNC rsemi_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));

    ATTRLIST(attrList,rsemi); 
    checkErr( Rs_SpecificDriverNew (instrSession, "rsemi", newOptionString, attrList));
   
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsemi);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));


    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsemi_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsemi_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSEMI_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSEMI_ATTR_ID_QUERY_RESPONSE,
                                   0, RSEMI_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSEMI_VALID_ID_RESPONSE_STRING, strlen(RSEMI_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    

    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (vi) session. */
    checkErr( rsemi_CheckStatus (instrSession));

Error:

    if (error < VI_SUCCESS)
    {
        if (instrSession != VI_NULL)
        {
            (void) viClose (instrSession);
            (void) Rs_Dispose (instrSession);
        }
        
        if (rmSession != VI_NULL)
            (void) viClose (rmSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;
}

/// HIFN  This function selects the spectrum analyzer mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_SetSANMode(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_SAN_MODE, ""));
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function creates, selects and deletes new spectrum displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR operation/This control selects the operation to be performed.
/// HIPAR spectrumDisplay/This control sets the spectrum display.
ViStatus _VI_FUNC rsemi_ConfigureSpectrumDisplay(
    ViSession    instrSession,
    ViInt32    operation,
    ViInt32    spectrumDisplay
)
{
    ViStatus    error   = VI_SUCCESS;
    ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    switch (operation)
    {
        case  RSEMI_VAL_SAN_CREATE:
            attribute = RSEMI_ATTR_SPECTRUM_DISPLAY_CREATE;
            break;
            
        case RSEMI_VAL_SAN_SELECT:
            attribute = RSEMI_ATTR_SPECTRUM_DISPLAY_SELECT;
            break;
            
        case RSEMI_VAL_SAN_DELETE:
            attribute = RSEMI_ATTR_SPECTRUM_DISPLAY_DELETE;
            break;
        default:
            viCheckParm (VI_ERROR_PARAMETER2, 2, "Operation");

    }
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               attribute, spectrumDisplay),
                                               3, "Spectrum Display");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the frequency range of the spectrum analyzer 
/// HIFN  using start frequency and stop frequency. If start frequency is equal 
/// HIFN  to the stop frequency, then the spectrum analyzer is in time-domain 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR startFrequency/The start frequency of the frequency sweep.
/// HIPAR stopFrequency/The stop frequency of the frequency sweep.
ViStatus _VI_FUNC rsemi_ConfigureSpectrumFrequencyStartStop(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    startFrequency,
    ViReal64    stopFrequency
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_FREQUENCY_START, 
                startFrequency), 3, "Start Frequency");

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_FREQUENCY_STOP, 
                stopFrequency), 4, "Stop Frequency");


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Sets the minimum/maximum start frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minMax/Sets the minimum/maximum start frequency.
ViStatus _VI_FUNC rsemi_ConfigureSpectrumFrequencyStartMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    minMax
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_START_MIN_MAX,
                                                minMax), 3, "Min Max"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}


/// HIFN  This function queries minimal and maximal start frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minimum/Queries the minimal start frequency.
/// HIPAR maximum/Queries the maximal start frequency.
ViStatus _VI_FUNC rsemi_QuerySpectrumFrequencyStartMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    minimum,
    ViReal64*    maximum
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_START_GET_MIN,
                                                minimum), 3, "minimum"); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_START_GET_MAX,
                                                maximum), 4, "maximum");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}


/// HIFN  Sets the minimum/maximum stop frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minMax/Sets the minimum/maximum stop frequency.
ViStatus _VI_FUNC rsemi_ConfigureSpectrumFrequencyStopMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    minMax
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_STOP_MIN_MAX,
                                                minMax), 3, "Min Max"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}


/// HIFN  This function queries minimal and maximal stop frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minimum/Queries the minimal stop frequency.
/// HIPAR maximum/Queries the maximal stop frequency.
ViStatus _VI_FUNC rsemi_QuerySpectrumFrequencyStopMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    minimum,
    ViReal64*    maximum
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_STOP_GET_MIN,
                                                minimum), 3, "minimum"); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_STOP_GET_MAX,
                                                maximum), 4, "maximum");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function configures the center frequency 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR centerFrequency/The center frequency of the frequency sweep.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyCenter(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    centerFrequency
)
{
    ViStatus    error   = VI_SUCCESS; 
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_FREQUENCY_CENTER, 
                centerFrequency), 3, "Center Frequency");


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the frequency range of the spectrum analyzer 
/// HIFN  using the center frequency and the frequency span.  If span corresponds 
/// HIFN  to zero hertz, then the spectrum analyzer is in time-domain.  
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) In auto-coupled mode, RBW, VBW and Sweep time may be affected by 
/// HIFN  this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR centerFrequency/The center frequency of the frequency sweep.
/// HIPAR span/The frequency span of the frequency sweep.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyCenterSpan(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    centerFrequency,
    ViReal64    span
)
{
    ViStatus    error   = VI_SUCCESS; 
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_FREQUENCY_CENTER, 
                centerFrequency), 3, "Center Frequency");

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_FREQUENCY_SPAN, 
                span), 4, "Span");


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the step width of the center frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR stepSize/Sets the step width of the center frequency.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyStepSize(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    stepSize
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_FREQUENCY_CENTER_LINK,
                                              RSEMI_VAL_CENT_FREQ_LINK_OFF));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, 
            RSEMI_ATTR_CENTER_FREQUENCY_STEP, stepSize), 3, "Step Size");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function couples the step size of the center frequency to the 
/// HIFN  span (ON) or sets the value of the center frequency entered via 
/// HIFN  SENSe<1|2>:]FREQuency:CENTer:STEP (OFF).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR stepSizeAuto/Couples the step size of the center frequency to the span (ON) or sets 
/// HIPAR stepSizeAuto/the value of the center frequency entered via 
/// HIPAR stepSizeAuto/SENSe<1|2>:]FREQuency:CENTer:STEP (OFF).
ViStatus _VI_FUNC rsemi_ConfigureFrequencyStepSizeAuto(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    stepSizeAuto
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, 
            RSEMI_ATTR_FREQUENCY_STEP_AUTO, stepSizeAuto), 3, "Step Size Auto");

    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function couples the step width of the center frequency to span 
/// HIFN  (span >0) or to the resolution bandwidth (span = 0) or cancels the 
/// HIFN  couplings and sets the coupling factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR coupling/Couples the step width of the center frequency to span (span >0) or to 
/// HIPAR coupling/the resolution bandwidth (span = 0) or cancels the couplings.
/// HIPAR couplingFactor/Sets the coupling factor.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyCouplingFactor(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    coupling,
    ViInt32    couplingFactor
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_FREQUENCY_CENTER_LINK,
                                              coupling), 3, "Coupling");
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_FREQUENCY_CENTER_LINK_FACTOR, 
                                            couplingFactor), 4, "Coupling Factor");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the frequency span in the analyzer mode to its 
/// HIFN  maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ConfigureFrequencySpanFull(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckErr (rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_FREQUENCY_SPAN_FULL,
                                               VI_NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the span to 0 Hz. The x-axis becomes the time axis 
/// HIFN  with the grid lines corresponding to 1/10 of the current sweep time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyZeroSpan(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckErr (rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_FREQUENCY_SPAN,
                                               0.0));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the frequency offset attribute of the 
/// HIFN  spectrum analyzer. This function affects the setting of the spectrum 
/// HIFN  analyzer's absolute frequencies, such as start, stop, center, and 
/// HIFN  marker. It does not affect values, which are the difference of 
/// HIFN  frequencies, such as span and delta marker. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR frequencyOffset/Frequency offset.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyOffset(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    frequencyOffset
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_OFFSET, frequencyOffset),
                                                3, "Frequency Offset");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches between frequency domain (SWEep) and time 
/// HIFN  domain (CW | FIXed) in the analyzer mode. For CW and FIXed, the 
/// HIFN  frequency setting is via command FREQuency:CENTer. In the SWEep mode, 
/// HIFN  the setting is via commands FREQuency:STARt, STOP, CENTer and SPAN. In 
/// HIFN  EMI receiver mode, it switches between single frequency measurement 
/// HIFN  (CW) and scan (SCAN).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR frequencyMode/Frequency mode.
ViStatus _VI_FUNC rsemi_ConfigureSpectrumFrequencyMode(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    frequencyMode
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SAN_FREQUENCY_MODE, frequencyMode),
                                                3, "Frequency Mode");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the sweep time values of the spectrum 
/// HIFN  analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR sweepTimeAuto/This control sets automatic sweep time. The driver uses this value to 
/// HIPAR sweepTimeAuto/set the RSEMI_ATTR_SWEEP_TIME_AUTO attribute.
/// HIPAR sweepTime/The length of time to complete one sweep. The driver uses this value 
/// HIPAR sweepTime/to set the RSEMI_ATTR_SWEEP_TIME attribute.
ViStatus _VI_FUNC rsemi_ConfigureSweepTime(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    sweepTimeAuto,
    ViReal64    sweepTime
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SWEEP_TIME_AUTO, sweepTimeAuto),
                                                3, "Sweep Time Auto");
    
    if (sweepTimeAuto == VI_FALSE)
    {
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                    RSEMI_ATTR_SWEEP_TIME, sweepTime),
                                                    4, "Sweep Time");
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function activates auto coupling of selected coupling method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR sweepCoupling/This control selects coupling type.
ViStatus _VI_FUNC rsemi_ConfigureSweepCouplingAuto(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    sweepCoupling
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    switch (sweepCoupling)
    {
        case  RSEMI_VAL_COUPLING_RBW:
            attribute = RSEMI_ATTR_SAN_RESOLUTION_BANDWIDTH_AUTO;
            break;
            
        case RSEMI_VAL_COUPLING_VBW:
            attribute = RSEMI_ATTR_VIDEO_BANDWIDTH_AUTO;
            break;
            
        case RSEMI_VAL_COUPLING_SWEEP_TIME:
            attribute = RSEMI_ATTR_SWEEP_TIME_AUTO;
            break;
        default:
            viCheckParm (VI_ERROR_PARAMETER3, 3, "Sweep Coupling");
    }
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                attribute, VI_TRUE),
                                                1, "Dummy");
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the coupling values of the spectrum analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR sweepCoupling/This control selects coupling type.
/// HIPAR couplingValue/Resolution bandwidth:
ViStatus _VI_FUNC rsemi_ConfigureSweepCoupling(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    sweepCoupling,
    ViReal64    couplingValue
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViAttr      attribute = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    switch (sweepCoupling)
    {
        case RSEMI_VAL_COUPLING_RBW:
            attribute = RSEMI_ATTR_SAN_RESOLUTION_BANDWIDTH;
            break;
            
        case RSEMI_VAL_COUPLING_VBW:
            attribute = RSEMI_ATTR_VIDEO_BANDWIDTH;
            break;
            
        case RSEMI_VAL_COUPLING_SWEEP_TIME:
            attribute = RSEMI_ATTR_SWEEP_TIME;
            break;
        default:
            viCheckParm (VI_ERROR_PARAMETER3, 3, "Sweep Coupling");
    }
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                            attribute, couplingValue),
                                            4, "Coupling Value");                                  
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures advanced parameters of coupling and sweeping.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR ratioSelection/This control selects bandwidth ratio to be set.
/// HIPAR couplingValue/Video Bandwidth Ratio:
ViStatus _VI_FUNC rsemi_ConfigureSweepCouplingAdvanced(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    ratioSelection,
    ViReal64    couplingValue
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViAttr      attribute = 0;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    switch (ratioSelection)
    {
        case RSEMI_VAL_COUPLING_RATIO_RBW:
            attribute = RSEMI_ATTR_RESOLUTION_BANDWIDTH_RATIO;
            break;
        case RSEMI_VAL_COUPLING_RATIO_VBW:
            attribute = RSEMI_ATTR_VIDEO_BANDWIDTH_RATIO;
            break;
        default:
            viCheckParm (VI_ERROR_PARAMETER3, 3, "Ratio Selection");
    }
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                attribute, couplingValue),
                                                4, "Coupling Value");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Sets the minimum or maximum ratio resolution bandwidth (Hz) / span 
/// HIFN  (Hz). The ratio to be entered is reciprocal to the ratio span/RBW used 
/// HIFN  in manual control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR ratio/Sets the minimum or maximum ratio resolution bandwidth (Hz) / span 
/// HIPAR ratio/(Hz). The ratio to be entered is reciprocal to the ratio span/RBW used 
/// HIPAR ratio/in manual control.
ViStatus _VI_FUNC rsemi_ConfigureBandwidthRatioMinMax (ViSession instrSession,
                                                       ViInt32 window,
                                                       ViInt32 minMax)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_RESOLUTION_BANDWIDTH_RATIO_MIN_MAX,
                                                minMax), 3, "Min Max"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function Queries minimal and maximal ratio resolution bandwidth 
/// HIFN  (Hz) / span (Hz).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR minimum/Queries the minimal ratio resolution bandwidth (Hz) / span (Hz).
/// HIPAR maximum/Queries the maximal ratio resolution bandwidth (Hz) / span (Hz).
ViStatus _VI_FUNC rsemi_QueryBandwidthRatioMinMax (ViSession instrSession,
                                                   ViInt32 window,
                                                   ViReal64 *minimum,
                                                   ViReal64 *maximum)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_RESOLUTION_BANDWIDTH_RATIO_GET_MIN,
                                                minimum), 3, "minimum"); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_RESOLUTION_BANDWIDTH_RATIO_GET_MAX,
                                                maximum), 4, "maximum");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines the resolution bandwidth for IF analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth/This control defines the resolution bandwidth for IF analysis.
ViStatus _VI_FUNC rsemi_ConfigureIFAnalysisResolutionBandwidth(
    ViSession    instrSession,
    ViReal64    bandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_IF_ANALYSIS_RESOLUTION_BANDWIDTH,
                                                bandwidth), 2, "Bandwidth"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function sets the filter type for the resolution bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR filterType/Sets the filter type for the resolution bandwidth.
ViStatus _VI_FUNC rsemi_ConfigureFilterType(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    filterType
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SAN_RESOLUTION_BANDWIDTH_FILTER_TYPE,
                                                filterType), 3, "Filter Type"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects the position of the video filter in the signal 
/// HIFN  path, provided that the resolution bandwidth is <=100 kHz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR videoFilterType/Set the position of the video filter in the signal path
ViStatus _VI_FUNC rsemi_ConfigureVBWMode(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    videoFilterType
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_VIDEO_BANDWIDTH_TYPE,
                                                videoFilterType), 3, "Video Filter Type");
                                                    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function allows to specify FFT filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR fftFilterMode/Allows to specify FFT filter.
ViStatus _VI_FUNC rsemi_ConfigureFFTFilterMode(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    fftFilterMode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_FFT_FILTER_MODE,
                                                fftFilterMode), 3, "FFT Filter Mode");
                                                    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches on or off the average calculation for the 
/// HIFN  selected trace in the selected measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR state/Switches on or off the average calculation for the selected trace in 
/// HIPAR state/the selected measurement window.
ViStatus _VI_FUNC rsemi_ConfigureAveraging(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");
    
    sprintf (buffer, "Win%ld,TR%ld", window, trace);  
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_AVG_STATE,
                                                state), 4, "State");
                                                    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects the type of average function. If Video is 
/// HIFN  selected, the logarithmic power is averaged and, if Linear is selected, 
/// HIFN  the power values are averaged before they are converted to logarithmic 
/// HIFN  values.
/// HIFN     The type of average calculation is equally set for all traces in one 
/// HIFN  measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR averagingType/Selects the type of average function. If Video is selected, the 
/// HIPAR averagingType/logarithmic power is averaged and, if Linear is selected, the power 
/// HIPAR averagingType/values are averaged before they are converted to logarithmic values.
ViStatus _VI_FUNC rsemi_ConfigureAveragingType(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    averagingType
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_AVG_TYPE,
                                                averagingType), 3, "Averaging Type");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the averaging count. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR count/This control defines the number of measurements which contribute to 
/// HIPAR count/the average value.
ViStatus _VI_FUNC rsemi_ConfigureAveragingCount(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    count
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_AVG_COUNT,
                                                count), 3, "Count");    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the acquisition attributes of the spectrum 
/// HIFN  analyzer. Single sweep, continuous sweep and sweep count can be set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR sweepModeContinuous/Turns on/off continuous sweeping.
/// HIPAR numberOfSweeps/The number of sweeps to take.
ViStatus _VI_FUNC rsemi_ConfigureSpectrumAcquisition(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    sweepModeContinuous,
    ViInt32    numberOfSweeps
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SAN_SWEEP_MODE_CONTINUOUS,
                                                sweepModeContinuous), 3, "Sweep Mode Continuous");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_SAN_NUM_OF_SWEEPS,
                                                numberOfSweeps), 4, "Number Of Sweeps");                                                

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the vertical scale of analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR verticalScale/Pass the vertical scale that you want the spectrum analyzer to use.
ViStatus _VI_FUNC rsemi_ConfigureVerticalScale(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    verticalScale
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_VERTICAL_SCALE,
                                                verticalScale), 3, "Vertical Scale");                                                  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function toggles between linear and logarithmic display of the 
/// HIFN  x-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR horizontalScale/Toggles between linear and logarithmic display of the x-axis. 
ViStatus _VI_FUNC rsemi_ConfigureHorizontalScale(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    horizontalScale
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_HORIZONTAL_SCALE,
                                                horizontalScale), 3, "Horizontal Scale"); 
                                                   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the number of measurement points for one sweep 
/// HIFN  run. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR sweepPoints/Sets defines the number of measurement points for one sweep run. 
ViStatus _VI_FUNC rsemi_ConfigureSweepPoints(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    sweepPoints
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SWEEP_POINTS,
                                                sweepPoints), 3, "Sweep Points"); 
                                                   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the number of measurement points for one sweep 
/// HIFN  run. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR sweepPoints/Returns the number of measurement points for one sweep run. The driver 
/// HIPAR sweepPoints/uses this value to get the RSEMI_ATTR_SWEEP_POINTS attribute.
ViStatus _VI_FUNC rsemi_QuerySweepPoints(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32*    sweepPoints
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, buffer, 
                                            RSEMI_ATTR_SWEEP_POINTS,
                                            sweepPoints), 3, "Sweep Points");
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
   return error;
}

/// HIFN  This function defines the filter mode to be used for FFT sweep
/// HIFN  mode by defining the partial span size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR sweepOptimization/Sets the sweep optimization.
ViStatus _VI_FUNC rsemi_ConfigureSweepOptimization (
    ViSession instrSession,
    ViInt32 sweepOptimization
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_SWEEP_OPTIMIZATION,
                                                sweepOptimization), 2, "Sweep Optimization"); 
                                                   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the sweep type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR sweepType/Sets the sweep type.
ViStatus _VI_FUNC rsemi_ConfigureSweepType(
    ViSession    instrSession,
    ViInt32    sweepType
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_SWEEP_TYPE,
                                                sweepType), 2, "sweep Type"); 
                                                   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the input attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR attenuationAuto/Specifies whether the spectrum analyzer is enabled for auto 
/// HIPAR attenuationAuto/attenuation.
/// HIPAR attenuation/Specifies the input attenuation (in positive dB). This means that if 
/// HIPAR attenuation/10dB is selected, the result is a reduction in the signal level of 10 
/// HIPAR attenuation/dB.
ViStatus _VI_FUNC rsemi_ConfigureAttenuation(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    attenuationAuto,
    ViInt32    attenuation
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_ATTENUATION_AUTO,
                                                attenuationAuto), 3, "Attenuation Auto");                                                 
    if (attenuationAuto == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_ATTENUATION,
                                                attenuation), 4, "Attenuation");                                                
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the input IF attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR attenuationAuto/Specifies whether the spectrum analyzer is enabled 
/// HIPAR attenuationAuto/for auto IF attenuation
/// HIPAR attenuation/This attribute sets the IF Gain Value (rec mode)
ViStatus _VI_FUNC rsemi_ConfigureIFAttenuation (
    ViSession instrSession,
    ViInt32 window,
    ViBoolean attenuationAuto,
    ViInt32 attenuation
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_ATTENUATION_IF_AUTO,
                                                attenuationAuto), 3, "Attenuation Auto");                                                 
    if (attenuationAuto == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_ATTENUATION_IF,
                                                attenuation), 4, "Attenuation");                                                
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the pulse limiter for input 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR attenuationLimiter/Activates the pulse limiter for input 2
ViStatus _VI_FUNC rsemi_ConfigureAttenuationLimiter (
    ViSession instrSession,
    ViInt32 window,
    ViBoolean attenuationLimiter
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_ATTENUATION_LIMITER,
                                                attenuationLimiter), 3, "Attenuation Limiter");                                                 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Sets the input attenuator to minimum/maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minMax/This control sets the input attenuator to minimum/maximum.
ViStatus _VI_FUNC rsemi_ConfigureAttenuationMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    minMax
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_ATTENUATION_MIN_MAX,
                                                minMax), 3, "Min Max"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}


/// HIFN  Increases or decreases the input attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR upDown/This control increases or decreases the input attenuation.
ViStatus _VI_FUNC rsemi_ConfigureAttenuationUpDown(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    upDown
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_ATTENUATION_UP_DOWN,
                                                upDown), 3, "Up Down"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;      
}


/// HIFN  This function queries the minimum/maximum input attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minimum/Queries the minimal ratio resolution bandwidth (Hz) / span (Hz).
/// HIPAR maximum/Queries the maximal ratio resolution bandwidth (Hz) / span (Hz).
ViStatus _VI_FUNC rsemi_QueryAttenuationMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    minimum,
    ViReal64*    maximum
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_ATTENUATION_GET_MIN,
                                                minimum), 3, "minimum"); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_ATTENUATION_GET_MAX,
                                                maximum), 4, "maximum");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function configures the reference level
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR referenceLevel/The calibrated vertical position of the captured data used as a 
/// HIPAR referenceLevel/reference for amplitude measurements. This is typically set to a value 
/// HIPAR referenceLevel/slightly higher than the highest expected signal level. 
ViStatus _VI_FUNC rsemi_ConfigureReferenceLevel(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    referenceLevel
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_REFERENCE_LEVEL,
                                                referenceLevel), 3, "Reference Level"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the offset value to the reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR referenceLevelOffset/Specifies the offset value to the reference level. The driver uses 
/// HIPAR referenceLevelOffset/this value to set the RSEMI_ATTR_REFERENCE_LEVEL_OFFSET attribute.
ViStatus _VI_FUNC rsemi_ConfigureReferenceLevelOffset(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    referenceLevelOffset
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_REFERENCE_LEVEL_OFFSET,
                                                referenceLevelOffset), 3, "Reference Level Offset"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the vertical attributes of the spectrum 
/// HIFN  analyzer.  This corresponds to attributes like amplitude units, input 
/// HIFN  attenuation, input impedance, reference level, and reference level 
/// HIFN  offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR amplitudeUnits/Specifies the amplitude units for input, output and display. Pass the 
/// HIPAR amplitudeUnits/units you want the spectrum analyzer to use.
/// HIPAR inputImpedance/Specifies the input impedance.
/// HIPAR referenceLevel/The calibrated vertical position of the captured data used as a 
/// HIPAR referenceLevel/reference for amplitude measurements. This is typically set to a value 
/// HIPAR referenceLevel/slightly higher than the highest expected signal level. 
/// HIPAR referenceLevelOffset/Specifies the offset value to the reference level.
/// HIPAR attenuationAuto/Specifies whether the spectrum analyzer is enabled for auto 
/// HIPAR attenuationAuto/attenuation.
/// HIPAR attenuation/Specifies the input attenuation (in positive dB). This means that if 
/// HIPAR attenuation/10dB is selected, the result is a reduction in the signal level of 10 
/// HIPAR attenuation/dB.
ViStatus _VI_FUNC rsemi_ConfigureLevel(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    amplitudeUnits,
    ViInt32    inputImpedance,
    ViReal64    referenceLevel,
    ViReal64    referenceLevelOffset,
    ViBoolean    attenuationAuto,
    ViInt32    attenuation
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_AMPLITUDE_UNITS,
                                                amplitudeUnits), 3, "Amplitude Units"); 
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SAN_INPUT_IMPEDANCE,
                                                inputImpedance), 4, "Input Impedance"); 
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_REFERENCE_LEVEL,
                                                referenceLevel), 5, "Reference Level"); 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_REFERENCE_LEVEL_OFFSET,
                                                referenceLevelOffset), 6, "Reference Level Offset"); 
    
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                    RSEMI_ATTR_ATTENUATION_AUTO,
                                                    attenuationAuto), 7, "Attenuation Auto");                                                 
    if (attenuationAuto == VI_FALSE)
    {
            viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                    RSEMI_ATTR_ATTENUATION,
                                                    attenuation), 8, "Attenuation");                                                
    }
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function swicthes the behaviour for the attenuator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR mode/This control switches the behaviour of the attenuator.
ViStatus _VI_FUNC rsemi_ConfigureAttenuationMode(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    mode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));  
    
    sprintf (buffer, "Win%ld", window);   
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                   RSEMI_ATTR_ATTENUATION_MODE,
                                   mode), 3, "Mode"); 
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
   return error;
}


/// HIFN  This function defines wether the reference value for the Y axis is 
/// HIFN  coupled to the reference level (default) or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR coupledReferenceLevel/This control defines wether the reference value for the Y axis is 
/// HIPAR coupledReferenceLevel/coupled to the reference level (default) or not.
ViStatus _VI_FUNC rsemi_ConfigureReferenceLevelCoupling(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    coupledReferenceLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));  
    
    sprintf (buffer, "Win%ld", window);   
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_DISP_REF_VALUE_AUTO,
                                                coupledReferenceLevel), 3, "Coupled Reference Level"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches the source of the IF output between the 
/// HIFN  demodulated signal (2 possible values) and the IF signal.
/// HIFN     
/// HIFN     Note(s): 
/// HIFN     
/// HIFN     (1) The AF output available at the frontpanel can only be used if the 
/// HIFN  IF output source is set to video.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR outputSource/This control switches the source of the IF output between the 
/// HIPAR outputSource/demodulated signal (2 possible values) and the IF signal.
ViStatus _VI_FUNC rsemi_ConfigureIFOutputSource(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    outputSource
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_IF_OUTPUT_SOURCE,
                                                 "Output",
                                                 window - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSEMI_ATTR_IF_OUTPUT_SOURCE,
                                                outputSource), 3, "Output Source"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function turns the 28 V supply of the noise source on the front 
/// HIFN  panel on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR noiseSource/This control turns the 28 V supply of the noise source on the front 
/// HIPAR noiseSource/panel on and off.
ViStatus _VI_FUNC rsemi_ConfigureNoiseSource(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    noiseSource
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));  
    
    sprintf (buffer, "Win%ld", window);   
 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_NOISE_SOURCE_STATE,
                                                noiseSource), 3, "Noise Source"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects the unit for power in the selected measurement 
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR unitPower/Sets the unit for power in the selected measurement window.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ConfigureUnitPower(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    unitPower
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DISP_UNIT_POWER,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap), 2, "Window");
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_DISP_UNIT_POWER, 
                 unitPower), 3, "Unit Power");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This attribute duplicates the currently selected measurement channel, i.e 
/// HIFN  starts a new measurement channel of the same type and with the identical 
/// HIFN  measurement settings. The name of the new channel is the same as the copied channel, 
/// HIFN  extended by a consecutive number (e.g. "Spectrum" -> "Spectrum 2").
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_DuplicateMeasurementChannel (
    ViSession instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_DUPLICATE_MEASUREMENT_CHANNEL, 
                VI_NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This method replaces a measurement channel with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR oldChannelName/String containing the name of the measurement channel 
/// HIPAR oldChannelName/you want to replace.
/// HIPAR channelType/Channel type of the new channel.
/// HIPAR newChannelName/String containing the name of the new channel.
ViStatus _VI_FUNC rsemi_ReplaceMeasurementChannel (
    ViSession instrSession,
    ViString oldChannelName,
    ViInt32 channelType,
    ViString newChannelName
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (oldChannelName == VI_NULL) 
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Old Channel Name");    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, channelType, 0, 3), 3, "Channel Type"); 
    if (newChannelName == VI_NULL) 
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "New Channel Name");    
    
    checkErr (viPrintf (instrSession, "INST:CRE:REPL '%s',%s,'%s'\n", oldChannelName, channelTypeArr[channelType], newChannelName));
    
    checkErr( rsemi_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This method renames a measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR oldChannelName/String containing the name of the channel you want to rename. 
/// HIPAR newChannelName/String containing the new channel name.
ViStatus _VI_FUNC rsemi_RenameMeasurementChannel (
    ViSession instrSession,
    ViString oldChannelName,
    ViString newChannelName
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (oldChannelName == VI_NULL) 
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Old Channel Name");    
    if (newChannelName == VI_NULL) 
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "New Channel Name");    
    
    checkErr (viPrintf (instrSession, "INST:REN '%s','%s'\n", oldChannelName, newChannelName));
    
    checkErr( rsemi_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function restores the default instrument settings in the current channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_PresetMeasurementChannel (
    ViSession instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL)); 
    
    viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_PRESET_MEASUREMENT_CHANNEL, 
                 NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function queries all active measurement channels. This is
/// HIFN  useful in order to obtain the names of the existing measurement
/// HIFN  channels, which are required in order to replace or delete the
/// HIFN  channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_QueryAllMeasurementChannels (
    ViSession instrSession,
    ViInt32 bufferSize,
    ViChar channelList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32    count = 0;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (viPrintf(instrSession, ":INST:LIST?\n"));

    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &count)); 
    checkErr( rsemi_CheckStatus (instrSession));

    if (count>=(ViUInt32)bufferSize)
    {
        strncpy(channelList, pbuffer, (size_t) (bufferSize-1));
        channelList[bufferSize-1]='\0';
        error = (ViStatus) (count+1);
    } 
    else
    {
        strncpy(channelList, pbuffer, (size_t) count); 
        channelList[count]='\0';   
    }
    
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function selects the scope of the output settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/This control selects the output.
/// HIPAR coupling/is control selects the scope of the output settings.
ViStatus _VI_FUNC rsemi_ConfigureOutputCoupling (ViSession instrSession,
                                                 ViInt32 output,
                                                 ViBoolean coupling)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_LINK,
                                                 "Output",
                                                 output - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LINK, 
                 coupling), 3, "Coupling");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures output voltage.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/This control selects the output.
/// HIPAR lowerVoltage/Lower value of the voltage range.
/// HIPAR upperVoltage/Upper value of the voltage range
ViStatus _VI_FUNC rsemi_ConfigureOutputVoltage (ViSession instrSession,
                                                ViInt32 output,
                                                ViReal64 lowerVoltage,
                                                ViReal64 upperVoltage)
{
    ViStatus    error = VI_SUCCESS;

    viCheckParm (rsemi_invalidViInt32Range (instrSession, output, 1, 2), 2, "Output");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, lowerVoltage, -1.5, -0.5), 3, "Lower Voltage");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, upperVoltage, 0.5, 1.5), 4, "Upper Voltage");
    viCheckErr (viPrintf (instrSession, "OUTP%ld:IF:VOLT %.12lG,%.12lG\n", output, lowerVoltage, upperVoltage));
    
    checkErr( rsemi_CheckStatus (instrSession));   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function defines the AC cutoff frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/This control selects the output.
/// HIPAR frequency/This control sets the AC cutoff frequency.
ViStatus _VI_FUNC rsemi_ConfigureACCutoffFrequency (ViSession instrSession,
                                                    ViInt32 output,
                                                    ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_AC_CUTOFF_FREQUENCY,
                                                 "Output",
                                                 output - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_AC_CUTOFF_FREQUENCY, 
                 frequency), 3, "Frequency");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures output for IF signal
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/This control selects the output.
/// HIPAR frequency/Defines the frequency for the IF output.
/// HIPAR lowerVoltage/Lower value of the voltage range.
/// HIPAR upperVoltage/Upper value of the voltage range.
ViStatus _VI_FUNC rsemi_ConfigureIFOutput (ViSession instrSession,
                                           ViInt32 output, 
                                           ViReal64 frequency,
                                           ViReal64 lowerVoltage,
                                           ViReal64 upperVoltage)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    viCheckParm (rsemi_invalidViReal64Range (instrSession, lowerVoltage, -1.5, -0.5), 4, "Lower Voltage");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, upperVoltage, 0.5, 1.5), 5, "Upper Voltage");
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_IF_OUTPUT_FREQUENCY,
                                                 "Output",
                                                 output - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_IF_OUTPUT_FREQUENCY, 
                 frequency), 3, "Frequency");
    
    
    viCheckErr (rsemi_ConfigureOutputVoltage(instrSession,  output, lowerVoltage,  upperVoltage));
    
    checkErr( rsemi_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures output for video signal
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/This control selects the output.
/// HIPAR mode/This control selects the mode by which the gain is applied to
/// HIPAR mode/the output signal.
/// HIPAR coupling/This control selects the output coupling type.
/// HIPAR gainMode/This control selects the mode by which the gain is applied to
/// HIPAR gainMode/the output signal.
/// HIPAR gainValue/This control defines the gain level by which the output signal
/// HIPAR gainValue/is amplified.
/// HIPAR ACCutoffFrequency/This control sets the AC cutoff frequency.
/// HIPAR lowpassState/This control turns a low pass filter to control the output on
/// HIPAR lowpassState/and off.
/// HIPAR lowpassFrequency/This control defines the cutoff frequency of the low pass filter
/// HIPAR lowpassFrequency/available for signal output.
/// HIPAR lowerVoltage/Lower value of the voltage range.
/// HIPAR upperVoltage/Upper value of the voltage range. 
/// HIPAR phones/This control turns additional signal output on the headphone
/// HIPAR phones/jack on and off.
ViStatus _VI_FUNC rsemi_ConfigureVideoOutput (ViSession instrSession,
                                              ViInt32 output, 
                                              ViInt32 mode,
                                              ViInt32 coupling, 
                                              ViInt32 gainMode,
                                              ViReal64 gainValue,
                                              ViReal64 ACCutoffFrequency,
                                              ViBoolean lowpassState, 
                                              ViReal64 lowpassFrequency,
                                              ViReal64 lowerVoltage,
                                              ViReal64 upperVoltage,
                                              ViBoolean phones)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    viCheckParm (rsemi_invalidViReal64Range (instrSession, lowerVoltage, -1.5, -0.5), 10, "Lower Voltage");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, upperVoltage, 0.5, 1.5), 11, "Upper Voltage");
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_MODE,
                                                 "Output",
                                                 output - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_MODE, 
                 mode), 3, "Mode");
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_COUPLING, 
                 coupling), 4, "Coupling");
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_GAIN_MODE, 
                 gainMode), 5, "Gain Mode");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_GAIN_VALUE, 
                 gainValue), 6, "Gain Value");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_AC_CUTOFF_FREQUENCY, 
                 ACCutoffFrequency), 7, "AC Cutoff Frequency");
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_ENABLED, 
                 lowpassState), 8, "Lowpass State");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_FREQUENCY, 
                 lowpassFrequency), 9, "AC Lowpass Frequency");    
    viCheckErr (rsemi_ConfigureOutputVoltage(instrSession,  output, lowerVoltage,  upperVoltage));  
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_PHONES_ENABLED, 
                 phones), 12, "Phones");
        
    checkErr( rsemi_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures output for AM signal
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/This control selects the output.
/// HIPAR scale/This control defines the scale of the transferred signal.
/// HIPAR ACCutoffFrequency/This control sets the AC cutoff frequency.
/// HIPAR lowpassState/This control turns a low pass filter to control the output on
/// HIPAR lowpassState/and off.
/// HIPAR lowpassFrequency/This control defines the cutoff frequency of the low pass filter
/// HIPAR lowpassFrequency/available for signal output.
/// HIPAR lowerVoltage/Lower value of the voltage range.
/// HIPAR upperVoltage/Upper value of the voltage range
/// HIPAR phones/This control turns additional signal output on the headphone
/// HIPAR phones/jack on and off.
ViStatus _VI_FUNC rsemi_ConfigureAMOutput (ViSession instrSession,
                                           ViInt32 output, 
                                           ViReal64 scale,
                                           ViReal64 ACCutoffFrequency,
                                           ViBoolean lowpassState,
                                           ViReal64 lowpassFrequency,
                                           ViReal64 lowerVoltage,
                                           ViReal64 upperVoltage,
                                           ViBoolean phones)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    viCheckParm (rsemi_invalidViReal64Range (instrSession, lowerVoltage, -1.5, -0.5), 7, "Lower Voltage");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, upperVoltage, 0.5, 1.5), 8, "Upper Voltage");

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_SCALE,
                                                 "Output",
                                                 output - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_SCALE, 
                 scale), 3, "Scale");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_AC_CUTOFF_FREQUENCY, 
                 ACCutoffFrequency), 4, "AC Cutoff Frequency");
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_ENABLED, 
                 lowpassState), 5, "Lowpass State");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_FREQUENCY, 
                 lowpassFrequency), 6, "AC Lowpass Frequency");    
    viCheckErr (rsemi_ConfigureOutputVoltage(instrSession,  output, lowerVoltage,  upperVoltage));  
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_PHONES_ENABLED, 
                 phones), 9, "Phones");
        
    checkErr( rsemi_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures output for FM signal
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/This control selects the output.
/// HIPAR coupling/This control selects the output coupling type.
/// HIPAR scale/This control defines the scale of the transferred signal.
/// HIPAR ACCutoffFrequency/This control sets the AC cutoff frequency.
/// HIPAR lowpassState/This control turns a low pass filter to control the output on
/// HIPAR lowpassState/and off.
/// HIPAR lowpassFrequency/This control defines the cutoff frequency of the low pass filter
/// HIPAR lowpassFrequency/available for signal output.
/// HIPAR lowerVoltage/Lower value of the voltage range.
/// HIPAR upperVoltage/Upper value of the voltage range
/// HIPAR phones/This control turns additional signal output on the headphone
/// HIPAR phones/jack on and off.
ViStatus _VI_FUNC rsemi_ConfigureFMOutput (ViSession instrSession,
                                           ViInt32 output, 
                                           ViInt32 coupling,
                                           ViReal64 scale,
                                           ViReal64 ACCutoffFrequency,
                                           ViBoolean lowpassState, 
                                           ViReal64 lowpassFrequency,
                                           ViReal64 lowerVoltage,
                                           ViReal64 upperVoltage,
                                           ViBoolean phones)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    viCheckParm (rsemi_invalidViReal64Range (instrSession, lowerVoltage, -1.5, -0.5), 8, "Lower Voltage");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, upperVoltage, 0.5, 1.5), 9, "Upper Voltage");
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_COUPLING,
                                                 "Output",
                                                 output - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_COUPLING, 
                 coupling), 3, "Coupling");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_SCALE, 
                 scale), 4, "Scale");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_AC_CUTOFF_FREQUENCY, 
                 ACCutoffFrequency), 5, "AC Cutoff Frequency");
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_ENABLED, 
                 lowpassState), 6, "Lowpass State");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_FREQUENCY, 
                 lowpassFrequency), 7, "AC Lowpass Frequency");    
    viCheckErr (rsemi_ConfigureOutputVoltage(instrSession,  output, lowerVoltage,  upperVoltage));  
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_PHONES_ENABLED, 
                 phones), 10, "Phones");
        
    checkErr( rsemi_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures output for PM signal
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/This control selects the output.
/// HIPAR coupling/This control selects the output coupling type.
/// HIPAR scale/This control defines the scale of the transferred signal.
/// HIPAR ACCutoffFrequency/This control sets the AC cutoff frequency.
/// HIPAR lowpassState/This control turns a low pass filter to control the output on
/// HIPAR lowpassState/and off.
/// HIPAR lowpassFrequency/This control defines the cutoff frequency of the low pass filter
/// HIPAR lowpassFrequency/available for signal output.
/// HIPAR lowerVoltage/Lower value of the voltage range.
/// HIPAR upperVoltage/Upper value of the voltage range
/// HIPAR phones/This control turns additional signal output on the headphone
/// HIPAR phones/jack on and off.
ViStatus _VI_FUNC rsemi_ConfigurePMOutput (ViSession instrSession,
                                           ViInt32 output, ViInt32 coupling,
                                           ViReal64 scale,
                                           ViReal64 ACCutoffFrequency,
                                           ViBoolean lowpassState,
                                           ViReal64 lowpassFrequency,
                                           ViReal64 lowerVoltage,
                                           ViReal64 upperVoltage,
                                           ViBoolean phones)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    viCheckParm (rsemi_invalidViReal64Range (instrSession, lowerVoltage, -1.5, -0.5), 8, "Lower Voltage");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, upperVoltage, 0.5, 1.5), 9, "Upper Voltage");
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_COUPLING,
                                                 "Output",
                                                 output - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_COUPLING, 
                 coupling), 3, "Coupling");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_SCALE, 
                 scale), 4, "Scale");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_AC_CUTOFF_FREQUENCY, 
                 ACCutoffFrequency), 5, "AC Cutoff Frequency");
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_ENABLED, 
                 lowpassState), 6, "Lowpass State");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_FREQUENCY, 
                 lowpassFrequency), 7, "AC Lowpass Frequency");    
    viCheckErr (rsemi_ConfigureOutputVoltage(instrSession,  output, lowerVoltage,  upperVoltage));  
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_PHONES_ENABLED, 
                 phones), 10, "Phones");
        
    checkErr( rsemi_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures the low pass filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/This control selects the output.
/// HIPAR state/This control turns a low pass filter to control the output on
/// HIPAR state/and off.
/// HIPAR frequency/This control defines the cutoff frequency of the low pass filter
/// HIPAR frequency/available for signal output.
ViStatus _VI_FUNC rsemi_ConfigureLowPassFilter (ViSession instrSession,
                                                ViInt32 output, 
                                                ViBoolean state,
                                                ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_ENABLED,
                                                 "Output",
                                                 output - 1,
                                                 10,
                                                 rep_cap));
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_ENABLED, 
                                                                state), 3, "State");
    if (state)
    {
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_LOW_PASS_FILTER_FREQUENCY, 
                                                                frequency),4, "Frequency");
    }
                                             
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function queries the sideband provided at the IF OUT
/// HIFN connector compared to the sideband of the RF signal. The
/// HIFN sideband depends on the current center frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sideband/Returns the sideband type.
ViStatus _VI_FUNC rsemi_QueryIFOutputSideband (ViSession instrSession,
                                               ViInt32 *sideband)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeViInt32 (instrSession, "", RSEMI_ATTR_IF_OUTPUT_SIDEBAND, 
                                                                sideband), 2, "Sideband");
                                   
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function configures the trigger direction as input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR port/Selects the trigger port.
ViStatus _VI_FUNC rsemi_ConfigureTriggerInput (ViSession instrSession,
                                               ViInt32 port)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_TRIGGER_DIRECTION,
                                                 "TriggerPort",
                                                 port - 1,
                                                 10,
                                                 rep_cap));
    viCheckErr (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_TRIGGER_DIRECTION, 
                                                                RSEMI_VAL_DIRECTION_INPUT));
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function selects the trigger output type, length and if
/// HIFN user defined output type is selected, also a level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR port/Selects the trigger port.
/// HIPAR outputType/Selects the type of signal generated at the trigger output.
/// HIPAR level/Selects signal level.
/// HIPAR length/Pulse length in seconds.
ViStatus _VI_FUNC rsemi_ConfigureTriggerOutput (ViSession instrSession,
                                                ViInt32 port, 
                                                ViInt32 outputType,
                                                ViInt32 level, 
                                                ViReal64 length)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_TRIGGER_DIRECTION,
                                                 "TriggerPort",
                                                 port - 1,
                                                 10,
                                                 rep_cap));
    viCheckErr (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_TRIGGER_DIRECTION, 
                                                                RSEMI_VAL_DIRECTION_OUTPUT));
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_TRIGGER_TYPE, 
                                                                outputType), 3, "Output Type");
    if (outputType == RSEMI_VAL_TRIG_OTYPE_UDEFINED)
    {
        viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_TRIGGER_LEVEL, 
                                                                level), 4, "Level");
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_TRIGGER_PULSE_LENGTH, 
                                                                length), 5, "Length");
    }
    
    
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}


/// HIFN This function generates a pulse at the trigger output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR port/Selects the trigger port. 
ViStatus _VI_FUNC rsemi_GenerateTriggerPulse (ViSession instrSession,
                                              ViInt32 port)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_TRIGGER_PULSE,
                                                 "TriggerPort",
                                                 port - 1,
                                                 10,
                                                 rep_cap));
    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_TRIGGER_PULSE, 
                                                                VI_NULL));
    
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function selects the probe connector that is supplied with power
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR probePower/Selects the probe connector that is supplied with power.
ViStatus _VI_FUNC rsemi_ConfigureProbePowerSupply (ViSession instrSession,
                                                   ViInt32 probePower)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    switch (probePower){
        case RSEMI_VAL_PROBE_OFF:
            viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_PROBE_POWER_ENABLED,
                                                 "Probe",
                                                 0,
                                                 10,
                                                 rep_cap));
            viCheckErr (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_PROBE_POWER_ENABLED, 
                                                                VI_FALSE));
        break;
        case RSEMI_VAL_PROBE_1:
            viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_PROBE_POWER_ENABLED,
                                                 "Probe",
                                                 0,
                                                 10,
                                                 rep_cap));
            viCheckErr (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_PROBE_POWER_ENABLED, 
                                                                VI_TRUE));
        break;
        case RSEMI_VAL_PROBE_2:
            viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SIGNAL_OUTPUT_PROBE_POWER_ENABLED,
                                                 "Probe",
                                                 1,
                                                 10,
                                                 rep_cap));
            viCheckErr (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_SIGNAL_OUTPUT_PROBE_POWER_ENABLED, 
                                                                VI_TRUE));
        break;
        default:
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 2, "Probe Power");
    }
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function configures the trace to acquire.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR traceType/The type of trace to take. The driver sets the
ViStatus _VI_FUNC rsemi_ConfigureTrace(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViInt32    traceType
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");
    
    sprintf (buffer, "Win%ld,TR%ld", window, trace); 
    
    
    viCheckErr ( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                        RSEMI_ATTR_TRACE_STATE,
                                                        VI_TRUE));
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_TRACE_TYPE,
                                                traceType), 4, "Trace Type"); 
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the trace detector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR detectorTypeAuto/Turns auto detector on/off.
/// HIPAR detectorType/Specifies the method of capturing and processing
ViStatus _VI_FUNC rsemi_ConfigureTraceDetector(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViBoolean    detectorTypeAuto,
    ViInt32    detectorType
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");
    sprintf (buffer, "Win%ld,TR%ld", window, trace); 
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_DETECTOR_TYPE_AUTO,
                                                detectorTypeAuto), 4, "Detector Type Auto");
    if (detectorTypeAuto == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_DETECTOR_TYPE,
                                                detectorType), 5, "Detector Type"); 
    }
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function specifies whether or not the traces with peak or minimum 
/// HIFN  value detection are reset after specific parameter changes.
/// HIFN     Usually the measurement must be restarted after a parameter change, 
/// HIFN  before an evaluation of the measurement results is performed (e.g. with 
/// HIFN  a marker). In cases in which a change causes a compulsory new 
/// HIFN  measurement, the trace is automatically reset in order to prevent 
/// HIFN  erroneous measurements of previous measurement results (e.g. when the 
/// HIFN  span changes). For applications in which this behavior is not desired, 
/// HIFN  this mechanism can be switched off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR resetatChange/Specifies whether or not the traces with peak or minimum value 
/// HIPAR resetatChange/detection are reset after specific parameter changes.
ViStatus _VI_FUNC rsemi_ConfigureTraceResetBehavior(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViBoolean    resetatChange
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");
    sprintf (buffer, "Win%ld,TR%ld", window, trace); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_TRACE_RESET_BEHAVIOR,
                                                resetatChange), 4, "Reset At Change"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the data to be included in a data export file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR exportMode/Selects the data to be included in a data export file.
/// HIPAR exportHeader/If enabled, additional instrument and measurement settings are 
/// HIPAR exportHeader/included in the header of the export file for result data. If disabled, 
/// HIPAR exportHeader/only the pure result data from the selected traces and tables is 
/// HIPAR exportHeader/exported.
/// HIPAR decimalSeparator/This control sets the decimal separator for the ASCII data
/// HIPAR decimalSeparator/stored to the file.
ViStatus _VI_FUNC rsemi_ConfigureTraceExport (ViSession instrSession,
                                              ViInt32 exportMode,
                                              ViBoolean exportHeader,
                                              ViInt32 decimalSeparator)
{
    ViStatus    error   = VI_SUCCESS; 
    
    checkErr( Rs_LockSession (instrSession, NULL)); 
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_TRACE_EXPORT_ALL, 
                exportMode), 2, "Export Mode");
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_TRACE_EXPORT_HEADER, 
                exportHeader), 3, "Export Header");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_FILE_DEC_SEPARATOR, 
                decimalSeparator), 4, "Decimal Separator");

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}                           

/// HIFN  This function stores the selected trace (1 to 3) in the measurement 
/// HIFN  window indicated by window in a file with ASCII format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR fileName/Specifies full path of the file, where trace data will be stored.
ViStatus _VI_FUNC rsemi_StoreSANTraceToFile(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViString    fileName
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");
    if (fileName == VI_NULL) 
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "File Name");

    viCheckErr (viPrintf (instrSession,"MMEM:STOR%ld:TRAC %ld,'%s'\n", window, trace, fileName));
        
    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the settings of the analyzer hardware for the 
/// HIFN  measurement of I/Q data. This allows setting the bandwidth of the 
/// HIFN  analog filters in front of the A/D converter as well as setting the 
/// HIFN  sample rate, trigger conditions and the record length.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) Function Configure Trace IQ Data Acquisition 
/// HIFN  (rsemi_ConfigureTraceIQDataAcquisition) has to be switched ON before 
/// HIFN  using this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR samplingRate/This control sets the sampling rate for the I/Q data acquisition.
/// HIPAR triggerMode/This parameter selects the trigger source for starting data 
/// HIPAR triggerMode/acquisition.
/// HIPAR triggerSlope/This parameter selects the slope of the trigger signal for external 
/// HIPAR triggerSlope/trigger.
/// HIPAR bandwidth/This control sets bandwidth of the analog filters in front of the A/D 
/// HIPAR bandwidth/converter.
/// HIPAR pretriggerSamples/This control sets the number of measurement values to be recorded 
/// HIPAR pretriggerSamples/before the trigger point.
/// HIPAR noofSamples/This control sets the number of measurement values to record.
ViStatus _VI_FUNC rsemi_TraceIQSet(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    samplingRate,
    ViInt32    triggerMode,
    ViInt32    triggerSlope,
    ViReal64    bandwidth,
    ViInt32    pretriggerSamples,
    ViInt32    noofSamples
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      cmd[RSEMI_IO_BUFFER_SIZE] = "";
     
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (rsemi_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSEMI_IO_BUFFER_SIZE, buffer));
    
    viCheckParm(rsemi_invalidViInt32Range (instrSession, triggerMode, 0, 4), 4, "Trigger Mode");
    if (triggerMode==2)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Trigger Mode");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, triggerSlope, 0, 1), 5, "Trigger Slope");  
    
    sprintf (cmd, "TRAC%ld:IQ:SET NORM,%.12lG,%.12lG,%s,%s,%ld,%ld\n",
            window, bandwidth, samplingRate, rsemi_rngTriggerSource.rangeValues[triggerMode].cmdString,
            rsemi_rngPolarity.rangeValues[triggerSlope].cmdString, pretriggerSamples, noofSamples);
    
    viCheckErr (viWrite(instrSession, (ViBuf)cmd, (ViUInt32)strlen(cmd), NULL));
    
    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the sampling rate for the I/Q data acquisition. 
/// HIFN  Thus the sampling rate can be modified without affecting the other 
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR samplingRate/This control sets the sampling rate for the I/Q data acquisition.
ViStatus _VI_FUNC rsemi_TraceIQSamplingRate(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    samplingRate
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_IQ_SAMPLE_RATE, 
                                                samplingRate), 3, "Sampling Rate");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the record length for the acquired I/Q data. 
/// HIFN  Increasing the record length automatically also increases the 
/// HIFN  measurement time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR recordLength/This control sets the number of measurement values to record.
ViStatus _VI_FUNC rsemi_TraceIQRecordLength(
    ViSession    instrSession,
    ViInt32    recordLength
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "Win1",
        RSEMI_ATTR_IQ_RECORD_LENGTH, recordLength), 2, "Record Length");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the IQ data measurement averaging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR averageState/This command switches the averaging function for the recorded I/Q data 
/// HIPAR averageState/on or off. Preconditions are that the I/Q data recording function has 
/// HIPAR averageState/been switched on previously and that the sampling rate has been set to 
/// HIPAR averageState/32 MHz. The driver uses this value to set the 
/// HIPAR averageState/RSEMI_ATTR_IQ_DATA_AVER_STATE attribute.
/// HIPAR averageCount/This command defines the number of averages to perform on the recorded 
/// HIPAR averageCount/I/Q data. The driver uses this value to set the 
/// HIPAR averageCount/RSEMI_ATTR_IQ_DATA_AVER_COUNT attribute.
ViStatus _VI_FUNC rsemi_TraceIQAveraging(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    averageState,
    ViInt32    averageCount
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_IQ_DATA_AVER_STATE, averageState),
                                                3, "Average State");
    if (averageState == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_IQ_DATA_AVER_COUNT, averageCount),
                                                4, "Average Count");    
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  Enables or disables the IQ data evaluation mode and is a prerequisite 
/// HIFN  for the IQ Analyzer functions.
/// HIFN     Before this control can be set, IQ data acquistion must be enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR iqDataAnalyzerEnabled/Enables or disables the IQ data evaluation mode and is a prerequisite 
/// HIPAR iqDataAnalyzerEnabled/for the IQ Analyzer functions.
ViStatus _VI_FUNC rsemi_ConfigureTraceIQDataAnalyzerEnabled(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    iqDataAnalyzerEnabled
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm(rsemi_SetAttributeViBoolean(instrSession, buffer,
        RSEMI_ATTR_IQ_DATA_ANALYZER_STATE, iqDataAnalyzerEnabled), 3, "IQ Data Analyzer Enabled"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the trace IQ data acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR traceState/This control switches IQ data acquisition on or off. The driver uses 
/// HIPAR traceState/this value to set the RSEMI_ATTR_IQ_DATA_STATE attribute.
ViStatus _VI_FUNC rsemi_ConfigureTraceIQDataAcquisition(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    traceState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_IQ_DATA_STATE, traceState),
                                                3, "Trace State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the gate function for IQ data measurement.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     
/// HIFN     (1) This function is only available for R&S FSV instrument.
/// HIFN     
/// HIFN     (2) This function is only valid for TRAC:IQ ON and trigger modes "EXT" 
/// HIFN  and "IFP".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gate/This control activates or deactivates the gate function for IQ data 
/// HIPAR gate/measurements.
/// HIPAR type/This control specifies the gate trigger type used for data collection.
/// HIPAR gateLength/This control defines the gate length in samples in edge mode.
/// HIPAR gateGap/This control defines the interval between several gate periods in 
/// HIPAR gateGap/samples.
/// HIPAR numberOfGatePeriods/This control defines the number of gate periods after a trigger 
/// HIPAR numberOfGatePeriods/signal.
ViStatus _VI_FUNC rsemi_ConfigureTraceIQGate(
    ViSession    instrSession,
    ViBoolean    gate,
    ViInt32    type,
    ViInt32    gateLength,
    ViInt32    gateGap,
    ViInt32    numberOfGatePeriods
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
                                                 RSEMI_ATTR_IQ_DATA_GATE_STATE, 
                                                 gate), 2, "Gate");
                                                 
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_IQ_DATA_GATE_TYPE, 
                                               type), 3, "Type");
                                                 
    if (type == RSEMI_VAL_GATE_TYPE_EDGE)
    {
        viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                                   RSEMI_ATTR_IQ_DATA_GATE_LENGTH, 
                                                   gateLength), 4, "Gate Length");
    }
                                                 
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_IQ_DATA_GATE_GAP, 
                                               gateGap), 5, "Gate Gap");
                                                 
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_IQ_DATA_GATE_NUMBER_OF_PERIODS, 
                                               numberOfGatePeriods), 6, "Number Of Gate Periods");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the display type of the IQ data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayType/This control defines the display type of the IQ data.
ViStatus _VI_FUNC rsemi_ConfigureTraceIQDataDisplayType(
    ViSession    instrSession,
    ViInt32    displayType
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_IQ_DATA_DISPLAY_TYPE, 
                                               displayType), 2, "Display Type");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the IQ bandwidth 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Configures the bandwidth mode.
/// HIPAR resolution/Sets the bandwidth resolution in manual mode.
ViStatus _VI_FUNC rsemi_ConfigureIQBandwidth (
    ViSession instrSession, 
    ViInt32 mode,
    ViReal64 resolution
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_IQ_BANDWIDTH_MODE, 
                                               mode), 2, "Mode");
    if (mode == RSEMI_VAL_IQ_BWID_MODE_MAN) 
                viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                                               RSEMI_ATTR_IQ_BANDWIDTH_RESOLUTION, 
                                               resolution), 3, "Resolution");

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/// HIFN  This function activates the bandwidth extension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR maximumBWExtension/Activates the bandwidth extension.
/// HIPAR mode/Defines the maximum analysis bandwidth for maximum allowed bandwidth.
/// HIPAR bandwidth/Defines the maximum analysis bandwidth.
ViStatus _VI_FUNC rsemi_ConfigureIQMaximumBW (
    ViSession instrSession,
    ViBoolean maximumBWExtension, 
    ViInt32 mode,
    ViReal64 bandwidth
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_IQ_MAX_BANDWIDTH_EXTENSION, 
                                               maximumBWExtension), 2, "Maximum BW Extension");
    if (maximumBWExtension == VI_TRUE)
    {
        if (mode == RSEMI_VAL_MAN)
        {
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                                                   RSEMI_ATTR_IQ_MAX_BANDWIDTH_VALUE, 
                                                   bandwidth), 4, "Bandwidth");
        }
        else
        {
            viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                                       RSEMI_ATTR_IQ_MAX_BANDWIDTH_MAX, 
                                                       mode), 3, "Mode");            
        }
    }

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/// HIFN  This function configures "Advanced Fourier Transformation" parameters 
/// HIFN  for I/Q analyzer 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR algorithm/Defines the FFT calculation method.
/// HIPAR length/Defines the number of frequency points determined by each FFT calculation.
/// HIPAR windowType/Defines the window type for I/Q analyzer.
/// HIPAR windowOverlap/Defines the part of a single FFT window that is re-calculated by
/// HIPAR windowOverlap/the next FFT calculation.
/// HIPAR windowLength/Defines the number of samples to be included in a single FFT
/// HIPAR windowLength/window when multiple FFT windows are used.
ViStatus _VI_FUNC rsemi_ConfigureIQFFTMode (
    ViSession instrSession, 
    ViInt32 algorithm,
    ViInt32 length, 
    ViInt32 windowType,
    ViReal64 windowOverlap, 
    ViInt32 windowLength
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_IQ_FFT_ALGORITHM, 
                                               algorithm), 2, "Algorithm");
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_IQ_FFT_LENGTH, 
                                               length), 3, "Length");
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_IQ_FFT_WINDOW_TYPE, 
                                               windowType), 4, "Window Type");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                                               RSEMI_ATTR_IQ_FFT_WIN_OVERLAP, 
                                               windowOverlap), 5, "Window Overlap");
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_IQ_FFT_WIN_LENGTH, 
                                               windowLength), 6, "Window Length");

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;  
}

/// HIFN  This function configures the average power consumption calculated at the end 
/// HIFN  of I/Q data measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Enables/disables the average power consumption calculation at the end 
/// HIPAR state/of the I/Q data measurement.
/// HIPAR factorA/Defines the conversion factor A.
/// HIPAR factorB/Defines the conversion factor B.
ViStatus _VI_FUNC rsemi_ConfigureIQAveragePowerConsumption (
    ViSession instrSession,
    ViBoolean state,
    ViReal64 factorA,
    ViReal64 factorB
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_IQ_APC_STATE, 
                                               state), 2, "State");
    if (state == VI_TRUE)
    {
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                                               RSEMI_ATTR_IQ_APC_FACTOR_A, 
                                               factorA), 3, "Factor A");
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                                               RSEMI_ATTR_IQ_APC_FACTOR_B, 
                                               factorB), 4, "Factor B");
    }

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/// HIFN  This function queries the average power consumption for an analog baseband input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR result/Queries the average power consumption for an analog baseband input.
ViStatus _VI_FUNC rsemi_QueryIQAveragePowerConsumptionResult (
    ViSession instrSession,
    ViReal64 *result
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, "", 
                                               RSEMI_ATTR_IQ_APC_RESULT, 
                                               result), 2, "Result");

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;  
}

/// HIFN  This function queries the filter bandwidth of the resampling filter, 
/// HIFN  which is dependent on the sampling rate. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterBandwidth/This control queries the filter bandwidth of the resampling filter.
ViStatus _VI_FUNC rsemi_QueryTraceIQFilterBandwidth(
    ViSession    instrSession,
    ViReal64*    filterBandwidth
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, "", 
                                               RSEMI_ATTR_IQ_DATA_FILTER_BANDWIDTH, 
                                               filterBandwidth), 2, "Filter Bandwidth");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function swaps the real (I branch) and the imaginary (Q branch) 
/// HIFN  parts of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR swapIQ/Swaps the real (I branch) and the imaginary (Q branch) parts of the signal.
ViStatus _VI_FUNC rsemi_TraceIQSwapIQ (
    ViSession instrSession, 
    ViBoolean swapIQ
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                                RSEMI_ATTR_IQ_SWAP_IQ, swapIQ),
                                                2, "Swap IQ");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function loads the IQ data from the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file, the IQ data are load from.
ViStatus _VI_FUNC rsemi_LoadIQData(
    ViSession    instrSession,
    ViString    fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "",
                                                RSEMI_ATTR_LOAD_IQ_DATA, fileName), 
                                                2, "File Name");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function stores the IQ data from/to the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file, where IQ data are stored.
ViStatus _VI_FUNC rsemi_StoreIQData(
    ViSession    instrSession,
    ViString    fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "",
                                                RSEMI_ATTR_SAVE_IQ_DATA, fileName), 
                                                2, "File Name");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines a description of the export file which is stored 
/// HIFN  with the data and also displayed in the file selection dialog box for 
/// HIFN  I/Q data import and export
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR description/Specifies a description of the export file.
ViStatus _VI_FUNC rsemi_ConfigureIQDataExportFileDescription(
    ViSession    instrSession,
    ViString    description
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "",
                    RSEMI_ATTR_IQ_DATA_EXPORT_FILE_DESCRIPTION, description), 
                                                2, "Description");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function copies one trace array to another trace array.  Any data 
/// HIFN  in the destination trace is over written. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR destinationTrace/Selects the destination trace.
/// HIPAR sourceTrace/Selects the source trace.
ViStatus _VI_FUNC rsemi_CopyTrace(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    destinationTrace,
    ViInt32    sourceTrace
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, destinationTrace, 1, 6), 3, "Destination Trace");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, sourceTrace, 1, 6), 4, "Source Trace");        
    
    viCheckErr(viPrintf(instrSession, "TRAC%ld:COPY TRACE%ld,TRACE%ld\n", window, 
    destinationTrace, sourceTrace));
    
    checkErr( rsemi_CheckStatus (instrSession));  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures subtraction of traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR position/Defines the position of the result of the trace mathematics in the 
/// HIPAR position/selected measurement window. The indication is in % of the screen 
/// HIPAR position/height, with 100% corresponding to the upper diagram border. The driver 
/// HIPAR position/uses this value to set the RSEMI_ATTR_TRACE_MATH_POSITION attribute.
/// HIPAR mode/Selects linear or logarithmic (= video) calculation of the 
/// HIPAR mode/mathematical functions related to the traces. The calculation of the 
/// HIPAR mode/average is one of the affected functions. The setting is valid for all 
/// HIPAR mode/measurement windows. The driver uses this value to set the 
/// HIPAR mode/RSEMI_ATTR_TRACE_MATH_MODE attribute.
/// HIPAR traceMathState/Switches the mathematical relation of traces on or off.
ViStatus _VI_FUNC rsemi_ConfigureSubtractTraces(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    position,
    ViInt32    mode,
    ViBoolean    traceMathState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     math_mode = RSEMI_VAL_LOG;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, mode, RSEMI_VAL_LIN, math_mode), 4, "Mode");
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_TRACE_MATH_POSITION, 
                    position), 3, "Position");
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_TRACE_MATH_MODE, 
                mode), 4, "Mode");
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_TRACE_MATH_STATE, 
                traceMathState), 5, "Trace Math State");
                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function subtracts the array elements of trace defined in 
/// HIFN  argument Trace 2 from TRACE1 of the instrument and stores the result in 
/// HIFN  the TRACE1.
/// HIFN     
/// HIFN     Destination Trace = TRACE1 - {Trace 2}
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace2/Selects the trace for subtraction.
ViStatus _VI_FUNC rsemi_SubtractTraces(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace2
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace2, 2, 6), 3, "Trace 2");
    
    viCheckErr(viPrintf(instrSession, "CALC%ld:MATH (TRACE1-TRACE%ld)\n", window, trace2));
                
    checkErr( rsemi_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function enables the active marker on the specified trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerEnabled/Specifies whether the specified delta marker is enabled. 
/// HIPAR trace/Selects the trace for Detector definition. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrumMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViBoolean    markerEnabled,
    ViInt32    trace
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[20] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");

    sprintf (rep_cap, "Win%ld,M%ld", window, marker);  

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSEMI_ATTR_SAN_MARKER_ENABLED,
                                                 markerEnabled), 4, "Marker Enabled");
    
    if (markerEnabled == VI_TRUE)
    {
    
        viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSEMI_ATTR_ASSIGN_MARKER_TRACE,
                                               trace), 5, "Trace");
    }
    

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function defines the reference marker for the selected delta 
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR referenceMarker/Defines the reference marker for the selected delta marker. 
ViStatus _VI_FUNC rsemi_ConfigureReferenceMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViInt32    referenceMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_REFERENCE_MARKER,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_REFERENCE_MARKER,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSEMI_ATTR_DMARKER_REFERENCE_MARKER,
                                               referenceMarker), 4, "Reference Marker");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function turns configures signal-tracking. 
/// HIFN     
/// HIFN     With signal track activated, the maximum signal is determined after 
/// HIFN  each frequency sweep and the center frequency is set to the frequency 
/// HIFN  of this signal. Thus with drifting signals the center frequency follows 
/// HIFN  the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR signalTrackEnabled/Switches the signal-track function on or off for the selected 
/// HIPAR signalTrackEnabled/measurement window. 
/// HIPAR signalTrackBandwidth/Defines the bandwidth around the center frequency within which the 
/// HIPAR signalTrackBandwidth/largest signal is searched. 
/// HIPAR signalTrackThreshold/Defines the threshold above which the largest signal is searched for 
/// HIPAR signalTrackThreshold/in the selected measurement window. 
/// HIPAR trace/Defines the trace on which the largest signal is searched for in the 
/// HIPAR trace/selected measurement window. 
ViStatus _VI_FUNC rsemi_ConfigureSignalTrack(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    signalTrackEnabled,
    ViReal64    signalTrackBandwidth,
    ViReal64    signalTrackThreshold,
    ViInt32    trace
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    buffer[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_SIGNAL_TRACK_ENABLED, 
                signalTrackEnabled), 3, "Signal Track Enabled");
    if (signalTrackEnabled == VI_TRUE)
    {                
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SIGNAL_TRACK_BWID, 
                    signalTrackBandwidth), 4, "Signal Track Bandwidth");
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SIGNAL_TRACK_THRESHOLD, 
                    signalTrackThreshold), 5, "Signal Track Threshold");
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_SIGNAL_TRACK_TRACE, 
                    trace), 6, "Trace");
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the minimum or maximum bandwidth around the center frequency 
/// HIFN  within which the largest signal is searched. It is only available in 
/// HIFN  the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minMax/This control defines minimum/maximum bandwidth around the center 
/// HIPAR minMax/frequency within which the largest signal is searched. It is only 
/// HIPAR minMax/available in the frequency domain (span > 0).
ViStatus _VI_FUNC rsemi_ConfigureSignalTrackBandwidthMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    minMax
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SIGNAL_TRACK_BWID_MIN_MAX,
                                                minMax), 3, "Min Max"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}


/// HIFN  This function queries minimal and maximal signal track bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minimum/Queries the minimum signal track bandwidth.
/// HIPAR maximum/Queries the maximum signal track bandwidth.
ViStatus _VI_FUNC rsemi_QuerySignalTrackBandwidthMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    minimum,
    ViReal64*    maximum
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SIGNAL_TRACK_BWID_GET_MIN,
                                                minimum), 3, "minimum"); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SIGNAL_TRACK_BWID_GET_MAX,
                                                maximum), 4, "maximum");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;     
}

/// HIFN  This function turns off all the markers in selected measurement 
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_DisableAllSpectrumMarkers(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    buffer[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "Win%ld", window); 
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_SAN_MARKER_AOFF, 
                VI_NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function specifies the type of marker search and performs the 
/// HIFN  search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
ViStatus _VI_FUNC rsemi_MarkerSearch(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViInt32    markerSearch
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker); 
    switch (markerSearch)
    {
            case RSEMI_VAL_MARKER_SEARCH_HIGHEST:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_PEAK, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_PEAK_NEXT, 
                                     VI_NULL));
            break;

            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_PEAK_LEFT, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_PEAK_RIGHT, 
                                     VI_NULL));
                         break;

            case RSEMI_VAL_MARKER_SEARCH_MINIMUM:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_SEARCH_MIN, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_SEARCH_MIN_NEXT, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_SEARCH_MIN_LEFT, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_SEARCH_MIN_RIGHT, VI_NULL));
            break;

            default: 
                viCheckParm ( RS_ERROR_INVALID_PARAMETER, 4, "Parameter to set");
            break;
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures an automatic peak search action
/// HIFN     for Marker 1 at the end of each particular sweep. This function may be 
/// HIFN  used during adjustments of a device under test to keep track of the 
/// HIFN  actual peak marker position and level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR markerSearch/Specifies the type of marker search type.
/// HIPAR autoPeak/Switches the peak auto search state. 
ViStatus _VI_FUNC rsemi_MarkerPeakAuto(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    markerSearch,
    ViBoolean    autoPeak
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    switch (markerSearch){
        case RSEMI_VAL_MARKER_SEARCH_HIGHEST:
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_PEAK_AUTO, 
                autoPeak), 4, "Auto Peak");
        break;
        case RSEMI_VAL_MARKER_SEARCH_MINIMUM:
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_MIN_AUTO, 
                autoPeak), 4, "Auto Peak");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Marker Search"); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function moves the active marker to the specified horizontal 
/// HIFN  position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerPosition/Horizontal position.
ViStatus _VI_FUNC rsemi_MoveSpectrumMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64    markerPosition
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker); 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SAN_MARKER_POSITION, 
                markerPosition), 4, "Marker Position");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function makes the selected marker frequency to be the center 
/// HIFN  frequency or step width of center frequency. It can also make the 
/// HIFN  active marker amplitude to be the reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR instrumentSetting/Specifies the instrument setting to be set from the marker position.
ViStatus _VI_FUNC rsemi_SetInstrumentFromMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViInt32    instrumentSetting
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");       
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);  
    switch(instrumentSetting)
    {
        case RSEMI_VAL_INSTRUMENT_SETTING_FREQUENCY_CENTER:
            viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                RSEMI_ATTR_MARKER_TO_CENTER, VI_NULL));
        break;
        
        case RSEMI_VAL_INSTRUMENT_SETTING_FREQUENCY_STEP:
            viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                RSEMI_ATTR_SAN_MARKER_TO_STEP, VI_NULL));
        break;
        case RSEMI_VAL_INSTRUMENT_SETTING_REFERENCE_LEVEL:
              viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                RSEMI_ATTR_MARKER_TO_REFERENCE, VI_NULL));
        break;
        
        default: 
            viCheckParm ( RS_ERROR_INVALID_PARAMETER, 4, "Parameter to set");
        break;
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function links normal marker <FirstMarker> to any active
/// HIFN normal marker <SecondMarker>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR firstMarker/This control selects first marker.
/// HIPAR secondMarker/This control selects second marker.
/// HIPAR state/Specifies the marker link state.
ViStatus _VI_FUNC rsemi_LinkMarkerToAnotherMarker (ViSession instrSession,
                                                   ViInt32 firstMarker,
                                                   ViInt32 secondMarker,
                                                   ViBoolean state)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      marker[10] = "";
    ViChar      marker2[10] = "";          
    ViChar         rep_cap[50] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MARKER_LINK_TO_MARKER,
                                                 "Marker",
                                                 firstMarker - 1,
                                                 10,
                                                 marker), 3, "First Marker");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MARKER_LINK_TO_MARKER,
                                                 "SecondMarker",
                                                 secondMarker - 1,
                                                 10,
                                                 marker2), 4, "Second Marker");
    sprintf(rep_cap,"%s,%s", marker, marker2);
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap, 
                                                RSEMI_ATTR_MARKER_LINK_TO_MARKER, 
                                                state), 5, "State");
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function defines the range to be zoomed around marker 1 in the 
/// HIFN  selected measurement window. Marker 1 is activated first, if necessary.
/// HIFN     The subsequent frequency sweep is stopped at the marker position and 
/// HIFN  the frequency of the signal is counted. This frequency becomes the new 
/// HIFN  center frequency, and the zoomed span is set. In order to recognize the 
/// HIFN  end of the operation the synchronization to the sweep end should be 
/// HIFN  activated. This is only possible in single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerZoom/Defines the range to be zoomed around marker 1 in the selected 
/// HIPAR markerZoom/measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_ConfigureSpectrumMarkerZoom(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64    markerZoom,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        

    checkErr (rsemi_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rsemi_SetOPCTimeout (instrSession, timeout));

    sprintf (buffer, "Win%ld,M%ld", window, marker);  
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_MARKER_ZOOM, 
                markerZoom), 3, "Marker Zoom");
    
Error:
    if (old_timeout >= 0)    
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches on and off a link of delta marker 1 to marker 
/// HIFN  1. If the link is switched on and the x-axis value of the marker1 is 
/// HIFN  changed the delta marker 1 will follow on
/// HIFN     the same x-position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR state/Specifies the marker and delta marker link state.
ViStatus _VI_FUNC rsemi_LinkSpectrumMarkerAndDeltamarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_SAN_MARKER_LINK, 
                state), 4, "State");
   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets whether all markers will be linked across all 
/// HIFN  display windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR linkMarkers/Sets whether all markers will be linked across all display windows.
ViStatus _VI_FUNC rsemi_LinkAllMarkers(
    ViSession    instrSession,
    ViInt32    linkMarkers
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                                 RSEMI_ATTR_MARKER_LINK_ALL_MARKERS,
                                                 linkMarkers), 2, "Link Markers");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function returns the horizontal position and the marker amplitude 
/// HIFN  level of the selected marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerPosition/The horizontal position of the selected marker.
/// HIPAR markerAmplitude/The amplitude value of the selected marker (Amplitude Units or 
/// HIPAR markerAmplitude/relative amplitude units for delta markers).
ViStatus _VI_FUNC rsemi_QuerySpectrumMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64*    markerPosition,
    ViReal64*    markerAmplitude
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker); 
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SAN_MARKER_POSITION, 
                markerPosition), 4, "Marker Position");
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SAN_MARKER_AMPLITUDE, 
                markerAmplitude), 5, "Marker Amplitude"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the step size of the rotary knob for marker or 
/// HIFN  delta marker value changes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR stepSize/Defines the step size. 
ViStatus _VI_FUNC rsemi_ConfigureMarkerStepSize(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    stepSize
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_MARKER_STEP_SIZE, 
                stepSize), 3, "Step Size");
   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function positions the selected marker to the given probability.
/// HIFN     
/// HIFN     Note(s): 
/// HIFN     
/// HIFN     (1) This function is only available with the CCDF measurement switched 
/// HIFN  on. The associated level value can be determined with the 
/// HIFN  CALCulate<1|2>:MARKer<1...4>:X command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR probability/Defines the probability.
ViStatus _VI_FUNC rsemi_ProbabilityMarkerPosition(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64    probability
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker); 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_MARKER_PROBABILITY, 
                probability), 4, "Probability");
   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  Configure a band power marker. Band power markers allow you to measure 
/// HIFN  the integrated power for a defined span (band) around a marker.
/// HIFN     The result can be displayed either as a power (dBm) or density 
/// HIFN  (dBm/Hz). The span is indicated by lines in the diagram. Band power 
/// HIFN  markers are only available for standard frequency measurements in 
/// HIFN  Spectrum mode (not zero span, I/Q Analyzer etc.).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bandPowerEnabled/Activates the band power marker in the specified window. When switched 
/// HIPAR bandPowerEnabled/to on, if no marker is active yet, marker 1 is
/// HIPAR displayMode/Defines whether the result is displayed as a power or density.
/// HIPAR span/Defines the span (band) around the marker for which the power is 
/// HIPAR span/measured. The span is indicated by lines in the diagram.
ViStatus _VI_FUNC rsemi_ConfigureMarkerBandPower(
    ViSession    instrSession,
    ViBoolean    bandPowerEnabled,
    ViInt32    displayMode,
    ViReal64    span
)
{
    ViStatus    error = VI_SUCCESS;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
  viCheckParm(rsemi_SetAttributeViBoolean(instrSession, "",
      RSEMI_ATTR_MARKER_BAND_POWER_STATE, bandPowerEnabled), 2, "Band Power Enabled");
  viCheckParm(rsemi_SetAttributeViInt32(instrSession, "",
      RSEMI_ATTR_MARKER_BAND_POWER_MODE, displayMode), 3, "Display Mode");
  viCheckParm(rsemi_SetAttributeViReal64(instrSession, "",
      RSEMI_ATTR_MARKER_BAND_POWER_SPAN, span), 4, "Span");  
        
Error:
 (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  Queries the value of the band power marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the value of the band power marker.
ViStatus _VI_FUNC rsemi_QueryMarkerBandPower(
    ViSession    instrSession,
    ViReal64*    result
)
{ 
    ViStatus    error = VI_SUCCESS;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsemi_GetAttributeViReal64(instrSession, "",
        RSEMI_ATTR_QUERY_MARKER_BAND_POWER_RESULT, result), 2, "Result");    
        
Error:
 (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures marker threshold values. The marker threshold 
/// HIFN  specifies a lower bound for ALL marker search functions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR thresholdState/Switches on or off the threshold line in the selected measurement 
/// HIPAR thresholdState/window.
/// HIPAR markerThreshold/Sets the threshold value for the maximum/minimum search of markers 
/// HIPAR markerThreshold/with marker search functions MAX PEAK, NEXT PEAK, etc in the selected 
/// HIPAR markerThreshold/measurement window. The associated display line is automatically 
/// HIPAR markerThreshold/switched on. 
ViStatus _VI_FUNC rsemi_ConfigureMarkerSearchThreshold(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    thresholdState,
    ViReal64    markerThreshold
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_MARKER_THRESHOLD_STATE, 
                thresholdState), 3, "Threshold State");

    if (thresholdState == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_MARKER_THRESHOLD, 
                markerThreshold), 4, "Marker Threshold");
    }
              

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the marker peak excursion The marker peak 
/// HIFN  excursion specifies the minimum amplitude variation that can be 
/// HIFN  recognized as a peak or minimum by the marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR peakExcursion/Sets the minimum amplitude variation of the signal in dB that the 
/// HIPAR peakExcursion/marker can identify as a peak. 
ViStatus _VI_FUNC rsemi_ConfigureMarkerSearchPeakExcursion(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    peakExcursion
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 

    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_PEAK_EXCURSION, 
                peakExcursion), 3, "Peak Excursion");
              

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures state of local oscillator supression.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR localOscillatorSupression/Switches the local oscillator suppression for peak search on or off. 
/// HIPAR localOscillatorSupression/This setting is valid for all markers and delta markers in all 
/// HIPAR localOscillatorSupression/measurement windows. 
ViStatus _VI_FUNC rsemi_ConfigureMarkerSearchLocalOscillator(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    localOscillatorSupression
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_MARKER_LOEX, 
                localOscillatorSupression), 3, "Local Oscillator Supression");
              

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures marker search limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR searchLimits/Switches between a limited (ON) and unlimited (OFF) search range in 
/// HIPAR searchLimits/the selected measurement window. The function is independent of the 
/// HIPAR searchLimits/selection of a marker. 
/// HIPAR searchLimitLeft/Sets the left limit of the search range for markers and delta markers 
/// HIPAR searchLimitLeft/in the selected measurement window. Depending on the x axis domain the 
/// HIPAR searchLimitLeft/indicated value defines a frequency (span > 0) or time (span = 0). The 
/// HIPAR searchLimitLeft/function is independent of the selection of a marker.
/// HIPAR searchLimitRight/Sets the right limit of the search range for markers and delta markers 
/// HIPAR searchLimitRight/in the selected measurement window. Depending on the x axis domain the 
/// HIPAR searchLimitRight/indicated value defines a frequency (span > 0) or time (span = 0). The 
/// HIPAR searchLimitRight/function is independent of the selection of a marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrumMarkerSearchLimits(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    searchLimits,
    ViReal64    searchLimitLeft,
    ViReal64    searchLimitRight
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_SAN_MARKER_SEARCH_LIMITS_STATE, 
                searchLimits), 3, "Serach Limit");
    if (searchLimits == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SAN_MARKER_SEARCH_LIMITS_LEFT, 
                    searchLimitLeft), 4, "Search Limit Left");
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SAN_MARKER_SEARCH_LIMITS_RIGHT, 
                    searchLimitRight), 5, "Search Limit Right");
    }
              

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the limits of the search range for markers 
/// HIFN  and delta markers to the zoom area. 
/// HIFN     
/// HIFN     Note(s): 
/// HIFN     
/// HIFN     (1) The function is only available if the search limit for marker and 
/// HIFN  delta marker is switched on 
/// HIFN  (CALCulate<1|2>:MARKer<1...16>:X:SLIMits[:STATe]).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR searchLimitsZoom/Sets the limits of the search range for markers and delta markers to 
/// HIPAR searchLimitsZoom/the zoom area.
ViStatus _VI_FUNC rsemi_ConfigureMarkerSearchLimitsZoom(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    searchLimitsZoom
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_MARKER_SEARCH_LIMITS_ZOOM, searchLimitsZoom),
                                                3, "Search Limits Zoom");   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures marker demodulation parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR state/This control switches on or off the audio demodulator when the 
/// HIPAR state/indicated marker is reached in the selected measurement window. In the 
/// HIPAR state/frequency domain (span > 0) the hold time can be defined at the 
/// HIPAR state/corresponding marker position with Holdoff argument.
/// HIPAR demodulationType/Selects the demodulation type for the audio demodulator. 
/// HIPAR markerStopTime/Defines the hold time at the marker position for the demodulation in 
/// HIPAR markerStopTime/the frequency domain (span > 0). 
/// HIPAR continuousDemodulation/This control switches on or off the continuous demodulation in the 
/// HIPAR continuousDemodulation/frequency domain (span >0) in the selected measurement window. Thus 
/// HIPAR continuousDemodulation/acoustic monitoring of the signals can be performed in the frequency 
/// HIPAR continuousDemodulation/domain. The function does not depend on the selected marker.
ViStatus _VI_FUNC rsemi_ConfigureMarkerDemodulation(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViBoolean    state,
    ViInt32    demodulationType,
    ViReal64    markerStopTime,
    ViBoolean    continuousDemodulation
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_MARKER_DEMOD_STATE, state),
                                                4, "State");   
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_MARKER_DEMOD_TYPE, demodulationType),
                                                5, "Demodulation Type");                                                 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_MARKER_DEMOD_HOLDOFF, markerStopTime),
                                                6, "Marker Stop Time"); 
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                    RSEMI_ATTR_MARKER_DEMOD_CONT, continuousDemodulation),
                                                    7, "Continuous Demodulation");
    

    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function activates or deactivates the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR label/Activates or deactivates the peak list marker label.
ViStatus _VI_FUNC rsemi_ConfigureMarkerPeakListAnnotation(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    label
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);    
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                 RSEMI_ATTR_MARKER_PEAK_LIST_ANNOTATION_LABEL, label),
                                                 3, "Label");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function activates or deactivates the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Activates or deactivates the peak list.
ViStatus _VI_FUNC rsemi_ConfigureMarkerPeakListState(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);    
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                 RSEMI_ATTR_MARKER_PEAK_LIST_ANNOTATION_STATE, state),
                                                 3, "State");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function searches the selected trace for the indicated number of 
/// HIFN  maxima. The number of maxima found depends on the waveform and value 
/// HIFN  set for the Peak Excursion parameter, however, a maximum number of 50 
/// HIFN  maxima are determined. Only the signals which exceed their surrounding 
/// HIFN  values at least by the value indicated by the peak excursion parameter 
/// HIFN  will be recognized as maxima. Therefore, the number of maxima found is 
/// HIFN  not automatically the same as the number of maxima desired.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR peakListCount/Sets the number of maxima to be found. 
/// HIPAR peakListSort/Sets the sort mode for the search for maxima. 
/// HIPAR peakListSize/Sets the maximum length of the peak list.
/// HIPAR automaticPeaksSearch/Activates and deactivates automatic peaks search after a sweep is 
/// HIPAR automaticPeaksSearch/done.
ViStatus _VI_FUNC rsemi_ConfigureMarkerPeakList(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViInt32    peakListCount,
    ViInt32    peakListSort,
    ViInt32    peakListSize,
    ViBoolean    automaticPeaksSearch
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_MARKER_PEAK_LIST_COUNT, peakListCount),
                                                4, "Peak List Count");   
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_MARKER_PEAK_LIST_SORT, peakListSort),
                                                5, "Peak List Sort");                                                 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_PEAK_LIST_SIZE, peakListSize),
                                               6, "Peak List Size");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                 RSEMI_ATTR_MARKER_PEAK_LIST_AUTO, automaticPeaksSearch),
                                                 7, "Automatic Peaks Search");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function activates or deactivates the marker peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR peakSearch/Activates or deactivates the marker peak search.
ViStatus _VI_FUNC rsemi_ActivateMarkerPeakSearch(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViBoolean    peakSearch
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [15] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (rep_cap, "Win%ld,M%ld", window, marker);    

    viCheckParm(rsemi_SetAttributeViBoolean (instrSession, rep_cap,
        RSEMI_ATTR_MARKER_PEAK_LIST_STATE, peakSearch), 4, "Peak Search");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the settings of peak list - switches on the 
/// HIFN  peak labels and sets the maximum number of peaks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR peakLabels/If enabled, the peaks are labelled by the corresponding marker number 
/// HIPAR peakLabels/in the diagram.
/// HIPAR maximumPeaks/Sets the maximum number of peaks the marker peak list may contain,
ViStatus _VI_FUNC rsemi_ConfigureMarkerPeakListSettings(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViBoolean    peakLabels,
    ViInt32    maximumPeaks
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MARKER_PEAK_LIST_ANNOTATION_LABEL,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MARKER_PEAK_LIST_SIZE,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckParm(rsemi_SetAttributeViBoolean(instrSession, rep_cap_win,
        RSEMI_ATTR_MARKER_PEAK_LIST_ANNOTATION_LABEL, peakLabels), 4, "Peak Labels");
    
    viCheckParm(rsemi_SetAttributeViInt32(instrSession, rep_cap,
        RSEMI_ATTR_MARKER_PEAK_LIST_SIZE, maximumPeaks), 5, "Maximum Peaks"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function stores the current marker peak list in a *.dat file. The 
/// HIFN  file consists of a data section containing the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR path/This control specifies the file path and name
ViStatus _VI_FUNC rsemi_StoreMarkerPeakList(
    ViSession    instrSession,
    ViString    path
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_MARKER_STORE_PEAK_LIST, path),
                                                2, "Path");   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function reads out the number of maxima found during the search. 
/// HIFN  If no search for maxima has been performed, 0 is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR peakListFound/This control returns the number of maxima found during the search. If 
/// HIPAR peakListFound/no search for maxima has been performed, 0 is returned.
ViStatus _VI_FUNC rsemi_QueryMarkerPeakListFound(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViInt32*    peakListFound
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_MARKER_PEAK_LIST_FOUND, peakListFound),
                                                4, "Peak List Found");   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function reads out the list of X or Y values of the maxima found.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR arraySize/This control defines the size of peak list array. It should correspond 
/// HIPAR arraySize/with requested peak list count.
/// HIPAR peakListSelection/Selects the peak list values.
/// HIPAR peakList/This control returns the list of X values of the maxima found.
ViStatus _VI_FUNC rsemi_QueryMarkerPeakList(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViInt32    arraySize,
    ViInt32    peakListSelection,
    ViReal64    peakList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuff=VI_NULL;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    switch (peakListSelection){
        case RSEMI_VAL_MARKER_SORT_X:
            viCheckErr( viPrintf (instrSession, "CALC%ld:MARK%ld:FUNC:FPE:X?\n", window, marker));   
        break;
        case RSEMI_VAL_MARKER_SORT_Y:
            viCheckErr( viPrintf (instrSession, "CALC%ld:MARK%ld:FUNC:FPE:Y?\n", window, marker));   
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Peak List Selection"); 
        break;    
    }    
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &buffer, VI_NULL));                                             
    pbuff = strtok (buffer, ",");
    i = 0;
    do{
        peakList[i] = atof (pbuff);
        pbuff = strtok (NULL, ",");
        i++;
    }while((pbuff != NULL) && (i<arraySize));   
    
    checkErr( rsemi_CheckStatus (instrSession));  
    
Error:
    if (buffer) free(buffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function enables the active marker on the specified trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR state/Specifies whether the specified marker is enabled.
/// HIPAR trace/Selects the trace for Detector definition. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrumDeltaMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViBoolean    state,
    ViInt32    trace
)
{
    ViStatus    error = VI_SUCCESS;
        ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_REF_MARKER_STATE,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_REF_MARKER_STATE,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap, 
                                                RSEMI_ATTR_REF_MARKER_STATE, state),
                                                4, "State");
    
    if (state == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                            RSEMI_ATTR_REF_MARKER_TRACE, trace),
                                            5, "Trace");
    }            
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the delta marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR mode/Switches between relative and absolute frequency input of the delta 
/// HIPAR mode/marker (or time with span = 0). It affects all delta markers 
/// HIPAR mode/independent of the measurement window.. 
/// HIPAR position/Sets the position of selected delta marker in the indicated 
/// HIPAR position/measurement window. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrumDeltaMarkerPosition(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViInt32    mode,
    ViReal64    position
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_REF_MARKER_MODE,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_REF_MARKER_MODE,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                            RSEMI_ATTR_REF_MARKER_MODE, mode),
                                            4, "Mode");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSEMI_ATTR_REF_MARKER_POSITION, position),
                                                5, "Position");
                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function turns off all the delta markers in selected measurement 
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_DisableAllDeltaMarkers(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[25] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_REF_MARKER_MODE,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap));
    viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_REF_MARKER_AOFF, 
                VI_NULL));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);   
    return error;
}


/// HIFN This function links delta marker to any active normal marker         
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR deltaMarker/his control selects delta marker.
/// HIPAR secondMarker/This control selects second marker.
/// HIPAR state/Specifies the marker link state.
ViStatus _VI_FUNC rsemi_LinkingDeltaMarkerToMarker (ViSession instrSession,
                                           ViInt32 window, ViInt32 deltaMarker,
                                           ViInt32 secondMarker, ViBoolean state)
{
     ViStatus    error   = VI_SUCCESS;
    ViChar        win[10]     = "";
    ViChar      marker1[10] = "";
    ViChar      marker2[10] = "";                          
    ViChar         rep_cap[50] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DELTA_MARKER_LINK_TO_MARKER,
                                                 "Window",
                                                 window,
                                                 10,
                                                 win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DELTA_MARKER_LINK_TO_MARKER,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 marker1));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DELTA_MARKER_LINK_TO_MARKER,
                                                 "SecondMarker",
                                                 secondMarker - 1,
                                                 10,
                                                 marker2));
    sprintf(rep_cap,"%s,%s,%s", win, marker1, marker2);
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap, 
                                                RSEMI_ATTR_DELTA_MARKER_LINK_TO_MARKER, 
                                                state), 3, "State");
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;   
}

/// HIFN  This function specifies the type of delta marker search and performs 
/// HIFN  the search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
ViStatus _VI_FUNC rsemi_SpectrumDeltaMarkerSearch(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViInt32    markerSearch
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_REF_MARKER_PEAK,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_REF_MARKER_PEAK,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    switch (markerSearch)
    {
            case RSEMI_VAL_MARKER_SEARCH_HIGHEST:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_REF_MARKER_PEAK, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_REF_MARKER_PEAK_NEXT, 
                                     VI_NULL));
            break;

            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_REF_MARKER_PEAK_NEXT_LEFT, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_REF_MARKER_PEAK_NEXT_RIGHT, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_MINIMUM:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_REF_MARKER_MIN, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                RSEMI_ATTR_REF_MARKER_MIN_NEXT, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                RSEMI_ATTR_REF_MARKER_MIN_NEXT_LEFT, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                RSEMI_ATTR_REF_MARKER_MIN_NEXT_RIGHT, VI_NULL));
            break;

            default: 
                viCheckParm ( RS_ERROR_INVALID_PARAMETER, 4, "Parameter to set");
            break;
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the horizontal position and the marker amplitude 
/// HIFN  level of the selected delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR mode/Switches between absolute and relative position value.
/// HIPAR position/Returns the absolute or relative position of selected delta marker.
/// HIPAR amplitude/Returns the measured value of the selected delta marker in the 
/// HIPAR amplitude/indicated measurement window. The corresponding delta marker will be 
/// HIPAR amplitude/activated, if necessary. The output is always a relative value referred 
/// HIPAR amplitude/to marker 1 or to the reference position (reference fixed active). 
ViStatus _VI_FUNC rsemi_QuerySpectrumDeltaMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViInt32    mode,
    ViReal64*    position,
    ViReal64*    amplitude
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_REF_MARKER_AMPLITUDE,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_REF_MARKER_AMPLITUDE,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    switch (mode){
        case RSEMI_VAL_ABS:
            viCheckParm( rsemi_GetAttributeViReal64 (instrSession, rep_cap, 
                                                RSEMI_ATTR_REF_MARKER_POSITION, position),
                                                5, "Position");   
        break;
        case RSEMI_VAL_REL:
            viCheckParm( rsemi_GetAttributeViReal64 (instrSession, rep_cap, 
                                                RSEMI_ATTR_REF_MARKER_REL_POSITION, position),
                                                5, "Position");   
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Mode"); 
    }        
    
    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, rep_cap, 
                                                RSEMI_ATTR_REF_MARKER_AMPLITUDE, amplitude),
                                                6, "Amplitude");
                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  Configure a band power marker. Band power markers allow you to measure 
/// HIFN  the integrated power for a defined span (band) around a marker.
/// HIFN     The result can be displayed either as a power (dBm) or density 
/// HIFN  (dBm/Hz). The span is indicated by lines in the diagram. Band power 
/// HIFN  markers are only available for standard frequency measurements in 
/// HIFN  Spectrum mode (not zero span, I/Q Analyzer etc.).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bandPowerEnabled/Activates the band power marker in the specified window. When switched 
/// HIPAR bandPowerEnabled/to on, if no marker is active yet, marker 1 is
/// HIPAR displayMode/Defines whether the result is displayed as a power or density.
/// HIPAR span/Defines the span (band) around the marker for which the power is 
/// HIPAR span/measured. The span is indicated by lines in the diagram.
ViStatus _VI_FUNC rsemi_ConfigureDeltaMarkerBandPower(
    ViSession    instrSession,
    ViBoolean    bandPowerEnabled,
    ViInt32    displayMode,
    ViReal64    span
)
{
    ViStatus    error = VI_SUCCESS;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsemi_SetAttributeViBoolean(instrSession, "",
        RSEMI_ATTR_DELTA_MARKER_BAND_POWER_STATE, bandPowerEnabled), 2, "Band Power Enabled");        
    viCheckParm(rsemi_SetAttributeViInt32(instrSession, "",
        RSEMI_ATTR_DELTA_MARKER_BAND_POWER_MODE, displayMode), 3, "Display Mode");
    viCheckParm(rsemi_SetAttributeViReal64(instrSession, "",
        RSEMI_ATTR_DELTA_MARKER_BAND_POWER_SPAN, span), 4, "Span");       

Error:
 (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  Queries the value of the band power marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the value of the band power marker.
ViStatus _VI_FUNC rsemi_QueryDeltaMarkerBandPower(
    ViSession    instrSession,
    ViReal64*    result
)
{
    ViStatus    error = VI_SUCCESS;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsemi_GetAttributeViReal64(instrSession, "",
      RSEMI_ATTR_QUERY_DELTA_MARKER_BAND_POWER_RESULT, result), 2, "Result");
Error:
 (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures marker reference fixed state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR fixedReference/This control switches the relative measurement to a fixed reference 
/// HIPAR fixedReference/value on or off. Marker 1 will be activated previously and a peak 
/// HIPAR fixedReference/search will be performed, if necessary. If marker 1 is activated, its 
/// HIPAR fixedReference/position becomes the reference point for the measurement.
ViStatus _VI_FUNC rsemi_ConfigureReferenceFixed(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    fixedReference
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean(instrSession, buffer, 
                RSEMI_ATTR_MARKER_REF_STATE,  fixedReference), 3, "Fixed Reference");  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures marker reference fixed position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR refPointLevel/Defines a new reference point level for delta marker 2 in the selected 
/// HIPAR refPointLevel/measurement window.
/// HIPAR refPointLevelOffset/Defines an additional level offset which is included in the display of 
/// HIPAR refPointLevelOffset/delta marker 2 in the selected measurement window.
/// HIPAR refPointFrequencyTime/Defines a new reference frequency or time for delta marker 2 in the 
/// HIPAR refPointFrequencyTime/selected measurement window.
ViStatus _VI_FUNC rsemi_ConfigureReferenceFixedPoint(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    refPointLevel,
    ViReal64    refPointLevelOffset,
    ViReal64    refPointFrequencyTime
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
            RSEMI_ATTR_MARKER_REF_LEVEL_OFFSET,  refPointLevelOffset), 
            4, "Reference Point Level Offset");   
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
            RSEMI_ATTR_MARKER_REF_LEVEL,  refPointLevel), 3, "Reference Point Level");       
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
            RSEMI_ATTR_MARKER_REF_FREQ,  refPointFrequencyTime), 
            5, "Reference Point Frequency");                  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the reference point level for all delta markers in 
/// HIFN  the selected measurement window for a measurement with fixed reference 
/// HIFN  point (CALC:DELT:FUNC:FIX:STAT ON) to the peak of the selected trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ReferenceFixedPeakSearch(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckErr( rsemi_SetAttributeViString(instrSession, buffer, RSEMI_ATTR_MARKER_REF_PEAK, 
                VI_NULL));
   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the reference oscillator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/Sets the reference oscillator. If the external reference oscillator is 
/// HIPAR source/selected, the reference signal must be connected to the rear panel of 
/// HIPAR source/the instrument.
/// HIPAR externalFrequency/Informs the instrument on the frequency of the external reference 
/// HIPAR externalFrequency/oscillator. This frequency is used as starting point for the 
/// HIPAR externalFrequency/synchronization of the internal reference frequencies.
ViStatus _VI_FUNC rsemi_ConfigureReferenceOscillator(
    ViSession    instrSession,
    ViInt32    source,
    ViReal64    externalFrequency
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_ROSC_SOURCE, source),
                                                2, "Source");
    if (source == RSEMI_VAL_SOUR_EXT)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_ROSC_FREQUENCY, externalFrequency),
                                                3, "External Frequency");                                                 
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function returns the current selection of the reference signal 
/// HIFN  (internal / external) if the external reference source EXT [INT] is 
/// HIFN  used. The function is only available, if SENS:ROSC:SOUR EAUT is 
/// HIFN  selected. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/Returns the source of the reference oscillator.
ViStatus _VI_FUNC rsemi_QuerySelectedReferenceOscillatorSource(
    ViSession    instrSession,
    ViInt32*    source
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_ROSC_SOURCE_EAUTO, source),
                                                2, "Source");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function configures the reference frequency input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR tuningRange/Defines the tuning range.
/// HIPAR loopBandwidth/Defines the loop bandwidth, that is, the speed of internal
/// HIPAR loopBandwidth/synchronization with the reference frequency.
ViStatus _VI_FUNC rsemi_ConfigureReferenceFrequencyInput (
    ViSession instrSession,
    ViInt32 tuningRange,
    ViReal64 loopBandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_ROSC_TUNING_RANGE, tuningRange),
                                                2, "Tuning Range");

    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_ROSC_LOOP_BANDWIDTH, loopBandwidth),
                                                3, "Loop Bandwidth");                                                 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function configures the reference frequency output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR output100MHz/This attribute enabled/disabled a 100 MHz reference signal
/// HIPAR output100MHz/provided to the REF OUTPUT 100 MHZ connector.
/// HIPAR output640MHz/This attribute enabled/disabled a 640 MHz reference signal
/// HIPAR output640MHz/provided to the REF OUTPUT 640 MHZ connector.
/// HIPAR outputSyncTrigger/This attribute enabled/disabled a 100 MHz reference signal
/// HIPAR outputSyncTrigger/provided to the SYNC TRIGGER OUTPUT connector.
ViStatus _VI_FUNC rsemi_ConfigureReferenceFrequencyOutput (
    ViSession instrSession,
    ViBoolean output100MHz,
    ViBoolean output640MHz,
    ViBoolean outputSyncTrigger
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rsemi_SetAttributeViBoolean(instrSession, "", 
                RSEMI_ATTR_ROSC_O100_ENABLED,  output100MHz), 2, "Output 100 MHz");

    viCheckParm( rsemi_SetAttributeViBoolean(instrSession, "", 
                RSEMI_ATTR_ROSC_O640_ENABLED,  output640MHz), 3, "Output 640 MHz");    
    
    viCheckParm( rsemi_SetAttributeViBoolean(instrSession, "", 
                RSEMI_ATTR_ROSC_SYNC_ENABLED,  outputSyncTrigger), 4, "Output Sync Trigger");    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines the maximum volume level for audio output
/// HIFN  (for example over headphones). 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR maximumVolume/Sets the maximum volume level for audio output
ViStatus _VI_FUNC rsemi_AudioOutputMaximumVolume (
    ViSession instrSession,
    ViReal64 maximumVolume
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_AUDIO_OUTPUT_MAXIMUM_VOLUME, maximumVolume),
                                                2, "Maximum Volume");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function turns audio output off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_AudioOutputMute (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_AUDIO_OUTPUT_MUTE, VI_NULL));
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function sets the volume of the built-in loudspeaker for 
/// HIFN  demodulated signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR speakerVolume/Sets the volume of the built-in loudspeaker for demodulated signals.
ViStatus _VI_FUNC rsemi_SpeakerVolume(
    ViSession    instrSession,
    ViReal64    speakerVolume
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_SYST_SPEAKER, speakerVolume),
                                                2, "Speaker Volume");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function controls the instrument behaviour when changing the 
/// HIFN  active application, e.g from SPECTRUM to FM DEMOD and back from FM 
/// HIFN  DEMOD to SPECTRUM. In the default state OFF a few
/// HIFN     parameters of the current analyzer setting are passed to the 
/// HIFN  application (e.g. center frequency, level settings) or from the 
/// HIFN  application back to the analyzer mode. If APPL SETUP RECOVERY is 
/// HIFN  switched ON, the settings of the applications are independent of each 
/// HIFN  other. Leaving the FM DEMOD application will restore the previous state 
/// HIFN  of the ANALYZER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR applicationSetupRecovery/This control sets the instrument behaviour when changing the active 
/// HIPAR applicationSetupRecovery/application.
ViStatus _VI_FUNC rsemi_ApplicationSetupRecovery(
    ViSession    instrSession,
    ViBoolean    applicationSetupRecovery
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rsemi_SetAttributeViBoolean(instrSession, "", 
                RSEMI_ATTR_APPL_SETUP_REC,  applicationSetupRecovery), 2, "Application Setup Recovery");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function turns logging of remote commands on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns logging of remote commands on and off. If on, writes all remote 
/// HIPAR state/commands that have been sent to a file. The destination is 
/// HIPAR state/C:\R_S\instr\ScpiLogging\ScpiLog.txt.
ViStatus _VI_FUNC rsemi_ConfigureRemoteLoggingState(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                                RSEMI_ATTR_REMOTE_LOGGING_STATE, 
                                                state), 2, "State");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN This function configures the instrument coupling..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR coupling/Selects the parameter for coupling setting.
/// HIPAR state/Selects the coupling state
ViStatus _VI_FUNC rsemi_ConfigureInstrumentCoupling (ViSession instrSession,
                                                     ViInt32 coupling,
                                                     ViInt32 state)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    switch (coupling){
        case RSEMI_VAL_COUPLING_ATTENUATION:
                    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_COUPLING_ATTENUATION, state));
        break;
        case RSEMI_VAL_COUPLING_BANDWIDTH:
                    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_COUPLING_BANDWIDTH, state));
        break;
        case RSEMI_VAL_COUPLING_CENTER:
                    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_COUPLING_CENTER, state));
        break;
        case RSEMI_VAL_COUPLING_DEMODULATION:
                    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_COUPLING_DEMODULATION, state));
        break;
        case RSEMI_VAL_COUPLING_GAIN:
                    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_COUPLING_GAIN, state));
        break;
        case RSEMI_VAL_COUPLING_LIMIT:
                    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_COUPLING_LIMIT, state));
        break;
        case RSEMI_VAL_COUPLING_MARKER:
                    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_COUPLING_MARKER, state));
        break;
        case RSEMI_VAL_COUPLING_PRESELECTOR:
                    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_COUPLING_PRESELECTOR, state));
        break;
        case RSEMI_VAL_COUPLING_SPAN:
                    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_COUPLING_SPAN, state));
        break;
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "State");
    }
    
  Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function shuts down the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_SystemShutdown(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_SYSTEM_SHUTDOWN, VI_NULL));
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the current frontend temperature of the R&S FSWT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frontendTemperature/Temperature in degrees Celsius.
ViStatus _VI_FUNC rsemi_QueryFrontendTemperature (
    ViSession instrSession,
    ViReal64 *frontendTemperature
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_FRONTEND_TEMPERATURE,
                                                frontendTemperature), 2, "Frontend Temperature");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function defines which option is returned when the *OPT?
/// HIFN  query is executed, depending on the used preamplifier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR option/This attribute defines which option is returned when the *OPT?
/// HIPAR option/query is executed, depending on the used preamplifier.
ViStatus _VI_FUNC rsemi_SystemPreamplifierOption (
    ViSession instrSession,
    ViInt32 option
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_SYST_PREAMPLIFIER_OPTION, option),
                                                2, "Option");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function activates or queries secure user mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR secureUserModeEnabled/Activates secure user mode.
ViStatus _VI_FUNC rsemi_ConfigureSecureUserMode (
    ViSession instrSession,
    ViBoolean secureUserModeEnabled
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, NULL));
    
    viCheckParm(rsemi_SetAttributeViBoolean(instrSession, "",
        RSEMI_ATTR_SECURE_USER_MODE, secureUserModeEnabled), 2, "Secure User Mode Enabled");    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function specifies the trigger source that causes the spectrum 
/// HIFN  analyzer to leave the Wait-for-Trigger state.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR triggerSource/Specifies the source of the trigger signal that causes the analyzer to 
/// HIPAR triggerSource/leave the Wait-For-Trigger state.
ViStatus _VI_FUNC rsemi_ConfigureTriggerSource(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    triggerSource
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);             
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_SOURCE,
                                                triggerSource),
                                                2, "Trigger Source");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function specifies the delay and polarity for triggering.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR triggerDelay/Sets the length of the trigger delay.
/// HIPAR triggerPolarity/Specifies the polarity for triggering.
ViStatus _VI_FUNC rsemi_ConfigureTrigger(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    triggerDelay,
    ViInt32    triggerPolarity
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);             
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_DELAY,
                                                triggerDelay),
                                                2, "Trigger Delay");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_SLOPE,
                                                triggerPolarity),
                                                3, "Trigger Polarity");                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function specifies the dropout time for triggering.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR dropoutTime/Defines the time the input signal must stay below the trigger level 
/// HIPAR dropoutTime/before triggering again.
ViStatus _VI_FUNC rsemi_ConfigureTriggerDropoutTime (
    ViSession instrSession,
    ViReal64 dropoutTime
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_TRIGGER_DROPOUT_TIME,
                                                dropoutTime),
                                                2, "Dropout Time");
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL); 
    return error;
}

/// HIFN  This function defines the external trigger level. This is applicable 
/// HIFN  when the trigger source is set to external.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR externalTriggerLevel/Sets the external level for triggering. 
ViStatus _VI_FUNC rsemi_ConfigureExternalTrigger(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    externalTriggerLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);             
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_EXTERNAL_TRIGGER_LEVEL,
                                                externalTriggerLevel),
                                                2, "External Trigger Level");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function specifies the video level for triggering.  This is 
/// HIFN  applicable when the trigger source is set to video.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR videoTriggerLevel/Sets the videp level for triggering.
ViStatus _VI_FUNC rsemi_ConfigureVideoTrigger(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    videoTriggerLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);             
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_VIDEO_TRIGGER_LEVEL,
                                                videoTriggerLevel),
                                                2, "Video Trigger Level");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function sets the level of the IF power trigger source. This is 
/// HIFN  applicable when the trigger source is set to IF Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR triggerLevel/Sets the level for triggering.
ViStatus _VI_FUNC rsemi_ConfigureIFPowerTrigger(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    triggerLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);             
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_IFP_LEVEL,
                                                triggerLevel),
                                                2, "Trigger Level");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function sets the IF power trigger parameters. This is applicable 
/// HIFN  when the trigger source is set to IF Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR triggerLevel/Sets the level for triggering.
/// HIPAR holdoff/This command sets the holding time before the next IF power trigger 
/// HIPAR holdoff/event. This command is available for R&S FSL from firmware version 1.30 
/// HIPAR holdoff/and R&S FSW-K6.
/// HIPAR hysteresis/This command sets the limit that the hysteresis value has to fall 
/// HIPAR hysteresis/below in order to trigger the next measurement.
ViStatus _VI_FUNC rsemi_ConfigureIFPowerTriggerParameters(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    triggerLevel,
    ViReal64    holdoff,
    ViReal64    hysteresis
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);             
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_IFP_LEVEL,
                                                triggerLevel),
                                                2, "Trigger Level");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_IFP_OFFSET,
                                                holdoff),
                                                3, "Holdoff");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_IFP_HYSTERESIS,
                                                hysteresis),
                                                4, "Hysteresis");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function configures the coupling of the external trigger to
/// HIFN  the oscilloscope.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/Configures the coupling of the external trigger to the oscilloscope.
ViStatus _VI_FUNC rsemi_ConfigureIFPowerTriggerCoupling (
    ViSession instrSession,
    ViInt32 coupling
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_TRIGGER_IFP_COUPLING,
                                                coupling),
                                                2, "Coupling");                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function specifies sets the level of the RF power trigger source. 
/// HIFN  This is applicable when the trigger source is set to RF Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR triggerLevel/Sets the level for triggering.
ViStatus _VI_FUNC rsemi_ConfigureRFPowerTrigger(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    triggerLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);             
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_RFP_LEVEL,
                                                triggerLevel),
                                                3, "Trigger Level");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function defines the holding time before the next trigger event. 
/// HIFN  Note that this command is available for any trigger source, not just RF 
/// HIFN  Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR holdoff/This control sets the holding time before the next trigger event. 
ViStatus _VI_FUNC rsemi_ConfigureRFPowerTriggerHoldoff(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    holdoff
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);             
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_DELAY,
                                                holdoff), 3, "Holdoff");
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function sets the level of the IQ power trigger source.
/// HIFN  This is applicable when the trigger source is set to IQ Power. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR triggerLevel/Sets the level of the IQ power trigger.
ViStatus _VI_FUNC rsemi_ConfigureIQPowerTrigger (
    ViSession instrSession,
    ViInt32 window, 
    ViReal64 triggerLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);    
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_IQP_LEVEL,
                                                triggerLevel), 3, "Trigger Level");
    
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function configures the time trigger. This is applicable when the 
/// HIFN  trigger source is set to time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeInterval/This control sets the time interval for the time trigger source.
ViStatus _VI_FUNC rsemi_ConfigureTimeTrigger(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    timeInterval
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);             
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_TRIGGER_TIME_INTERVAL,
                                                timeInterval),
                                                2, "Time Interval");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function determines the time offset of the trigger in the sample 
/// HIFN  (trigger position in sample = TPIS). This value can only be determined 
/// HIFN  in triggered measurements using external or IFPower triggers, otherwise 
/// HIFN  the value is 0.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     
/// HIFN     (1) This function is not available if the digital baseband interface 
/// HIFN  (R&S B17 option) is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerPositionInSample/This control returns the time offset of the trigger in the sample 
/// HIPAR triggerPositionInSample/(trigger position in sample = TPIS).
ViStatus _VI_FUNC rsemi_QueryTriggerPositionInSample(
    ViSession    instrSession,
    ViInt32*    triggerPositionInSample
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, "", 
        RSEMI_ATTR_TRIGGER_POSITION_IN_SAMPLE, triggerPositionInSample), 
        2, "Trigger Position In Sample");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function configures the external gate parameters. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR gating/This parameter switches on/off the gating.
/// HIPAR gateSource/This control toggles between external gate signal and RF power signal.
/// HIPAR mode/Sets sets the type of triggering (level or edge) by the external gate 
/// HIPAR mode/signal. 
/// HIPAR polarity/Sets determines the polarity of the external gate signal.
/// HIPAR delay/Defines the delay time between the external gate signal and the 
/// HIPAR delay/continuation of the sweep. 
/// HIPAR length/Specifies defines the gate time in the capture buffer in time units or 
/// HIPAR length/samples. The range of this value is dependent on the measurement that 
/// HIPAR length/was last run.
ViStatus _VI_FUNC rsemi_ConfigureExternalGate(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    gating,
    ViInt32    gateSource,
    ViInt32    mode,
    ViInt32    polarity,
    ViReal64    delay,
    ViReal64    length
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);   
    
    if (gating == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                    RSEMI_ATTR_EXTERNAL_GATE_SIGNAL_SOURCE,
                                                    gateSource), 4, "Gate Source");   
         
        
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                    RSEMI_ATTR_EXTERNAL_GATE_TRIGGER_TYPE,
                                                    mode), 5, "Mode"); 
        if (gateSource == RSEMI_VAL_EGAT_SOUR_EXT) { 
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                    RSEMI_ATTR_EXTERNAL_GATE_POLARITY,
                                                    polarity), 6, "Polarity");     
        }
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                        RSEMI_ATTR_EXTERNAL_GATE_HOLD,
                                                        delay), 7, "Delay"); 
        
        if (mode==RSEMI_VAL_EGAT_TRIG_EDGE)                                                    
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                    RSEMI_ATTR_EXTERNAL_GATE_LENGTH,
                                                    length), 8, "Length");                                                     
    }                                                    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_EXTERNAL_GATE,
                                                gating),
                                                3, "Gating");                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function configures the external gate trace parameters. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR range/This control sets the external gate trace range.
/// HIPAR traceState/This parameter activates or deactivates tracing for a specific range 
/// HIPAR traceState/using gated triggering.
/// HIPAR comment/This control defines a comment for one of the traces for gated 
/// HIPAR comment/triggering.
/// HIPAR traceStart/Defines the starting point for the range to be traced using gated 
/// HIPAR traceStart/triggering.
/// HIPAR traceStop/Defines the stopping point for the range to be traced using gated 
/// HIPAR traceStop/triggering.
/// HIPAR periodLength/Defines the length of the period to be traced using gated triggering.
ViStatus _VI_FUNC rsemi_ConfigureExternalGateTrace(
    ViSession    instrSession,
    ViInt32    trace,
    ViInt32    range,
    ViBoolean    traceState,
    ViString    comment,
    ViReal64    traceStart,
    ViReal64    traceStop,
    ViReal64    periodLength
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 2, "Trace");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 3), 3, "Range");
        
    sprintf (buffer, "TR%ld,ER%ld", trace, range);   
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                 RSEMI_ATTR_EXTERNAL_GATE_TRACE_STATE,
                                                 traceState), 4, "Trace State");

    viCheckParm (rsemi_SetAttributeViString (instrSession, buffer,
                                                 RSEMI_ATTR_EXTERNAL_GATE_TRACE_COMMENT,
                                                 comment), 5, "Comment");
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                 RSEMI_ATTR_EXTERNAL_GATE_TRACE_START,
                                                 traceStart), 6, "Trace Start");
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                 RSEMI_ATTR_EXTERNAL_GATE_TRACE_STOP,
                                                 traceStop), 7, "Trace Stop");

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                 RSEMI_ATTR_EXTERNAL_GATE_TRACE_PERIOD_LENGTH,
                                                 periodLength), 8, "Period Length");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function turns external mixing on/off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR externalMixingEnabled/Specifies whether external mixing is enabled.
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerEnabled(
    ViSession    instrSession,
    ViBoolean    externalMixingEnabled
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                                RSEMI_ATTR_EXTERNAL_MIXER_ENABLED,
                                                externalMixingEnabled),
                                                2, "External Mixer Enabled");                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function specifies a 2-port or a 3-port external mixer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR numberofPorts/Specifies a two-port mixer or a three-port mixer. 
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerNumberOfPorts(
    ViSession    instrSession,
    ViInt32    numberofPorts
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_EXTERNAL_MIXER_NUMBER_OF_PORTS,
                                                numberofPorts), 2, "Number Of Ports");                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function specifies the mixer harmonic and average conversion 
/// HIFN  loss.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR band/Defines waveguide band in the Conversion Loss table.
/// HIPAR harmonic/Specifies harmonic number. 
/// HIPAR harmonicType/Sets the type of harmonic.
ViStatus _VI_FUNC rsemi_ConfigureExternalMixer(
    ViSession    instrSession,
    ViInt32    band,
    ViInt32    harmonic,
    ViInt32    harmonicType
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_EXTERNAL_MIXER_HARMONIC_BAND,
                                                band), 2, "Band");                                                
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_EXTERNAL_MIXER_HARMONIC_TYPE,
                                                harmonicType), 4, "Harmonic Type");         
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_EXTERNAL_MIXER_HARMONIC,
                                                harmonic), 3, "Harmonic");        
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function queries the frequency at which the external mixer band 
/// HIFN  starts and stops.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR rfStart/Queries the frequency at which the external mixer band starts.
/// HIPAR rfStop/Queries the frequency at which the external mixer band stops.
ViStatus _VI_FUNC rsemi_QueryExternalMixerRFStartStop(
    ViSession    instrSession,
    ViReal64*    rfStart,
    ViReal64*    rfStop
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, "",
                                                 RSEMI_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_START,
                                                 rfStart), 2, "RF Start");
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, "",
                                                 RSEMI_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_STOP,
                                                 rfStop), 3, "RF Stop");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function defines the frequency at which the mixer switches from 
/// HIFN  one range to the next (if two different ranges are selected). The 
/// HIFN  handover frequency for each band can be selected freely within the 
/// HIFN  overlapping frequency range.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     
/// HIFN     (2) This function is only available if the external mixer is active.
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR handoverFrequency/Defines the frequency at which the mixer switches from one range to 
/// HIPAR handoverFrequency/the next.
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerHandoverFrequency(
    ViSession    instrSession,
    ViReal64    handoverFrequency
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                 RSEMI_ATTR_EXTERNAL_MIXER_HANDOVER_FREQUENCY,
                                                 handoverFrequency), 2, "Handover Frequency");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function specifies whether the frequencies beyond the defined 
/// HIFN  band limits are to be used.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     
/// HIFN     (1) This function is only available if the external mixer is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR rfOverrange/Specifies whether the frequencies beyond the defined band limits are 
/// HIPAR rfOverrange/to be used.
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerRFOverrange(
    ViSession    instrSession,
    ViBoolean    rfOverrange
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_EXTERNAL_MIXER_RF_OVERRANGE,
                                                 rfOverrange), 2, "RF Overrange");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function restores the preset frequency ranges for the standard 
/// HIFN  waveguide bands.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     
/// HIFN     (2) Changes to the band and mixer settings are maintained even after 
/// HIFN  using this function. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerPresetBand(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rsemi_SetAttributeViString (instrSession, "",
                                               RSEMI_ATTR_EXTERNAL_MIXER_PRESET_BAND,
                                               NULL));
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function specifies a second (high) harmonic.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR highHarmonic/Specifies whether a second (high) harmonic is to be used to cover the 
/// HIPAR highHarmonic/band's frequency range.
/// HIPAR harmonicOrder/Specifies the harmonic order to be used for the high range. 
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerHighHarmonic(
    ViSession    instrSession,
    ViBoolean    highHarmonic,
    ViInt32    harmonicOrder
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC_STATE,
                                                 highHarmonic), 2, "High Harmonic");
    
    if (highHarmonic == VI_TRUE)
    {
        viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                                     RSEMI_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC,
                                                     harmonicOrder), 3, "Harmonic Order");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function sets the Signal ID or Auto ID mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Sets the type of harmonic.
/// HIPAR autoIDThreshold/Determines the maximum permissible level difference between 
/// HIPAR autoIDThreshold/measurement and reference sweep in AUTO ID mode. 
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerSignalIDMode(
    ViSession    instrSession,
    ViInt32    mode,
    ViReal64    autoIDThreshold
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_EXTERNAL_MIXER_SIGNAL_ID_MODE,
                                                mode), 2, "Mode");                                                
    if (mode==RSEMI_VAL_STATE_AUTO)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_EXTERNAL_MIXER_THRESHOLD,
                                                autoIDThreshold), 3, "Auto ID Threshold");        
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function configures the bias current for the low and high range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/This control selects the conversion loss range.
/// HIPAR bias/Specifies the bias current.
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerBias(
    ViSession    instrSession,
    ViInt32    range,
    ViReal64    bias
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (range == 1)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                    RSEMI_ATTR_EXTERNAL_MIXER_BIAS_LOW,
                                                    bias), 2, "Bias");
    }
    else
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_EXTERNAL_MIXER_BIAS_HIGH,
                                                bias), 2, "Bias");        
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function specifies the conversion loss for selected band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/This control selects the conversion loss range.
/// HIPAR conversionLossMode/This control selects the converstion loss mode.
/// HIPAR conversionLossAverage/Specifies the average conversion loss.
/// HIPAR conversionLossTable/Sets the conversion loss table for the active band.
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerConversionLoss(
    ViSession    instrSession,
    ViInt32    range,
    ViInt32    conversionLossMode,
    ViReal64    conversionLossAverage,
    ViString    conversionLossTable
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 2), 2, "Range");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, conversionLossMode, RSEMI_VAL_MIXER_LOSS_AVER, RSEMI_VAL_MIXER_LOSS_TABL), 3, "Conversion Loss Mode");
        
    if (range == 1)
    {
        if (conversionLossMode == RSEMI_VAL_MIXER_LOSS_AVER)
        {
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                    RSEMI_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_LOW,
                                                    conversionLossAverage), 4, "Conversion Loss Average");
        }
        else
        {
            viCheckParm( rsemi_SetAttributeViString (instrSession, "", 
                                                    RSEMI_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_LOW,
                                                    conversionLossTable), 5, "Conversion Loss Table");                                                
        }
    }
    else
    {
        if (conversionLossMode == RSEMI_VAL_MIXER_LOSS_AVER)
        {
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                    RSEMI_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_HIGH,
                                                    conversionLossAverage), 4, "Conversion Loss");
        }
        else
        {
            viCheckParm( rsemi_SetAttributeViString (instrSession, "", 
                                                    RSEMI_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_HIGH,
                                                    conversionLossTable), 5, "Conversion Loss Table");        
        }
    }
        
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the LO level of the external mixer LO port from 
/// HIFN  13.0 dBm to 17.0 dBm in 0.1 dB steps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR lowPower/Sets the LO level of the external mixer LO port. 
ViStatus _VI_FUNC rsemi_ConfigureExternalMixerLowPower(
    ViSession    instrSession,
    ViReal64    lowPower
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_EXTERNAL_MIXER_LO_POWER,
                                                lowPower), 2, "Low POwer");                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function creates or overwrites conversion loss table of external 
/// HIFN  mixer.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (2) If the external mixer is not turned on, this function will return 
/// HIFN  an error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR tableName/Enter name of table (without extention) to edit or create. A new table 
/// HIPAR tableName/will be created if the specified name does not exist.
/// HIPAR mixerType/Defines the type designation of the mixer in the Conversion Loss 
/// HIPAR mixerType/Table.
/// HIPAR band/Defines waveguide band in the Conversion Loss table.
/// HIPAR serialNumber/Enter the serial number of the mixer.
/// HIPAR harmonic/Defines the harmonic of the mixer in the Conversion Loss Table. 
/// HIPAR ports/Specifies the mixer type (two-port mixer or three-port mixer).
/// HIPAR bias/Enters the bias current.
/// HIPAR comment/Enter the conevrsion loss table comment.
/// HIPAR numberOfValues/Inputs number of reference value pairs to be send to the  instrument.
/// HIPAR frequencyValues/Specifies array of frequency values in Hz. 
/// HIPAR conversionLossValues/Specifies array of level values in dB.
ViStatus _VI_FUNC rsemi_CreateExternalMixerConversionLossTable(
    ViSession    instrSession,
    ViString    tableName,
    ViString    mixerType,
    ViInt32    band,
    ViString    serialNumber,
    ViInt32    harmonic,
    ViInt32    ports,
    ViReal64    bias,
    ViString    comment,
    ViInt32    numberOfValues,
    ViReal64    frequencyValues[],
    ViReal64    conversionLossValues[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuf;
    ViInt32     i;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_CVL_NAME, 
                tableName), 3, "Table Name");
    viCheckParm( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_CVL_MIXER, 
                mixerType), 4, "Mixer Type");                
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_CVL_BAND, 
                band), 5, "Band");     
    viCheckParm( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_CVL_MIXER_SNUM, 
                serialNumber), 6, "Serial Number"); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_CVL_HARM, 
                harmonic), 7, "Harmonic");                
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_CVL_PORT_MIXER, 
                ports), 8, "Ports");      
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", RSEMI_ATTR_CVL_BIAS, 
                bias), 9, "Bias");
    viCheckParm( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_CVL_COMMENT, 
                comment), 10, "Comment"); 
    
    pbuf = buffer;
    if (numberOfValues>0)
    {
        pbuf += sprintf (buffer, "CORR:CVL:DATA ");
        
        for (i = 0; i < numberOfValues; i++)
            pbuf += sprintf (pbuf, "%.12f,%.12f,", frequencyValues[i], conversionLossValues[i]);       
        
        *pbuf = '\0';
        *--pbuf = '\n';
        
        viCheckErr (viPrintf (instrSession, "%s\n", buffer));
    }    
    checkErr( rsemi_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects a conversion loss table of external mixer.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) If the external mixer is not turned on, this function will return 
/// HIFN  an error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR tableName/Enter name of table (without extention) to delete.
ViStatus _VI_FUNC rsemi_ExternalMixerConversionLossTableSelect(
    ViSession    instrSession,
    ViInt32    window,
    ViString    tableName
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);   
    viCheckParm( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_CVL_NAME,
                                                tableName),
                                                2, "Table Name");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function deletes a conversion loss table of external mixer.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) If the external mixer is not turned on, this function will return 
/// HIFN  an error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR tableName/Enter name of table (without extention) to delete.
ViStatus _VI_FUNC rsemi_ExternalMixerConversionLossTableDelete(
    ViSession    instrSession,
    ViInt32    window,
    ViString    tableName
)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);   
    viCheckParm( rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_CVL_NAME,
                                                tableName),
                                                2, "Table Name");
    viCheckParm( rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_CVL_CLEAR,
                                                tableName),
                                                2, "Table Name");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function queries the names of all conversion loss tables saved on 
/// HIFN  the hard disk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofCVL/This control returns number of the names of all digital standards 
/// HIPAR numberofCVL/available.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog 
/// HIPAR bufferSize/list parameter.
/// HIPAR cvlNamesList/This control the names of all conversion loss tables saved on the hard 
/// HIPAR cvlNamesList/disk.
ViStatus _VI_FUNC rsemi_ExternalMixerConversionLossTableCatalog(
    ViSession    instrSession,
    ViInt32*    numberofCVL,
    ViInt32    bufferSize,
    ViChar    cvlNamesList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      *buf = VI_NULL;
    ViChar      *buf2 = VI_NULL;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";   
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsemi_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSEMI_IO_BUFFER_SIZE, buffer));
    
    if (!((strstr (buffer, "K90")  != NULL) || (strstr (buffer, "B21") != NULL)))
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
        
    if (bufferSize < 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");         
    if ((bufferSize > 0) && (cvlNamesList == VI_NULL))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "CVL Names List");  
       
    viCheckErr (viPrintf (instrSession, ":CORR:CVL:CAT?\n"));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &buf, &retCnt));
    //viCheckErr (rsemi_ParseCatalog(buf, bufferSize, CVLNamesList, numberofCVL));
    buf2 = malloc (retCnt);
    sscanf(buf, "'%[^']'", buf2);
    viCheckErr (rsemi_ParseLimitLineCatalog (buf2, bufferSize, cvlNamesList, numberofCVL));
    
    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    if (buf) free(buf);
    if (buf2) free(buf2);
    (void) Rs_UnlockSession(instrSession, VI_NULL);      
    return error;
}


/// HIFN This function activates input from the RF INPUT connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/Selects the signal source for measurements. If no additional
/// HIPAR source/options are installed, only RF input is supported.
ViStatus _VI_FUNC rsemi_ConfigureRFInputState (ViSession instrSession,
                                               ViInt32 source)
{
   ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_RF_INPUT,
                                                source),
                                                2, "Source");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function switches the input coupling of the RF input between AC 
/// HIFN  and DC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR rfInputCoupling/This control selects selects the coupling of RF Input. The driver uses 
/// HIPAR rfInputCoupling/this value to set the RSEMI_ATTR_AMPL_RF_COUPLING attribute.
ViStatus _VI_FUNC rsemi_ConfigureAmplitudeRFInputCoupling(
    ViSession    instrSession,
    ViInt32    rfInputCoupling
)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_AMPL_RF_COUPLING,
                                                rfInputCoupling),
                                                2, "RF Input Coupling");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

 
/// HIFN This function configures input impedance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR impedance/This control selects the nominal input impedance.
ViStatus _VI_FUNC rsemi_ConfigureAmplitudeRFInputImpedance (ViSession instrSession,
                                                            ViInt32 impedance)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_SAN_INPUT_IMPEDANCE,
                                                impedance),
                                                2, "Impedance");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function configures electronic attenuator parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR input/Defines the input number.
/// HIPAR mode/This control selects selects the state of electronic attenuator. 
/// HIPAR manualValue/Programs the attenuation of the electronic input attenuator. Other 
/// HIPAR manualValue/entries are rounded to the next lower integer value. If the attenuation 
/// HIPAR manualValue/is programmed directly, the coupling to the reference level is switched 
/// HIPAR manualValue/off. If the defined reference level cannot be set with the given RF 
/// HIPAR manualValue/attenuation, this level is adapted to the maximum possible value.
ViStatus _VI_FUNC rsemi_ConfigureElectronicAttenuator(
    ViSession    instrSession,
    ViInt32    input,
    ViInt32    mode,
    ViInt32    manualValue
)
{
    ViStatus    error = VI_SUCCESS;  
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, input, 1, 2), 2, "Input");
 
    sprintf (buffer, "Win%ld", input);    
    switch (mode){
        case RSEMI_VAL_EATT_OFF:
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                         RSEMI_ATTR_AMPL_EATT_STATE,
                                                         VI_FALSE), 3, "Mode");  
        break;
        case RSEMI_VAL_EATT_AUTO:
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                         RSEMI_ATTR_AMPL_EATT_STATE,
                                                         VI_TRUE), 3, "Mode");  
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                         RSEMI_ATTR_AMPL_EATT_AUTO,
                                                         VI_TRUE), 3, "Mode");                                                           
        break;
        case RSEMI_VAL_EATT_MAN:
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                         RSEMI_ATTR_AMPL_EATT_STATE,
                                                         VI_TRUE), 3, "Mode");  
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                         RSEMI_ATTR_AMPL_EATT_AUTO,
                                                         VI_FALSE), 3, "Mode");                                                           
            viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                         RSEMI_ATTR_AMPL_EATT_VALUE,
                                                         manualValue), 4, "Manual Value");                                                          
        break;
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches on the preamplifier for the instrument. The 
/// HIFN  switchable gain is fixed to 20 dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Switches on the preamplifier for the instrument. The switchable gain 
/// HIPAR state/is fixed to 20 dB.
ViStatus _VI_FUNC rsemi_ConfigurePreamplifierState(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "Win%ld", window);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_AMPL_PREAMPLIFIER,
                                                state), 2, "State");
                                       
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN This function configures the preamplifier for the RF input
/// HIFN signal. The switchable gain is fixed to 20 dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR preamplifierLevel/Selects the preamplification level.
ViStatus _VI_FUNC rsemi_ConfigurePreamplifierValue (ViSession instrSession,
                                                    ViInt32 preamplifierLevel)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_AMPL_PREAMPLIFIER_LEVEL, 
                                                                preamplifierLevel), 2, "Preamplifier Level");
    
                                             
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function switches on the state for the LNA Preamp.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Switches on the LNA preamplifier for the instrument.
ViStatus _VI_FUNC rsemi_ConfigureLNAPreamplifierState (
    ViSession instrSession,
    ViInt32 window,
    ViBoolean state
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_AMPL_LNA_PREAMPLIFIER,
                                                state), 3, "State");                                                 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures auto mode for the LNA preamplifier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR autoMode/Auto Mode for the LNA Preamplifier
ViStatus _VI_FUNC rsemi_ConfigureLNAPreamplifier (
    ViSession instrSession,
    ViInt32 window,
    ViBoolean autoMode
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_AMPL_LNA_PREAMPLIFIER_AUTO,
                                                autoMode), 3, "Auto Mode");                                                 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Resets the attenuator and reconnects the RF input with the input mixer
/// HIFN after an overload condition occured and the protection mechanism intervened.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_InputProtectionReset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
            RSEMI_ATTR_AMPL_PROTECTION_RESET, VI_NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Turns the availability of attenuation levels of 10 dB or less on
/// HIFN and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR attenuationProtection/Enables and disables the attenuation level of 10 dB.
ViStatus _VI_FUNC rsemi_ConfigureAttenuationProtection (ViSession instrSession,
                                                        ViBoolean attenuationProtection)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_AMPL_PROTECTION_ENABLED, attenuationProtection), 2, "Attenuation Protection");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/// HIFN Turns the highpass filter at the RF input on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR highpassFilter/Turns the highpass filter at the RF input on and off.
ViStatus _VI_FUNC rsemi_ConfigureHighpassFilter (ViSession instrSession,
                                                 ViBoolean highpassFilter)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_AMPL_HIGHPASS_FILTER, highpassFilter), 2, "Highpass Filter");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches the YIG filter for image rejection into the 
/// HIFN  signal path (ON) or removes it from the signal path (OFF).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yigFilter/Switches the YIG filter for image rejection into the signal path (ON) 
/// HIPAR yigFilter/or removes it from the signal path (OFF).
ViStatus _VI_FUNC rsemi_ConfigureYIGFilter(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    yigFilter
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "Win%ld", window);
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, 
            RSEMI_ATTR_AMPL_YIG_FILTER, yigFilter), 3, "YIG Filter");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function enables or disables the use of the direct path for frequencies 
/// HIFN close to 0 Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR directPath/Enables or disables the use of the direct path for frequencies
/// HIPAR directPath/close to 0 Hz.
ViStatus _VI_FUNC rsemi_ConfigureDirectPath (
    ViSession instrSession,
    ViInt32 directPath
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_DIRECT_PATH, 
                                                                directPath), 2, "Direct Path");
    
                                             
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function switches the preselection on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches the preselection on or off.
ViStatus _VI_FUNC rsemi_ConfigurePreselectorState (ViSession instrSession,
                                                   ViBoolean state)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_NOISE_INP_PRES_STAT, state), 2, "State");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function selects the preselector mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Document parameter here.
ViStatus _VI_FUNC rsemi_ConfigurePreselectorMode (ViSession instrSession,
                                                  ViInt32 mode)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_INP_PRESELECTOR_FILTER_ENABLED, VI_FALSE));
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
            RSEMI_ATTR_INP_PRESELECTOR_MODE, mode), 2, "Mode");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function turns a preselecting notch filter on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR filter/Selects the filter.
/// HIPAR state/Turns a preselecting notch filter on and off.
ViStatus _VI_FUNC rsemi_ConfigurePreselectorNotchFilterState (
    ViSession instrSession,
    ViInt32 filter,
    ViBoolean state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "NFilter%ld", filter);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                  RSEMI_ATTR_INP_PRESELECTOR_NOTCH_FILTER_STATE,
                                                  state), 3, "State");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function activates the split for the preselectation at 2 MHz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Activates the split for the preselectation at 2 MHz.
ViStatus _VI_FUNC rsemi_ConfigurePreselectorSplitState (
    ViSession instrSession,
    ViBoolean state
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_INP_PRESELECTOR_FILTER_SPLIT_STATE, state), 2, "State");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function selects the preselector highpass/lowpass frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR lowpass/Sets the frequency of the low pass filter available for
/// HIPAR lowpass/preselection.
/// HIPAR highpass/Sets the frequency of the high pass filter available for
/// HIPAR highpass/preselection.
ViStatus _VI_FUNC rsemi_ConfigurePreselectorHPLPFilterFrequency (ViSession instrSession,
                                                                 ViReal64 lowpass,
                                                                 ViReal64 highpass)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_INP_PRESELECTOR_FILTER_ENABLED, VI_TRUE));
    viCheckErr (rsemi_SetAttributeViInt32 (instrSession, "", 
            RSEMI_ATTR_INP_PRESELECTOR_FILTER_TYPE, RSEMI_VAL_PRESELECTOR_FILTER_HIGHLOW));
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
            RSEMI_ATTR_INP_PRESELECTOR_FILTER_LPAS_FREQ, lowpass), 2, "Lowpass");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
            RSEMI_ATTR_INP_PRESELECTOR_FILTER_HPAS_FREQ, highpass), 3, "Highpass");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN This function selects the preselector highpass/lowpass
/// HIFN frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Sets the lower frequency of the bandpass filter.
/// HIPAR upper/Sets the upper frequency of the bandpass filter.
ViStatus _VI_FUNC rsemi_ConfigurePreselectorBandpassFilterFrequency (ViSession instrSession,
                                                                     ViReal64 lower,
                                                                     ViReal64 upper)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_INP_PRESELECTOR_FILTER_ENABLED, VI_TRUE));
    viCheckErr (rsemi_SetAttributeViInt32 (instrSession, "", 
            RSEMI_ATTR_INP_PRESELECTOR_FILTER_TYPE, RSEMI_VAL_PRESELECTOR_FILTER_BANDPASS));
    viCheckErr (viPrintf(instrSession, "INP:PRES:FILT:BPAS %.12lG,%.12lG", lower, upper));
    
    checkErr( rsemi_CheckStatus (instrSession));   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function creates new limit or selects existing one and configures 
/// HIFN  basic parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects limit.
/// HIPAR name/This control sets the limit line name.
/// HIPAR domain/This control defines the frequency or time domain for the x axis 
/// HIPAR domain/values.
/// HIPAR comment/This control sets the limit line comment.
/// HIPAR assigntoTrace/Assigns a limit line to a trace in the indicated measurement window.
/// HIPAR deleteExistingLimitLine/Removes specified limit line before new is created.
ViStatus _VI_FUNC rsemi_CreateSpectrumLimitLine(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    limit,
    ViString    name,
    ViInt32    domain,
    ViString    comment,
    ViInt32    assigntoTrace,
    ViBoolean    deleteExistingLimitLine
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      error_message[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 3, "Limit");
    
    if (strlen (name) > 8)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Name");
    }

    if (strlen (comment) > 40)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Comment");
    }

    sprintf (buffer, "L%ld", limit);
    
    if (deleteExistingLimitLine==VI_TRUE)
    {
        error = rsemi_SetAttributeViString (instrSession, buffer, RSEMI_SAN_LIMIT_DELETE,"");
    
        do
        {
            (void) rsemi_error_query (instrSession, &error, error_message);
        } while (error != 0);
    }
    viCheckParm (rsemi_SetAttributeViString (instrSession, buffer, RSEMI_SAN_LIMIT_NAME, name), 4, "Name");
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_SAN_LIMIT_CONTROL_DOMAIN,domain), 5, "Domain");
    viCheckParm (rsemi_SetAttributeViString (instrSession, buffer, RSEMI_SAN_LIMIT_COMMENT,comment), 6, "Comment");

    sprintf (buffer, "Win%ld,L%ld", window, limit);
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_SAN_LIMIT_ASSIGN_TRACE, assigntoTrace), 7, "Assign to Trace");


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures specified limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit line. 
/// HIPAR type/An upper line will be used as the maximum allowable value when 
/// HIPAR type/comparing with the data. A lower limit line defines the minimum value.
/// HIPAR units/This control defines the unit of the selected limit line.
/// HIPAR xAxisInterpolation/Selects linear or logarithmic interpolation for the calculation of 
/// HIPAR xAxisInterpolation/limit lines from frequency points.
/// HIPAR yAxisInterpolation/Selects linear or logarithmic interpolation for the upper or lower 
/// HIPAR yAxisInterpolation/limit line.
/// HIPAR xAxisScaling/Selects the relative or absolute scaling for the X axis of the 
/// HIPAR xAxisScaling/selected limit line.
/// HIPAR margin/Defines a margin to a lower limit line, at which out-of-limit values 
/// HIPAR margin/are signalled (if the limit check is active), but not handled as a 
/// HIPAR margin/violation of the limit value.
/// HIPAR threshold/Defines an absolute threshold value for limit lines with relative Y 
/// HIPAR threshold/axis scaling independently of the measurement window. The absolute 
/// HIPAR threshold/threshold value is used in the limit check
ViStatus _VI_FUNC rsemi_ConfigureLimitLine(
    ViSession    instrSession,
    ViInt32    limit,
    ViInt32    type,
    ViInt32    units,
    ViInt32    xAxisInterpolation,
    ViInt32    yAxisInterpolation,
    ViInt32    xAxisScaling,
    ViReal64    margin,
    ViReal64    threshold
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     scaling;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 2, "Limit");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, type, 0, 1), 3, "Type");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, units, 0, 17), 4, "Units");

    sprintf (buffer, "L%ld", limit);
    
    scaling = (units == RSEMI_VAL_UNIT_DB ) ? RSEMI_VAL_REL : RSEMI_VAL_ABS;
    
    viCheckParm (rsemi_SetAttributeViInt32(instrSession, buffer, RSEMI_LIMIT_CONTROL_SPACING, xAxisInterpolation), 5, "X Axis Interpolation");
    viCheckParm (rsemi_SetAttributeViInt32(instrSession, buffer, RSEMI_LIMIT_CONTROL_SCALING, xAxisScaling), 7, "X Axis Scaling");

    viCheckParm (rsemi_SetAttributeViInt32(instrSession, buffer, RSEMI_LIMIT_UNITS, units), 4, "Units");

    switch (type)
    {
        case RSEMI_VAL_LIMIT_LOWER:
            viCheckParm (rsemi_SetAttributeViInt32(instrSession, buffer, RSEMI_LIMIT_LOWER_SPACING, yAxisInterpolation), 6, "Y Axis Interpolation");
            viCheckParm (rsemi_SetAttributeViReal64(instrSession, buffer, RSEMI_LIMIT_LOWER_MARGIN, margin), 8, "Marging");
            if (scaling == RSEMI_VAL_REL)
            {
                viCheckParm (rsemi_SetAttributeViReal64(instrSession, buffer, RSEMI_LIMIT_LOWER_THRESHOLD, threshold), 9, "Threshold");
            }   
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession,buffer,RSEMI_LIMIT_LOWER_SCALING, scaling));
            break;
            
        case RSEMI_VAL_LIMIT_UPPER:
            viCheckParm (rsemi_SetAttributeViInt32(instrSession, buffer, RSEMI_LIMIT_UPPER_SPACING, yAxisInterpolation), 6, "Y Axis Interpolation");
            viCheckParm (rsemi_SetAttributeViReal64(instrSession, buffer, RSEMI_LIMIT_UPPER_MARGIN, margin), 8, "Marging");
            if (scaling == RSEMI_VAL_REL)
            {
                viCheckParm (rsemi_SetAttributeViReal64(instrSession, buffer, RSEMI_LIMIT_UPPER_THRESHOLD, threshold), 9, "Threshold");
            }
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession,buffer,RSEMI_LIMIT_UPPER_SCALING, scaling));
            break;
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines specified limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit line. 
/// HIPAR type/An upper line will be used as the maximum allowable value when 
/// HIPAR type/comparing with the data. A lower limit line defines the minimum value.
/// HIPAR count/Specifies number of values in X Axis and Amplitude arrays.
/// HIPAR xAxis/Specifies the frequency or time value for a limit point. The units are 
/// HIPAR xAxis/depended on the attribute RSEMI_ATTR_LLINE_X_AXIS_UNITS.
/// HIPAR amplitude/Specifies the amplitude value for the current limit point.
ViStatus _VI_FUNC rsemi_DefineSpectrumLimitLine(
    ViSession    instrSession,
    ViInt32    limit,
    ViInt32    type,
    ViInt32    count,
    ViReal64    xAxis[],
    ViReal64    amplitude[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *pbuffer= VI_NULL;
    ViChar      *p2buf;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 2, "Limit");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, type, 0, 1), 3, "Type");
    if (count < 1)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Count");
    }
    if (xAxis == NULL)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "X Axis");
    }
    if (amplitude == NULL)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Amplitude");
    }

    viCheckAlloc (pbuffer = (ViChar*) malloc ((size_t)(count * 20 + 200)));
    
    p2buf = pbuffer + sprintf (pbuffer, "CALC:LIM%ld:CONT ", limit);
    
    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%le,", xAxis[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    viCheckErr (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), VI_NULL));

    p2buf = pbuffer + sprintf (pbuffer, "CALC:LIM%ld:%s ", limit, SANlimTypeArr [type]);
    
    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%le,", amplitude[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    viCheckErr (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), VI_NULL));

    checkErr( rsemi_CheckStatus (instrSession));

Error:
    if (pbuffer) free (pbuffer);
    
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects the limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects limit.
/// HIPAR name/This control sets the limit line name.
/// HIPAR limitEnabled/Turns the limit line display on and off.
ViStatus _VI_FUNC rsemi_SelectLimitLine(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    limit,
    ViString    name,
    ViBoolean    limitEnabled
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 3, "Limit");
    if (strlen (name) > 8)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Name");
    }
    
    sprintf (buffer, "L%ld", limit); 
    viCheckParm (rsemi_SetAttributeViString (instrSession, buffer, RSEMI_SAN_LIMIT_NAME, name), 4, "Name");
    sprintf (buffer, "Win%ld,L%ld", window, limit); 
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_LIMIT_STATE, limitEnabled), 5, "Limit Enabled");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches on or off the limit check and/or limit line for 
/// HIFN  the selected limit line in the selected measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit line. 
/// HIPAR type/An upper line will be used as the maximum allowable value when 
/// HIPAR type/comparing with the data. A lower limit line defines the minimum value.
/// HIPAR limitEnabled/Turns the limit line display on and off.
/// HIPAR checkEnabled/Turns limit line testing on/off. 
ViStatus _VI_FUNC rsemi_EnableLimitCheck(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    limit,
    ViInt32    type,
    ViBoolean    limitEnabled,
    ViBoolean    checkEnabled
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     attribute = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 3, "Limit");

    switch (type)
    {
        case RSEMI_VAL_LIMIT_LOWER:
            attribute = RSEMI_LIMIT_LOWER_STATE;
            break;
        case RSEMI_VAL_LIMIT_UPPER:
            attribute = RSEMI_LIMIT_UPPER_STATE;
            break;
    
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Type");
    }
    
    sprintf (buffer, "Win%ld,L%ld", window, limit);

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_LIMIT_STATE, limitEnabled), 5, "Limit Enabled");
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, (ViUInt32) attribute, checkEnabled), 6, "Check Enabled");
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function moves specified limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be type to be moved.
/// HIPAR method/This control selects the method of moving.
/// HIPAR moveValue/Defines the move value.
ViStatus _VI_FUNC rsemi_MoveLimitLine(
    ViSession    instrSession,
    ViInt32    limit,
    ViInt32    type,
    ViInt32    method,
    ViReal64 moveValue
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     attributes[3][2] = {{RSEMI_LIMIT_LOWER_OFFSET,RSEMI_LIMIT_LOWER_SHIFT},
                                   {RSEMI_LIMIT_UPPER_OFFSET,RSEMI_LIMIT_UPPER_SHIFT},
                                   {RSEMI_LIMIT_CONTROL_OFFSET,RSEMI_LIMIT_CONTROL_SHIFT}};

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 2, "Limit");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, type, 0, 2), 3, "Type");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, method, 0, 1), 4, "Method");

    sprintf (buffer, "L%ld", limit);

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, (ViUInt32)attributes[type][method], moveValue), 5, "Move Value");


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function copies specified limit line into another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR copyTo/This control selects limit.
ViStatus _VI_FUNC rsemi_CopySpectrumLimitLine(
    ViSession    instrSession,
    ViInt32    limit,
    ViInt32    copyTo
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 2, "Limit");

    sprintf (buffer, "L%ld", limit);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_SAN_LIMIT_COPY, copyTo), 3, "Copy To");


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function removes specified limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
ViStatus _VI_FUNC rsemi_DeleteLimitLine(
    ViSession    instrSession,
    ViInt32    limit
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 2, "Limit");

    sprintf (buffer, "L%ld", limit);

    viCheckErr (rsemi_SetAttributeViString (instrSession, buffer, RSEMI_SAN_LIMIT_DELETE, VI_NULL));


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function deletes the result of the current limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects limit.
ViStatus _VI_FUNC rsemi_ClearLimitLineResults(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    limit
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      lim_line_no[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 3, "Limit");
    
    sprintf(lim_line_no,"Win%ld,L%ld",window, limit);

    viCheckErr(rsemi_SetAttributeViString (instrSession, lim_line_no,
                                               RSEMI_LIMIT_CLEAR, VI_NULL));


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures state of specified display line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/This control selects line.
/// HIPAR type/Selects the line to be shown.
/// HIPAR state/Specifies whether selected line is enabled.
ViStatus _VI_FUNC rsemi_ConfigureDisplayLimitLineState(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    line,
    ViInt32    type,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (type){
        case RSEMI_VAL_LINE_THR:
            sprintf (buffer, "Win%ld", window);    
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                         RSEMI_ATTR_THRLINE_STATE,
                                                         state), 5, "State"); 
        break;
        case RSEMI_VAL_LINE_DISPLAY:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");
            sprintf (buffer, "Win%ld,L%ld", window, line);
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                         RSEMI_ATTR_DLINE_STATE,
                                                         state), 5, "State"); 
        break;
        case RSEMI_VAL_LINE_FREQ:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");
            sprintf (buffer, "Win%ld,L%ld", window, line);
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                         RSEMI_ATTR_FLINE_STATE,
                                                         state), 5, "State"); 
        break;
        case RSEMI_VAL_LINE_TIME:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");
            sprintf (buffer, "Win%ld,L%ld", window, line);
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                         RSEMI_ATTR_TLINE_STATE,
                                                         state), 5, "State"); 
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Type");     
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures position of specified display line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/This control selects line.
/// HIPAR type/Selects the line to be shown.
/// HIPAR position/Defines position of selected line.
ViStatus _VI_FUNC rsemi_ConfigureDisplayLimitLinePosition(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    line,
    ViInt32    type,
    ViReal64    position
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (type){
        case RSEMI_VAL_LINE_THR:
            sprintf (buffer, "Win%ld", window);    
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                         RSEMI_ATTR_THRLINE_POSITION,
                                                         position), 5, "Position");                                                           
        break;
        case RSEMI_VAL_LINE_DISPLAY:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");
            sprintf (buffer, "Win%ld,L%ld", window, line);
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                         RSEMI_ATTR_DLINE_POSITION,
                                                         position), 5, "Position");                                                           
        break;
        case RSEMI_VAL_LINE_FREQ:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");
            sprintf (buffer, "Win%ld,L%ld", window, line);
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                         RSEMI_ATTR_FLINE_POSITION,
                                                         position), 5, "Position");                                                              
        break;
        case RSEMI_VAL_LINE_TIME:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");
            sprintf (buffer, "Win%ld,L%ld", window, line);
             viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                         RSEMI_ATTR_TLINE_POSITION,
                                                         position), 5, "Position");                                                          
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Type");     
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the names of all active limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR stringSize/This control sets the allocated length of Active Limit Lines string.
/// HIPAR activeLimitLines/This control returns the control lines of the user ports.
/// HIPAR returnedStringLength/Returns the length of string returned from the instrument.
ViStatus _VI_FUNC rsemi_GetActiveLimitLines(
    ViSession    instrSession,
    ViInt32    stringSize,
    ViChar    activeLimitLines[],
    ViInt32*    returnedStringLength
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer,
                *pbuf;
    ViInt32    retCnt;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!activeLimitLines)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Active Limit Lines");
    
    viCheckErr( viPrintf (instrSession, ":CALC:LIM:ACT?\n")); 
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL)); 
    retCnt = (ViUInt32) (strrchr (pbuffer, '\'') - pbuffer);
    if (returnedStringLength) 
    {
        *returnedStringLength = retCnt;
    }   
    
    retCnt=(stringSize>retCnt)?retCnt:stringSize;
    pbuf=pbuffer;
    pbuf++;
    strncpy(activeLimitLines, pbuf, retCnt-1);
    activeLimitLines[retCnt-1]='\0';
    
    if (pbuffer) free(pbuffer);
    
    checkErr( rsemi_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}




/// HIFN This function enables and disables the limit check for a  specific trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects limit.
/// HIPAR trace/Specifies a trace
/// HIPAR checkEnabled/Turns limit line testing on/off.   
ViStatus _VI_FUNC rsemi_EnableTraceLimitCheck (ViSession instrSession,
                                               ViInt32 window, 
                                               ViInt32 limit,
                                               ViInt32 trace,
                                               ViBoolean checkEnabled)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_lim[10] = "";  
    ViChar      rep_cap_trace[10] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_TRACE_CHECK_ENABLED,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_TRACE_CHECK_ENABLED,
                                                 "LimitLine",
                                                 limit - 1,
                                                 10,
                                                 rep_cap_lim), 3, "Limit");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_TRACE_CHECK_ENABLED,
                                                 "Trace",
                                                 trace - 1,
                                                 10,
                                                 rep_cap_trace), 4, "Trace");
        
    sprintf (buffer, "%s,%s,%s", rep_cap_win, rep_cap_lim, rep_cap_trace);

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_LIMIT_LINE_TRACE_CHECK_ENABLED, checkEnabled), 5, "Check Enabled");
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the active measurement window.
/// HIFN     In FULL SCREEN mode, the measurements are only performed in the active 
/// HIFN  measurement window. Measurements are therefore initiated in the active 
/// HIFN  window and result queries (marker, trace data and other results) 
/// HIFN  answered also in the active window.
/// HIFN     Initiating measurements and querying results in the inactive window 
/// HIFN  yields an error message (execution error).
/// HIFN     In split screen mode, the selection of the active window for result 
/// HIFN  queries is irrelevant.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     
/// HIFN     (1) In FULL SCREEN mode, settings can also be performed in the 
/// HIFN  inactive measurement window. They become effective as soon as the 
/// HIFN  corresponding window becomes active.
/// HIFN     
/// HIFN     (2) This function is not available in GSM/EDGE mode. In GSM/EDGE mode, 
/// HIFN  the display is always set to FULL SCREEN, which corresponds to SCREEN 
/// HIFN  A, which is automatically selected when the GSM/EDGE mode is activated 
/// HIFN  with command INSTrument:SELect:MGSM.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_DisplaySetActiveWindow(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);      
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_SELECT_WINDOW,
                                                VI_NULL));                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function queries the currently selected subwindow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subwindow/This control queries the currently selected subwindow.
ViStatus _VI_FUNC rsemi_GetSelectedSubwindow(
    ViSession    instrSession,
    ViInt32*    subwindow
)
{
    ViStatus    error = VI_SUCCESS;
    
     checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (subwindow == VI_NULL)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Subwindow");
    }
    viCheckErr( rsemi_GetAttributeViInt32(instrSession, "", RSEMI_ATTR_QUERY_SELECT_SUBWINDOW,
                                                subwindow));                                                

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function moves the focus area to the selected window and 
/// HIFN  subwindow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
ViStatus _VI_FUNC rsemi_ConfigureDisplayFocusedArea(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    subwindow
)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subwindow, 1, 2), 3, "Subwindow");
    sprintf (buffer, "Win%ld,SW%ld", window,subwindow);      
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_DISPLAY_FOCUS_AREA,
                                                VI_NULL));                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function controls the apperance of display elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/This control switches the x-axis annotation on or off.
/// HIPAR logo/This control switches the factory logo on the screen on or off.
/// HIPAR time/This control switches on or off the screen display of date and time.
ViStatus _VI_FUNC rsemi_ConfigureDisplayControl(
    ViSession    instrSession,
    ViBoolean    frequency,
    ViBoolean    logo,
    ViBoolean    time
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rsemi_SetAttributeViBoolean (instrSession, "", 
                                                RSEMI_ATTR_DISP_FREQ_STATE,
                                                frequency), 2, "Frequency");                                                
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                                    RSEMI_ATTR_DISP_TIME_STATE,
                                                    time), 4, "Time");
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                                RSEMI_ATTR_DISP_LOGO_STATE,
                                                logo), 3, "Logo");      
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function controls the apperance of display title.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control switches the x-axis annotation on or off. The driver uses 
/// HIPAR state/this value to set the RSEMI_ATTR_DISP_COMMENT_STATE attribute.
/// HIPAR comment/Defines a comment (max. 20 characters) which can be displayed on the 
/// HIPAR comment/screen in the selected measurement window.
ViStatus _VI_FUNC rsemi_ConfigureDisplayComment(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state,
    ViString    comment
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);      
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_DISP_COMMENT_STATE,
                                                state), 2, "State"); 
    viCheckParm( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_DISP_COMMENT,
                                                comment), 3, "Comment");                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function switches the measurement window for channel and 
/// HIFN  adjacent-channel power measurements to full screen or half screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR windowSize/Switches the measurement window for channel and adjacent-channel power 
/// HIPAR windowSize/measurements to full screen or half screen.
ViStatus _VI_FUNC rsemi_ConfigureDisplayWindowSize(
    ViSession    instrSession,
    ViInt32    windowSize
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_DISP_WINDOW_SIZE,
                                                windowSize), 2, "Window Size"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function configures the measurement display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR windowSize/Configures the measurement display.
ViStatus _VI_FUNC rsemi_ConfigureDisplaySubwindowSize(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    subwindow,
    ViInt32    windowSize
)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subwindow, 1, 2), 3, "Subwindow");
    sprintf (buffer, "Win%ld,SW%ld", window, subwindow);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer,
        RSEMI_ATTR_DISPLAY_SUBWINDOW_SIZE, windowSize), 3, "Window Size");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function defines the display range of the Y axis (level axis) in 
/// HIFN  the selected measurement window with logarithmic scaling 
/// HIFN  (DISP:TRAC:Y:SPAC LOG).
/// HIFN     For linear scaling, (DISP:TRAC:Y:SPAC LIN | PERC) the display range is 
/// HIFN  fixed and cannot be modified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR range/Defines the display range of the Y axis (level axis) in the selected 
/// HIPAR range/measurement window with logarithmic scaling (DISP:TRAC:Y:SPAC LOG).
ViStatus _VI_FUNC rsemi_ConfigureDisplayLogRange(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    range
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_DISP_LOG_RANGE, 
                range), 3, "Range");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the scale type of the Y axis (absolute or 
/// HIFN  relative) in the selected measurement window.
/// HIFN     When SYSTem:DISPlay is set to OFF, this command has no immediate 
/// HIFN  effect on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxisGridMode/Defines the scale type of the Y axis (absolute or relative) in the 
/// HIPAR yAxisGridMode/selected measurement window.
ViStatus _VI_FUNC rsemi_ConfigureDisplayAmplitudeGridMode(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    yAxisGridMode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                RSEMI_ATTR_DISP_AMPLITUDE_GRID_MODE, yAxisGridMode), 
                3, "Range");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  If the external generator control option (FSP-B10) is mounted and the 
/// HIFN  normalization in the NETWORK mode is activated, this value defines the 
/// HIFN  power value assigned to the reference position in the selected 
/// HIFN  measurement window. This value corresponds to the parameter REFERENCE 
/// HIFN  VALUE in manual operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR referenceValue/If the external generator control option (FSP-B10) is mounted and the 
/// HIPAR referenceValue/normalization in the NETWORK mode is activated, this value defines the 
/// HIPAR referenceValue/power value assigned to the reference position in the selected 
/// HIPAR referenceValue/measurement window. This value corresponds to the parameter REFERENCE 
/// HIPAR referenceValue/VALUE in manual operation.
ViStatus _VI_FUNC rsemi_ConfigureDisplayReferenceValue(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    referenceValue
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                RSEMI_ATTR_DISP_REF_VALUE, referenceValue), 
                3, "Reference Position");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the position of the reference value in the 
/// HIFN  selected measurement window.
/// HIFN     In operating mode NETWORK (Ext. Generator Option FSP-B10) with active 
/// HIFN  normalization, RPOSition defines the reference point for the output of 
/// HIFN  the normalized measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR referencePosition/Defines the position of the reference value in the selected 
/// HIPAR referencePosition/measurement window.
ViStatus _VI_FUNC rsemi_ConfigureDisplayReferencePosition(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    referencePosition
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                RSEMI_ATTR_DISP_REF_POSITION, referencePosition), 
                3, "Reference Value Position");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches block data transmission during a scan on
/// HIFN  and off. The block size depends on the scan time, the trace
/// HIFN  number is not evaluated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR control/This control defines the control block.
/// HIPAR transmissionMode/This control selects the transmission mode.
ViStatus _VI_FUNC rsemi_ConfigureScanTransmissionMode (ViSession instrSession,
                                                       ViInt32 control,
                                                       ViInt32 transmissionMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, control, 1, 4), 2, "Control");
    
    sprintf (buffer, "CB%ld", control);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                RSEMI_ATTR_SCAN_TRANSMISSION_MODE, transmissionMode), 
                3, "Transmission Mode");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the external generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generatorAddress/This control changes the IEC/IEEE-bus address of the device
/// HIPAR generatorAddress/selected as external generator.
/// HIPAR TTLHandshake/This control selects the link type of the external generator if
/// HIPAR TTLHandshake/the GPIB interface is used.
/// HIPAR generatorType/This control selects the type of external generator.
/// HIPAR reference/This control sets the reference signal for the external generator.
ViStatus _VI_FUNC rsemi_ConfigureExternalGeneratorGPIB (
    ViSession instrSession,
    ViInt32 generatorAddress,
    ViInt32 TTLHandshake,
    ViString generatorType,
    ViInt32 reference
)
{
    ViStatus    error = VI_SUCCESS;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_EXT_GEN_TYPE, 
                                                generatorType), 4, "Generator Type");  
   
    viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_EXT_GEN_INTERFACE_TYPE, 
                                                RSEMI_VAL_INTERFACE_GPIB));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_EXT_GEN_IEEE_ADDR, 
                                                generatorAddress), 2, "Generator Address");
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_EXT_GEN_INTF_TYPE, 
                                                TTLHandshake), 3, "TTL Handshake");
                                                  
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_EXT_GEN_ROSC_SOURCE, 
                                                reference), 5, "Reference");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches the selected tracking generator on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches the selected tracking generator on or off.
ViStatus _VI_FUNC rsemi_TrackingGeneratorEnable(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                              RSEMI_ATTR_TRACKING_GENERATOR_ENABLED,
                                              state), 2, "State");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures tracking generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects the type of measurement to be performed with the tracking 
/// HIPAR measurementType/generator.
/// HIPAR normalization/Turns correction of measurement results (normalization) on and off.
ViStatus _VI_FUNC rsemi_ConfigureTrackingGenerator(
    ViSession    instrSession,
    ViInt32    measurementType,
    ViBoolean    normalization
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                            RSEMI_ATTR_TRACKING_GENERATOR_MEASUREMENT_TYPE,
                                            measurementType), 2, "Measurement Type");
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                              RSEMI_ATTR_TRACKING_GENERATOR_NORMALIZATION,
                                              normalization), 3, "Normalization");
    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function initiates a reference measurement (calibration). The 
/// HIFN  reference measurement is the basis for the measurement normalization.
/// HIFN     To obtain a correct reference measurement, a complete sweep with 
/// HIFN  synchronization to the end of the sweep must have been carried out. 
/// HIFN  This is only possible in the single sweep mode.
/// HIFN     The command is available if a tracking generator is installed and has 
/// HIFN  been turned on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR calibration/Initiates a reference measurement (calibration). The reference 
/// HIPAR calibration/measurement is the basis for the measurement normalization.
ViStatus _VI_FUNC rsemi_ConfigureTrackingGeneratorCalibration(
    ViSession    instrSession,
    ViInt32    calibration
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                            RSEMI_ATTR_TRACKING_GENERATOR_CALIBRATION,
                                            calibration), 2, "Calibration");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function restores the measurement configuration used for 
/// HIFN  calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_TrackingGeneratorRestore(
    ViSession    instrSession
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "",
                                            RSEMI_ATTR_TRACKING_GENERATOR_RESTORE, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function turns external amplitude modulation of the tracking 
/// HIFN  generator on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR am/Turns external amplitude modulation of the tracking generator on and 
/// HIPAR am/off.
ViStatus _VI_FUNC rsemi_ConfigureTrackingGeneratorModulationAM(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    am
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                              RSEMI_ATTR_TRACKING_GENERATOR_MODULATION_AM,
                                              am), 3, "AM");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function turns external I/Q modulation of the tracking generator 
/// HIFN  on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR iq/Turns external I/Q modulation of the tracking generator on and off.
ViStatus _VI_FUNC rsemi_ConfigureTrackingGeneratorModulationIQ(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    iq
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                              RSEMI_ATTR_TRACKING_GENERATOR_MODULATION_IQ,
                                              iq), 3, "IQ");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures external frequency modulation of the tracking 
/// HIFN  generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR fm/Turns external frequency modulation of the tracking generator on and 
/// HIPAR fm/off.
/// HIPAR deviation/Defines the maximum frequency deviation at 1 V input voltage at the FM 
/// HIPAR deviation/input of the tracking generator.
ViStatus _VI_FUNC rsemi_ConfigureTrackingGeneratorModulationFM(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    fm,
    ViReal64    deviation
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                              RSEMI_ATTR_TRACKING_GENERATOR_MODULATION_FM,
                                              fm), 3, "FM");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_TRACKING_GENERATOR_MODULATION_FM_DEVIATION,
                                             deviation), 4, "Deviation");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function defines a frequency offset of the tracking generator. 
/// HIFN  Frequency-converting DUTs can be measured with this setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR offset/Defines a frequency offset of the tracking generator. 
/// HIPAR offset/Frequency-converting DUTs can be measured with this setting.
ViStatus _VI_FUNC rsemi_ConfigureTrackingGeneratorModulationFrequencyOffset(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    offset
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_TRACKING_GENERATOR_MODULATION_FREQUENCY_OFFSET,
                                             offset), 3, "Offset");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures tracknig generator levels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR outputLevel/Defines the output level of the tracking generator.
/// HIPAR offset/Defines a level offset for the tracking generator level. Thus, for 
/// HIPAR offset/example, attenuators or amplifiers at the output of the tracking 
/// HIPAR offset/generator can be taken into account for the setting.
ViStatus _VI_FUNC rsemi_ConfigureTrackingGeneratorModulationLevel(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    outputLevel,
    ViReal64    offset
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_TRACKING_GENERATOR_MODULATION_OUTPUT_LEVEL,
                                             outputLevel), 3, "Output Level");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_TRACKING_GENERATOR_MODULATION_LEVEL_OFFSET,
                                             offset), 4, "Offset");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures tracking generator power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR start/Defines the start power of the power sweep.
/// HIPAR stop/Defines the stop power of the power sweep.
ViStatus _VI_FUNC rsemi_ConfigureTrackingGeneratorPower(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    start,
    ViReal64    stop
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_TRACKING_GENERATOR_POWER_START,
                                             start), 3, "Start");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_TRACKING_GENERATOR_POWER_STOP,
                                             stop), 4, "Stop");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function turns the power sweep on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR mode/Turns the power sweep on and off.
ViStatus _VI_FUNC rsemi_ConfigureTrackingGeneratorPowerMode(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    mode
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                            RSEMI_ATTR_TRACKING_GENERATOR_POWER_MODE,
                                            mode), 3, "Mode");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function switches on or off the update of all display elements 
/// HIFN  during remote control.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     The best performance is obtained when the display output is switched 
/// HIFN  off during remote control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayInRemote/This control enables or disables instrument display when in remote 
/// HIPAR displayInRemote/control mode.
ViStatus _VI_FUNC rsemi_ConfigureDisplayUpdate(
    ViSession    instrSession,
    ViBoolean    displayInRemote
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                RSEMI_ATTR_DISP_UPDATE, displayInRemote), 
                2, "Display In Remote");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function activates or deactivates the display of the front panel 
/// HIFN  keys on the screen. With the display activated, the instrument can be 
/// HIFN  operated on the screen using the mouse by pressing the corresponding 
/// HIFN  buttons. This may be useful if the instrument is operated in a detached 
/// HIFN  station by means of a remote program such as PCANYWHERE.
/// HIFN     
/// HIFN     Notes: 
/// HIFN     
/// HIFN     (1) With the display of the front panel keys activated, the screen 
/// HIFN  resolution of the unit is set to 1024x768. Thus, only a section of the 
/// HIFN  whole screen is visible on the internal LCD display, which will be 
/// HIFN  moved by mouse moves. For a full display of the user interface, an 
/// HIFN  external monitor has to be connected to the rear panel. When the front 
/// HIFN  panel display is deactivated, the original screen resolution is 
/// HIFN  restored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR softFrontpanel/This control enables or disables instrument display when in remote 
/// HIPAR softFrontpanel/control mode.
ViStatus _VI_FUNC rsemi_ConfigureDisplaySoftFrontpanel(
    ViSession    instrSession,
    ViBoolean    softFrontpanel
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                RSEMI_ATTR_DISP_FP_KEYS, softFrontpanel), 
                2, "Soft Front Panel");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the scaling of the y-axis in the currently 
/// HIFN  selected unit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR scaling/Defines the scaling of the Y axis.
ViStatus _VI_FUNC rsemi_ConfigureDisplayYAxisScaling(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    scaling
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_DISP_Y_AXIS_SCALING,
                                              scaling), 3,"Scaling");
                                               
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the display settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR statusBarState/This control switches the statusbar under the diagram on or off.
/// HIPAR softkeyBarState/This control switches the softkey bar on the right side of the diagram 
/// HIPAR softkeyBarState/on or off.
/// HIPAR toolBarState/This control switches the toolbar above the diagram on or off. 
/// HIPAR touchscreenState/This control switches the touch screen functionality on, off, or 
/// HIPAR touchscreenState/partially on.
ViStatus _VI_FUNC rsemi_ConfigureDisplaySettings(
    ViSession    instrSession,
    ViBoolean    statusBarState,
    ViBoolean    softkeyBarState,
    ViBoolean    toolBarState,
    ViInt32    touchscreenState
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_DISP_STATUS_BAR,
                                              statusBarState), 2, "Status Bar State");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_DISP_SOFTKEY_BAR,
                                              softkeyBarState), 3, "Softkey Bar State");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_DISP_TOOL_BAR,
                                              toolBarState), 4, "Tool Bar State");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_DISP_TOUCHSCREEN_STATE,
                                              touchscreenState), 5, "Touchscreen State");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Turns the display of the Input State icon on the toolbar on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR inputStateShow/Document parameter here.
ViStatus _VI_FUNC rsemi_ConfigureDisplayShowInputState (ViSession instrSession,
                                                        ViBoolean inputStateShow)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm(rsemi_SetAttributeViBoolean(instrSession, "",
        RSEMI_ATTR_DISPLAY_INPUT_STATE, inputStateShow), 2, "Input State Show");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function hides or displays the channel bar information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelBarState/Hides or displays the channel bar information.
ViStatus _VI_FUNC rsemi_ConfigureDisplayChannelBarState (
    ViSession instrSession,
    ViBoolean channelBarState
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_DISP_CHANNEL_BAR_STATE,
                                              channelBarState), 2, "Channel Bar State");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches between the MultiView tab and the most
/// HIFN  recently displayed measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR multiViewState/Switches between the MultiView tab and the most recently
/// HIPAR multiViewState/displayed measurement channel.
ViStatus _VI_FUNC rsemi_ConfigureDisplayMultiViewState (
    ViSession instrSession,
    ViBoolean multiViewState
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_DISP_MULTIVIEW_ENABLED,
                                              multiViewState), 2, "Multi View State");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function displays the selection list of available themes for the 
/// HIFN  screen display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofThemes/This control returns number of names of all themes.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog 
/// HIPAR bufferSize/list parameter.
/// HIPAR themeList/This control returns the names of all themes stored on the hard disk.
ViStatus _VI_FUNC rsemi_GetDisplayThemeCatalog(
    ViSession    instrSession,
    ViInt32*    numberofThemes,
    ViInt32    bufferSize,
    ViChar    themeList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      *buf=VI_NULL;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if (bufferSize < 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");         
    if ((bufferSize > 0) && (themeList == VI_NULL))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Theme List");  
       
    viCheckErr (viPrintf (instrSession, "DISP:THEM:CAT?\n"));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &buf, &retCnt));
    viCheckErr (rsemi_ParseCatalog(buf, bufferSize, themeList, numberofThemes));

    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    if (buf) free(buf);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects a theme for the screen display. The theme 
/// HIFN  defines the colors used for keys and screen elements, for example. Use 
/// HIFN  command DISPlay:THEMe:CATalog? to get a list of possible themes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR theme/This control selects a theme for the screen display.
ViStatus _VI_FUNC rsemi_ConfigureDisplayThemeSelect(
    ViSession    instrSession,
    ViString    theme
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_DISP_SELECT_THEME, theme), 2, "Theme");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function turns the marker information in all diagrams on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR markerInfo/Turns the marker information in all diagrams on and off.
ViStatus _VI_FUNC rsemi_ConfigureMarkerInfo (
    ViSession instrSession,
    ViBoolean markerInfo
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_DISPLAY_MARKER_INFO,
                                                 markerInfo), 2, "Marker Info");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the display zooming parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR zoomState/This control activates and deactivates the zoom mode.
/// HIPAR x1/Defines the x1 value of the area for which the display is enlarged. 
/// HIPAR x1/The area is defined via the coordinates of the lower left (x1, y1) and 
/// HIPAR x1/top right (x2,y2) corners of a rectangle, where x and y are percentages 
/// HIPAR x1/of the complete diagram. (0,0) defines the lower left corner of the 
/// HIPAR x1/diagram area, (100,100) defines the top right corner.
/// HIPAR y1/Defines the y1 value of the area for which the display is enlarged. 
/// HIPAR y1/The area is defined via the coordinates of the lower left (x1, y1) and 
/// HIPAR y1/top right (x2,y2) corners of a rectangle, where x and y are percentages 
/// HIPAR y1/of the complete diagram. (0,0) defines the lower left corner of the 
/// HIPAR y1/diagram area, (100,100) defines the top right corner.
/// HIPAR x2/Defines the x1 value of the area for which the display is enlarged. 
/// HIPAR x2/The area is defined via the coordinates of the lower left (x1, y1) and 
/// HIPAR x2/top right (x2,y2) corners of a rectangle, where x and y are percentages 
/// HIPAR x2/of the complete diagram. (0,0) defines the lower left corner of the 
/// HIPAR x2/diagram area, (100,100) defines the top right corner.
/// HIPAR y2/Defines the y2 value of the area for which the display is enlarged. 
/// HIPAR y2/The area is defined via the coordinates of the lower left (x1, y1) and 
/// HIPAR y2/top right (x2,y2) corners of a rectangle, where x and y are percentages 
/// HIPAR y2/of the complete diagram. (0,0) defines the lower left corner of the 
/// HIPAR y2/diagram area, (100,100) defines the top right corner.
ViStatus _VI_FUNC rsemi_ConfigureDisplayZoom(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    zoomState,
    ViReal64    x1,
    ViReal64    y1,
    ViReal64    x2,
    ViReal64    y2
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";     
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViReal64Range (instrSession, x1, 0.0, 100.0), 4, "x1");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, y1, 0.0, 100.0), 5, "y1"); 
    viCheckParm (rsemi_invalidViReal64Range (instrSession, x2, 0.0, 100.0), 6, "x2");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, y2, 0.0, 100.0), 7, "y2");  
       
    sprintf (buffer, "Win%ld", window);  

    viCheckParm ( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                RSEMI_ATTR_DISPLAY_ZOOM_STATE, zoomState),
                3, "Zoom State");
    
    if (zoomState == VI_TRUE) {
        
        viCheckErr( viPrintf (instrSession, "DISP:WIND%ld:ZOOM:AREA %.12lG,%.12lG,%.12lG,%.12lG\n", window, x1, y1, x2, y2));
        
        checkErr( rsemi_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function changes the presentation of 'capacitance C<i> in parallel with 
/// HIFN  resistance R<i>' circuit blocks in lumped de/embedding networks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayConductances/Changes the presentation of 'capacitance C<i> in parallel with
/// HIPAR displayConductances/resistance R<i>' circuit blocks in lumped de/embedding networks.
ViStatus _VI_FUNC rsemi_ConfigureDisplayConductances (
    ViSession instrSession,
    ViBoolean displayConductances
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_DISPLAY_CONDUCTANCES,
                                                 displayConductances), 2, "Display Conductances");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the currently active measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control returns the index number of the currently active 
/// HIPAR window/measurement window.
ViStatus _VI_FUNC rsemi_QueryActiveMeasurementWindow(
    ViSession    instrSession,
    ViInt32*    window
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_GetAttributeViInt32 (instrSession, "",
                                                RSEMI_ATTR_QUERY_ACTIVE_MEASUREMENT_WINDOW,
                                                window), 2, "Window"); 
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 *------------------- Layout ------------------------------------------------*
 *===========================================================================*/
/// HIFN This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR direction/Direction the window is added relative to the window set with "Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/Returns the name of the new window.
ViStatus _VI_FUNC rsemi_AddWindow (ViSession instrSession, ViInt32 stringSize,
                                      ViString windowName, ViInt32 direction,
                                      ViInt32 windowType,
                                      ViChar returnedWindowName[])
{
    ViStatus    error   = VI_SUCCESS;
    size_t      length;
    ViChar*     pbuffer = NULL;
    ViChar*     p2buf = NULL;
    ViReal64     discrete;
    ViChar       *p2direction = NULL;
    ViChar       *p2type = NULL;   
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    do
    {
        discrete = rsemi_rngLayoutDirection.rangeValues[i].discreteOrMinValue;
        p2direction = rsemi_rngLayoutDirection.rangeValues[i].cmdString;

        if (p2direction == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Direction");

        if (fabs(discrete - direction) <= 0.01) break;

        i++;

    } while (i != 0);
    i=0;
    do
    {
        discrete = rsemi_rngLayoutQueryWindowType.rangeValues[i].discreteOrMinValue;
        p2type = rsemi_rngLayoutQueryWindowType.rangeValues[i].cmdString;

        if (p2type == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Window Type");

        if (fabs(discrete - windowType) <= 0.01) break;

        i++;

    } while (i != 0);        
    
    viCheckErr (viPrintf (instrSession, "LAY:ADD? '%s',%s,%s", windowName, p2direction, p2type));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, NULL));
    
    if (returnedWindowName != NULL)
    {
        size_t len = strstr (pbuffer+1,"'") - pbuffer - 1;
        length = (size_t) stringSize - 1 <= len ? (size_t) stringSize-1 : len;
        p2buf = strtok (pbuffer, "'");
        strncpy (returnedWindowName, p2buf, length);
    }
 
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    if (pbuffer != NULL) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
} 

/// HIFN This function removes a window from the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
ViStatus _VI_FUNC rsemi_RemoveWindow (ViSession instrSession,
                                         ViString windowName)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_LAYOUT_WINDOW_REMOVE, windowName),
                2, "Window Name");                

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsemi_ReplaceWindow (ViSession instrSession,
                                          ViString windowName,
                                          ViInt32 windowType)
{
    ViStatus    error   = VI_SUCCESS;
    ViReal64     discrete;
    ViChar       *p2type = NULL;   
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    do
    {
        discrete = rsemi_rngLayoutQueryWindowType.rangeValues[i].discreteOrMinValue;
        p2type = rsemi_rngLayoutQueryWindowType.rangeValues[i].cmdString;

        if (p2type == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Window Type");

        if (fabs(discrete - windowType) <= 0.01) break;

        i++;

    } while (i != 0);
        
    viCheckErr (viPrintf (instrSession, "LAY:REPL '%s',%s", windowName, p2type));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function queries the name and index of all active windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR bufferSize/This control specifies the size of the buffer.
/// HIPAR results/Returns the name and index of all active windows.
ViStatus _VI_FUNC rsemi_QueryWindowsList (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar results[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL)); 
    
    viCheckParm (rsemi_GetAttributeViString (instrSession, "", RSEMI_ATTR_LAYOUT_WINDOW_LIST, 
                                                                bufferSize, results), 3, "Results");                


Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}


/// HIFN This function queries the index of a particular display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR index/Index number of the window.
ViStatus _VI_FUNC rsemi_QueryWindowIndex (ViSession instrSession,
                                             ViString windowName, ViInt32 *index)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *buffer = NULL;
    ViChar      *pbuff = NULL; 
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (viPrintf (instrSession, "LAY:IDEN? '%s'\n", windowName));
    
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &buffer, NULL));
    
    pbuff = strtok (buffer, ",\n");
    *index = atol(pbuff);
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function queries the name of a particular display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR index/Index number of the window.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to. 
ViStatus _VI_FUNC rsemi_QueryWindowName (ViSession instrSession, ViInt32 stringSize,
                                            ViInt32 index, ViChar windowName[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LAYOUT_QUERY_WINDOW_NAME,
                                                 "Window",
                                                 index - 1,
                                                 10,
                                                 rep_cap));
    viCheckParm (rsemi_GetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_LAYOUT_QUERY_WINDOW_NAME, 
                                                                stringSize, windowName), 3, "Window Name");
    
                                             
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}


/// HIFN This function changes the position of a splitter
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR firstWindowName/String containing the number or name of the first window the splitter controls.
/// HIPAR secondWindowName/String containing the number or name of the second window the splitter controls. 
/// HIPAR position/New vertical or horizontal position of the splitter in percent of the screen area.
ViStatus _VI_FUNC rsemi_ConfigureSplitterPosition (ViSession instrSession,
                                                      ViString firstWindowName,
                                                      ViString secondWindowName,
                                                      ViInt32 position)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      cmd [100] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
        
    sprintf (cmd, "LAY:SPL '%s','%s',%ld", firstWindowName, secondWindowName, position);
    
    viCheckErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function resets the screen colors of all display items to their 
/// HIFN  default settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR defaultSetting/This control selects the predefined setting to be applied.
ViStatus _VI_FUNC rsemi_DisplayColorDefault(
    ViSession    instrSession,
    ViInt32    defaultSetting
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, defaultSetting, 1, 2), 2, "Default Settings");
    sprintf (buffer, "DEF%ld", defaultSetting);  
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_DISP_COL_PRESET, VI_NULL));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the color table of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR colorMap/This control defines the color table of the instrument.
/// HIPAR tint/This control sets the hue = TINT.
/// HIPAR saturation/This control sets the sat = SATURATION.
/// HIPAR brightness/This control sets the lum = BRIGHTNESS.
ViStatus _VI_FUNC rsemi_ConfigureDisplayColor(
    ViSession    instrSession,
    ViInt32    colorMap,
    ViReal64    tint,
    ViReal64    saturation,
    ViReal64    brightness
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, colorMap, 1, 49), 2, "Color Map");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, tint, 0.0, 1.0), 3, "TINT");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, saturation, 0.0, 1.0), 4, "SATURATION"); 
    viCheckParm (rsemi_invalidViReal64Range (instrSession, brightness, 0.0, 1.0), 5, "BRIGHTNESS");        
    
    viCheckErr( viPrintf (instrSession, "DISP:CMAP%ld:HSL %.12f,%.12f,%.12f\n",
                colorMap, tint, saturation, brightness));

    checkErr( rsemi_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the color table of the instrument using 
/// HIFN  predefined colors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR colorMap/This control defines the color table of the instrument.
/// HIPAR predefinedColors/This control defines the color table of the instrument using 
/// HIPAR predefinedColors/predefined color values.
ViStatus _VI_FUNC rsemi_ConfigureDisplayColorByPredefined(
    ViSession    instrSession,
    ViInt32    colorMap,
    ViInt32    predefinedColors
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, colorMap, 1, 49), 2, "Color Map");
    sprintf (buffer, "CM%ld", colorMap);  
    
    viCheckParm ( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                RSEMI_ATTR_DISP_COL_PREDEFINED, predefinedColors),
                3, "Predefined Colors");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function generates a transducer factor <name> using normalized 
/// HIFN  trace data. 
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     (1) The function is only available when normalization is switched on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR transducerState/This command switches the selected transducer factor on or off.
/// HIPAR transducerName/This command selects the transducer factor name.
ViStatus _VI_FUNC rsemi_SelectTransducerFactor(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    transducerState,
    ViString    transducerName
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    window;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_TRAN_SEL_NAME, transducerName),
                4, "Transducer Name");                

    viCheckParm ( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                RSEMI_ATTR_TRAN_STATE, transducerState),
                3, "Transducer State");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the transducer factor. The transducer is 
/// HIFN  stored on harddisk any may be activated by function 
/// HIFN  SelectTransducerFactor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR name/This command selects the transducer factor designated by <name>. If 
/// HIPAR name/<name> does not exist yet, a new transducer factor is created.
/// HIPAR unit/This control defines the unit of the transducer factor selected.
/// HIPAR interpolation/This control defines whether the frequency scaling of the transducer 
/// HIPAR interpolation/factor is linear or logarithmic.
/// HIPAR comment/Sets the comment for the selected transducer factor. 
/// HIPAR noofTestPoints/Specifies the number of frequency,level pairs. This value corresponds 
/// HIPAR noofTestPoints/with number of arrays members (Frequency and Level controls). 
/// HIPAR frequencyData/This array defines the frequency part (in Hz) of the test points for 
/// HIPAR frequencyData/the selected transducer factor.  The frequencies must be in ascending 
/// HIPAR frequencyData/order.
/// HIPAR levelData/This array defines the level part of the test points for the selected 
/// HIPAR levelData/transducer factor.
ViStatus _VI_FUNC rsemi_ConfigureTransducerFactor(
    ViSession    instrSession,
    ViInt32    window,
    ViString    name,
    ViInt32    unit,
    ViInt32    interpolation,
    ViString    comment,
    ViInt32    noofTestPoints,
    ViReal64    frequencyData[],
    ViReal64    levelData[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      *pbuf = NULL;
    ViChar      *p2buf = NULL;
    ViInt32     i;
    window;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
   
    viCheckParm ( rsemi_SetAttributeViString (instrSession, rep_cap, 
                RSEMI_ATTR_TRAN_SEL_NAME, name),
                3, "Name");
    viCheckParm ( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                RSEMI_ATTR_TRAN_UNIT, unit),
                4, "Unit");                
    viCheckParm ( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                RSEMI_ATTR_TRAN_SCALING, interpolation),
                5, "Interpolation");    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, rep_cap, 
                RSEMI_ATTR_TRAN_COMMENT, comment),
                6, "Comment");                    
                    
    if (noofTestPoints < 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "No Of Test Points");
    
    viCheckAlloc (pbuf = (ViChar*) malloc((size_t)(23 + 2 * noofTestPoints * 15 + 300)));
    
    p2buf = pbuf;
    
    if (noofTestPoints > 0)
    {
        pbuf += sprintf (pbuf, "SENS%ld:CORR:TRAN:DATA ", window);
        for (i=0;i< noofTestPoints;i++)
        {
            pbuf += sprintf (pbuf, "%le,%lf,", frequencyData[i], levelData[i]); 
        }
        *--pbuf = '\0';    
        viCheckErr (viPrintf (instrSession, "%s\n", p2buf));
    }     
    
    checkErr( rsemi_CheckStatus (instrSession)); 

Error:
    if (p2buf) free (p2buf);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function allows automatic reference level offset adaption to 
/// HIFN  restore the original dynamic range by also shifting the reference level 
/// HIFN  by the maximum value of the transducer factor. 
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     Command SENS:CORR:TRAN:SEL has to be sent prior to this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR refLevAdj/Allows automatic reference level offset adaption to restore the 
/// HIPAR refLevAdj/original dynamic range by also shifting the reference level by the 
/// HIPAR refLevAdj/maximum value of the transducer factor.
ViStatus _VI_FUNC rsemi_ConfigureTransducerFactorRefLevAdj(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    refLevAdj
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    window;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    

    viCheckParm ( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                RSEMI_ATTR_TRAN_ADJ_STATE, refLevAdj),
                3, "Ref Lev Adj");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches on the display of the active transducer factor 
/// HIFN  or set.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     (1) Prior to this command, the command SENS:CORR:TRAN:SEL must be 
/// HIFN  sent.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control switches on or off the power-save mode of the display.
ViStatus _VI_FUNC rsemi_ConfigureDisplayTransducerState(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    window;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                RSEMI_ATTR_TRAN_DISPLAY, state), 
                3, "State");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function reads out the names of all transducer factors stored on 
/// HIFN  the harddisk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofTransducerFactors/This control returns number of names of all transducer factors.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog 
/// HIPAR bufferSize/list parameter.
/// HIPAR transducerFactorsList/This control returns the names of all transducer factors stored on the 
/// HIPAR transducerFactorsList/hard disk.
ViStatus _VI_FUNC rsemi_QueryTransducerFactorCatalog(
    ViSession    instrSession,
    ViInt32*    numberofTransducerFactors,
    ViInt32    bufferSize,
    ViChar    transducerFactorsList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      *buf=VI_NULL;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if (bufferSize < 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");         
    if ((bufferSize > 0) && (transducerFactorsList == VI_NULL))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Transducer Factor List");  
       
    viCheckErr (viPrintf (instrSession, ":CORR:TRAN:CAT?\n"));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &buf, &retCnt));
    viCheckErr (rsemi_ParseCatalog(buf, bufferSize, transducerFactorsList, numberofTransducerFactors));

    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    if (buf) free(buf);
    (void) Rs_UnlockSession(instrSession, VI_NULL);      
    return error;
}


/// HIFN  This function deletes the active transducer factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR transducerName/This command selects the transducer factor name. The driver uses this 
/// HIPAR transducerName/value to set the RSEMI_ATTR_TFAC_SEL_NAME attribute.
ViStatus _VI_FUNC rsemi_DeleteTransducerFactor(
    ViSession    instrSession,
    ViInt32    window,
    ViString    transducerName
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    window;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_TRAN_SEL_NAME, transducerName),
                3, "Transducer Name");                
    viCheckErr ( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_TRAN_DELETE, VI_NULL)); 
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the active transducer factor. Any empty string 
/// HIFN  is returned, if no transducer factor is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR activeTransducer/This control the active transducer factor. Any empty string is 
/// HIPAR activeTransducer/returned, if no transducer factor is active.
ViStatus _VI_FUNC rsemi_GetTransducerActive(
    ViSession    instrSession,
    ViChar    activeTransducer[]
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    /*viCheckParm (rsemi_GetAttributeViString (instrSession, "Win0", RSEMI_ATTR_TFAC_ACT_NAME, 512,
                 activeTransducer), 2, "Active Transducer");
    */
    viCheckParm (rsemi_GetAttributeViString (instrSession, "", RSEMI_ATTR_TRAN_ACT_NAME, 512,
                 activeTransducer), 2, "Active Transducer");
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN This function  restores the default configuration of the test
/// HIFN report regarding the information that is part of the report. It
/// HIFN also restores the default names of the header titles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_TestReportDefault (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_TEST_REPORT_DEFAULT, VI_NULL));
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function selects the information that each dataset in the
/// HIFN test report contains.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Sets the number of items in Items array.
/// HIPAR items/This control contains array of values that each dataset in the
/// HIPAR items/test report contains.
ViStatus _VI_FUNC rsemi_TestReportContentsConfiguration (ViSession instrSession,
                                                         ViInt32 arraySize,
                                                         ViInt32 items[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar        buffer[RS_MAX_MESSAGE_BUF_SIZE] = "\0";
    ViChar        *p2buffer = buffer;
    ViReal64     discrete;          
    ViChar        *p2string = VI_NULL;
    ViInt32        i=0,
                j=0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    if (arraySize<=0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 2, "Array Size");
    if (items == VI_NULL)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Items");
    
    p2buffer += sprintf(buffer,"HCOP:TREP:ITEM:SEL '");
    for (j=0;j<arraySize;j++)
    {
        i=0;
        do
        {
            discrete = rsemi_rngTestReportItem.rangeValues[items[i]].discreteOrMinValue;
            p2string = rsemi_rngTestReportItem.rangeValues[items[i]].cmdString;

            if (p2string == RS_RANGE_TABLE_END_STRING)
                viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Items");

            if (fabs(discrete - items[j]) <= 0.01) break;

            i++;

        } while (i != 0);
        p2buffer += sprintf (p2buffer, "%s,", p2string);
    }
    *p2buffer='\0';
    *--p2buffer='\'';
    viCheckErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    
   checkErr( rsemi_CheckStatus (instrSession)); 
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the selected information to be included in the test report for a specific channel type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelType/Selects the channel type.
/// HIPAR bufferSize/Sets the number of items in 'Information' array.
/// HIPAR information/Returns the selected information to be included in the test report for a specific channel type.
ViStatus _VI_FUNC rsemi_QueryTestReportInformation (ViSession instrSession,
                                                    ViInt32 channelType,
                                                    ViInt32 bufferSize,
                                                    ViChar information[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RS_MAX_MESSAGE_BUF_SIZE] = "\0";
    ViChar*  pbuf = NULL;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	viCheckParm (rsemi_invalidViInt32Range (instrSession, channelType, RSEMI_VAL_TREP_CHANNEL_TYPE_CAPD, RSEMI_VAL_TREP_CHANNEL_TYPE_SAN), 2, "Channel Type");
   
    sprintf(buffer,"HCOP:TREP:ITEM:LIST? %s",gs_TestReportChannelTypeArr[channelType]);
    viCheckErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
	viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuf, NULL));
    checkErr( rsemi_CheckStatus (instrSession)); 
	memset (information, 0, bufferSize);
	strncpy (information, pbuf, bufferSize - 1);
   
Error:
    if (pbuf) free (pbuf);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Includes or excludes the complete set of measurement information from the test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR enabled/Includes or excludes the complete set of measurement information from the test report.
ViStatus _VI_FUNC rsemi_TestReportHeaderState (ViSession instrSession,
                                               ViBoolean enabled)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
	viCheckParm(rsemi_SetAttributeViBoolean(instrSession, "",
        RSEMI_ATTR_TEST_REPORT_HEADER_STATE, enabled), 2, "Enabled");    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/// HIFN This function defines the information displayed in the page
/// HIFN header.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR line/Defines the header line.
/// HIPAR title/Defines a custom name for one of the items part of the report
/// HIPAR title/header.
/// HIPAR text/Defines a descriptive text for one of the items part of the
/// HIPAR text/report header.
/// HIPAR visibility/Selects how often the items in the report header are displayed
/// HIPAR visibility/in the document.
ViStatus _VI_FUNC rsemi_TestReportHeaderLine (ViSession instrSession,
                                              ViInt32 line, 
                                              ViString title,
                                              ViString text, 
                                              ViInt32 visibility)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
   
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_TEST_REPORT_HEADER_TITLE,
                                                 "HeaderLine",
                                                 line - 1,
                                                 10,
                                                 rep_cap), 2, "Line");

    viCheckParm (rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_TEST_REPORT_HEADER_TITLE, 
                                                title), 3, "Title");
    
    viCheckParm (rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_TEST_REPORT_HEADER_TEXT, 
                                                text), 4, "Text");
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSEMI_ATTR_TEST_REPORT_HEADER_CONTROL, 
                                                visibility), 5, "Visibility");
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;    
}


/// HIFN This function defines the page header logo.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR logo/Defines a path to graphic (for example a company logo) that is
/// HIPAR logo/shown at the top of each page in the test report.
/// HIPAR visibility/Selects how often the logo is displayed in the document.
ViStatus _VI_FUNC rsemi_TestReportHeaderLogo (ViSession instrSession,
                                              ViString logo, 
                                              ViInt32 visibility)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    
    viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_LOGO, 
                                                logo), 2, "Logo");
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_LOGO_CONTROL, 
                                                visibility), 3, "Visibility");
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/// HIFN This function defines general properties of test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR titleEnabled/Includes or excludes the title page from the test report.
/// HIPAR showPageCountEnabled/Includes or excludes page number from the test report.
/// HIPAR printerFriendlyColorsEnabled/Document parameter here.
/// HIPAR timeStampEnabled/Document parameter here.
ViStatus _VI_FUNC rsemi_ConfigureTestReportGeneralProperties (ViSession instrSession,
                                                              ViBoolean titleEnabled,
                                                              ViBoolean showPageCountEnabled,
                                                              ViBoolean printerFriendlyColorsEnabled,
                                                              ViBoolean timeStampEnabled)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm(rsemi_SetAttributeViBoolean(instrSession, "",
        RSEMI_ATTR_TEST_REPORT_TITLE_ENABLED, titleEnabled), 2, "Title Enabled");
    viCheckParm(rsemi_SetAttributeViBoolean(instrSession, "",
        RSEMI_ATTR_TEST_REPORT_SHOW_PAGE_COUNT_ENABLED, showPageCountEnabled), 3, "Show Page Count Enabled");	
    viCheckParm(rsemi_SetAttributeViBoolean(instrSession, "",
        RSEMI_ATTR_TEST_REPORT_PRINTER_FRIENDLY_COLOURS_ENABLED, printerFriendlyColorsEnabled), 4, "Printer Friendly Colours Enabled");
    viCheckParm(rsemi_SetAttributeViBoolean(instrSession, "",
        RSEMI_ATTR_TEST_REPORT_TIME_STAMP_ENABLED, timeStampEnabled), 5, "Time Stamp Enabled");    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/// HIFN This function defines the page title.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR title/Defines the title for the test report as shown on its title page.
/// HIPAR description/Defines the description of the test report as shown on its title page.
/// HIPAR pageSize/Selects the size of the test report document.
ViStatus _VI_FUNC rsemi_TestReportTitle (ViSession instrSession,
                                         ViString title, 
                                         ViString description,
                                         ViInt32 pageSize)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    
    viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_TITLE, 
                                                title), 2, "Title");
    
    viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_DESCRIPTION, 
                                                description), 3, "Description");
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_PAGE_SIZE, 
                                                pageSize), 4, "Page Size");
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}
   


/// HIFN This function allows you manage test report templates.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of operation to be performed.
/// HIPAR path/This control specifies the source of the operation to be performed.
ViStatus _VI_FUNC rsemi_TestReportTemplateOperations (ViSession instrSession,
                                                      ViInt32 operation,
                                                      ViString path)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    
    switch(operation){
        case  RSEMI_VAL_TEMPLATE_LOAD:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_TEMPLATE_LOAD, 
                                                path), 3, "Path");
        break;
        case  RSEMI_VAL_TEMPLATE_SAVE:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_TEMPLATE_SAVE, 
                                                path), 2, "Path");
        break;
        case  RSEMI_VAL_TEMPLATE_DELETE:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_TEMPLATE_DELETE, 
                                                path), 2, "Path");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation"); 
    }
            
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/// HIFN This function queries the names of all test report templates.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfTempplates/This control returns number of the names of all templates.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the
/// HIPAR bufferSize/catalog list parameter.
/// HIPAR templateList/This control the names of all conversion loss tables saved on
/// HIPAR templateList/the hard disk.
ViStatus _VI_FUNC rsemi_TestReportTemplateCatalog (ViSession instrSession,
                                                   ViInt32 *numberOfTempplates,
                                                   ViInt32 bufferSize,
                                                   ViChar templateList[])
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      *buf=VI_NULL;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if (bufferSize < 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");         
    if ((bufferSize > 0) && (templateList == VI_NULL))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Template List");  
       
    viCheckErr (viPrintf (instrSession, ":HCOP1:TREP:ITEM:TEMP:CAT?\n"));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &buf, &retCnt));
    viCheckErr (rsemi_ParseCatalog(buf, bufferSize, templateList, numberOfTempplates));

    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);      
    return error;
}

/// HIFN This function adds/removes datasets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR selector/This control selects the items to save or recall.
/// HIPAR dataset/Sets the dataset number.
ViStatus _VI_FUNC rsemi_TestReportDataSetsConfiguration (ViSession instrSession,
                                                      ViInt32 selector,
                                                      ViInt32 dataset)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    
    switch(selector){
        case  RSEMI_VAL_TREP_NEW:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_NEW, VI_NULL));
        break;
        case  RSEMI_VAL_TREP_APPEND:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_DATASET_ADD, VI_NULL));
        break;
        case  RSEMI_VAL_TREP_REMOVE:
            viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_DATASET_REMOVE, 
                                                dataset), 3, "Dataset");
        break;
        case  RSEMI_VAL_TREP_REMOVE_ALL:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_TEST_REPORT_DATASET_REMOVE_ALL, VI_NULL));
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation"); 
    }
            
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/// HIFN This function includes or excludes datasets from the test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dataset/Sets the dataset number.
/// HIPAR state/Sets the dataset state in report.
ViStatus _VI_FUNC rsemi_TestReportDataSetSelect (ViSession instrSession,
                                                 ViInt32 dataset,
                                                 ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (dataset<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Dataset");
    
    checkErr (viPrintf (instrSession, "HCOP:TREP:TEST:SEL %ld,%s\n", dataset, switchArr[state]));
    
    checkErr( rsemi_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;     
}

/// HIFN  This function selects the printer output medium and the data format of 
/// HIFN  the printout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR device/Sets the device number.
/// HIPAR destination/Defines the output associated with device configuration.
/// HIPAR pageOrientation/The command selects the format of the output (portrait and landscape) 
/// HIPAR pageOrientation/(hardcopy unit 1 or 2).
ViStatus _VI_FUNC rsemi_ConfigureHardcopyDevice(
    ViSession    instrSession,
    ViInt32    device,
    ViInt32    destination,
    ViInt32    pageOrientation
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, device, 1, 2), 2, "Device");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, destination, 0, 8), 3, "Destination");
        
    sprintf (buffer, "DE%ld", device);
    switch(destination){
        case  RSEMI_VAL_HCOPY_DEST_BMP:
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                                  RSEMI_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, 
                                                    RSEMI_VAL_HCOPY_DEVICE_LANG_BMP));                
        break;
        case  RSEMI_VAL_HCOPY_DEST_WMF:
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                                  RSEMI_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                      RSEMI_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSEMI_VAL_HCOPY_DEVICE_LANG_WMF));
        break;
        case  RSEMI_VAL_HCOPY_DEST_EMF:
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                              RSEMI_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                      RSEMI_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSEMI_VAL_HCOPY_DEVICE_LANG_EWMF));
        break;
        case  RSEMI_VAL_HCOPY_DEST_CLP:
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                                 RSEMI_VAL_HCOPY_DEVICE_CLP));
        break;
        case  RSEMI_VAL_HCOPY_DEST_PRN:
            
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                      RSEMI_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSEMI_VAL_HCOPY_DEVICE_LANG_GDI));
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                                  RSEMI_VAL_HCOPY_DEVICE_PRN));
            viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                  RSEMI_ATTR_HCOPY_DEVICE_ORIENTATION,
                                                  pageOrientation), 4, "Page Orientation");
        break;
        case  RSEMI_VAL_HCOPY_DEST_JPG:
                 viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSEMI_VAL_HCOPY_DEVICE_MEM));
                 viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                      RSEMI_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSEMI_VAL_HCOPY_DEVICE_LANG_JPG));
        break;
        case  RSEMI_VAL_HCOPY_DEST_PNG:
                viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSEMI_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                      RSEMI_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSEMI_VAL_HCOPY_DEVICE_LANG_PNG));
        break;
        case  RSEMI_VAL_HCOPY_DEST_PDF:
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSEMI_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                      RSEMI_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSEMI_VAL_HCOPY_DEVICE_LANG_PDF));
        break;
        case  RSEMI_VAL_HCOPY_DEST_RTF:
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSEMI_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                      RSEMI_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSEMI_VAL_HCOPY_DEVICE_LANG_RTF));
        break;
        case  RSEMI_VAL_HCOPY_DEST_DOC:
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,  RSEMI_ATTR_HCOPY_DEVICE_DESTINATION,
                                                      RSEMI_VAL_HCOPY_DEVICE_MEM));
            viCheckErr (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                      RSEMI_ATTR_HCOPY_DEVICE_LANG_OUT_FORM,
                                                      RSEMI_VAL_HCOPY_DEVICE_LANG_DOC));
        break;
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function aborts a running hardcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_HardcopyAbort(
    ViSession    instrSession
)
{
    
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_HCOPY_ABORT, VI_NULL));                
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN This function selects the output for hardcopy print.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session./// HIPAR output/Selects the output for hardcopy print.
ViStatus _VI_FUNC rsemi_HardcopyOutput (ViSession instrSession,
                                        ViInt32 output)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_HCOPY_OUTPUT, output), 2, "Output");                
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function starts a hardcopy output of the selected items.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR device/Sets the device number.
/// HIPAR items/Selects the items to be output.
ViStatus _VI_FUNC rsemi_HardcopyPrint(
    ViSession    instrSession,
    ViInt32    device,
    ViInt32    items
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, device, 1, 2), 2, "Device");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, items, 0, 4), 3, "Items");
        
    sprintf (buffer, "DE%ld", device);
    switch (items){
        case RSEMI_VAL_HCOP_ALL:
            viCheckErr ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_HCOPY_PRINT_SCREEN, VI_NULL));    
        break;
        case RSEMI_VAL_HCOP_TABLE:
            viCheckErr ( rsemi_SetAttributeViBoolean (instrSession, "", 
                RSEMI_ATTR_HCOPY_PRINT_TABLE, VI_TRUE));
        break;
        case RSEMI_VAL_HCOP_TRACE:
            viCheckErr ( rsemi_SetAttributeViBoolean (instrSession, "", 
                RSEMI_ATTR_HCOPY_PRINT_TRACE_STATE, VI_TRUE));
        break;
    }
    viCheckErr ( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_HCOPY_PRINT, VI_NULL));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function starts a hardcopy output. The numeric suffix selects 
/// HIFN  which printer configuration (device 1 or 2) is to be used for the 
/// HIFN  hardcopy output. If there is no suffix, configuration 1 is 
/// HIFN  automatically selected. If the output is printed to a file (see 
/// HIFN  HCOPy:DESTination<1|2> command), the file name used in the last saving 
/// HIFN  process is automatically counted up to the next unused name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR device/Sets the device number.
ViStatus _VI_FUNC rsemi_HardcopyPrintNext(
    ViSession    instrSession,
    ViInt32    device
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, device, 1, 2), 2, "Device");
        
    sprintf (buffer, "DE%ld", device);
    
    viCheckErr ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_HCOPY_PRINT_NEXT, NULL));                
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines a destination file for the hadcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR name/Sets the hardcopy file name.
ViStatus _VI_FUNC rsemi_HardcopySetFileName(
    ViSession    instrSession,
    ViString    name
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_HCOPY_FILE_NAME, name));                
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the list of printers under Windows NT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/This control indicates the allocated memory in Printer List.
/// HIPAR printerList/This control returns list of printers available under  Windows NT 
/// HIPAR printerList/system. Names are separated using commas (,).
ViStatus _VI_FUNC rsemi_HardcopyGetPrinterList(
    ViSession    instrSession,
    ViInt32    bufferSize,
    ViChar    printerList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViInt32     count;
    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (bufferSize<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Buffer Size"); 
    if (printerList == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Printer List");
    
    p2buf=printerList;
    count=0;
    viCheckErr (rsemi_GetAttributeViString (instrSession, "", RSEMI_ATTR_HCOPY_PRINTER_FIRST,
                RSEMI_IO_BUFFER_SIZE, buffer));
    
    while (strlen(buffer) > 1)
    {
        count+=(ViInt32)strlen(buffer);
        count++; //add one position for comma
        if (count<=bufferSize)
            p2buf += sprintf (p2buf, "%s,", buffer);
        viCheckErr (rsemi_GetAttributeViString (instrSession, "", RSEMI_ATTR_HCOPY_PRINTER_NEXT,
                RSEMI_IO_BUFFER_SIZE, buffer));      
    }
    *--p2buf='\0';
    
    if (count>bufferSize)
        error = count;      
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects one of the printers installed under Windows NT. 
/// HIFN  Prior to use this function get list of available printers using 
/// HIFN  function Hardcopy Get Printer List.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR device/Sets the device number.
/// HIPAR printerName/This control sets printer name.
ViStatus _VI_FUNC rsemi_HardcopySetPrinter(
    ViSession    instrSession,
    ViInt32    device,
    ViString    printerName
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, device, 1, 2), 2, "Device");

    sprintf (buffer, "DE%ld", device);
    viCheckParm (rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_HCOPY_PRINTER,
                                                printerName), 3, "Printer Name");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the comment text for measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR comment/Sets the hardcopy comment.
ViStatus _VI_FUNC rsemi_HardcopyComment(
    ViSession    instrSession,
    ViString    comment
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckErr ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_HCOPY_COMM_SCREEN, comment));    
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function includes or removes the time and date in the printout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR device/Sets the device number.
/// HIPAR timeandDateStamp/Includes or removes the time and date in the printout.
ViStatus _VI_FUNC rsemi_HardcopyTimeDate(
    ViSession    instrSession,
    ViInt32    device,
    ViBoolean    timeandDateStamp
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "DE%ld", device);
    viCheckErr ( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                RSEMI_ATTR_HCOPY_TIME_DATE, timeandDateStamp));    
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function resets the colors for a hardcopy to the selected default 
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR color/This control sets color or monochrome hardcopy of the screen.
/// HIPAR defaultSetting/This control selects the predefined setting to be applied.
ViStatus _VI_FUNC rsemi_HardcopyColor(
    ViSession    instrSession,
    ViBoolean    color,
    ViInt32    defaultSetting
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_HCOPY_COLOR, color), 
                2, "Color");
    
    if (color == VI_TRUE)
    {
        viCheckParm ( rsemi_SetAttributeViInt32 (instrSession, "", 
                RSEMI_ATTR_HCOPY_COLOR_DEF, defaultSetting), 3, "Default Setting");                
    }             
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the color table in USER DEFINED COLORS mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR colorMap/This control defines the color table of the instrument.
/// HIPAR tint/This control sets the hue = TINT.
/// HIPAR saturation/This control sets the sat = SATURATION.
/// HIPAR brightness/This control sets the lum = BRIGHTNESS.
ViStatus _VI_FUNC rsemi_ConfigureHardcopyColor(
    ViSession    instrSession,
    ViInt32    colorMap,
    ViReal64    tint,
    ViReal64    saturation,
    ViReal64    brightness
)
{
   ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, colorMap, 1, 49), 2, "Color Map");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, tint, 0.0, 1.0), 3, "TINT");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, saturation, 0.0, 1.0), 4, "SATURATION"); 
    viCheckParm (rsemi_invalidViReal64Range (instrSession, brightness, 0.0, 1.0), 5, "BRIGHTNESS");        
    
    viCheckErr( viPrintf (instrSession, "HCOP:CMAP%ld:HSL %.12f,%.12f,%.12f\n",
                colorMap, tint, saturation, brightness));

    checkErr( rsemi_CheckStatus (instrSession)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the color table in USER DEFINED COLORS using 
/// HIFN  predefined color values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR colorMap/This control defines the color table of the instrument.
/// HIPAR predefinedColors/This control defines the color table of the instrument using 
/// HIPAR predefinedColors/predefined color values.
ViStatus _VI_FUNC rsemi_ConfigureHardcopyColorByPredefined(
    ViSession    instrSession,
    ViInt32    colorMap,
    ViInt32    predefinedColors
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, colorMap, 1, 49), 2, "Color Map");
    
    sprintf (buffer, "CM%ld", colorMap);  
    
    viCheckParm ( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                RSEMI_ATTR_HCOPY_COLOR_PREDEFINED, predefinedColors),
                3, "Predefined Colors");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function initiates the acquisition of system error correction 
/// HIFN  data. A "0" is returned if the acquisition was successful.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     
/// HIFN     (1)During the acquisition of correction data the instrument does not 
/// HIFN  accept any remote control commands, except
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR sync/Unused
/// HIPAR timeout_ms/Sets the timeout for the calibration routine to be finished and have 
/// HIPAR timeout_ms/obtained calibration result. If the length of time required for 
/// HIPAR timeout_ms/calibration exceeds the timeout value, then the function will return 
/// HIPAR timeout_ms/with a timeout error and the instrument will continue with calibration.
/// HIPAR result/This control returns the result of acquisition of system error 
/// HIPAR result/correction data. A "0" is returned if the acquisition was successful.
ViStatus _VI_FUNC rsemi_Calibration(
    ViSession    instrSession,
    ViBoolean    sync,
    ViUInt32    timeout,
    ViInt32*    result
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     old_timeout=-1;
    sync;
    
    checkErr( Rs_LockSession (instrSession, NULL));
   
    viCheckParm (rsemi_invalidViInt32Range (instrSession, timeout, 0, 600000), 3, "Timeout [ms]");     
    
        viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));   
        viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout));  
        viCheckParm (rsemi_GetAttributeViInt32 (instrSession, "", 
                RSEMI_ATTR_CALIBRATION, result), 4, "Result");
   
Error:
    if (old_timeout >= 0)    
        rsemi_SetOPCTimeout (instrSession, old_timeout);  
    
    (void) Rs_UnlockSession(instrSession, NULL);
    
    return error;    
}


/// HIFN  This function aborts the acquisition of correction data and restores 
/// HIFN  the last complete correction data set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_CalibrationAbort(
    ViSession    instrSession
)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
            RSEMI_ATTR_CALIBRATION_ABORT, VI_NULL));
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function reads the results of the correction data acquisition. 
/// HIFN  The lines of the result table (see manual section "Recording the 
/// HIFN  correction data of FSQ - CAL key") are output as string data separated 
/// HIFN  by commas:
/// HIFN     "Total Calibration Status: PASSED","Date (dd/mm/yyyy): 12/07/1999", 
/// HIFN  "Time: 16:24:54","Runtime:00.06"
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of array passed to Result argument.
/// HIPAR result/This control returns the results of the correction data acquisition. 
/// HIPAR result/The lines of the result table (see manual section "Recording the 
/// HIPAR result/correction data of FSQ - CAL key") are output as string data separated 
/// HIPAR result/by
ViStatus _VI_FUNC rsemi_CalibrationResult(
    ViSession    instrSession,
    ViInt32    arraySize,
    ViChar    result[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *pbuffer=VI_NULL;
    ViUInt32    ret_val;
    ViUInt32    length;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    //viCheckParm (rsemi_GetAttributeViString (instrSession, "", 
    //        RSEMI_ATTR_CALIBRATION_RESULT_QUERY, arraySize, result), 2, "Result");
    viCheckErr (viPrintf (instrSession, "CAL:RES?\n"));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, &ret_val));
    length=((ViUInt32)arraySize<=ret_val)?(ViUInt32)arraySize-1:ret_val;
    memcpy (result,pbuffer,length);
    result[length]='\0';
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function determines whether the current calibration data are 
/// HIFN  taken into account by the instrument (ON) or not (OFF).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Determines whether the current calibration data are taken into account 
/// HIPAR state/by the instrument (ON) or not (OFF).
ViStatus _VI_FUNC rsemi_CalibrationState(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_CALIBRATION_STATE, state), 2, "State");
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function toggles the control lines of the user ports between 
/// HIFN  INPut and OUTPut. With ON, the user port is switched to INPut, with OFF 
/// HIFN  to OUTPut.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Toggles the control lines of the user ports between INPut and OUTPut.
ViStatus _VI_FUNC rsemi_ConfigureControlLinesState(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    window;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, 
            RSEMI_ATTR_INPUT_UPORT_STATE, state), 3, "State");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the control lines of the user ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR userPorts/This control sets the control lines of the user ports.
ViStatus _VI_FUNC rsemi_ConfigureOutputControlLines(
    ViSession    instrSession,
    ViInt32    window,
    ViString    userPorts
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViString (instrSession, buffer, 
            RSEMI_ATTR_OUT_UPORT_VALUE, userPorts), 3, "User Ports");
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function toggles the control lines of the user ports between 
/// HIFN  INPut and OUTPut. With ON, the user port is switched to INPut, with OFF 
/// HIFN  to OUTPut.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR lines/This control selects the control lines to be returned.
/// HIPAR userPorts/Returns the control lines of the user ports. 
ViStatus _VI_FUNC rsemi_GetControlLines(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    lines,
    ViChar    userPorts[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    window;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, lines, 0, 1), 3, "Lines");
    
    switch (lines){
        
        case RSEMI_VAL_USER_PORT_INP:
            viCheckParm (rsemi_GetAttributeViString (instrSession, buffer, 
                    RSEMI_ATTR_INPUT_UPORT_VALUE, 8, userPorts), 3, "User Ports");
        break;
        case RSEMI_VAL_USER_PORT_OUT:
            viCheckParm (rsemi_GetAttributeViString (instrSession, buffer, 
                    RSEMI_ATTR_OUT_UPORT_VALUE, 8, userPorts), 3, "User Ports");
        break;
    }       

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the auto adjust settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR adjustSettings/This control selects the auto adjust setting.
/// HIPAR measurementTimeMode/This control defines whether the duration of the level measurement 
/// HIPAR measurementTimeMode/used to determine the optimal reference level is determined 
/// HIPAR measurementTimeMode/automatically or manually.
/// HIPAR measurementTime/This control defines the duration of the level measurement used to 
/// HIPAR measurementTime/determine the optimal reference level automatically.
ViStatus _VI_FUNC rsemi_ConfigureAutoAdjust(
    ViSession    instrSession,
    ViInt32    adjustSettings,
    ViInt32    measurementTimeMode,
    ViReal64    measurementTime
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSEMI_IO_BUFFER_SIZE] = "ESR";

    rsemi_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSEMI_IO_BUFFER_SIZE, model);
    ;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (adjustSettings)
    {
        case RSEMI_VAL_ADJ_ALL:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
                    RSEMI_ATTR_AUTO_ADJUST_ALL, NULL));
        break;
        case RSEMI_VAL_ADJ_FREQ:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
                    RSEMI_ATTR_AUTO_ADJUST_FREQ, NULL));
        break;
        case RSEMI_VAL_ADJ_LEV:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
                    RSEMI_ATTR_AUTO_ADJUST_LEVEL, NULL));
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Adjust Settings"); 
    }
    if (strstr (model, "FSWT") != NULL)
    {
        viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                RSEMI_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_MODE_FSWT, measurementTimeMode), 3, "Measurement Time Mode");
    
        if (measurementTimeMode == RSEMI_VAL_MAN)
        {   
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                    RSEMI_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_FSWT, measurementTime), 4, "Measurement Time");
        }
    }
    else
    {
        viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                RSEMI_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_MODE, measurementTimeMode), 3, "Measurement Time Mode");
    
        if (measurementTimeMode == RSEMI_VAL_MAN)
        {   
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                    RSEMI_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME, measurementTime), 4, "Measurement Time");
        }
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the auto adjust hystersis settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR lowerHystersis/This control defines a lower threshold the signal must drop below 
/// HIPAR lowerHystersis/before the reference level is automatically adjusted when the "Auto 
/// HIPAR lowerHystersis/Level" function is performed.
/// HIPAR upperHystersis/This control defines an upper threshold the signal must exceed before 
/// HIPAR upperHystersis/the reference level is automatically adjusted when the "Auto Level" 
/// HIPAR upperHystersis/function is performed.
ViStatus _VI_FUNC rsemi_ConfigureAutoAdjustHystersis(
    ViSession    instrSession,
    ViReal64    lowerHystersis,
    ViReal64    upperHystersis
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                RSEMI_ATTR_AUTO_ADJUST_HYSTERSIS_LOWER, lowerHystersis), 2, "Lower Hystersis");
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", 
                RSEMI_ATTR_AUTO_ADJUST_HYSTERSIS_UPPER, upperHystersis), 3, "Upper Hystersis");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the behaviour of the measurement when adjusting 
/// HIFN  a setting automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Defines the behaviour of the measurement when adjusting a setting 
/// HIPAR state/automatically.
ViStatus _VI_FUNC rsemi_ConfigureAutoAdjustTriggerState(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
                        RSEMI_ATTR_AUTO_ADJUST_TRIGGER_STATE, state), 2, "state");  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function initiates a signal acquisition based on the present 
/// HIFN  instrument configuration.  It then waits for the acquisition to 
/// HIFN  complete, and returns the trace as an array of amplitude values. The 
/// HIFN  amplitude array returns data that represent the amplitude of the 
/// HIFN  signals of the sweep from the start frequency to the stop frequency (in 
/// HIFN  frequency domain, in time domain the amplitude array is ordered from 
/// HIFN  beginning of sweep to end).  The Amplitude Units attribute determines 
/// HIFN  the units of the points in the amplitude array.  This function resets 
/// HIFN  the sweep count.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) If the spectrum analyzer did not complete the acquisition within 
/// HIFN  the time period the user specified with the maxTime parameter, the 
/// HIFN  function returns Max Time Exceeded error.
/// HIFN     
/// HIFN     (2) If the data was captured in an uncalibrated spectrum analyzer 
/// HIFN  mode, the function will generate a Measure Uncalibrated warning flag. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform 
/// HIPAR maximumTime/operation to complete.    
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the Amplitude array and X Array 
/// HIPAR actualPoints/array.
/// HIPAR amplitude/A user allocated buffer into which the trace amplitudes is stored.
ViStatus _VI_FUNC rsemi_ReadYTrace(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViUInt32    maximumTime,
    ViInt32    arrayLength,
    ViInt32*    actualPoints,
    ViReal64    amplitude[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");
       
    checkErr( rsemi_Initiate (instrSession, window, maximumTime));
   
    sprintf (buffer, "TRACE%ld", trace);
    viCheckErr (rsemi_dataReadTrace (instrSession, window, buffer, arrayLength, 
                    amplitude, actualPoints));

    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the trace the spectrum analyzer acquires. The 
/// HIFN  trace is from a previously initiated acquisition.  You use the 
/// HIFN  rsemi_Initiate function to start an acquisition.  
/// HIFN     
/// HIFN     You can call the Read Y Trace function instead of the Initiate 
/// HIFN  function. This function starts an acquisition, waits for the 
/// HIFN  acquisition to complete, and returns the trace in one function call.
/// HIFN     
/// HIFN     The array returns data that represents the amplitude of the signals of 
/// HIFN  the sweep from the start frequency to the stop frequency (in frequency 
/// HIFN  domain, in time domain the amplitude array is ordered from beginning of 
/// HIFN  sweep to end).  The Amplitude Units attribute determines the units of 
/// HIFN  the points in the amplitude array.
/// HIFN     
/// HIFN     This function does not check the instrument status. Typically, the 
/// HIFN  end-user calls this function only in a sequence of calls to other 
/// HIFN  low-level driver functions. The sequence performs one operation. The 
/// HIFN  end-user uses the low-level functions to optimize one or more aspects 
/// HIFN  of interaction with the instrument. Call the rsemi_error_query function 
/// HIFN  at the conclusion of the sequence to check the instrument status. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR amplitude/A user allocated buffer into which the trace amplitudes is stored.
ViStatus _VI_FUNC rsemi_FetchYTrace(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViInt32    arrayLength,
    ViInt32*    actualPoints,
    ViReal64    amplitude[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");

    sprintf (buffer, "TRACE%ld", trace);
    
    viCheckErr (rsemi_dataReadTrace (instrSession,window, buffer, arrayLength, 
                    amplitude, actualPoints));

    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the previously acquired trace data for the 
/// HIFN  specified trace from the memory. As an offset and number of sweep 
/// HIFN  points to be retrieved can be specified, the trace data can be 
/// HIFN  retrieved in smaller portions, making the function faster than the 
/// HIFN  rsemi_ReadYTrace function. This is useful if only specific parts of the 
/// HIFN  trace data are of interest.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR offsetSweepPoint/The offset in sweep points related to the start of the measurement at 
/// HIPAR offsetSweepPoint/which data retrieval is to start.
/// HIPAR numberofSweeps/Number of sweep points to be retrieved from the trace.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR amplitude/A user allocated buffer into which the trace amplitudes is stored.
ViStatus _VI_FUNC rsemi_ReadYTracePrevious(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViInt32    offsetSweepPoint,
    ViInt32    numberofSweeps,
    ViInt32    arrayLength,
    ViInt32*    actualPoints,
    ViReal64    amplitude[]
)
{
    ViChar      *extBuf=VI_NULL;
    ViReal32    *temp;
    ViUInt32    i = 0;
    ViStatus    error = VI_SUCCESS;
    ViUInt32    count=0;
    ViChar      *p2buf=VI_NULL;
    ViUInt32    ret_cnt;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");

    
    sprintf (buffer, ":FORM REAL,32;:TRAC%ld:MEM? TRACE%ld,%ld,%ld\n", window, trace, offsetSweepPoint, numberofSweeps);
    viCheckErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    viCheckErr (rsemi_readDataBlock(instrSession, &extBuf, &ret_cnt));
    
    if (error == VI_SUCCESS)
    {     /* binary data, apply conversion */
        /* Convert binary data into array of ViReal64 types. */
        count = (ViInt32)(ret_cnt/4);
        /* Binary data byte order (MOTOROLA or INTEL). */
        rsemi_SWAP_32((ViPBuf)extBuf, count);
        /* Pointer to data */
        temp = (ViReal32 *)extBuf;
        if (actualPoints) *actualPoints=count;
    
        count=(count<(ViUInt32)arrayLength)?count:(ViUInt32)arrayLength;
    
        for (i = 0; i < count; i++)
            if (i<(ViUInt32)arrayLength)
            *amplitude++ = (ViReal64)*temp++;
    }
    else
        
    {   /* ASCII data */
        //read rest of first value

        p2buf = strtok(extBuf, ",");
        i = 0;
        while (p2buf)
        {
            if (i < (ViUInt32)arrayLength)
            {
                amplitude[i] = atof (p2buf);
            }   
            p2buf = strtok(NULL, ",");
            i++;
        }
    
        if (actualPoints) *actualPoints=i; 
    
    }

Error:
    if (extBuf) free(extBuf);
    return error;
}


/// HIFN  Reads the x values of selected measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR x/A user allocated buffer into which the trace X values is stored.
ViStatus _VI_FUNC rsemi_FetchXTrace(
    ViSession    instrSession,
    ViInt32    trace,
    ViInt32    arrayLength,
    ViInt32*    actualPoints,
    ViReal64    x[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 2, "Timeout");
    
    sprintf (buffer, "FORM REAL,32;TRAC:DATA:X? TRACe%ld\n", trace);
    
    viCheckErr (viPrintf (instrSession, buffer));
    viCheckErr (rsemi_dataReadTraceOnly (instrSession, arrayLength, x, actualPoints));
    
    checkErr( rsemi_CheckStatus (instrSession));     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function will start a measurement and read I/Q trace data 
/// HIFN  results. The result values are scaled linear in unit Volt and 
/// HIFN  correspond to the voltage at the RF input of the instrument.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) For large I/Q trace data is recommended to increase default 
/// HIFN  timeout value to the appropriate value which allows instrument handle 
/// HIFN  all pending tasks. To change default timeout value call following VISA 
/// HIFN  function:
/// HIFN     
/// HIFN         viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 60000);
/// HIFN     
/// HIFN     This attribute specifies the minimum timeout value to use (in 
/// HIFN  milliseconds) when accessing the device associated with the given 
/// HIFN  session.
/// HIFN     
/// HIFN     (2) If there are not I/Q data available in memory because the 
/// HIFN  corresponding measurement had not been started, the function will cause 
/// HIFN  a Query Error.
/// HIFN     
/// HIFN     (3) For FFT Analysis Mode data are scaled linearly with Volt as the 
/// HIFN  unit and corresponds to the voltage at the RF input of the instrument, 
/// HIFN  irrespective of the output format selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR bufferSize/Pass the number of doubles in the array you specify for the real parts 
/// HIPAR bufferSize/and imaginary parameters.
/// HIPAR noofPoints/Returns the number of I/Q trace data points.
/// HIPAR realParts_I/Returns an array of trace data (I-value x, real part) point. The 
/// HIPAR realParts_I/buffer must contain at least as many elements as the value you specify 
/// HIPAR realParts_I/with the Buffer Size parameter.
/// HIPAR imaginaryParts_Q/Returns an array of trace data (Q-value y, imaginary part) point. The 
/// HIPAR imaginaryParts_Q/buffer must contain at least as many elements as the value you specify 
/// HIPAR imaginaryParts_Q/with the Buffer Size parameter.
ViStatus _VI_FUNC rsemi_ReadTraceIQData(
    ViSession    instrSession,
    ViInt32    window,
    ViUInt32    timeout,
    ViInt32    bufferSize,
    ViInt32*    noofPoints,
    ViReal64    realParts_I[],
    ViReal64    imaginaryParts_Q[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      cmd [100] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (cmd, ":FORM REAL,32;:TRAC:IQ:DATA:FORM COMP;:TRAC%ld:IQ:DATA?\n", window);
    
    viCheckErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));

    viCheckErr (rsemi_CheckBeforeRead(instrSession, timeout));

    viCheckErr (rsemi_dataReadTraceIQ (instrSession, bufferSize, noofPoints, 
                    realParts_I, imaginaryParts_Q));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function permits the readout of previously acquired (and 
/// HIFN  frequency response corrected) I/Q data from the memory, with indication 
/// HIFN  of the offset related to the start of measurement and with indication 
/// HIFN  of the number of measurement values. Therefore a previously acquired 
/// HIFN  data set can be read out in smaller portions. The result values are 
/// HIFN  scaled linear in unit Volt and correspond to the voltage at the RF 
/// HIFN  input of the instrument.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) For large I/Q trace data is recommended to increase default 
/// HIFN  timeout value to the appropriate value which allows instrument handle 
/// HIFN  all pending tasks. To change default timeout value call following VISA 
/// HIFN  function:
/// HIFN     
/// HIFN         viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 60000);
/// HIFN     
/// HIFN     This attribute specifies the minimum timeout value to use (in 
/// HIFN  milliseconds) when accessing the device associated with the given 
/// HIFN  session.
/// HIFN     
/// HIFN     (2) If there are not I/Q data available in memory because the 
/// HIFN  corresponding measurement had not been started, the function will cause 
/// HIFN  a Query Error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR offsetSamples/This control sets offset of the values to be read related to the start 
/// HIPAR offsetSamples/of the acquired data.
/// HIPAR noofSamples/This control sets the number of measurement values to be read.
/// HIPAR bufferSize/Pass the number of doubles in the array you specify for the real parts 
/// HIPAR bufferSize/and imaginary parameters.
/// HIPAR noofPoints/Returns the number of I/Q trace data points.
/// HIPAR realParts_I/Returns an array of trace data (I-value x, real part) point. The 
/// HIPAR realParts_I/buffer must contain at least as many elements as the value you specify 
/// HIPAR realParts_I/with the Buffer Size parameter.
/// HIPAR imaginaryParts_Q/Returns an array of trace data (Q-value y, imaginary part) point. The 
/// HIPAR imaginaryParts_Q/buffer must contain at least as many elements as the value you specify 
/// HIPAR imaginaryParts_Q/with the Buffer Size parameter.
ViStatus _VI_FUNC rsemi_FetchTraceIQData(
    ViSession    instrSession,
    ViInt32    offsetSamples,
    ViInt32    noofSamples,
    ViInt32    bufferSize,
    ViInt32*    noofPoints,
    ViReal64    realParts_I[],
    ViReal64    imaginaryParts_Q[]
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckErr (viPrintf (instrSession, ":FORM REAL,32;:TRAC:IQ:DATA:MEM? %ld,%ld\n",
                offsetSamples, noofSamples));
                
    viCheckErr (rsemi_dataReadTraceIQ (instrSession, bufferSize, noofPoints, 
                    realParts_I, imaginaryParts_Q));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the transfer parameters and reads the data header.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) No error checking is done in this function. It is recommended to 
/// HIFN  perform error checking after all data are read (with funciton 
/// HIFN  rsemi_ReadMemoryIQLargeDataBlock).
/// HIFN     
/// HIFN     (2) This function starts measurement and for REAL,32 data format reads 
/// HIFN  the data header. Data has to be read with funciton 
/// HIFN  rsemi_ReadMemoryIQLargeDataBlock. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR format/Sets the format of transferred data.
/// HIPAR dataOrder/Sets the transferred data order.
/// HIPAR noofPoints/Returns the number of I/Q trace data bytes.
ViStatus _VI_FUNC rsemi_ReadMemoryIQLargeData(
    ViSession    instrSession,
    ViInt32    window,
    ViUInt32    timeout,
    ViInt32    format,
    ViInt32    dataOrder,
    ViInt32*    noofPoints
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";  
    ViInt32     count = 0;
    ViInt32     lenLength;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, format, 0, 1), 4, "Format");        
    viCheckParm (rsemi_invalidViInt32Range (instrSession, dataOrder, 0, 2), 5, "Data Order");         
        
    switch (format){
        case RSEMI_VAL_FORMAT_ASC:
            viCheckErr (viPrintf (instrSession, ":FORM ASC\n"));
        break;  
        case RSEMI_VAL_FORMAT_REAL:
            viCheckErr (viPrintf (instrSession, ":FORM REAL,32\n"));
        break;
    }
    
    sprintf (buffer, "Win%ld", window); 
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_IQ_DATA_FORMAT, dataOrder),
                4, "Data Order");
       
    viCheckErr (viPrintf (instrSession, ":TRAC%ld:IQ:DATA?\n", window));
    viCheckErr (rsemi_CheckBeforeRead(instrSession, timeout));   
    switch (format){
        case RSEMI_VAL_FORMAT_ASC:
            count=-1;   
        break;  
        case RSEMI_VAL_FORMAT_REAL:
            checkErr (viRead (instrSession, (ViPBuf)buffer, 2, (ViPUInt32) &count));                
            if (sscanf (buffer, "#%1ld", &lenLength) != 1) 
                viCheckErr (VI_ERROR_INV_RESPONSE);
            checkErr (viRead (instrSession, (ViPBuf)buffer, lenLength, (ViPUInt32) &count));
            buffer[lenLength]='\0';
            if (sscanf (buffer, "%lu", &count) != 1) 
                viCheckErr (VI_ERROR_INV_RESPONSE);
        break;
    }
    if (noofPoints) *noofPoints = count;
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function will read I/Q trace data according format defined with 
/// HIFN  function rsemi_ReadMemoryIQLargeData.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) Funcion rsemi_ReadMemoryIQLargeData has to be called before this 
/// HIFN  function.
/// HIFN     
/// HIFN     (2) If all data are read, function returns VI_SUCCESS, else it returns 
/// HIFN  warning 3FFF0006  The specified number of bytes was read. 
/// HIFN  (VI_SUCCESS_MAX_CNT)
/// HIFN     
/// HIFN     (3) No error checking is done. It is recommended to check error after 
/// HIFN  all data are read out.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/Pass the array size you specify for the returned data parameter.
/// HIPAR returnedData/Returns an byte array of trace data. The buffer must contain at least 
/// HIPAR returnedData/as many elements as the value you specify with the Buffer Size 
/// HIPAR returnedData/parameter.
/// HIPAR returnedCounts/Number of bytes actually transferred.
ViStatus _VI_FUNC rsemi_ReadMemoryIQLargeDataBlock(
    ViSession    instrSession,
    ViInt32    bufferSize,
    ViChar    returnedData[],
    ViInt32*    returnedCounts
)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    count;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    error = viRead (instrSession, (ViPBuf)returnedData, bufferSize, &count);
    
    if (returnedCounts)
        *returnedCounts=count;

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the result of the limit check of the limit line 
/// HIFN  indicated in the selected measurement window. It should be noted that a 
/// HIFN  complete sweep must have been performed for obtaining a valid result. A 
/// HIFN  synchronization with  *OPC, *OPC? or *WAI should therefore be provided.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects limit.
/// HIPAR state/The result of the limit check responds with 0 for PASS, 1 for FAIL, 
/// HIPAR state/and 2 for MARGIN.
ViStatus _VI_FUNC rsemi_GetLimitCheckResult(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    limit,
    ViInt32*    state
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 3, "Limit");

    sprintf (buffer, "Win%ld,L%ld", window, limit);

    viCheckParm (rsemi_GetAttributeViInt32 (instrSession, buffer, RSEMI_LIMIT_CHECK_RESULT, state), 4, "State");
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function initiates an acquisition. After calling this function, 
/// HIFN  the spectrum analyzer leaves the idle state and waits for a trigger.
/// HIFN     
/// HIFN     This function does not check the instrument status. Typically, the 
/// HIFN  end-user calls this function only in a sequence of calls to other 
/// HIFN  low-level driver functions. The sequence performs one operation. The 
/// HIFN  end-user uses the low-level functions to optimize one or more aspects 
/// HIFN  of interaction with the instrument.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) Call Acquisition Status to determine when the acquisition is 
/// HIFN  complete.
/// HIFN     
/// HIFN     (2) Call the Error Query function at the conclusion of the sequence to 
/// HIFN  check the instrument status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_MeasurementInitiate(
    ViSession    instrSession,
    ViInt32    window,
    ViUInt32    timeout
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";     
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout));    
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr (rsemi_ClearBeforeRead (instrSession));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_INIT, 
                VI_NULL));


Error:
    if (old_timeout > 0) 
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function starts a new spurious measurement and waits till 
/// HIFN  sweeping is finished. 
/// HIFN     
/// HIFN     Note(s): 
/// HIFN     
/// HIFN     (1) Set sweep configuration with rsemi_ConfigureAcquisition
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_InitiateSpurious(
    ViSession    instrSession,
    ViInt32    window,
    ViUInt32    timeout
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";     
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout)); 
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_INIT_SPURIOUS, 
                VI_NULL));


Error:
    if (old_timeout > 0) 
        rsemi_SetOPCTimeout (instrSession, old_timeout); 
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function starts a Spectrum Emission Mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_InitiateSEMMeasurement(
    ViSession    instrSession,
    ViInt32    window,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";     
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout));    
    sprintf (buffer, "Win%ld", window);
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_INIT_SEM_MEAS, 
                VI_NULL));


Error:
    if (old_timeout > 0) 
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function aborts a previously initiated measurement and returns 
/// HIFN  the spectrum analyzer to the idle state.
/// HIFN     
/// HIFN     This function does not check the instrument status. Typically, the 
/// HIFN  end-user calls this function only in a sequence of calls to other 
/// HIFN  low-level driver functions. The sequence performs one operation. The 
/// HIFN  end-user uses the low-level functions to optimize one or more aspects 
/// HIFN  of interaction with the instrument. 
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) To check the instrument status, call the Error Query function at 
/// HIFN  the conclusion of the sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_Abort(
    ViSession    instrSession
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_MEAS_ABORT, 
                VI_NULL));


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function continues a stopped measurement at the current position 
/// HIFN  in single sweep mode. The function is useful especially for trace 
/// HIFN  functions MAXHold, MINHold and AVERage if the previous results are not 
/// HIFN  to be cleared with Sweep Count > 0 or Average Count > 0 on restarting 
/// HIFN  the measurement (INIT:IMMediate resets the previous results on 
/// HIFN  restarting the measurement).
/// HIFN     The single-sweep mode is automatically switched on. Synchronization to 
/// HIFN  the end of the indicated number of measurements can then be performed 
/// HIFN  with the command *OPC, *OPC? or *WAI. In the continuous-sweep mode, 
/// HIFN  synchronization to the sweep end is not possible since the overall 
/// HIFN  measurement never ends.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_MeasurementContinue(
    ViSession    instrSession,
    ViInt32    window,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout=-1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout)); 
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_INIT_CONMEAS, VI_NULL));   

    
Error:
    if (old_timeout > 0) 
        rsemi_SetOPCTimeout (instrSession, old_timeout); 
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sends a command to trigger the SpecAn.  Call this 
/// HIFN  function if the trigger source is set to software trigger.  
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) If both the trigger source are not set to software trigger, this 
/// HIFN  function returns an error.
/// HIFN     
/// HIFN     (2) This function does not check the instrument status.   Typically, 
/// HIFN  you call this function only in a sequence of calls to other low-level 
/// HIFN  driver functions.  The sequence performs one operation.  You use the 
/// HIFN  low-level functions to optimize one or more aspects of interaction with 
/// HIFN  the instrument.  If you want to check the instrument status, call the 
/// HIFN  rsemi_error_query function at the conclusion of the sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_SendSoftwareTrigger(
    ViSession    instrSession,
    ViUInt32    timeout
)
{

    ViStatus    error   = VI_SUCCESS;
    ViInt32     old_timeout=-1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout)); 
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_INIT_SW_TRIGGER, 
                VI_NULL));


Error:
    if (old_timeout > 0) 
        rsemi_SetOPCTimeout (instrSession, old_timeout); 
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*===========================================================================*
 *------------------- Sequencer ---------------------------------------------*
 *===========================================================================*/
/// HIFN  This function starts a new sequence of measurements by the Sequencer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_InitiateSequencer (
    ViSession instrSession,
    ViUInt32 timeout
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout));    
    
    viCheckErr (rsemi_ClearBeforeRead (instrSession));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_INIT_SEQUENCER, NULL));

Error:
    if (old_timeout > 0) 
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function stops the currently active sequence of measurements. The 
/// HIFN  Sequencer itself is not deactivated, so you can start a new sequence 
/// HIFN  immediately.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_AbortSequencer (
    ViSession instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "",
                                               RSEMI_ATTR_ABORT_SEQUENCER, NULL));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function is only available if the Sequencer is deactivated 
/// HIFN  (SYSTem:SEQuencer OFF) and only in MSRA mode. The data in the capture 
/// HIFN  buffer is re-evaluated by all active MSRA applications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_RefreshSequencer (
    ViSession instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "",
                                               RSEMI_ATTR_REFRESH_SEQUENCER, NULL));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function turns the Sequencer on and off. The Sequencer must be 
/// HIFN  active before any other Sequencer commands (INIT:SEQ...) are executed, 
/// HIFN  otherwise an error will occur.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control switches the state of the sequencer.
ViStatus _VI_FUNC rsemi_ConfigureSequencerState (
    ViSession instrSession,
    ViBoolean state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_SEQUENCER_STATE,
                                                 state), 2, "State");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function selects the way the R&S FSWT application performs 
/// HIFN  measurements sequentially. Before this function can be executed, the 
/// HIFN  Sequencer must be activated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR sequencerMode/This control selects the way the R&S FSWT application performs 
/// HIPAR sequencerMode/measurements sequentially.
ViStatus _VI_FUNC rsemi_ConfigureSequencerMode (
    ViSession instrSession,
    ViInt32 sequencerMode
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SEQUENCER_MODE,
                                               sequencerMode), 2, "Sequencer Mode");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function returns the current number of started sweeps. A sweep 
/// HIFN  count value should be set and the device should be in single sweep 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR numberOfSweeps/Returns the current sweep number.
ViStatus _VI_FUNC rsemi_GetSweepNumber(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32*    numberOfSweeps
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SWEEP_COUNT_CURRENT,
                                                numberOfSweeps), 3, "Number Of Sweeps"); 
                                                   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches the noise measurement on or off for all markers 
/// HIFN  of the indicated measurement window. The noise power density is 
/// HIFN  measured at the position of the markers.
/// HIFN     
/// HIFN     Note: This function is not available for GSM/EDGE measurements 
/// HIFN  modulation accuracy (MAC), phase/frequency error (PFE) and power versus 
/// HIFN  time (PVT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR state/This control switches the noise measurement on or off for all markers 
/// HIPAR state/of the indicated measurement window. The noise power density is 
/// HIPAR state/measured at the position of the markers. 
ViStatus _VI_FUNC rsemi_ConfigureMarkerNoiseMeasurement(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);  
    viCheckParm( rsemi_SetAttributeViBoolean(instrSession, buffer, RSEMI_ATTR_MARKER_NOISE_STATE, 
                state), 4, "State");
   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the result of the noise measurement.
/// HIFN     A complete sweep with synchronization to the sweep end must be 
/// HIFN  performed between switching on the function and querying the measured 
/// HIFN  value in order to obtain a valid query result. This is only possible in 
/// HIFN  single sweep mode.
/// HIFN     
/// HIFN     Note: This function is not available for GSM/EDGE measurements 
/// HIFN  modulation accuracy (MAC), phase/frequency error (PFE) and power versus 
/// HIFN  time (PVT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR result/This control returns the result of the noise measurement.
ViStatus _VI_FUNC rsemi_QueryMarkerNoiseMeasurementResult(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64*    result
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);  
    viCheckParm( rsemi_GetAttributeViReal64(instrSession, buffer, RSEMI_ATTR_MARKER_NOISE_RESULT, 
                result), 4, "Result");
   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures phase noise marker measurement. To setup 
/// HIFN  reference point use function rsemi_ConfigureReferenceFixedPoint.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control switches on or off the phase-noise measurement with all 
/// HIPAR state/actve delta markers in the selected measurement window. The correction 
/// HIPAR state/values for the bandwidth and the log amplifier are taken into account 
/// HIPAR state/in the measurement.
ViStatus _VI_FUNC rsemi_ConfigureMarkerPhaseNoiseMeasurement(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean(instrSession, buffer, 
                RSEMI_ATTR_REFERENCE_MARKER_PNO_STATE,  state), 3, "State");

    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines a new reference point level for delta marker 2 
/// HIFN  in the selected measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_PhaseNoisePeakSearch(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckErr( rsemi_SetAttributeViString(instrSession, buffer, 
                RSEMI_ATTR_MARKER_REF_PEAK,  VI_NULL));
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sests an automatic peak search action for the reference 
/// HIFN  fixed marker 1 at the end of each particular sweep. This function may 
/// HIFN  be used for tracking of a drifting source whilst phase noise 
/// HIFN  measurement. The delta marker 2 which shows the phase noise measurement 
/// HIFN  result keeps the delta frequency value. Therefore the phase noise 
/// HIFN  measurement in a certain offset is valid although the source is 
/// HIFN  drifting. Only when the marker 2 is reaching the border of the span the 
/// HIFN  delta marker value is adjusted to be within the span. Choose a larger 
/// HIFN  span in such
/// HIFN     situations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control switches an automatic peak search action for the 
/// HIPAR state/reference fixed marker 1 at the end of each particular sweep.
ViStatus _VI_FUNC rsemi_PhaseNoisePeakSearchAuto(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean(instrSession, buffer, 
                RSEMI_ATTR_REFERENCE_MARKER_PNO_AUTO,  state), 2, "State");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the result of the phase-noise measurement in the 
/// HIFN  selected measurement window. The measurement will be switched on, if 
/// HIFN  necessary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR phaseNoiseResult/Returns the result of the phase-noise measurement in the selected 
/// HIPAR phaseNoiseResult/measurement window. The measurement will be switched on, if necessary.
ViStatus _VI_FUNC rsemi_QueryPhaseNoiseResult(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    phaseNoiseResult
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_GetAttributeViReal64(instrSession, buffer, 
                RSEMI_ATTR_REFERENCE_MARKER_PNO_RESULT,  phaseNoiseResult), 
                3, "Phase Noise Result");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the marker frequency counter resolution and turns 
/// HIFN  the marker frequency counter on/off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerFrequencyCounter/Specifies whether the frequency counter is enabled.
/// HIPAR frequencyCounterResolution/Specifies the frequency counter resolution.
ViStatus _VI_FUNC rsemi_ConfigureMarkerFrequencyCounter(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViBoolean    markerFrequencyCounter,
    ViReal64    frequencyCounterResolution
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_MARKER_FREQUENCY_COUNTER_ENABLED, 
                markerFrequencyCounter), 4, "Marker Frequency Counter");
    
    sprintf (buffer, "Win%ld", window);                
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_MARKER_FREQUENCY_COUNTER_RESOLUTION, 
                    frequencyCounterResolution), 5, "Freqeuncy Counter Resolution");
              

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the result of the frequency counter for the 
/// HIFN  indicated marker in the selected measurement window. Before the 
/// HIFN  command, the frequency counter should be switched on and a complete 
/// HIFN  measurement performed to obtain a valid count result. Therefore, a 
/// HIFN  single sweep with synchronization must be performed between switching 
/// HIFN  on the frequency counter and querying the count result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR counterFrequency/This control returns the result of the frequency counter for the 
/// HIPAR counterFrequency/indicated marker.
ViStatus _VI_FUNC rsemi_QueryMarkerFrequencyCounterResult(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64*    counterFrequency
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window,marker); 
    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_MARKER_COUNT, 
                counterFrequency), 4, "Counter Freqeuncy");
              
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the N dB function.
/// HIFN     
/// HIFN     Note: This function is not available for GSM/EDGE measurements 
/// HIFN  modulation accuracy (MAC), phase/frequency error (PFE) and power versus 
/// HIFN  time (PVT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR ndBState/Switches the "N dB Down" function on or off in the selected 
/// HIPAR ndBState/measurement window. Marker 1 is activated first, if necessary.
/// HIPAR ndBLevel/Defines the level spacing of the two delta markers to the right and 
/// HIPAR ndBLevel/left of marker 1 in the selected measurement window.
ViStatus _VI_FUNC rsemi_ConfigureNdBPoints(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    ndBState,
    ViReal64    ndBLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, 
                RSEMI_ATTR_MARKER_NDB_STATE,  ndBState), 
                3, "N dB State");
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, 
                RSEMI_ATTR_MARKER_NDB_VAL,  ndBLevel), 
                4, "N dB Level");                
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the frequency spacing (bandwidth) of the 
/// HIFN  N-dB-down markers in the selected measurement window.
/// HIFN     A complete sweep with synchronization to sweep end must be performed 
/// HIFN  between switching on the function and querying the measured value in 
/// HIFN  order to obtain a valid query result. This is only possible in single 
/// HIFN  sweep mode.
/// HIFN     
/// HIFN     Note: This function is not available for GSM/EDGE measurements 
/// HIFN  modulation accuracy (MAC), phase/frequency error (PFE) and power versus 
/// HIFN  time (PVT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR ndBResult/Returns the frequency spacing (bandwidth) of the N-dB-down markers in 
/// HIPAR ndBResult/the selected measurement window.
ViStatus _VI_FUNC rsemi_QueryNdBResult(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    ndBResult
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_GetAttributeViReal64(instrSession, buffer, 
                RSEMI_ATTR_MARKER_NDB_RESULT,  ndBResult), 
                3, "N dB Result");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the two frequencies of the N-dB-down marker in 
/// HIFN  the selected measurement window. The two frequency values are output in 
/// HIFN  ascending order.
/// HIFN     A complete sweep with synchronization to sweep end must be performed 
/// HIFN  between switching on the function and querying the measured value to 
/// HIFN  obtain a valid query result. This is only possible in single sweep 
/// HIFN  mode.
/// HIFN     
/// HIFN     Note: This function is not available for GSM/EDGE measurements 
/// HIFN  modulation accuracy (MAC), phase/frequency error (PFE) and power versus 
/// HIFN  time (PVT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR ndBFrequencyLower/Returns the lower frequency of N dB down markers in the selected 
/// HIPAR ndBFrequencyLower/measurement window.
/// HIPAR ndBFrequencyHigher/Returns the higher frequency of N dB down markers in the selected 
/// HIPAR ndBFrequencyHigher/measurement window.
ViStatus _VI_FUNC rsemi_QueryNdBFrequencies(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    ndBFrequencyLower,
    ViReal64*    ndBFrequencyHigher
)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!ndBFrequencyLower)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "N dB Frequency Lower");    
    if (!ndBFrequencyHigher)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "N dB Frequency Higher");         
   
    viCheckErr( viQueryf(instrSession, "CALC%ld:MARK:FUNC:NDBD:FREQ?\n","%Le,%Le", window, ndBFrequencyLower, ndBFrequencyHigher ));
    
    checkErr( rsemi_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the two times of the N-dB-down marker in the 
/// HIFN  selected measurement window. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR ndBTimeLower/Returns the lower frequency of N dB down markers in the selected 
/// HIPAR ndBTimeLower/measurement window.
/// HIPAR ndBTimeHigher/Returns the higher frequency of N dB down markers in the selected 
/// HIPAR ndBTimeHigher/measurement window.
ViStatus _VI_FUNC rsemi_QueryNdBTimes(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    ndBTimeLower,
    ViReal64*    ndBTimeHigher
)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!ndBTimeLower)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "N dB Frequency Lower");    
    if (!ndBTimeHigher)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "N dB Frequency Higher");         
   
    viCheckErr( viQueryf(instrSession, "CALC%ld:MARK:FUNC:NDBD:TIME?\n","%Le,%Le", window, ndBTimeLower, ndBTimeHigher ));
    
    checkErr( rsemi_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the Q factor (quality) of the measured bandwidth 
/// HIFN  for span>0. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR ndBQFactor/Returns the Q factor (quality) of the measured bandwidth for span>0.
ViStatus _VI_FUNC rsemi_QueryNdBQFactor(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    ndBQFactor
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!ndBQFactor)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "N dB Frequency Lower");    
    
    sprintf (buffer, "Win%ld", window); 
    
    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_MARKER_NDB_Q_FACTOR,
                                                ndBQFactor), 3, "N dB Q Factor");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the list of settings (max. 100 entries for an 
/// HIFN  individual frequency point) for the multiple power measurement and 
/// HIFN  starts a measurement sequence. To reduce the setting time, all 
/// HIFN  indicated parameters are set up simultaneously at each test point.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) Each one test point is defined on the same position in all 
/// HIFN  parameter arrays.
/// HIFN     
/// HIFN     (2) The measurement is performed in the time domain and therefore the 
/// HIFN  span is set to 0 Hz. If the time domain is quit, the function is 
/// HIFN  automatically switched off. The measurement is not compatible with 
/// HIFN  other measurements, especially as far as marker, adjacent channel power 
/// HIFN  measurement or statistics are concerned. The corresponding commands 
/// HIFN  thus automatically deactivate the function. The function is only 
/// HIFN  available in REMOTE operation. It is deactivated when switching the 
/// HIFN  instrument back to LOCAL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR noofListItems/Set number of list items. This value must be equal to array size of 
/// HIPAR noofListItems/each list parameter.
/// HIPAR analyzerFrequency/Array of receive frequency for the signal to be measured (= center 
/// HIPAR analyzerFrequency/frequency in manual operation).
/// HIPAR referenceLevel/Array of reference level values for each list item.
/// HIPAR rfInputAttenuation/Array of RF input attenuation values for each list item.
/// HIPAR rfInputElectronicAttn/Array of RF input attenuation values of electronic attenuator for each 
/// HIPAR rfInputElectronicAttn/list item.
/// HIPAR filterType/Array of filter types for each list item.
/// HIPAR resolutionBandwidth/Array of resolution bandwidth values for each list item.
/// HIPAR videoBandwidth/Array of video bandwidth values for each list item.
/// HIPAR measTime/Array of measurement time values for each list item.
/// HIPAR triggerLevel/Array of trigger level values for each list item.
ViStatus _VI_FUNC rsemi_ConfigureListPowerSequence(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    noofListItems,
    ViReal64    analyzerFrequency[],
    ViReal64    referenceLevel[],
    ViReal64    rfInputAttenuation[],
    ViReal64    rfInputElectronicAttn[],
    ViInt32    filterType[],
    ViReal64    resolutionBandwidth[],
    ViReal64    videoBandwidth[],
    ViReal64    measTime[],
    ViReal64    triggerLevel[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     pwrite_buffer = NULL;
    ViChar*     pbuffer;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, noofListItems, 1, 100), 3, "No Of List Items");   
    if (analyzerFrequency == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Analyzer Frequency");  
    if (referenceLevel == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Reference Level");  
    if (rfInputAttenuation == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "RF Input Attenuation");  
    if (rfInputElectronicAttn == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "RF Input Electronic Attn");  
    if (filterType == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Filter Type");  
    if (resolutionBandwidth == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Resolutin Bandwidth");  
    if (videoBandwidth == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Video Bandwidth");  
    if (measTime == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Meas Time");  
    if (triggerLevel == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 12, "Trigger Level");  

    /*
     * Alloc buffer. Size is calclulated as:
     * size of header + size required for one entry * number of entries
     */
    pwrite_buffer = (ViChar*) malloc((size_t)(20 + 240 * noofListItems));
    if (pwrite_buffer == NULL)
        return VI_ERROR_ALLOC;

    pbuffer = pwrite_buffer;

    pbuffer += sprintf (pbuffer, ":SENS%ld:LIST:POW:SEQ ", window);

    for (i = 0; i < noofListItems; i++)
    {
        /* Electronic attenuator is not in the signal path. */
        if (rfInputElectronicAttn[i] < 0)
        {
            pbuffer += sprintf (pbuffer, "%.12f,%.12f,%.12f,OFF,%s,%.12f,%.12f,%.12f,%.12f,",
                                analyzerFrequency[i],
                                referenceLevel[i],
                                rfInputAttenuation[i],
                                listFilterTypeArr[filterType[i]],/* RFInputEAttn[i], */
                                resolutionBandwidth[i],
                                videoBandwidth[i],
                                measTime[i],
                                triggerLevel[i]);
        }
        else
        {
            pbuffer += sprintf (pbuffer, "%.12f,%.12f,%.12f,%.12f,%s,%.12f,%.12f,%.12f,%.12f,",
                                analyzerFrequency[i],
                                referenceLevel[i],
                                rfInputAttenuation[i],
                                rfInputElectronicAttn[i],
                                listFilterTypeArr[filterType[i]],
                                resolutionBandwidth[i],
                                videoBandwidth[i],
                                measTime[i],
                                triggerLevel[i]);
        }
    }

    *--pbuffer = '\0'; /* Remove remaining comma */

    pbuffer += sprintf (pbuffer, "\n");

    viCheckErr(viWrite (instrSession, (ViBuf) pwrite_buffer, (ViUInt32)strlen (pwrite_buffer), NULL));
    
    checkErr( rsemi_CheckStatus (instrSession));  
    
Error:
    if (pwrite_buffer) free (pwrite_buffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the constant settings for the list during 
/// HIFN  multiple power measurement. Parameters Peak Meas, RMS Meas and AVG Meas 
/// HIFN  define, which measurements are to be performed at the same time at the 
/// HIFN  frequency point. Correspondingly, one, two or three results per 
/// HIFN  frequency point are returned for the function Get List Power Sequence 
/// HIFN  (actListPwrSeq). If all three parameters are set to OFF, the command 
/// HIFN  generates an execution error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR peakMeas/Activates (deactivates) the measurement of the peak power (peak
/// HIPAR rmsMeas/Activates (deactivates) the measurement of the RMS power (RMS 
/// HIPAR rmsMeas/detector).
/// HIPAR avgMeas/Activates (deactivates) the measurement of the average power (average 
/// HIPAR avgMeas/detector).
/// HIPAR triggerMode/Selection of the trigger source used for the list measurement.
/// HIPAR triggerSlope/This control selects the slope of the trigger signal.
/// HIPAR triggerOffset/This control sets the offset between the detection of the trigger 
/// HIPAR triggerOffset/signal and the start of the measurement at the next frequency point.
/// HIPAR gateLength/Gate length with Gated Sweep.
ViStatus _VI_FUNC rsemi_ConfigureListPowerSet(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    peakMeas,
    ViBoolean    rmsMeas,
    ViBoolean    avgMeas,
    ViInt32    triggerMode,
    ViInt32    triggerSlope,
    ViReal64    triggerOffset,
    ViReal64    gateLength
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, triggerMode, 0, 3), 6, "Trigger Mode");   
    viCheckParm (rsemi_invalidViInt32Range (instrSession, triggerSlope, 0, 1), 7, "Trigger Slope");           
    viCheckParm (rsemi_invalidViReal64Range (instrSession, triggerOffset, 0.0, 100.0), 8, "Trigger Offset");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, gateLength, 0.0, 100.0), 9, "Gate Length");
        
    viCheckErr (viPrintf (instrSession, "SENS%ld:LIST:POW:SET %s,%s,%s,%s,%s,%.12f,%.12f\n",
            window, switchArr[peakMeas], switchArr[rmsMeas], switchArr[avgMeas],
            rsemi_rngTriggerSource.rangeValues[triggerMode].cmdString, rsemi_rngPolarity.rangeValues[triggerSlope].cmdString,
            triggerOffset, gateLength));
            
    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function deactivates the list measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ListPowerMeasurementOff(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckErr ( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_LIST_POW_STATE_OFF, ""));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the list of settings (max. 100 entries for an 
/// HIFN  individual frequency point) for the multiple power measurement, starts 
/// HIFN  a measurement sequence and returns the measured data. To reduce the 
/// HIFN  setting time, all indicated parameters are set up simultaneously at 
/// HIFN  each test point.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) Each one test point is defined on the same position in all 
/// HIFN  parameter arrays.
/// HIFN     
/// HIFN     (2) The measurement is performed in the time domain and therefore the 
/// HIFN  span is set to 0 Hz. If the time domain is quit, the function is 
/// HIFN  automatically switched off. The measurement is not compatible with 
/// HIFN  other measurements, especially as far as marker, adjacent channel power 
/// HIFN  measurement or statistics are concerned. The corresponding commands 
/// HIFN  thus automatically deactivate the function. The function is only 
/// HIFN  available in REMOTE operation. It is deactivated when switching the 
/// HIFN  instrument back to LOCAL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR noofListItems/Set number of list items. This value must be equal to array size of 
/// HIPAR noofListItems/each list parameter.
/// HIPAR analyzerFrequency/Array of receive frequency for the signal to be measured (= center 
/// HIPAR analyzerFrequency/frequency in manual operation).
/// HIPAR referenceLevel/Array of reference level values for each list item.
/// HIPAR rfInputAttenuation/Array of RF input attenuation values for each list item.
/// HIPAR rfInputElectronicAttn/Array of RF input attenuation values of electronic attenuator for each 
/// HIPAR rfInputElectronicAttn/list item.
/// HIPAR filterType/Array of filter types for each list item.
/// HIPAR resolutionBandwidth/Array of resolution bandwidth values for each list item.
/// HIPAR videoBandwidth/Array of video bandwidth values for each list item.
/// HIPAR measTime/Array of measurement time values for each list item.
/// HIPAR triggerLevel/Array of trigger level values for each list item.
/// HIPAR listPowerResults/This control returns array of elements as the result of a previously 
/// HIPAR listPowerResults/configured and initiated list measurement.
ViStatus _VI_FUNC rsemi_QueryListPowerSequence(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    noofListItems,
    ViReal64    analyzerFrequency[],
    ViReal64    referenceLevel[],
    ViReal64    rfInputAttenuation[],
    ViReal64    rfInputElectronicAttn[],
    ViInt32    filterType[],
    ViReal64    resolutionBandwidth[],
    ViReal64    videoBandwidth[],
    ViReal64    measTime[],
    ViReal64    triggerLevel[],
    ViReal64    listPowerResults[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     pwrite_buffer = NULL;
    ViChar*     pbuffer;
    ViInt32     i = 0,
                results;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, noofListItems, 1, 100), 3, "No Of List Items");   
    if (analyzerFrequency == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Analyzer Frequency");  
    if (referenceLevel == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Reference Level");  
    if (rfInputAttenuation == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "RF Input Attenuation");  
    if (rfInputElectronicAttn == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "RF Input Electronic Attn");  
    if (filterType == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Filter Type");  
    if (resolutionBandwidth == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Resolutin Bandwidth");  
    if (videoBandwidth == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Video Bandwidth");  
    if (measTime == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Meas Time");  
    if (triggerLevel == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 12, "Trigger Level"); 
    if (listPowerResults == VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 13, "List Power Results");         
    
 
    pwrite_buffer = (ViChar*) malloc((size_t)(20 + 240 * noofListItems));
    if (pwrite_buffer == NULL)
        return VI_ERROR_ALLOC;

    pbuffer = pwrite_buffer;

    pbuffer += sprintf (pbuffer, ":SENS%ld:LIST:POW:SEQ? ", window);

    for (i=0; i<noofListItems; i++)
        {
        /* Electronic attenuator is not in the signal path. */
        if (rfInputElectronicAttn[i] < 0)
            pbuffer += sprintf (pbuffer, "%.12f,%.12f,%.12f,OFF,%s,%.12f,%.12f,%.12f,%.12f,",
                                analyzerFrequency[i], referenceLevel[i], rfInputAttenuation[i],
                                /* RFInputEAttn[i], */ listFilterTypeArr[filterType[i]], resolutionBandwidth[i],
                                videoBandwidth[i], measTime[i], triggerLevel[i]);
        else
            pbuffer += sprintf (pbuffer, "%.12f,%.12f,%.12f,%.12f,%s,%.12f,%.12f,%.12f,%.12f,",
                                analyzerFrequency[i], referenceLevel[i], rfInputAttenuation[i],
                                rfInputElectronicAttn[i], listFilterTypeArr[filterType[i]], resolutionBandwidth[i],
                                videoBandwidth[i], measTime[i], triggerLevel[i]);
        }

    *--pbuffer = '\0'; /* Remove remaining comma */

    pbuffer += sprintf (pbuffer, "\n");

    viCheckErr (viWrite (instrSession, (ViBuf) pwrite_buffer, (ViUInt32) strlen (pwrite_buffer), NULL));

    //viCheckErr (rsemi_WaitForOPC (instrSession, timeout)); 
    /* Read float numbers separated by commas x 3 possibly activated detectors */
    
    
    results = (noofListItems);
    viCheckErr (viScanf (instrSession, "%,#lf", &results, listPowerResults));
    
    checkErr( rsemi_CheckStatus (instrSession)); 
    
Error:
    if (pwrite_buffer) free (pwrite_buffer); 
    
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the result of a previous list measurement as 
/// HIFN  configured and initiated with List Power Sequence. This function may be 
/// HIFN  used to obtain measurement results in an asynchronous way, using the 
/// HIFN  service request mechanism for synchronization with the end of the 
/// HIFN  measurement. If no measurement results are available, the function will 
/// HIFN  return a query error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR noofResults/Expected number of values to be returned from the instrument. This 
/// HIPAR noofResults/value must be equal to the List Power results array number of items.
/// HIPAR listPowerResults/This control returns array of elements as the result of a previous 
/// HIPAR listPowerResults/list measurement as configured and initiated with List Power Sequence 
/// HIPAR listPowerResults/(confListPwrSeq).
/// HIPAR returnedValues/Returns the number of Power Results.
ViStatus _VI_FUNC rsemi_QueryListPowerResult(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    noofResults,
    ViReal64    listPowerResults[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     ptag;
    ViInt32     index;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (noofResults<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "No Of Results");
        
    viCheckErr (viQueryf(instrSession, "SENS%ld:LIST:POW:RES?\n","%s",window, buffer));
    ptag = strtok(buffer, ",");
    for (index = 0; ptag; index++)
    {
        if (index < noofResults)
            listPowerResults[index] = atof(ptag);
        ptag = strtok(NULL, ",");
    }
    if (returnedValues)
        *returnedValues=index;
    checkErr( rsemi_CheckStatus (instrSession));  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches on or off the previously selected time domain 
/// HIFN  power measurements. Thus one or several measurements can be first 
/// HIFN  selected and then switched on and off together with this attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Switches on or off the previously selected time domain power 
/// HIPAR state/measurements.
ViStatus _VI_FUNC rsemi_ConfigureTimeDomainPowerMeasurementState(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_MEAS_TDOM_STATE,
                                                state), 3, "State"); 
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures parameters of time domain power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR peak/Sets the time domain power peak masurement.
/// HIPAR rms/Sets the time domain power RMS masurement.
/// HIPAR mean/Sets the time domain power mean masurement.
/// HIPAR standardDeviation/Sets the time domain power standard deviation masurement.
/// HIPAR average/Switches on or off averaging for the active time domain power 
/// HIPAR average/measurement in the indicated window.
/// HIPAR maxHold/Switches on or off the peak-hold function for the active time domain 
/// HIPAR maxHold/power measurement in the indicated measurement window.
/// HIPAR power/Selects absolute or relative time domain power measurement in the 
/// HIPAR power/indicated measurement window.
ViStatus _VI_FUNC rsemi_ConfigureTimeDomainPowerMeasurement(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    peak,
    ViBoolean    rms,
    ViBoolean    mean,
    ViBoolean    standardDeviation,
    ViBoolean    average,
    ViBoolean    maxHold,
    ViInt32    power
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_MEAS_TDOM_PEAK,
                                                peak), 3, "Peak"); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_MEAS_TDOM_RMS,
                                                rms), 4, "RMS");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_MEAS_TDOM_MEAN,
                                                mean), 5, "Mean");
    if (mean == VI_TRUE)
        viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_MEAS_TDOM_SDEV,
                                                standardDeviation), 6, "Standard Deviation");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_MEAS_TDOM_AVG,
                                                average), 7, "Average");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_MEAS_TDOM_MAX_HOLD,
                                                maxHold), 8, "Max Hold"); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_MEAS_TDOM_MODE,
                                                power), 9, "Power");                                                
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches off all time domain measurements in the 
/// HIFN  selected measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_TimeDomainPowerMeasurementAllOff(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_TDOM_AOFF,
                                                VI_NULL));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  With this function the currently measured average value and RMS value 
/// HIFN  are declared as reference values for relative measurements in the 
/// HIFN  indicated measurement window.
/// HIFN     If the function of Averaging or Peak Hold is switched on, the current 
/// HIFN  value is the accumulated measurement value at the time considered.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     (1) This function is not available for GSM/EDGE measurements 
/// HIFN  modulation accuracy (MAC) and phase/frequency error (PFE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_TimeDomainPowerMeasurementSetReference(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window); 
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_TDOM_SET_REFERENCE,
                                                VI_NULL));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries selected time domain power measurement result in 
/// HIFN  the indicated measurement window.
/// HIFN     
/// HIFN     A complete sweep with synchronization to sweep end must be performed 
/// HIFN  between switching on the function and querying the measured value to 
/// HIFN  obtain a valid query result. This is only possible in single sweep 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurement/Selects the time domain power measurement.
/// HIPAR resultType/Selects the time domain power measurement result type.
/// HIPAR result/Returns the selected result of time domain power measurement.
ViStatus _VI_FUNC rsemi_QueryTDomPowerResult(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    measurement,
    ViInt32    resultType,
    ViReal64*    result
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, measurement, 0, 3), 3, "Measurement");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, resultType, 0, 2), 4, "Result Type");        
    
    sprintf (buffer, "Win%ld", window);  
    switch (measurement){
        case RSEMI_VAL_TDOM_PEAK:
            switch (resultType){
                case RSEMI_VAL_TDOM_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_PEAK_RESULT,
                                                                result), 5, "Result");
                break;
                case RSEMI_VAL_TDOM_AVG_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_AVG_PEAK_RESULT,
                                                                result), 5, "Result");
                break;
                case RSEMI_VAL_TDOM_MAX_HOLD_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_PEAK_HOLD_RESULT,
                                                                result), 5, "Result");
                break;
            }    
        break;
        case RSEMI_VAL_TDOM_RMS:
            switch (resultType){
                case RSEMI_VAL_TDOM_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_RMS_RESULT,
                                                                result), 5, "Result");
                break;
                case RSEMI_VAL_TDOM_AVG_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_AVG_RMS_RESULT,
                                                                result), 5, "Result");
                break;
                case RSEMI_VAL_TDOM_MAX_HOLD_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_PEAK_HOLD_RMS_RESULT,
                                                                result), 5, "Result");
                break;
            }    
        break;
        case RSEMI_VAL_TDOM_MEAN:
            switch (resultType){
                case RSEMI_VAL_TDOM_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_MEAN_RESULT,
                                                                result), 5, "Result");
                break;
                case RSEMI_VAL_TDOM_AVG_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_AVG_MEAN_RESULT,
                                                                result), 5, "Result");
                break;
                case RSEMI_VAL_TDOM_MAX_HOLD_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_PEAK_HOLD_MEAN_RESULT,
                                                                result), 5, "Result");
                break;
            }    
        break;
        case RSEMI_VAL_TDOM_SDEV:
            switch (resultType){
                case RSEMI_VAL_TDOM_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_SDEV_RESULT,
                                                                result), 5, "Result");
                break;
                case RSEMI_VAL_TDOM_AVG_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_AVG_SDEV_RESULT,
                                                                result), 5, "Result");
                break;
                case RSEMI_VAL_TDOM_MAX_HOLD_RESULT:
                    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                                RSEMI_ATTR_MEAS_TDOM_PEAK_HOLD_SDEV_RESULT,
                                                                result), 5, "Result");
                break;
            }    
        break;
    }    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function is used to determine the power of a sequence of signal 
/// HIFN  pulses having the same interval, as are typical for the slots of a GSM 
/// HIFN  signal, for example. The number of pulses to be measured as well as the 
/// HIFN  measurement time and the period can be set. To define the position of 
/// HIFN  the first pulse in the trace, a suitable offset can be entered.
/// HIFN     The evaluation is performed on the measurement data of a previously 
/// HIFN  recorded trace. The data recorded during the set measurement time is 
/// HIFN  combined to a measured value for each pulse according to the detector 
/// HIFN  specified and the indicated number of results is output as a list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeOffsetofFirstPulse/Defines the time offset of first pulse to define the position of the 
/// HIPAR timeOffsetofFirstPulse/first pulse in the trace.
/// HIPAR measurementTime/Defines the measurement time.
/// HIPAR period/Defines the period.
/// HIPAR numberofPulses/Defines number of pulses to be measured.
/// HIPAR result/Returns the array of power of a sequence of signal pulses having the 
/// HIPAR result/same interval.
ViStatus _VI_FUNC rsemi_QueryTDomPowerPulseSequenceResult(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    timeOffsetofFirstPulse,
    ViReal64    measurementTime,
    ViReal64    period,
    ViInt32    numberofPulses,
    ViReal64    result[]
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (numberofPulses<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Number Of Pulses");
    if (result == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Result");        
    
    viCheckErr(viPrintf (instrSession, "CALC%ld:MARK:FUNC:MSUM? %.12f,%.12f,%.12f,%ld\n", 
                 window, timeOffsetofFirstPulse, measurementTime, period, numberofPulses));
    viCheckErr (viScanf (instrSession, "%,#lf", &numberofPulses, result));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the state of selected power measurement.
/// HIPAR powerMeasurement/Selects the ACP (adjacent channel power) standard.
ViStatus _VI_FUNC rsemi_SelectPowerMeasurement(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state,
    ViInt32    powerMeasurement
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    if (state == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                    RSEMI_ATTR_MEAS_POW_SELECT,
                                                    powerMeasurement), 4, "Power Measurement");
    }                                                    
    else
    {
        viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_OFF,
                                                VI_NULL));
    }                                                

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR powerSet/
ViStatus _VI_FUNC rsemi_ConfigurePowerSet (
    ViSession instrSession,
    ViInt32 window, 
    ViInt32 powerSet
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_SET,
                                                powerSet), 3, "Power Set");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the adjacent power channels measurement.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) This function is only available in the frequency domain (span > 
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR noOfAdjChannels/The start frequency of the frequency sweep.
/// HIPAR channelMode/Sets absolute and relative adjacent channel measurement.
/// HIPAR powerMode/Sets the Clear Write or Maxhold for Channel Power values. 
/// HIPAR resultMode/Sets the query response of the power measurement results in the 
/// HIPAR resultMode/indicated measurement window between output of absolute values (OFF) 
/// HIPAR resultMode/and output referred to the measurement bandwidth (ON).. 
ViStatus _VI_FUNC rsemi_ConfigurePowerMeasurement(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    noOfAdjChannels,
    ViInt32    channelMode,
    ViInt32    powerMode,
    ViBoolean    resultMode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ADJ_NUM,
                                                noOfAdjChannels), 3, "No Of Adj Channels");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ADJ_MODE,
                                                channelMode), 4, "Channel Mode");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_MODE,
                                                powerMode), 5, "Power Mode");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_RESULT_MODE,
                                                resultMode), 6, "Result Mode");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the adjacent power channel spacing.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) This function is only available in the frequency domain (span > 
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR channel/Selects the channel type. The driver uses this value to selects the  
/// HIPAR channel/RSEMI_ATTR_POW_CHANNEL_SPACING or RSEMI_ATTR_POW_ADJ_CHANNEL_SPACING or 
/// HIPAR channel/
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR spacing/Sets the selected channel spacing. 
ViStatus _VI_FUNC rsemi_ConfigurePowerChannelSpacing(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    channel,
    ViInt32    channelNumber,
    ViReal64    spacing
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, channel, 0, 2), 3, "Channel");        
    
    switch (channel){
        case RSEMI_VAL_ACP_TX:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, channelNumber, 1, 11), 4, "Channel");  
            sprintf (buffer, "Win%ld,TX%ld", window, channelNumber); 
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_CHAN_SPACING ,
                                                spacing), 5, "Spacing");
        break;
        case RSEMI_VAL_ACP_ADJ:
            sprintf (buffer, "Win%ld", window); 
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ADJ_CHANNEL_SPACING,
                                                spacing), 5, "Spacing");
        break;
        case RSEMI_VAL_ACP_ALT:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, channelNumber, 1, 11), 4, "Channel");  
            sprintf (buffer, "Win%ld,CH%ld", window, channelNumber); 
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ALT_CHANNEL_SPACING,
                                                spacing), 5, "Spacing");
        break;
    }    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the adjacent power channel bandwidth.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) This function is only available in the frequency domain (span > 
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR channel/Selects the channel type.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR bandwidth/Sets the selected channel bandwidth.
ViStatus _VI_FUNC rsemi_ConfigurePowerChannelBandwidth(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    channel,
    ViInt32    channelNumber,
    ViReal64    bandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, channel, 0, 2), 3, "Channel");        
    
    switch (channel){
        case RSEMI_VAL_ACP_TX:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, channelNumber, 1, 11), 4, "Channel");  
            sprintf (buffer, "TX%ld", channelNumber); 
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                RSEMI_ATTR_POW_CHANNEL_BANDWIDTH,
                                                bandwidth), 5, "Bandwidth");
        break;
        case RSEMI_VAL_ACP_ADJ:
            sprintf (buffer, "Win%ld", window); 
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ADJ_CHANNEL_BANDWIDTH,
                                                bandwidth), 5, "Bandwidth");
        break;
        case RSEMI_VAL_ACP_ALT:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, channelNumber, 1, 11), 4, "Channel");  
            sprintf (buffer, "Win%ld,CH%ld", window, channelNumber); 
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ALT_CHANNEL_BANDWIDTH,
                                                bandwidth), 5, "Bandwidth");
        break;
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the power channel name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR channel/Selects the channel type.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR name/Sets the power channel name.
ViStatus _VI_FUNC rsemi_ConfigurePowerChannelName(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    channel,
    ViInt32    channelNumber,
    ViString    name
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViAttr      attr[] = {RSEMI_ATTR_MEAS_POW_CHANNEL_NAME,
                          RSEMI_ATTR_MEAS_POW_ADJ_CHANNEL_NAME,
                          RSEMI_ATTR_MEAS_POW_ALT_CHANNEL_NAME}; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, channel, RSEMI_VAL_ACP_TX, RSEMI_VAL_ACP_ALT), 3, "Channel");

    if (channel == RSEMI_VAL_ACP_ADJ)
    {
        sprintf (buffer, "Win%ld", window);
    }
    else if (channel == RSEMI_VAL_ACP_TX)
    {
        viCheckParm (rsemi_invalidViInt32Range (instrSession, channelNumber, 1, 18), 4, "Channel Number");

        sprintf (buffer, "Win%ld,TX%ld", window, channelNumber);
    }
    else
    {
        viCheckParm (rsemi_invalidViInt32Range (instrSession, channelNumber, 1, 11), 4, "Channel Number");
    
        sprintf (buffer, "Win%ld,CH%ld", window, channelNumber);
    }

    viCheckParm( rsemi_SetAttributeViString(instrSession, buffer, attr[channel], name),5, "Name");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches on or off the correction of the instrument 
/// HIFN  inherent noise for channel power measurement. On activating this 
/// HIFN  function, a reference measurement of the instrument inherent noise is 
/// HIFN  performed. The measured noise power is then subtracted from the power 
/// HIFN  in the examined channel. The measurement of the instrument inherent 
/// HIFN  noise is then repeated after any change of the center frequency, 
/// HIFN  resolution bandwidth, sweep time and level setting.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     (1) This function is only available in the frequency domain (span > 
/// HIFN  0).
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR noiseCorrectionState/Turns on or off the noise correction. The driver uses this value to 
/// HIPAR noiseCorrectionState/set the RSEMI_ATTR_POW_NOISE_CORR attribute.
ViStatus _VI_FUNC rsemi_ConfigureChannelPowerNoiseCorrection(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    noiseCorrectionState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_NOISE_CORR,
                                                noiseCorrectionState), 3, "Noise Correction State");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects on, off or auto correction of the instrument 
/// HIFN  inherent noise for channel power measurement. On activating this 
/// HIFN  function, a reference measurement of the instrument inherent noise is 
/// HIFN  performed. The measured noise power is then subtracted from the power 
/// HIFN  in the examined channel. The measurement of the instrument inherent 
/// HIFN  noise is then repeated after any change of the center frequency, 
/// HIFN  resolution bandwidth, sweep time and level setting.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) This function is only available in the frequency domain (span > 
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR mode/Activates or deactivates a noise correction measurement.
ViStatus _VI_FUNC rsemi_ConfigureChannelPowerNoiseCorrectionAuto(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    mode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_NOISE_CORR_AUTO,
                                                mode), 3, "mode");
   
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}


/// HIFN  This function assigns the channel/adjacent channel power measurement 
/// HIFN  of the indicated trace in the selected measurement window. The 
/// HIFN  corresponding trace should be active, ie its states should differ from 
/// HIFN  blank.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR traceNumber/Selects trace number.
ViStatus _VI_FUNC rsemi_ConfigureChannelPowerTrace(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    traceNumber
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_TRACE,
                                                traceNumber), 3, "Trace Number");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the standard for channel power measurement.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     FAST ACP is not available if a WLAN standard is selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR acpStandard/Selects the ACP (adjacent channel power) standard.
ViStatus _VI_FUNC rsemi_ConfigureChannelPowerStandard(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    acpStandard
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_STANDARD,
                                                acpStandard), 3, "ACP Standard");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns all predefined ACP standards and all user 
/// HIFN  standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog 
/// HIPAR bufferSize/list parameter.
/// HIPAR powerStandardCatalog/This control returns all predefined ACP standards and all user 
/// HIPAR powerStandardCatalog/standards.
ViStatus _VI_FUNC rsemi_QueryChannelPowerStandardCatalog(
    ViSession    instrSession,
    ViInt32    bufferSize,
    ViChar    powerStandardCatalog[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = VI_NULL;
    ViUInt32    count = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (viPrintf(instrSession, "CALC:MARK:FUNC:POW:STAN:CAT?\n"));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &count)); 
    checkErr( rsemi_CheckStatus (instrSession));
    
    if (count>=(ViUInt32)bufferSize)
    {
        strncpy(powerStandardCatalog, pbuffer, (size_t) (bufferSize-1));
        powerStandardCatalog[bufferSize-1]='\0';
        error = (ViStatus) (count+1);
    } 
    else
    {
        strncpy(powerStandardCatalog, pbuffer, (size_t) count); 
        powerStandardCatalog[count]='\0';   
    }
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the user standard for channel power 
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR operation/This control selects the operation to be performed with user standard.
/// HIPAR userStandardName/This control selects name of ACP user standard.
ViStatus _VI_FUNC rsemi_ConfigureChannelPowerUserStandard(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    operation,
    ViString    userStandardName
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViAttr      attr_arr[3] = {RSEMI_ATTR_ACP_USER_STD_SET,
                               RSEMI_ATTR_ACP_USER_STD_SAVE,
                               RSEMI_ATTR_ACP_USER_STD_DELETE};
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, operation, RSEMI_VAL_USTD_SET, RSEMI_VAL_USTD_DEL), 3, "Operation");
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViString (instrSession, buffer, attr_arr[operation],
                                                userStandardName), 4, "User Standard Name");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This sets the reference value for the relative measurement to the 
/// HIFN  currently measured channel power.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     This function sets instrument to Channel Power with a Single Carrier 
/// HIFN  measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_PowerAdjustReferenceLevel(
    ViSession    instrSession,
    ViInt32    window,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    checkErr (rsemi_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rsemi_SetOPCTimeout (instrSession, timeout));
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_REF_VALUE,
                                                VI_NULL));

Error:
    if (old_timeout >= 0)    
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function adjusts the frequency span, the measurement bandwidths 
/// HIFN  and the detector as required for the number of channels, the channel 
/// HIFN  bandwidths and the channel spacings selected in the active power 
/// HIFN  measurement. If necessary, adjacent-channel power measurement is 
/// HIFN  switched on prior to the adjustment.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) This attribute is available only for measurements in the frequency 
/// HIFN  domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR channelPowerType/Specifies the channel power standard. 
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_PowerPresetMeasurement(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    channelPowerType,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    checkErr (rsemi_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rsemi_SetOPCTimeout (instrSession, timeout));
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ADJ_PRESET,
                                                channelPowerType), 3, "Channel Power Type");
   

Error:
    if (old_timeout >= 0)    
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the power measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR powerMeasurement/Selects the ACP (adjacent channel power) standard. 
/// HIPAR arraySize/Defines the memory size allocated for Results. 
/// HIPAR results/Returns the measured values:
/// HIPAR returnedValues/Returns number of returned values in Results array.
ViStatus _VI_FUNC rsemi_QueryPowerResults(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    powerMeasurement,
    ViInt32    arraySize,
    ViReal64    results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuff = VI_NULL; 
    ViInt32     i;
    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, powerMeasurement, 0, 7), 3, "Power Measurement");
    
    if (arraySize<0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Array Size"); 
    if ((arraySize>0) && (results == NULL))    
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Results"); 
    
    viCheckErr( viPrintf (instrSession, "CALC%ld:MARK:FUNC:POW:RES? %s\n", window, 
        rsemi_rngMeasPowerSelect.rangeValues[powerMeasurement].cmdString));
    
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &buffer, VI_NULL));  
    
    pbuff = strtok (buffer, ",");
    i = 0;
    do{
        if (i<arraySize) results[i] = atof (pbuff);
        pbuff = strtok (NULL, ",");
        i++;
    }while(pbuff != NULL);   
    
    if (returnedValues) *returnedValues = i;
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    if (buffer) free(buffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the power channel weighting filters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterType/This control selects the type of the filter to be modified.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR filterState/This control sets the state of the selected filter.
/// HIPAR filterValue/This control sets the weighting filter alpha value.
ViStatus _VI_FUNC rsemi_ConfPowerChannelWeightingFilters (ViSession instrSession,
                                                          ViInt32 filterType,
                                                          ViInt32 channelNumber,
                                                          ViBoolean filterState,
                                                          ViReal64 filterValue)
{
     ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch(filterType){
        case RSEMI_VAL_WFILTER_ALL:
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_SEM_WEIGHT_FILTER_ALL_STATE, filterState),
                                                 4, "Filter State");
    
            if (filterState == VI_TRUE)
            {
                viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",RSEMI_ATTR_SEM_WEIGHT_FILTER_ALL_ALPHA, filterValue),
                                                            5, "Filter Value");
            }    
        break;
        case RSEMI_VAL_WFILTER_ADJ:
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_SEM_WEIGHT_FILTER_ACH_STATE, filterState),
                                                 4, "Filter State");
    
            if (filterState == VI_TRUE)
            {
                viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",RSEMI_ATTR_SEM_WEIGHT_FILTER_ACH_ALPHA, filterValue),
                                                            5, "Filter Value");
            }
        break;
        case RSEMI_VAL_WFILTER_ALT:
            viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_STATE,
                                                 "Channel",
                                                 channelNumber - 1,
                                                 10,
                                                 rep_cap), 3, "Channel Number");
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_STATE, filterState),
                                                 4, "Filter State");
    
            if (filterState == VI_TRUE)
            {
                viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_ALPHA_VALUE, filterValue),
                                                            5, "Filter Value");
            }
        break;
        case RSEMI_VAL_WFILTER_TX:
                viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_POWER_CHANNEL_WEIGHTING_FILTER,
                                                 "TXChannel",
                                                 channelNumber - 1,
                                                 10,
                                                 rep_cap), 3, "Channel Number");
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_POWER_CHANNEL_WEIGHTING_FILTER, filterState),
                                                 4, "Filter State");
    
            if (filterState == VI_TRUE)
            {
                viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_POWER_CHANNEL_WEIGHTING_FILTER_ALPHA, filterValue),
                                                            5, "Filter Value");
            }
        break;
        default:
           viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Filter Type");
    }
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures the MSR power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  
/// HIPAR ACLRMode/Sets absolute and relative adjacent channel measurement. 
/// HIPAR powerMode/Sets the Clear Write or Maxhold for Channel Power values.
/// HIPAR powerUnit/Sets the query response of the power measurement results in the
/// HIPAR powerUnit/indicated measurement window between output of absolute values
/// HIPAR powerUnit/(OFF) and output referred to the measurement bandwidth (ON).
/// HIPAR noiseCorrection/Activates or deactivates a noise correction measurement.
/// HIPAR traceNumber/Selects trace number.
ViStatus _VI_FUNC rsemi_ConfigureMSRPowerMeasurement (ViSession instrSession,
                                                      ViInt32 ACLRMode,
                                                      ViInt32 powerMode,
                                                      ViBoolean powerUnit,
                                                      ViInt32 noiseCorrection,
                                                      ViInt32 traceNumber)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, NULL));
 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                RSEMI_ATTR_MEAS_POW_ADJ_MODE,
                                                ACLRMode), 2, "ACLR Mode");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                RSEMI_ATTR_MEAS_POW_MODE,
                                                powerMode), 3, "Power Mode");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "",
                                                RSEMI_ATTR_MEAS_POW_RESULT_MODE,
                                                powerUnit), 4, "Power Unit");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                RSEMI_ATTR_MEAS_POW_NOISE_CORR_AUTO,
                                                noiseCorrection),5, "Noise Correction");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                RSEMI_ATTR_MEAS_POW_TRACE,
                                                traceNumber), 6, "Trace Number");

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;    
}

/// HIFN  This function defines the number of subblocks, i.e. groups of 
/// HIFN  transmission channels in MSR signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfSubblocks/Defines the number of subblocks, i.e. groups of transmission channels 
/// HIPAR numberOfSubblocks/in MSR signal.
ViStatus _VI_FUNC rsemi_ConfigureMSRNumberOfSubblocks (ViSession instrSession,
                                                       ViInt32 numberOfSubblocks)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                RSEMI_ATTR_MSR_NUMBER_OF_SUBBLOCKS,
                                                numberOfSubblocks), 2, "Number Of Subblocks");
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  Subblocks are groups of transmit channels in an MSR signal. Up to 5 
/// HIFN  subblocks can be defined. They are defined as an RF bandwidth around a 
/// HIFN  center frequency with a specific number of transmit channels (max. 18).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR centerFrequency/Defines the center of the specified MSR subblock. Note that the 
/// HIPAR centerFrequency/position of the subblock also affects the position of the adjacent gap 
/// HIPAR centerFrequency/(CACLR) channels.
/// HIPAR subblock/This control selects the subblock number.
/// HIPAR RFBandwidth/Defines the bandwidth of the individual MSR subblock. Note that 
/// HIPAR RFBandwidth/subblock ranges also affect the position of the adjacent gap channels 
/// HIPAR RFBandwidth/(CACLR).
/// HIPAR txChannelCount/Defines the bandwidth of the individual MSR subblock. Note that 
/// HIPAR txChannelCount/subblock ranges also affect the position of the adjacent gap channels 
/// HIPAR txChannelCount/(CACLR).
ViStatus _VI_FUNC rsemi_ConfigureMSRSubblockDefinition (ViSession instrSession,
                                                        ViInt32 subblock,
                                                        ViReal64 centerFrequency,
                                                        ViReal64 RFBandwidth,
                                                        ViInt32 txChannelCount)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [15] = "";
                               
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MSR_SUBBLOCK_CENTER_FREQUENCY,
                                                 "Subblock",
                                                 subblock - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_SUBBLOCK_CENTER_FREQUENCY,
                                                centerFrequency), 3, "Center Frequency");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_SUBBLOCK_RF_BANDWIDTH,
                                                RFBandwidth), 4, "RF Bandwidth");
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_SUBBLOCK_TX_CHANNEL_COUNT,
                                                txChannelCount), 5, "Tx Channel Count"); 

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  As opposed to common ACLR channel definitions, the TX channels are 
/// HIFN  defined at absolute frequencies, rather than by a spacing relative to 
/// HIFN  the (common) center frequency.
/// HIFN  Each transmit channel can be assigned a different technology, used to 
/// HIFN  predefine the required bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subblock/This control selects the subblock number.
/// HIPAR txChannel/Tx channel number.
/// HIPAR txCenterFrequency/This control defines the (absolute) center frequency of the specified 
/// HIPAR txCenterFrequency/MSR Tx channel.
/// HIPAR technology/This control defines the technology used for transmission by the 
/// HIPAR technology/specified MSR Tx channel.
/// HIPAR txBandwidth/Defines the bandwidth of the specified MSR Tx channel.
/// HIPAR weightingFilterState/This control turns the weighting filter transmission channel on and 
/// HIPAR weightingFilterState/off.
/// HIPAR rollOffFactor/This control defines the roll-off factor for the specified 
ViStatus _VI_FUNC rsemi_ConfigureMSRTxChannelDefinition (ViSession instrSession,
                                                         ViInt32 subblock,
                                                         ViInt32 txChannel,
                                                         ViReal64 txCenterFrequency,
                                                         ViInt32 technology,
                                                         ViReal64 txBandwidth,
                                                         ViBoolean weightingFilterState,
                                                         ViReal64 rollOffFactor)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [25] = "";
       ViChar      sub_rep_cap [10] = "";
    ViChar        tx_rep_cap[10] = "";
                               
    checkErr( Rs_LockSession (instrSession, NULL));

  
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MSR_TX_CENTER_FREQUENCY,
                                                 "Subblock",
                                                 subblock - 1,
                                                 10,
                                                 sub_rep_cap));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MSR_TX_CENTER_FREQUENCY,
                                                 "TXChannel",
                                                 txChannel - 1,
                                                 10,
                                                 tx_rep_cap));
    
    sprintf (rep_cap, "%s,%s", sub_rep_cap, tx_rep_cap); 
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_TX_CENTER_FREQUENCY,
                                                txCenterFrequency), 4, "Tx Center Frequency");
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_TX_TECHNOLOGY,
                                                technology), 5, "Technology");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_TX_BANDWIDTH,
                                                txBandwidth), 6, "Tx Bandwidth");
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_TX_WEIGHTING_FILTER_STATE,
                                                weightingFilterState), 7, "Weighting Filter State"); 
    
    if (weightingFilterState == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_TX_WEIGHTING_FILTER_ROLL_OFF_FACTOR,
                                                rollOffFactor), 8, "Roll Off Factor"); 
    }
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function queries the name of the specified MSR Tx channel.
/// HIFN  In MSR ACLR measurements, TX channel names correspond to the specified 
/// HIFN  technology, followed by a consecutive number. The assigned subblock 
/// HIFN  (A,B,C,D,E) is indicated as a prefix (e.g. A: WCDMA1). Channel names 
/// HIFN  cannot be defined manually.    
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subblock/This control selects the subblock number.
/// HIPAR txChannel/Tx channel number.
/// HIPAR name/String containing the name of the channel.
ViStatus _VI_FUNC rsemi_QueryMSRTxName (ViSession instrSession,
                                           ViInt32 subblock, 
                                           ViInt32 txChannel,
                                           ViInt32 bufferSize,
                                           ViChar name[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [25] = "";
       ViChar      sub_rep_cap [10] = "";
    ViChar        tx_rep_cap[10] = "";
                               
    checkErr( Rs_LockSession (instrSession, NULL));

  
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MSR_TX_CENTER_FREQUENCY,
                                                 "Subblock",
                                                 subblock - 1,
                                                 10,
                                                 sub_rep_cap));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MSR_TX_CENTER_FREQUENCY,
                                                 "TXChannel",
                                                 txChannel - 1,
                                                 10,
                                                 tx_rep_cap));
    
    sprintf (rep_cap, "%s,%s", sub_rep_cap, tx_rep_cap); 
    
    viCheckParm( rsemi_GetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_MSR_TX_CHANNEL_NAME, 
                                                bufferSize, name), 4, "Name"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function defines the distance from subblock to the specified gap 
/// HIFN  channel, the bandwidth of the specified MSR gap (CACLR) channel, and 
/// HIFN  the weighting filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR channelSpacing/Defines the distance from subblock to the specified gap channel.
/// HIPAR channelBandwidths/Defines the bandwidth of the specified MSR gap (CACLR) channel.
/// HIPAR weightingFilterState/This control turns the weighting filter for the specified gap (CACLR) 
/// HIPAR weightingFilterState/channel on and off.
/// HIPAR rollOffFactor/This control defines the roll-off factor for the specified gap (CACLR) 
/// HIPAR rollOffFactor/channel's weighting filter.
ViStatus _VI_FUNC rsemi_ConfigureMSRGapChannel (ViSession instrSession,
                                                ViInt32 gap,
                                                ViReal64 channelSpacing,
                                                ViReal64 channelBandwidths,
                                                ViBoolean weightingFilterState,
                                                ViReal64 rollOffFactor)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MSR_GAP_CHANNEL_SPACING,
                                                 "Gap",
                                                 gap - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_GAP_CHANNEL_SPACING,
                                                channelSpacing), 3, "channelSpacing");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_GAP_CHANNEL_BANDWIDTHS,
                                                channelBandwidths), 4, "channelBandwidths");
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_GAP_WEIGHTING_FILTER_STATE,
                                                weightingFilterState), 5, "Weighting Filter State"); 
    
    if (weightingFilterState == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_GAP_WEIGHTING_FILTER_ROLL_OFF_FACTOR,
                                                rollOffFactor), 6, "Roll Off Factor"); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function defines the limit value of the gap (CACLR) channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR relativeState/Turns the relative limit check for the specified gap (CACLR) channel 
/// HIPAR relativeState/on and off.
/// HIPAR relativeValue/Defines the relative limit of the specified gap (CACLR) channel. The 
/// HIPAR relativeValue/reference value for the relative limit is the measured channel power.
/// HIPAR absoluteState/Turns the absolute limit check for the specified gap (CACLR) channel 
/// HIPAR absoluteState/on and off.
/// HIPAR absoluteValue/Defines the absolute limit of the specified gap (CACLR) channel.
ViStatus _VI_FUNC rsemi_ConfigureMSRGapLimitChecking (ViSession instrSession,
                                                      ViInt32 gap,
                                                      ViBoolean relativeState,
                                                      ViReal64 relativeValue,
                                                      ViBoolean absoluteState,
                                                      ViReal64 absoluteValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MSR_GAP_LIMIT_RELATIVE_STATE,
                                                 "Gap",
                                                 gap - 1,
                                                 10,
                                                 rep_cap));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_GAP_LIMIT_RELATIVE_STATE,
                                                relativeState), 3, "Relative State");
    if (relativeState)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_GAP_LIMIT_RELATIVE,
                                                relativeValue), 4, "Relative Value");
    }
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                                RSEMI_ATTR_MSR_GAP_LIMIT_ABSOLUTE_STATE,
                                                absoluteState), 5, "Absolute State"); 
    if (absoluteState)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                 RSEMI_ATTR_MSR_GAP_LIMIT_ABSOLUTE,
                                                absoluteValue), 46, "Absolute Value");
    }
    
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;    
}

/// HIFN This function queries the state of the limit check for allgap
/// HIFN (CACLR) channels in an MSR ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Pass the allocated array size.
/// HIPAR limitResults/Array of limit checking results.
/// HIPAR returnedValues/Number of returned values.
ViStatus _VI_FUNC rsemi_QueryMSRGapLimitChecking (ViSession instrSession,
                                                  ViInt32 arraySize,
                                                  ViInt32 limitResults[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL; 
    ViChar*     pstring_value;
    ViUInt32    ret_cnt;
    ViInt32     cnt;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    
    viCheckErr (viPrintf (instrSession, "CALC:LIM:ACP:GAP:RES?\n"));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, &ret_cnt)); 

    cnt=0;
    pstring_value = strtok(pbuffer, ",\r\n");
    while (pstring_value){
        if (cnt<arraySize)
            limitResults[cnt]=rsemi_StringIndex (statusGapArr, pstring_value);
        pstring_value = strtok(NULL, ",\r\n");
        cnt++;
    }
    if (pbuffer) free(pbuffer);
    
    if (returnedValues) *returnedValues=cnt;
    checkErr( rsemi_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function switches on or off the high-speed channel/adjacent 
/// HIFN  channel power measurement.
/// HIFN     The measurement itself is performed in the time domain on the center 
/// HIFN  frequencies of the individual channels. The function automatically 
/// HIFN  switches to the time domain and back. Depending on the selected mobile 
/// HIFN  radio standard, weighting filters with sqrt(cos) characteristic or very 
/// HIFN  steep-sided channel filters are used for band limitation.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function is only available in the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR fastACPState/Turns on or off the high-speed channel/adjacent channel power 
/// HIPAR fastACPState/measurement..
ViStatus _VI_FUNC rsemi_ConfigureACPFastACPState(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    fastACPState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_ACP_HSP,
                                                fastACPState), 3, "Fast ACP State");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function adapts the reference level to the measured channel power 
/// HIFN  and - if required - switches on previously the adjacent channel power 
/// HIFN  measurement. This ensures that the signal path of the instrument is not 
/// HIFN  overloaded. Since the measurement bandwidth is significantly smaller 
/// HIFN  than the signal bandwidth in channel power measurements, the signal 
/// HIFN  path can be overloaded although the trace is still significantly below 
/// HIFN  the reference level. If the measured channel power equals the reference 
/// HIFN  level, the signal path is not overloaded.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) The attribute is only available in the frequency domain (span > 
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_ACPChannelPowerAutoAdjust(
    ViSession    instrSession,
    ViInt32    window,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout = -1;   
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    checkErr (rsemi_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rsemi_SetOPCTimeout (instrSession, timeout));
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_ACP_PRESET_REF_LEVEL,
                                                VI_NULL));

Error:
    if (old_timeout >= 0)    
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches on and off the limit check for adjacent channel 
/// HIFN  power measurements in the selected measurement window.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function is only available in the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Switches on and off the limit check for adjacent channel power 
/// HIPAR state/measurements.
ViStatus _VI_FUNC rsemi_ConfigureACPLimitState(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_ACP_LIMIT_STATE,
                                                state), 3, "State");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the limit value of the adjacent channel.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function is only available in the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR relativeState/Switches the limit check for the relative limit value of the adjacent 
/// HIPAR relativeState/channel when adjacent channel power measurement is performed. 
/// HIPAR relativeValue/Sets the relative limit value.
/// HIPAR absoluteState/Switches the limit check for the absolute limit value of the adjacent 
/// HIPAR absoluteState/channel when adjacent channel power measurement is performed.
/// HIPAR absoluteValue/Sets the relative limit value.
ViStatus _VI_FUNC rsemi_ConfigureACPAdjacentChannelLimit(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    relativeState,
    ViReal64    relativeValue,
    ViBoolean    absoluteState,
    ViReal64    absoluteValue
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
     
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViReal64Range (instrSession, relativeValue, 0.0, 100.0), 4, "Relative Value");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, absoluteValue, -200.0, 200.0), 6, "Absolute Value");         
    
    sprintf (buffer, "Win%ld", window); 
    
    viCheckErr (viPrintf (instrSession, "CALC%ld:LIM:ACP:ACH %.12lf,0.0\n", window, relativeValue));
    checkErr( rsemi_CheckStatus (instrSession));
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                    RSEMI_ATTR_MEAS_ACP_LIMIT_REL_STATE,
                                                    relativeState), 3, "Relative State");
    viCheckErr (viPrintf (instrSession, "CALC%ld:LIM:ACP:ACH:ABS %.12lf,0.0\n", window, absoluteValue));
    checkErr( rsemi_CheckStatus (instrSession));
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_ACP_LIMIT_ABS_STATE,
                                                absoluteState), 5, "Absolute State"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the limit value of the alternate adjacent 
/// HIFN  channel.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function is only available in the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR alternateChannelNumber/Sets the channel number.
/// HIPAR relativeState/Switches the limit check for the relative limit value of the alternate 
/// HIPAR relativeState/adjacent channel when adjacent channel power measurement is performed.  
/// HIPAR relativeState/The driver uses this value to set the 
/// HIPAR relativeState/RSEMI_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE attribute.
/// HIPAR relativeValue/Sets the relative limit value.
/// HIPAR absoluteState/Switches the limit check for the absolute limit value of the alternate 
/// HIPAR absoluteState/adjacent channel when adjacent channel power measurement is performed.
/// HIPAR absoluteValue/Sets the relative limit value.
ViStatus _VI_FUNC rsemi_ConfigureACPAlternateChannelLimit(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    alternateChannelNumber,
    ViBoolean    relativeState,
    ViReal64    relativeValue,
    ViBoolean    absoluteState,
    ViReal64    absoluteValue
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    

    viCheckParm (rsemi_invalidViInt32Range (instrSession, alternateChannelNumber, 1, 11), 3, "Channel Number");        
    viCheckParm (rsemi_invalidViReal64Range (instrSession, relativeValue, 0.0, 100.0), 5, "Relative Value");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, absoluteValue, -200.0, 200.0), 7, "Absolute Value");         
    
    sprintf (buffer, "Win%ld,CH%ld", window, alternateChannelNumber); 
    
    viCheckErr (viPrintf (instrSession, "CALC%ld:LIM:ACP:ALT%ld %.12lf,0.0\n", window, 
                            alternateChannelNumber, relativeValue));
    checkErr( rsemi_CheckStatus (instrSession));                             
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                    RSEMI_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE,
                                                    relativeState), 4, "Relative State");
    viCheckErr (viPrintf (instrSession, "CALC%ld:LIM:ACP:ALT%ld:ABS %.12lf,0.0\n", window, 
                                                    alternateChannelNumber, absoluteValue));
    checkErr( rsemi_CheckStatus (instrSession));   
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                    RSEMI_ATTR_MEAS_ACP_ALT_LIMIT_ABS_STATE,
                                                    absoluteState), 6, "Absolute State"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the adjacent channel limit results.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function is only available in the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR lowerResult/Returns the lower limit result.
/// HIPAR upperResult/Returns the upper limit result.
ViStatus _VI_FUNC rsemi_QueryACPAdjacentChannelLimitCheckResults(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32*    lowerResult,
    ViInt32*    upperResult
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      buf2[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!lowerResult)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Lower Result");
    if (!upperResult)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Upper Result");        
        
    viCheckErr (viQueryf (instrSession, "CALC%ld:LIM:ACP:ACH:RES?\n","%[^,],%[^\r\n]", 
    window, buf1, buf2));
    viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)lowerResult, buf1));     
    viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)upperResult, buf2));
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}


/// HIFN  This function returns the alternate channel limit results.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function is only available in the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR alternateChannelNumber/Sets the channel number.
/// HIPAR lowerResult/Returns the lower limit result.
/// HIPAR upperResult/Returns the upper limit result.
ViStatus _VI_FUNC rsemi_QueryACPAlternateChannelLimitCheckResults(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    alternateChannelNumber,
    ViInt32*    lowerResult,
    ViInt32*    upperResult
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      buf2[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, alternateChannelNumber, 1, 11), 3, "Alternate Channel Number");        
    if (lowerResult == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Lower Result");
    if (upperResult == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Upper Result");        
        
    viCheckErr (viQueryf (instrSession, "CALC%ld:LIM:ACP:ALT%ld:RES?\n","%[^,],%[^\r\n]", 
    window, alternateChannelNumber, buf1, buf2));
    viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)lowerResult, buf1));     
    viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)upperResult, buf2));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function queries the state of the limit check for all gap
/// HIFN  (CACLR) channels in an MSR ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR gap/This control selects the gap (CACLR) channel number
/// HIPAR lowerResult/Returns the lower limit result.
/// HIPAR upperResult/Returns the upper limit result.
ViStatus _VI_FUNC rsemi_QueryACPGapChannelLimitCheckResults (
    ViSession instrSession,
    ViInt32 window,
    ViInt32 gap,
    ViInt32 *lowerResult,
    ViInt32 *upperResult
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      buf2[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      buf3[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      buf4[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, gap, 1, 2), 3, "Gap");        
    if (lowerResult == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Lower Result");
    if (upperResult == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Upper Result");        
        
    viQueryf (instrSession, "CALC%ld:LIM:ACP:GAP%ld:RES?\n","%[^,],%[^\r\n],%[^,],%[^\r\n]", 
    window, gap, buf1, buf2, buf3, buf4);
    
    if ((gap == 2) && (strlen(buf3) > 0) && (strlen (buf4) > 0))
    {
        viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)lowerResult, buf3));     
        viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)upperResult, buf4));
    }
    else
    {
        viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)lowerResult, buf1));     
        viCheckErr (Rs_GetIndexOfToken (resultsString, ",", (ViUInt32 *)upperResult, buf2));
    }
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the adjacent power channels measurement.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) This function is only available in the frequency domain (span > 
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR noOfAdjChannels/The start frequency of the frequency sweep.
/// HIPAR noOfTXChannels/Sets the number of TX channels.
/// HIPAR channelMode/Sets absolute and relative adjacent channel measurement. The driver 
/// HIPAR channelMode/uses this value to set the RSEMI_ATTR_MEAS_POW_ADJ_MODE attribute.
/// HIPAR powerMode/Sets the Clear Write or Maxhold for Channel Power values. The driver 
/// HIPAR powerMode/uses this value to set the RSEMI_ATTR_MEAS_POW_MODE attribute.
/// HIPAR resultMode/Sets the query response of the power measurement results in the 
/// HIPAR resultMode/indicated measurement window between output of absolute values (OFF) 
/// HIPAR resultMode/and output referred to the measurement bandwidth (ON).. 
ViStatus _VI_FUNC rsemi_ConfigureMulticarrierACPPower(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    noOfAdjChannels,
    ViInt32    noOfTXChannels,
    ViInt32    channelMode,
    ViInt32    powerMode,
    ViBoolean    resultMode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ADJ_NUM,
                                                noOfAdjChannels), 3, "No Of Adj Channels");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_CARR_SIG_NUM,
                                                noOfTXChannels), 4, "No Of TX Channels");
    if (noOfAdjChannels>0)
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ADJ_MODE,
                                                channelMode), 5, "Channel Mode");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_MODE,
                                                powerMode), 6, "Power Mode");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_RESULT_MODE,
                                                resultMode), 7, "Result Mode");                                                
                                                

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  Activates the automatic selection of a transmission channel to be used 
/// HIFN  as a reference channel in relative adjacent-channel power measurements. 
/// HIFN  The transmission channel with the highest power, the transmission 
/// HIFN  channel with the lowest power, or the transmission channel nearest to 
/// HIFN  the adjacent channels can be defined as a reference channel.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) The attribute is available only for multicarrier channel and 
/// HIFN  adjacent-channel power measurements (CALC:MARK:FUNC:POW:SEL MCAC) in 
/// HIFN  the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR reference/This control sets the type of reference selection.
ViStatus _VI_FUNC rsemi_ConfigureACPReferenceChannelAuto(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    reference
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_REF_CHAN_SEL_AUTO,
                                                reference), 3, "Reference");
   

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects a transmission channel to be used as a reference 
/// HIFN  channel in relative adjacent-channel power measurements.
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) The attribute is available only for multicarrier channel and 
/// HIFN  adjacent-channel power measurements (CALC:MARK:FUNC:POW:SEL MCAC) in 
/// HIFN  the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR channel/This control select the reference channel for relative adjacent 
/// HIPAR channel/channel power measurement.
ViStatus _VI_FUNC rsemi_ConfigureACPReferenceChannelMan(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    channel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_ADJ_REF_TXCHANNEL,
                                                channel), 3, "Channel");
   

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines the percentage of the power with respect to the 
/// HIFN  total power. This value is the basis for the occupied bandwidth 
/// HIFN  measurement (command: POWer:ACHannel:PRESet OBW).
/// HIFN     
/// HIFN     Note:
/// HIFN     (1) The attribute is only available in the frequency domain (span > 
/// HIFN  0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR powerBandwidth/Sets the percentage of the power with respect to the total power. The 
/// HIPAR powerBandwidth/driver uses this value to set the RSEMI_ATTR_POW_BANDWIDTH attribute.
/// HIPAR channelBandwidth/Sets the selected channel bandwidth. The driver uses this value to set 
/// HIPAR channelBandwidth/the RSEMI_ATTR_POW_CHANNEL_BANDWIDTH attribute.
ViStatus _VI_FUNC rsemi_ConfigureOccupiedBandwidth(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    powerBandwidth,
    ViReal64    channelBandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_POW_BANDWIDTH,
                                                powerBandwidth), 4, "Power Bandwidth");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "TX1",
                                                RSEMI_ATTR_POW_CHANNEL_BANDWIDTH,
                                                channelBandwidth), 5, "Channel Bandwidth");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures parameters of signal statistic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the signal statistic measurement.
/// HIPAR numberofSamples/Defines the number of measurement points to be acquired for the 
/// HIPAR numberofSamples/statistical measurement functions.
/// HIPAR resolutionBandwidth/Specifies the measurement resolution bandwidth.
ViStatus _VI_FUNC rsemi_ConfigureSignalStatisticMeasurement(
    ViSession    instrSession,
    ViInt32    measurement,
    ViInt32    numberofSamples,
    ViReal64    resolutionBandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, measurement, 0, 2), 2, "Measurement");
 
    switch (measurement){
        case RSEMI_VAL_SSTAT_OFF:      
            viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_APD_STATE,
                                                VI_FALSE));
            viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_CCDF_STATE,
                                                VI_FALSE));
        break;
        case RSEMI_VAL_SSTAT_APD:      
            viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_CCDF_STATE,
                                                VI_FALSE));
            viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_APD_STATE,
                                                VI_TRUE));                                                
            viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_SAMPLES,
                                                numberofSamples), 4, "Number Of Samples");                                                
        break;
        case RSEMI_VAL_SSTAT_CCDF:      
            viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_APD_STATE,
                                                VI_FALSE));
            viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_CCDF_STATE,
                                                VI_TRUE));
            viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_SAMPLES,
                                                numberofSamples), 3, "Number Of Samples");                                                
        break;
    }
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "Win0",
                                                RSEMI_ATTR_SAN_RESOLUTION_BANDWIDTH,
                                                resolutionBandwidth), 4, "Resolution Bandwidth"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures parameters of signal statistic measurement 
/// HIFN  X-Axis scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR xAxisRefLevel/Defines the reference level for the X axis of the measurement diagram. 
/// HIPAR xAxisRefLevel/The setting is identical to the reference level setting using the 
/// HIPAR xAxisRefLevel/command DISPlay:WINDow:TRACe:Y:RLEVel.
/// HIPAR xAxisRange/Defines the level range for the X axis of the measurement diagram. The 
/// HIPAR xAxisRange/setting is identical to the level range setting defined with the 
/// HIPAR xAxisRange/command DISPlay:WINDow:TRACe:Y:SCALe.
ViStatus _VI_FUNC rsemi_ConfigureSignalStatisticMeasurementXAxis(
    ViSession    instrSession,
    ViReal64    xAxisRefLevel,
    ViReal64    xAxisRange
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_X_REF,
                                                xAxisRefLevel), 2, "X Axis Ref Level");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_X_RANGE,
                                                xAxisRange), 3, "X Axis Range");                                                
   

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures parameters of signal statistic measurement Y 
/// HIFN  Axis scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR yUnit/Toggles the scaling of Y axis between percentage and absolute.
/// HIPAR yAxisMaxValue/Defines the upper limit for the Y axis of the diagram in statitistical 
/// HIPAR yAxisMaxValue/measurements. Since probabilities are specified on the Y axis, the 
/// HIPAR yAxisMaxValue/entered numerical values are dimensionless. 
/// HIPAR yAxisMinValue/Defines the lower limit for the Y axis of the diagram in statistical 
/// HIPAR yAxisMinValue/measurements. Since probabilities are specified on the Y axis, the 
/// HIPAR yAxisMinValue/entered numerical values are dimensionless.
ViStatus _VI_FUNC rsemi_ConfigureSignalStatisticMeasurementYAxis(
    ViSession    instrSession,
    ViInt32    yUnit,
    ViReal64    yAxisMaxValue,
    ViReal64    yAxisMinValue
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                    RSEMI_ATTR_MEAS_STAT_Y_UNIT,
                                                    yUnit), 2, "Y Unit");    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_Y_MAX,
                                                yAxisMaxValue), 3, "Y Axis Max Value");  
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_Y_MIN,
                                                yAxisMinValue), 4, "Y Axis Min Value");                                                  
   

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the acquisition attributes of signal 
/// HIFN  statistic.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR sweepModeContinuous/Turns on/off continuous sweeping.
ViStatus _VI_FUNC rsemi_ConfigureSignalStatisticSweep(
    ViSession    instrSession,
    ViBoolean    sweepModeContinuous
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "Win0", 
                                                RSEMI_ATTR_SAN_SWEEP_MODE_CONTINUOUS,
                                                sweepModeContinuous), 2, "Sweep Mode Continuous");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function optimizes the level setting of the instrument depending 
/// HIFN  on the measured peak power, in order to obtain maximum instrument 
/// HIFN  sensitivity.
/// HIFN     To obtain maximum resolution, the level range is set as a function of 
/// HIFN  the measured spacing between peak power and the minimum power for the 
/// HIFN  APD measurement and of the spacing between peak power and mean power 
/// HIFN  for the CCDF measurement. In addition, the probability scale for the 
/// HIFN  number of test points is adapted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_SignalStatisticMeasurementAdjustSettings(
    ViSession    instrSession,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout = -1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    checkErr (rsemi_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rsemi_SetOPCTimeout (instrSession, timeout));
    viCheckErr( rsemi_SetAttributeViString (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_ADJ,
                                                VI_NULL));

Error:
    if (old_timeout >= 0)    
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function resets the scaling of the X and Y axes in a statistical 
/// HIFN  measurement. The following
/// HIFN     values are set:
/// HIFN     X axis ref level: -20 dBm
/// HIFN     X axis range APD: 100 dB
/// HIFN     X axis range CCDF: 20 dB
/// HIFN     Y axis upper limit: 1.0
/// HIFN     Y axis lower limit: 1E-6
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_SignalStatisticMeasurementPresetScaling(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "",
                                                RSEMI_ATTR_MEAS_STAT_PRESET,
                                                VI_NULL));
   

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function reads out the level values for the probabilities 0.1%, 
/// HIFN  1% and 10%.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR probability/Selects the probability.
/// HIPAR result/Returns the selected level value.
ViStatus _VI_FUNC rsemi_QuerySignalStatisticCCDFLevel(
    ViSession    instrSession,
    ViInt32    trace,
    ViInt32    probability,
    ViReal64*    result
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 2, "Trace");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, probability, 0, 3), 3, "Probability"); 
    if (result == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Result");         
    
    viCheckErr (viQueryf (instrSession, "CALC:STAT:CCDF:X%ld? %s\n", "%le", 
                trace, statProbArr[probability], result));
                
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function reads out the results of statistical measurements of a 
/// HIFN  recorded trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR resultType/Selects the signal statistic measurement result type.
/// HIPAR result/Returns the selected measurement.
ViStatus _VI_FUNC rsemi_QuerySignalStatisticResult(
    ViSession    instrSession,
    ViInt32    trace,
    ViInt32    resultType,
    ViReal64*    result
)
{
    
    ViStatus    error = VI_SUCCESS;
    ViReal64     discrete;
    ViChar       *p2string = NULL;
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 2, "Trace");
     
    do
    {
        discrete = rsemi_rngStatisticsResultType.rangeValues[i].discreteOrMinValue;
        p2string = rsemi_rngStatisticsResultType.rangeValues[i].cmdString;

        if (p2string == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Result Type");

        if (fabs(discrete - resultType) <= 0.01) break;

        i++;

    } while (i != 0);
    
    if (result == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Result");         
    
    viCheckErr (viQueryf (instrSession, "CALC:STAT:RES%ld? %s\n", "%le", 
                trace, p2string, result));
                
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}


/// HIFN  This function switches on the measurement of the AM modulation depth. 
/// HIFN  An AM-modulated carrier is required on the screen for correct 
/// HIFN  operation. If necessary, marker 1 is previously activated and set to 
/// HIFN  the largest signal available.
/// HIFN     The level value of marker 1 is regarded as the carrier level. On 
/// HIFN  activating the function, marker 2 and marker 3 are automatically set as 
/// HIFN  delta markers symetrically to the carrier to the adjacent maxima of the 
/// HIFN  trace.
/// HIFN     If the position of delta marker 2 is changed, delta marker 3 is moved 
/// HIFN  symetrically with respect to the reference marker (marker 1). If the 
/// HIFN  position of delta marker 3 is changed, fine adjustment can be performed 
/// HIFN  independently of delta marker 2.
/// HIFN     Instrument calculates the power at the marker positions from the 
/// HIFN  measured levels.
/// HIFN     The AM modulation depth is calculated from the ratio of power values 
/// HIFN  at the reference marker and the delta markers. If the two AM sidebands 
/// HIFN  differ in power, the average value of the two power values is used for 
/// HIFN  calculating the AM modulation depth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Switches on the measurement of the AM modulation depth.
ViStatus _VI_FUNC rsemi_ConfigureModDepthMeasurement(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                            RSEMI_ATTR_MEAS_MDEPTH_STATE,
                                            state), 3, "State");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the AM modulation depth in the indicated 
/// HIFN  measurement window.
/// HIFN     A complete sweep with synchronization to sweep end must be performed 
/// HIFN  between switching on the function and querying the measured value to 
/// HIFN  obtain a valid query result. This is only possible in single sweep 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR modulationDepth/Returns the AM modulation depth in the indicated measurement window.
ViStatus _VI_FUNC rsemi_QueryModulationDepth(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    modulationDepth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                            RSEMI_ATTR_MEAS_MDEPTH_RESULT,
                                            modulationDepth), 3, "Modulation Depth");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function starts the search of the signals required for the 
/// HIFN  modulation depth measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Reserved.
ViStatus _VI_FUNC rsemi_ModulationDepthSignalSearch(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    marker;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MEAS_MODDEPTH_SEARCH,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap,
                                                RSEMI_ATTR_MEAS_MODDEPTH_SEARCH ,
                                                NULL));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function initiates the measurement of the third-order intercept 
/// HIFN  point.
/// HIFN     A two-tone signal with equal carrier levels is expected at the RF 
/// HIFN  input of the instrument. Marker 1 and marker 2 (both normal markers) 
/// HIFN  are set to the maximum of the two signals. Delta marker 3 and delta 
/// HIFN  marker 4 are positioned to the intermodulation products. The delta 
/// HIFN  markers can be modified sperately afterwards.
/// HIFN     The third-order intercept is calculated from the level spacing between 
/// HIFN  the normal markers and the delta markers.
/// HIFN     
/// HIFN     Note: This attribute is not available during GSM measurements 
/// HIFN  modulation accuracy (MAC), phase-frequency error (PFE), power vs. time 
/// HIFN  (PVT) and carrier power (CPW).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Initiates the measurement of the third-order intercept point.
ViStatus _VI_FUNC rsemi_ConfigureTOIMeasurement(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_TOI_STATE,
                                                state), 3, "State");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the third-order intercept point measurement in 
/// HIFN  the indicated measurement window.
/// HIFN     
/// HIFN     A complete sweep with synchronization to sweep end must be performed 
/// HIFN  between switching on the function and querying the measured value to 
/// HIFN  obtain a valid query result. This is only possible in single sweep 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR toiMeasurement/Returns the third-order intercept point measurement in the indicated 
/// HIPAR toiMeasurement/measurement window.
ViStatus _VI_FUNC rsemi_QueryTOI(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    toiMeasurement
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_TOI_RESULT,
                                                toiMeasurement), 3, "TOI Measurement");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function starts the search of the signals required for the third 
/// HIFN  order intercept measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
ViStatus _VI_FUNC rsemi_TOISignalSearch(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker); 
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                                                RSEMI_ATTR_MEAS_TOI_SEARCH,
                                                NULL));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures harmonic distortion measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches on or off the measurement of the harmonics of a carrier 
/// HIPAR state/signal. The carrier signal is the first harmonic. The function is 
/// HIPAR state/independent of the marker selection. It is only available in the 
/// HIPAR state/frequency domain (span > 0).
/// HIPAR noofHarmonics/Defines the number of harmonics of a carrier signal to be measured.
/// HIPAR harmonicRBWAuto/Specifies whether the resolution bandwidth of the 2nd to the nth 
/// HIPAR harmonicRBWAuto/harmonic should be identical to the bandwidth of the first harmonic 
/// HIPAR harmonicRBWAuto/(OFF) or to the next largest bandwidth (corresponding to the harmonic) 
/// HIPAR harmonicRBWAuto/of the multiple of the bandwidth of the first harmonic (ON).
ViStatus _VI_FUNC rsemi_ConfigureHDistMeasurement(
    ViSession    instrSession,
    ViBoolean    state,
    ViInt32    noofHarmonics,
    ViBoolean    harmonicRBWAuto
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "",
                                                RSEMI_ATTR_MEAS_HDIST_STATE,
                                                state), 3, "State");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                RSEMI_ATTR_MEAS_HDIST_NOOFHARM,
                                                noofHarmonics), 4, "No Of Harmonic");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "",
                                                RSEMI_ATTR_MEAS_HDIST_RBWAUTO,
                                                harmonicRBWAuto), 5, "Harmonic RBW Auto");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function optimizes the device settings depending on the mode in 
/// HIFN  which the harmonic measurement was started:
/// HIFN     If the harmonic measurement was started in the frequency domain (span 
/// HIFN  > 0), the frequency and the
/// HIFN     level of the first harmonic are calculated, from which the measurement 
/// HIFN  list is set up.
/// HIFN     If the measurement was started in the time domain (span = 0), the 
/// HIFN  frequency of the first harmonic is
/// HIFN     not changed. The level, however, is not calculated.
/// HIFN     The function is independent of the marker selection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_AdjustHDistSettings(
    ViSession    instrSession,
    ViUInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     old_timeout=-1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout)); 
    viCheckErr( rsemi_SetAttributeViString (instrSession, "",
                                                RSEMI_ATTR_MEAS_HDIST_PRESET,
                                                VI_NULL));
   
Error:
    if (old_timeout > 0) 
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the results of the total harmonic distortion 
/// HIFN  (THD).
/// HIFN     
/// HIFN     A complete sweep with synchronization to sweep end must be performed 
/// HIFN  between switching on the function and querying the measured value to 
/// HIFN  obtain a valid query result. This is only possible in single sweep 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR harmonicDistortioninpercent/Returns the results of the total harmonic distortion (THD) in %.
/// HIPAR harmonicDistortionindB/Returns the results of the total harmonic distortion (THD) in dB.
ViStatus _VI_FUNC rsemi_QueryHDist(
    ViSession    instrSession,
    ViReal64*    harmonicDistortioninpercent,
    ViReal64*    harmonicDistortionindB
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!harmonicDistortioninpercent)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Harmonic Distortion in percent");
     if (!harmonicDistortionindB)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Harmonic Distortion in dB");                 
    
    viCheckErr (viQueryf (instrSession, ":CALC:MARK:FUNC:HARM:DIST? TOT\n", "%le,%le", 
                harmonicDistortioninpercent, harmonicDistortionindB));
                
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function reads out the list of harmonics. The first value is the 
/// HIFN  absolute power of the first harmonic in the unit set via UNIT. The 
/// HIFN  other values are relative to the carrier signal and are output in dB.
/// HIFN     
/// HIFN     A complete sweep with synchronization to sweep end must be performed 
/// HIFN  between switching on the function and querying the measured value to 
/// HIFN  obtain a valid query result. This is only possible in single sweep 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR harmonicsList/Returns the list of harmonics. The first value is the absolute power 
/// HIPAR harmonicsList/of the first harmonic in the unit set via UNIT. The other values are 
/// HIPAR harmonicsList/relative to the carrier signal and are output in dB.
ViStatus _VI_FUNC rsemi_QueryHDistHarmonicsList(
    ViSession    instrSession,
    ViReal64    harmonicsList[]
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (harmonicsList == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Harmonic List");
    
    viCheckErr (viQueryf (instrSession, "CALC:MARK:FUNC:HARM:LIST?\n", "%,255lf", 
                harmonicsList));
                
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the instrument setup for multiple burst power 
/// HIFN  measurement and starts a measurement sequence. To reduce the setting 
/// HIFN  time, the setup is performed simultaneously for all selected 
/// HIFN  parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR analyzerFrequency_Hz/Receive frequency for the burst signals to be measured (= center 
/// HIPAR analyzerFrequency_Hz/frequency in manual operation).
/// HIPAR resolutionBandwidth_Hz/Resolution bandwidth for the measurement.
/// HIPAR measTime_s/Time span during which measurement samples are sampled for RMS / peak 
/// HIPAR measTime_s/measurement The type of measurement is selected by Type of Meas.
/// HIPAR triggerSource/Trigger signal source.
/// HIPAR triggerLevel/Signal level of the video signal as a percentage of the diagram
/// HIPAR triggerOffset_s/Offset between the detection of the trigger signal and the start of 
/// HIPAR triggerOffset_s/the measurement.
/// HIPAR typeofMeas/Determines whether mean power (RMS) or peak power (PEAK) is to be 
/// HIPAR typeofMeas/measured. The FSQ detector is selected accordingly.
/// HIPAR noofMeasurements/Number of individual bursts to be measured.
ViStatus _VI_FUNC rsemi_BurstPowerSequence(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    analyzerFrequency_Hz,
    ViReal64    resolutionBandwidth_Hz,
    ViReal64    measTime_s,
    ViInt32    triggerSource,
    ViReal64    triggerLevel,
    ViReal64    triggerOffset_s,
    ViInt32    typeofMeas,
    ViInt32    noofMeasurements
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (analyzerFrequency_Hz<0.0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Analyzer Frequency [Hz]");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, resolutionBandwidth_Hz, 10.0, 10.0e+6), 4, "Resolution Bandwidth [Hz]");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, measTime_s, 1.0e-6, 30.0), 5, "Meas Time [s]");        
    if (!((triggerSource == RSEMI_VAL_TRG_EXT) || (triggerSource == RSEMI_VAL_TRG_VID)))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Trigger Source");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, triggerLevel, 0.0, 100.0), 7, "Trigger Level");  
    viCheckParm (rsemi_invalidViReal64Range (instrSession, triggerOffset_s, 125.0e-9, 100.0), 8, "Trigger Offset [s]");  
    viCheckParm (rsemi_invalidViInt32Range (instrSession, typeofMeas, 0, 1), 9, "Type Of Meas");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, noofMeasurements, 1, 501), 10, "No Of Measurements");
        
    viCheckErr( viPrintf (instrSession, "SENS%ld:MPOW:SEQ %.12lG,%.12lG,%.12lG,%s,%.12lG,%.12lG,%s,%ld\n",
    window, analyzerFrequency_Hz,resolutionBandwidth_Hz,measTime_s,rsemi_rngTriggerSource.rangeValues[triggerSource].cmdString,
    triggerLevel,triggerOffset_s,measTypeArr[typeofMeas],noofMeasurements));
   
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function the filter type for the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR filterType/This control defines the filter type for the measurement.
ViStatus _VI_FUNC rsemi_BurstPowerFilterType(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    filterType
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_MPOW_FTYPE,
                                                filterType), 2, "Filter Type"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}


/// HIFN  This function configures the instrument setup for multiple burst power 
/// HIFN  measurement, starts a measurement sequence and reads the results. To 
/// HIFN  reduce the setting time, the setup is performed simultaneously for all 
/// HIFN  selected parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR analyzerFrequency_Hz/Receive frequency for the burst signals to be measured (= center 
/// HIPAR analyzerFrequency_Hz/frequency in manual operation).
/// HIPAR resolutionBandwidth_Hz/Resolution bandwidth for the measurement.
/// HIPAR measTime_s/Time span during which measurement samples are sampled for RMS / peak 
/// HIPAR measTime_s/measurement The type of measurement is selected by Type of Meas.
/// HIPAR triggerSource/Trigger signal source.
/// HIPAR triggerLevel/Signal level of the video signal as a percentage of the diagram
/// HIPAR triggerOffset_s/Offset between the detection of the trigger signal and the start of 
/// HIPAR triggerOffset_s/the measurement.
/// HIPAR typeofMeas/Determines whether mean power (RMS) or peak power (PEAK) is to be 
/// HIPAR typeofMeas/measured. The FSQ detector is selected accordingly.
/// HIPAR noofMeasurements/Number of individual bursts to be measured.
/// HIPAR timeout/This control sets the timeout value for data readout in milliseconds.
/// HIPAR burstPowerResults/This control returns array of elements as the result of a previous 
/// HIPAR burstPowerResults/list measurement as configured and initiated with List Power Sequence 
/// HIPAR burstPowerResults/(confListPwrSeq).
ViStatus _VI_FUNC rsemi_GetBurstPowerSequence(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    analyzerFrequency_Hz,
    ViReal64    resolutionBandwidth_Hz,
    ViReal64    measTime_s,
    ViInt32    triggerSource,
    ViReal64    triggerLevel,
    ViReal64    triggerOffset_s,
    ViInt32    typeofMeas,
    ViInt32    noofMeasurements,
    ViUInt32    timeout,
    ViReal64    burstPowerResults[]
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	timeout;
    if (analyzerFrequency_Hz<0.0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Analyzer Frequency [Hz]");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, resolutionBandwidth_Hz, 10.0, 10.0e+6), 4, "Resolution Bandwidth [Hz]");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, measTime_s, 1.0e-6, 30.0), 5, "Meas Time [s]");        
    if (!((triggerSource == RSEMI_VAL_TRG_EXT) || (triggerSource == RSEMI_VAL_TRG_VID)))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Trigger Source");
    viCheckParm (rsemi_invalidViReal64Range (instrSession, triggerLevel, 0.0, 100.0), 7, "Trigger Level");  
    viCheckParm (rsemi_invalidViReal64Range (instrSession, triggerOffset_s, 125.0e-9, 100.0), 8, "Trigger Offset [s]");  
    viCheckParm (rsemi_invalidViInt32Range (instrSession, typeofMeas, 0, 1), 9, "Type Of Meas");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, noofMeasurements, 1, 501), 10, "No Of Measurements");
    if (burstPowerResults == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 12, "Burst Power Results");        
        
    viCheckErr( viPrintf (instrSession, ":SENS%ld:MPOW:SEQ? %.12lf,%.12lf,%.12lf,%s,%.12lf,%.12lf,%s,%ld\n",
    window, analyzerFrequency_Hz,resolutionBandwidth_Hz,measTime_s,rsemi_rngTriggerSource.rangeValues[triggerSource].cmdString,
    triggerLevel,triggerOffset_s,measTypeArr[typeofMeas],noofMeasurements));
    
    viCheckErr( viScanf (instrSession, "%,#lf", &noofMeasurements, burstPowerResults));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the results of a multiple burst power 
/// HIFN  measurement as configured and initiated with Get Burst Power Result 
/// HIFN  (actBurstPwrResult). The unit used for the return values is always dBm. 
/// HIFN  This function may be used to obtain measurement results in an 
/// HIFN  asynchronous way using the service request mechanism for 
/// HIFN  synchronization with the end of the measurement. If no measurement 
/// HIFN  results are available, the function will return a query error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR noofResults/Expected number of values to be returned from the instrument. This 
/// HIPAR noofResults/value must be equal to the Burst Power results array number of items.
/// HIPAR burstPowerResults/This control returns array of elements as the results of a multiple 
/// HIPAR burstPowerResults/burst power measurement as configured and initiated with Get Burst 
/// HIPAR burstPowerResults/Power Result (actBurstPwrResult).
ViStatus _VI_FUNC rsemi_GetBurstPowerResults(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    noofResults,
    ViReal64    burstPowerResults[]
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (noofResults<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "No Of Results");
    if (burstPowerResults == NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "burstPowerResults");        
        
    viCheckErr( viQueryf (instrSession, "*WAI;SENS%ld:MPOW:RES?\n", "%,#lf",
    window, &noofResults, burstPowerResults));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the minimum power value in a multiple burst 
/// HIFN  power measurement as configured and initiated with Burst Power Sequence 
/// HIFN  (confBurstPwrSeq). The unit used for the return values is always dBm. 
/// HIFN  If no measurement result is available, the function will return a query 
/// HIFN  error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR powerResultMin/This control queries the minimum power value in a multiple burst power 
/// HIPAR powerResultMin/measurement as configured and initiated with Burst Power Sequence.
ViStatus _VI_FUNC rsemi_GetBurstPwrResultMin(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    powerResultMin
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_MPOW_MIN, 
                powerResultMin), 3, "Power Result Min");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches on or off the spurious emissions measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches on or off the spurious emissions measurement.
ViStatus _VI_FUNC rsemi_ConfigureSEState(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
       
    if (state == VI_TRUE)
    {
        viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "Win0",
                                                  RSEMI_ATTR_SWEEP_MODE,
                                                  RSEMI_VAL_SWE_MODE_LIST));
    }
    else
        viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "Win0",
                                                  RSEMI_ATTR_SWEEP_MODE,
                                                  RSEMI_VAL_SWE_MODE_AUTO));
    
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This configures how detailed the list in the Result Summary is.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR details/Configures how detailed the list in the Result Summary is.
ViStatus _VI_FUNC rsemi_ConfigureSEPeakSearchDetails (
    ViSession instrSession,
    ViInt32 window,
    ViBoolean details
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                  RSEMI_ATTR_SE_PEAK_SEARCH_DETAILS,
                                                  details), 3, "Details");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/// HIFN  This function configures the peak search for spurious emission 
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR margin/Defines an overall margin which is subtracted from the limit line to 
/// HIPAR margin/make the peak search.
/// HIPAR peaksPerRange/Defines how many peaks are searched for within one range.
ViStatus _VI_FUNC rsemi_ConfigureSEPeakSearch(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    margin,
    ViInt32    peaksPerRange
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                  RSEMI_ATTR_SE_PEAK_SEARCH_MARGIN,
                                                  margin), 3, "Margin");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                                  RSEMI_ATTR_SE_PEAK_SEARCH_PEAKS,
                                                  peaksPerRange), 4, "Peaks Per Range");
    
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the start and stop frequency for the 
/// HIFN  specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR startFrequency/Sets the start frequency of a range in the spurious measurement. 
/// HIPAR stopFrequency/Sets the stop frequency of a range in the spurious measurement.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListStartStop(
    ViSession    instrSession,
    ViInt32    range,
    ViReal64    startFrequency,
    ViReal64    stopFrequency
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_START, startFrequency),
                                                3, "Start Frequency");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_STOP, stopFrequency),
                                                4, "Stop Frequency");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the filter type for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR filter/This control sets the filter in the spurious measurement.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListFilterType(
    ViSession    instrSession,
    ViInt32    range,
    ViInt32    filter
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_FILT_TYPE, filter),
                                                3, "Filter");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the resolution and video bandwidth values for 
/// HIFN  the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR resolutionBandwidth/Sets the the resolution bandwidth.
/// HIPAR videoBandwidth/Sets the the video bandwidth.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListBandwidth(
    ViSession    instrSession,
    ViInt32    range,
    ViReal64    resolutionBandwidth,
    ViReal64    videoBandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_BAND, resolutionBandwidth),
                                                3, "Resolution Bandwidth");                                                        
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_BAND_VID, videoBandwidth),
                                                4, "Video Bandwidth");                                                  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the sweep time values for the specified 
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR sweepTimeAuto/Sets the automatic coupling of the sweep duration on the frequency 
/// HIPAR sweepTimeAuto/span and the bandwidth settings in a spurious measurement range.
/// HIPAR sweepTime/Sets the sweep duration of a range in the spurious measurement.
ViStatus _VI_FUNC rsemi_ConfigureSESweepSweepListTime(
    ViSession    instrSession,
    ViInt32    range,
    ViBoolean    sweepTimeAuto,
    ViReal64    sweepTime
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_SWE_TIME_AUTO, sweepTimeAuto),
                                                3, "Sweep Time Auto");                                                        
    if (sweepTimeAuto == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_SWE_TIME, sweepTime),
                                                4, "Sweep Time");                                                  
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the detector value for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR detector/This control sets the detector in the spurious measurement. The driver 
/// HIPAR detector/sets the RSEMI_ATTR_SE_LIST_RANG_DET to this value.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListDetector(
    ViSession    instrSession,
    ViInt32    range,
    ViInt32    detector
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_DET, detector),
                                                3, "Detector");                                                        
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the reference level for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR refLevelValue/Specifies the reference level of a range in the spurious measurement.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListReferenceLevel(
    ViSession    instrSession,
    ViInt32    range,
    ViReal64    refLevelValue
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_RLEV, refLevelValue),
                                                3, "Ref Level Value");                                                        
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the RF attenuation settings for the specified 
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR attenuatorAuto/Sets the autoranging of a range in the spurious measurement on or 
/// HIPAR attenuatorAuto/off.. The driver sets the RSEMI_ATTR_SE_LIST_RANG_INP_ATT_AUTO to this 
/// HIPAR attenuatorAuto/value.
/// HIPAR manualAttenuatorValue_dB/This control sets the RF attenuation of a range.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListRFAttenuation(
    ViSession    instrSession,
    ViInt32    range,
    ViBoolean    attenuatorAuto,
    ViReal64    manualAttenuatorValue_dB
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_INP_ATT_AUTO, attenuatorAuto),
                                                3, "Attenuator Auto");                                                        
    if (attenuatorAuto == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_INP_ATT, manualAttenuatorValue_dB),
                                                4, "Manual Attenuator Value dB");                                                  
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures state of pre-amplifier for selected sweep 
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR preamplifier/Switches the preamplifier of a range on and off. The driver sets the 
/// HIPAR preamplifier/RSEMI_ATTR_LIST_RANG_INP_GAIN_STAT to this value.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListPreamplifier(
    ViSession    instrSession,
    ViInt32    range,
    ViBoolean    preamplifier
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    
    sprintf (buffer, "R%ld", range);

    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_INP_GAIN_STAT, preamplifier),
                                                3, "Preamplifier");  
                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures level of pre-amplifier for selected sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR preamplifierLevel/Configures level of pre-amplifier for selected sweep range.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListPreamplifierLevel (
    ViSession instrSession,
    ViInt32 range,
    ViInt32 preamplifierLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_INP_GAIN_LEVEL, preamplifierLevel),
                                                3, "Preamplifier Level");                                                        
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures state of LNA pre-amplifier for selected sweep 
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR preamplifier/Switches the LNA preamplifier of a range on and off.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListLNAPreamplifier (
    ViSession instrSession,
    ViInt32 range,
    ViBoolean preamplifier
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    
    sprintf (buffer, "R%ld", range);

    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANGE_INP_GAIN_LNA_STAT, preamplifier),
                                                3, "Preamplifier");  
                                                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the sweep settings for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR sweepPoints/Sets the number of sweep points of a range in the spurious 
/// HIPAR sweepPoints/measurement. The driver sets the RSEMI_ATTR_SE_LIST_RANG_POINTS to this 
/// HIPAR sweepPoints/value.
/// HIPAR stopAfterSweep/Determines whether a sweep is stopped in the spurious measurement when 
/// HIPAR stopAfterSweep/a range is changed. The driver sets the    
/// HIPAR stopAfterSweep/RSEMI_ATTR_SE_LIST_RANG_BREAK to this value.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListSweep(
    ViSession    instrSession,
    ViInt32    range,
    ViInt32    sweepPoints,
    ViBoolean    stopAfterSweep
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_POINTS, sweepPoints),
                                                3, "Sweep Points");                                                  
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_BREAK, stopAfterSweep),
                                                4, "Stop After Sweep");                                                        
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures a transducer for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR transducerFactor/Sets the transducer factor for a range in the spurious measurement.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListTransducerFactor(
    ViSession    instrSession,
    ViInt32    range,
    ViString    transducerFactor
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    if (strlen(transducerFactor)>0)
        viCheckParm( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_TRAN, transducerFactor),
                                                3, "Transducer Factor");   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the limits for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR limitChecking/Activates/deactivates limit checking of spurious emission measurement 
/// HIPAR limitChecking/LIST EVALUATION.
/// HIPAR limit/This control defines the limit of spurious emission measurement LIST 
/// HIPAR limit/EVALUATION.
/// HIPAR absoluteLimitStart/This control sets an absolute limit value at the start frequency of 
/// HIPAR absoluteLimitStart/the specified range. Different from manual operation, this setting is 
/// HIPAR absoluteLimitStart/independently of the defined limit check type. The driver uses this 
/// HIPAR absoluteLimitStart/value to set the RSEMI_ATTR_SE_LIST_RANG_LIMIT_START attribute.
/// HIPAR absoluteLimitStop/This control sets an absolute limit value at the stop frequency of the 
/// HIPAR absoluteLimitStop/specified range. Different from manual operation, this setting is 
/// HIPAR absoluteLimitStop/independently of the defined limit check type.
ViStatus _VI_FUNC rsemi_ConfigureSESweepListLimits(
    ViSession    instrSession,
    ViInt32    range,
    ViBoolean    limitChecking,
    ViReal64    limit,
    ViReal64    absoluteLimitStart,
    ViReal64    absoluteLimitStop
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_LIMIT_STATE, limitChecking),
                                                3, "Limit Checking");                                                 
    if (limitChecking == VI_TRUE)                                               
    {
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_LIMIT_START, absoluteLimitStart),
                                                5, "Absolute Limit Start");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_LIMIT_STOP, absoluteLimitStop),
                                                6, "Absolute Limit Stop");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SE_LIST_RANG_LIMIT, limit),
                                                4, "Limit");
        
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function deletes a range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/This control specifies the range that will be deleted.
ViStatus _VI_FUNC rsemi_SEDeleteRange(
    ViSession    instrSession,
    ViInt32    range
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 2, "Range");
    sprintf (buffer, "R%ld", range);
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                                               RSEMI_ATTR_SE_LIST_RANG_DEL, VI_NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the number of defined ranges of the spurious 
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfRanges/Returns indicates the number of defined ranges of the spurious 
/// HIPAR numberOfRanges/measurement. The driver sets the RSEMI_ATTR_LIST_RANG_COUNT to this 
/// HIPAR numberOfRanges/value.
ViStatus _VI_FUNC rsemi_GetSENumberOfListRanges(
    ViSession    instrSession,
    ViInt32*    numberOfRanges
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SE_LIST_RANG_COUNT, numberOfRanges),
                                               2, "Number Of Ranges");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function searches the peaks in spurious emissions data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_SESearchPeak(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "Win1",
                                                  RSEMI_ATTR_SE_PEAK_SEARCH,
                                                  VI_NULL));
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function stores the peak list of selected trace in ASCII format 
/// HIFN  to a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file, where spurios emissions data will be 
/// HIPAR fileName/stored.
ViStatus _VI_FUNC rsemi_StoreSEToFile(
    ViSession    instrSession,
    ViString    fileName
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "Win1",
                                               RSEMI_ATTR_SE_STORE_FILE, fileName),
                                               3, "File Name");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the spurious emissions peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR frequency/Returns an array of frequency list in Hz.
/// HIPAR level/Returns an array of level list in dBm.
/// HIPAR deltaLimit/Returns an array of delta to limit line values in dB. A delta limit of 
/// HIPAR deltaLimit/+200dB indicates no limit check is active.
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to limit 
/// HIPAR returnedValues/line) read (available to return) from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter No of Values.
ViStatus _VI_FUNC rsemi_ReadSEPeakList(
    ViSession    instrSession,
    ViInt32    noOfValues,
    ViReal64    frequency[],
    ViReal64    level[],
    ViReal64    deltaLimit[],
    ViInt32*    returnedValues
)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     arrayLength = 0;
    ViReal64    *data = VI_NULL;
    ViInt32     retVal = 0;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
       
    arrayLength = 3*noOfValues;
    data = (ViReal64 *) malloc (sizeof(ViReal64)*arrayLength);
    if (data == NULL)
        return VI_ERROR_ALLOC;
    
   

    viCheckErr (rsemi_dataReadTrace (instrSession, 1, "SPUR", arrayLength, 
                    data, &retVal));
    
    if (returnedValues)
        *returnedValues = retVal / 3; /* Three result arrays */    
    
    retVal = (retVal>arrayLength)?arrayLength:retVal;
    
    retVal /=3;
    
    for (i = 0; i < retVal; i++)
    {
        frequency[i] = data[i*3];
        level[i] = data[i*3+1];
        deltaLimit[i] = data[i*3+2];
    }   
    
        
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    if (data) free(data);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function activates or deactivates spurious emission measurement 
/// HIFN  LIST EVALUATION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches on or off the spurious emissions measurement.
ViStatus _VI_FUNC rsemi_SEListEvaluation(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "",
                                               RSEMI_ATTR_SE_LIST_EVAL_STATE, state),
                                               2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the resulting peak list of the spurious emission 
/// HIFN  measurement LIST EVALUATION
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR rangeNumber/Returns an array of range numbers
/// HIPAR startFrequency/Returns an array of range start frequency
/// HIPAR stopFrequency/Returns an array of range stop frequency.
/// HIPAR resolutionBandwidth/Returns an array of resolution bandwidth
/// HIPAR frequency/Returns an array of rrequency of the peak in the range
/// HIPAR levelAbs/Returns an array of absolute peak power of the range in dBm
/// HIPAR levelRel/Returns an array of relative peak power of the range in dBm
/// HIPAR delta/Returns an array of delta of the peak power to the limit line in dB
/// HIPAR limitCheck/Returns an array of limit check state (0 = PASSED, 1 = FAILED
/// HIPAR reserved1/Reserved (0.0)
/// HIPAR reserved2/Reserved (0.0)
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to limit 
/// HIPAR returnedValues/line) read (available to return) from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter No of Values.
ViStatus _VI_FUNC rsemi_ReadSEListEvaluation(
    ViSession    instrSession,
    ViInt32    noOfValues,
    ViReal64    rangeNumber[],
    ViReal64    startFrequency[],
    ViReal64    stopFrequency[],
    ViReal64    resolutionBandwidth[],
    ViReal64    frequency[],
    ViReal64    levelAbs[],
    ViReal64    levelRel[],
    ViReal64    delta[],
    ViReal64    limitCheck[],
    ViReal64    reserved1[],
    ViReal64    reserved2[],
    ViInt32*    returnedValues
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuf=VI_NULL;
    ViInt32     count=0;
    ViUInt32    retCnt;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rangeNumber == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Range number");
    if (startFrequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Start Frequency"); 
    if (stopFrequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Stop Frequency"); 
    if (resolutionBandwidth == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Resolution Bandwidth"); 
    if (frequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Frequency"); 
    if (levelAbs == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Level Abs");
    if (levelRel == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Level Rel"); 
    if (delta == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Delta"); 
    if (limitCheck == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Limit Check");
    if (reserved1 == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Reserved 1");
    if (reserved2 == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 12, "Reserved 2"); 
     
    viCheckErr (viPrintf (instrSession, "TRAC:DATA? LIST\n"));
    viCheckErr( Rs_ReadDataUnknownLength(instrSession, &buffer, &retCnt));
    
    pbuf=strtok(buffer, ",");
    if (pbuf)
        do{
            if (count<noOfValues)
            {
                sscanf (pbuf, "%le", &rangeNumber[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &startFrequency[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &stopFrequency[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &resolutionBandwidth[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &frequency[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &levelAbs[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &levelRel[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &delta[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &limitCheck[count]);
                pbuf=strtok(NULL, ",");
                sscanf (pbuf, "%le", &reserved1[count]);
                pbuf=strtok(NULL, ","); 
                sscanf (pbuf, "%le", &reserved2[count]);
                pbuf=strtok(NULL, ","); 
            }               
            else
            {
                for (i=0;i<11;i++)
                    pbuf=strtok(NULL, ","); 
            }       
            count++;
        }while (pbuf);  

    if (returnedValues)
        *returnedValues = count;
     
    checkErr( rsemi_CheckStatus (instrSession));     
Error:
    if (buffer) free(buffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function changes from Spectrum Analyzer to Spectrum Emission Mask 
/// HIFN  measurement mode and back.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches on or off the spectrum emission mask measurement.
ViStatus _VI_FUNC rsemi_ConfigureSEMState(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    if (state == VI_TRUE)
    {
        viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "Win0",
                                                  RSEMI_ATTR_SWEEP_MODE,
                                                  RSEMI_VAL_SWE_MODE_ESP));
    }
    else
        viCheckErr( rsemi_SetAttributeViInt32 (instrSession, "Win0",
                                                  RSEMI_ATTR_SWEEP_MODE,
                                                  RSEMI_VAL_SWE_MODE_AUTO));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the Spectrum EM Mask Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR limitLineType/This control selects the Limit Line Type. The driver uses this value 
/// HIPAR limitLineType/to set the RSEMI_ATTR_ESP_MODE attribute.
/// HIPAR limitLineValue/This control sets the Limit Line Value. The driver uses this value to 
/// HIPAR limitLineValue/set the RSEMI_ATTR_ESP_MAN_VAL attribute.
ViStatus _VI_FUNC rsemi_ConfigureSEM(
    ViSession    instrSession,
    ViInt32    limitLineType,
    ViReal64    limitLineValue
)
{
    ViStatus    error   = VI_SUCCESS;


    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_ESP_MODE,
                 limitLineType), 2, "Limit Line Type");

    if (limitLineType == RSEMI_VAL_ESP_MODE_MAN)
    {
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", RSEMI_ATTR_ESP_MAN_VAL,
                     limitLineValue), 3, "Limit Line Value");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the peak search for spectrum emission mask 
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR margin/Defines an overall margin which is subtracted from the limit line to 
/// HIPAR margin/make the peak search.
/// HIPAR peaksPerRange/Defines how many peaks are searched for within one range.
ViStatus _VI_FUNC rsemi_ConfigureSEMPeakSearch(
    ViSession    instrSession,
    ViReal64    margin,
    ViInt32    peaksPerRange
)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "",
                                                  RSEMI_ATTR_SEM_PEAK_SEARCH_MARGIN,
                                                  margin), 2, "Margin");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                                  RSEMI_ATTR_SEM_PEAK_SEARCH_PEAKS,
                                                  peaksPerRange), 3, "Peaks Per Range");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function restores the standard limit lines for the spectrum 
/// HIFN  emission mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_RestoreSEMStandardLines(
    ViSession    instrSession
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_ESP_RESTORE,
                 NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function searches the peaks in spectrum emission data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_SEMSearchPeak(
    ViSession    instrSession
)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rsemi_SetAttributeViString (instrSession, "Win1", RSEMI_ATTR_SEM_PEAK_SEARCH,
                 NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function activates or deactivates spectrum emission mask  LIST 
/// HIFN  EVALUATION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches on or off the spectrum emission mask measurement.
ViStatus _VI_FUNC rsemi_SEMListEvaluation(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "",
                                               RSEMI_ATTR_SEM_LIST_EVAL_STATE, state),
                                               2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the list evaluation results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR rangeNumber/Returns an array of range numbers
/// HIPAR startFrequency/Returns an array of range start frequency
/// HIPAR stopFrequency/Returns an array of range stop frequency.
/// HIPAR resolutionBandwidth/Returns an array of resolution bandwidth
/// HIPAR frequency/Returns an array of rrequency of the peak in the range
/// HIPAR powerAbs/Returns an array of absolute power in dBm of peak.
/// HIPAR powerRel/Returns the relative power in dBc (related to the channel
/// HIPAR delta/Returns an array of delta of the peak power to the limit line in dB
/// HIPAR limitCheck/Returns an array of limit check state (0 = PASSED, 1 = FAILED
/// HIPAR reserved1/Reserved (0.0)
/// HIPAR reserved2/Reserved (0.0)
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to limit 
/// HIPAR returnedValues/line) read (available to return) from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter No of Values.
ViStatus _VI_FUNC rsemi_ReadSEMListEvaluationResults(
    ViSession    instrSession,
    ViInt32    noOfValues,
    ViReal64    rangeNumber[],
    ViReal64    startFrequency[],
    ViReal64    stopFrequency[],
    ViReal64    resolutionBandwidth[],
    ViReal64    frequency[],
    ViReal64    powerAbs[],
    ViReal64    powerRel[],
    ViReal64    delta[],
    ViReal64    limitCheck[],
    ViReal64    reserved1[],
    ViReal64    reserved2[],
    ViInt32*    returnedValues
)
{
    ViStatus    error   = VI_SUCCESS;
    ViReal64    *pArray=VI_NULL;
    ViInt32     count=0;
    ViInt32     retCnt;
    ViInt32     i;
    ViInt32     values = noOfValues * 11;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (rangeNumber == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Range number");
    if (startFrequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Start Frequency"); 
    if (stopFrequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Stop Frequency"); 
    if (resolutionBandwidth == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Resolution Bandwidth"); 
    if (frequency == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Frequency"); 
    if (powerAbs == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Power Abs");
    if (powerRel == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 8, "Power Rel"); 
    if (delta == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 9, "Delta"); 
    if (limitCheck == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 10, "Limit Check");
    if (reserved1 == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 11, "Reserved 1");
    if (reserved2 == VI_NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 12, "Reserved 2"); 
     
    viCheckErr (viPrintf (instrSession, "TRAC:DATA? LIST\n"));
    pArray = malloc (values*sizeof (ViReal64));
    viCheckErr (rsemi_dataReadTraceOnly (instrSession, values, pArray, &retCnt));
    values = (retCnt>values)?values:retCnt;
    i=0;
    while (count<values)
    {
        rangeNumber[i]=pArray[count];
        count++;
        startFrequency[i]=pArray[count];
        count++;
        stopFrequency[i]=pArray[count];
        count++;
        resolutionBandwidth[i]=pArray[count];
        count++;
        frequency[i]=pArray[count];
        count++;
        powerAbs[i]=pArray[count];
        count++;
        powerRel[i]=pArray[count];
        count++;
        delta[i]=pArray[count];
        count++;
        limitCheck[i]=pArray[count];
        count++;
        reserved1[i]=pArray[count];
        count++;
        reserved2[i]=pArray[count];
        count++;
        i++;
    }             
    if (returnedValues)
        *returnedValues = retCnt/11;
    checkErr( rsemi_CheckStatus (instrSession));     
Error:
    if (pArray) free(pArray);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function marks all peaks with blue squares in the diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Marks all peaks with blue squares in the diagram. 
ViStatus _VI_FUNC rsemi_SEMMarkAllPeaks(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "",
                                               RSEMI_ATTR_SEM_MARK_ALL_PEAKS, state),
                                               2, "State");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the reference range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR powerReferenceType/This control sets the power reference type.
/// HIPAR txBandwidth/This control defines the bandwidth used for measuring the channel 
/// HIPAR txBandwidth/power (reference range). The driver uses this value to set the 
/// HIPAR txBandwidth/RSEMI_ATTR_SEM_BANDWIDTH attribute.
/// HIPAR rrcFilterState/This control activates or deactivates the use of an RRC filter. The 
/// HIPAR rrcFilterState/driver uses this value to set the RSEMI_ATTR_SEM_RRC_FILTER_STATE 
/// HIPAR rrcFilterState/attribute.
/// HIPAR rrcFilterAlpha/This control sets the alpha value of the RRC filter.
ViStatus _VI_FUNC rsemi_ConfigureSEMReferenceRange(
    ViSession    instrSession,
    ViInt32    powerReferenceType,
    ViReal64    txBandwidth,
    ViBoolean    rrcFilterState,
    ViReal64    rrcFilterAlpha
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SEM_POWER_REF_TYPE, powerReferenceType),
                                               2, "Power Reference Type");
    
    if (powerReferenceType == RSEMI_VAL_SEM_REF_TYPE_CPOW)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "",
                                                   RSEMI_ATTR_SEM_BANDWIDTH, txBandwidth),
                                                   3, "TX Bandwidth");
    
        viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "",
                                                   RSEMI_ATTR_SEM_RRC_FILTER_STATE, rrcFilterState),
                                                   4, "RRC Filter State");
    
        if (rrcFilterState == VI_TRUE)
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "",
                                                       RSEMI_ATTR_SEM_RRC_FILTER_ALPHA_VALUE, rrcFilterAlpha),
                                                       5, "RRC Filter Alpha");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the current position (number) of the reference 
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceRangePosition/This control returns the current position (number) of the reference 
/// HIPAR referenceRangePosition/range.
ViStatus _VI_FUNC rsemi_QuerySEMReferenceRangePosition(
    ViSession    instrSession,
    ViInt32*    referenceRangePosition
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SEM_REF_RANGE_POSITION, referenceRangePosition),
                                               2, "Reference Range Position");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function activates Fast SEM mode to accelerate spectrum emission 
/// HIFN  mask measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR fastSEM/This control activates Fast SEM mode to accelerate spectrum emission 
/// HIPAR fastSEM/mask measurements.
ViStatus _VI_FUNC rsemi_ConfigureFastSEM(
    ViSession    instrSession,
    ViBoolean    fastSEM
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "",
                                               RSEMI_ATTR_SEM_FAST_SEM, fastSEM),
                                               2, "Fast SEM");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function selects the specified XML file under 
/// HIFN  C:\r_s\instr\sem_std. If the file is stored in a subdirectory, include 
/// HIFN  the relative path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR presetStandard/This control selects the specified XML file The driver uses this value 
/// HIPAR presetStandard/to set the RSEMI_ATTR_SEM_PRESET_STANDARD attribute.
ViStatus _VI_FUNC rsemi_ConfigureSEMPresetStandard(
    ViSession    instrSession,
    ViString    presetStandard
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "",
                                               RSEMI_ATTR_SEM_PRESET_STANDARD, presetStandard),
                                               2, "Preset Standard");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function copies the XML files from the C:\R_S\instr\sem_backup 
/// HIFN  folder to the C:\R_S\instr\sem_std folder. Files of the same name are 
/// HIFN  overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_RestoreSEMStandardFiles(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "",
                                               RSEMI_ATTR_SEM_RESTORE_STANDARD, NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function stores the current settings as presettings in the 
/// HIFN  specified XML file under C:\r_s\instr\sem_backup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR settingsFileName/This control specifies XML file that stores the current settings as 
/// HIPAR settingsFileName/presettings.
ViStatus _VI_FUNC rsemi_StoreSEMPresetSettings(
    ViSession    instrSession,
    ViString    settingsFileName
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "",
                                               RSEMI_ATTR_SEM_STORE_PRESET, settingsFileName),
                                               2, "Settings File Name");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function stores the current list evaluation results in a *.dat 
/// HIFN  file. The file consists of a data section containing the list 
/// HIFN  evaluation results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/This control specifies the name.
ViStatus _VI_FUNC rsemi_SEMSaveListToFile(
    ViSession    instrSession,
    ViString    fileName
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "",
                                               RSEMI_ATTR_SEM_LIST_FILE_EXPORT, fileName),
                                               2, "File Name");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the start and stop frequency for the 
/// HIFN  specified range. In order to change the start/stop frequency of the 
/// HIFN  first/last range, select the appropriate span. If you set a span that 
/// HIFN  is smaller than the overall span of the ranges, the measurement 
/// HIFN  includes only the ranges that lie within the defined span and have a 
/// HIFN  minimum span of 20 Hz. The first and last range are adapted to the 
/// HIFN  given span as long as the minimum span of 20 Hz is not violated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR rangeStart/This control sets the start frequency for the specified range.
/// HIPAR rangeStop/This control sets the stop frequency for the specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMRangeStartStop(
    ViSession    instrSession,
    ViInt32    range,
    ViReal64    rangeStart,
    ViReal64    rangeStop
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_FREQ_START, rangeStart),
                                                3, "Range Start");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_FREQ_STOP, rangeStop),
                                                4, "Range Stop");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the filter type for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR filterType/This control sets the filter type for the specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMRangeFilterType(
    ViSession    instrSession,
    ViInt32    range,
    ViInt32    filterType
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_FILTER_TYPE, filterType),
                                                3, "Filter Type");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the resolution and video bandwidth values for 
/// HIFN  the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR resolutionBandwidth/This control sets the RBW value for the specified range. 
/// HIPAR videoBandwidth/This control sets the VBW value for the specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMRangeBandwidth(
    ViSession    instrSession,
    ViInt32    range,
    ViReal64    resolutionBandwidth,
    ViReal64    videoBandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_BANDWIDTH, resolutionBandwidth),
                                                3, "Resolution Bandwidth");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_VBW, videoBandwidth),
                                                4, "Video Bandwidth");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the sweep time values for the specified 
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR sweepTimeMode/This control activates or deactivates the automatic sweep time setting 
/// HIPAR sweepTimeMode/for the specified range. The driver uses this value to set the 
/// HIPAR sweepTimeMode/RSEMI_ATTR_SEM_RANGE_SWEEP_TIME_AUTO attribute.
/// HIPAR sweepTime/This control sets the sweep time for the specified range. Allowed 
/// HIPAR sweepTime/values depend on the ratio of span to RBW and RBW to VBW. The driver 
/// HIPAR sweepTime/uses this value to set the RSEMI_ATTR_SEM_RANGE_SWEEP_TIME attribute.
ViStatus _VI_FUNC rsemi_ConfigureSEMRangeSweepTime(
    ViSession    instrSession,
    ViInt32    range,
    ViBoolean    sweepTimeMode,
    ViReal64    sweepTime
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_SWEEP_TIME_AUTO, sweepTimeMode),
                                                3, "Sweep Time Mode");
    
    if (sweepTimeMode == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                    RSEMI_ATTR_SEM_RANGE_SWEEP_TIME, sweepTime),
                                                    4, "Sweep Time");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the reference level for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR referenceLevel/This control sets the reference level for the specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMRangeReferenceLevel(
    ViSession    instrSession,
    ViInt32    range,
    ViReal64    referenceLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_REF_LEVEL, referenceLevel),
                                                3, "Reference Level");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the RF attenuation setting for the specified 
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR rfAttenuationMode/This control activates or deactivates the automatic RF attenuation 
/// HIPAR rfAttenuationMode/setting for the specified range.
/// HIPAR rfAttenuator/This control sets the attenuation for the specified range.
/// HIPAR preamplifiier/This control switches the preamplifier on or off for the specified 
/// HIPAR preamplifiier/range.
ViStatus _VI_FUNC rsemi_ConfigureSEMRangeRFAttenuation(
    ViSession    instrSession,
    ViInt32    range,
    ViBoolean    rfAttenuationMode,
    ViReal64    rfAttenuator,
    ViBoolean    preamplifiier
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_INPUT_ATTENUATION_AUTO, rfAttenuationMode),
                                                3, "RF Attenuation Mode");
    if (rfAttenuationMode == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                    RSEMI_ATTR_SEM_RANGE_INPUT_ATTENUATION, rfAttenuator),
                                                    4, "RF Attenuator");
    }
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_PREAMPLIFIER, preamplifiier),
                                                5, "Preamplifiier");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures state of LNA pre-amplifier for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR preamplifier/Switches the LNA preamplifier of a range on and off.
ViStatus _VI_FUNC rsemi_ConfigureSEMRangeLNAPreamplifier (
    ViSession instrSession,
    ViInt32 range,
    ViBoolean preamplifier
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_LNA_PREAMPLIFIER, preamplifier),
                                                3, "Preamplifier");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures a transducer for the specified range. You can 
/// HIFN  only choose a transducer that fulfills the following conditions:
/// HIFN     - The transducer overlaps or equals the span of the range.
/// HIFN     - The x-axis is linear.
/// HIFN     - The unit is dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR transducerFactor/This control sets a transducer for the specified range. The driver 
/// HIPAR transducerFactor/uses this value to set the RSEMI_ATTR_SEM_RANGE_TRANSDUCER attribute.
ViStatus _VI_FUNC rsemi_ConfigureSEMRangeTransducerFactor(
    ViSession    instrSession,
    ViInt32    range,
    ViString    transducerFactor
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_TRANSDUCER, transducerFactor),
                                                3, "Transducer Factor");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the limit setting for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR source/Selects the source.
/// HIPAR limitCheckType/This control sets the type of limit check for all ranges.
/// HIPAR absoluteLimitStart/This control sets an absolute limit value at the start frequency of 
/// HIPAR absoluteLimitStart/the specified range. Different from manual operation, this setting is 
/// HIPAR absoluteLimitStart/independently of the defined limit check type.
/// HIPAR absoluteLimitStop/This control sets an absolute limit value at the stop frequency of the 
/// HIPAR absoluteLimitStop/specified range. Different from manual operation, this setting is 
/// HIPAR absoluteLimitStop/independently of the defined limit check type.
/// HIPAR relativeLimitStart/This control sets a relative limit value at the start frequency of the 
/// HIPAR relativeLimitStart/specified range. Different from manual operation, this setting is 
/// HIPAR relativeLimitStart/independently of the defined limit check type. The driver uses this 
/// HIPAR relativeLimitStart/value to set the RSEMI_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT attribute.
/// HIPAR relativeLimitStop/This control sets a relative limit value at the stop frequency of the 
/// HIPAR relativeLimitStop/specified range. Different from manual operation, this setting is 
/// HIPAR relativeLimitStop/independently of the defined limit check type. The driver uses this 
/// HIPAR relativeLimitStop/value to set the RSEMI_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT attribute.
ViStatus _VI_FUNC rsemi_ConfigureSEMRangeLimits(
    ViSession    instrSession,
    ViInt32    range,
    ViInt32    source,
    ViInt32    limitCheckType,
    ViReal64    absoluteLimitStart,
    ViReal64    absoluteLimitStop,
    ViReal64    relativeLimitStart,
    ViReal64    relativeLimitStop
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      rep_cap[25] = ""; 
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, source, 1, 4), 3, "Source");
    
    sprintf (rep_cap, "PC%ld", source);
    
    sprintf (buffer, "R%ld,PC%ld", range, source);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSEMI_ATTR_SEM_RANGE_LIMIT_CHECK_TYPE, limitCheckType),
                                                4, "Limit Check Type");
    
    switch (limitCheckType)
    {
        case RSEMI_VAL_SEM_LIM_CHECK_TYPE_ABS:
        {
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_START_FREQ_ABS_LIMIT, absoluteLimitStart),
                                                5, "Absolute Limit Start");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_STOP_FREQ_ABS_LIMIT, absoluteLimitStop),
                                                6, "Absolute Limit Stop");
            break;
        }
        case RSEMI_VAL_SEM_LIM_CHECK_TYPE_REL:
        {
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT, relativeLimitStart),
                                                7, "Relative Limit Start");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT, relativeLimitStop),
                                                8, "Relative Limit Stop");
            break;
        }
        case RSEMI_VAL_SEM_LIM_CHECK_TYPE_AND:
        case RSEMI_VAL_SEM_LIM_CHECK_TYPE_OR:
        {
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_START_FREQ_ABS_LIMIT, absoluteLimitStart),
                                                5, "Absolute Limit Start");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_STOP_FREQ_ABS_LIMIT, absoluteLimitStop),
                                                6, "Absolute Limit Stop");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT, relativeLimitStart),
                                                7, "Relative Limit Start");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT, relativeLimitStop),
                                                8, "Relative Limit Stop");
            break;
        }
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Type");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns the number of defined ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR rangeCount/Returns the number of defined ranges.
ViStatus _VI_FUNC rsemi_GetSEMRangeCount(
    ViSession    instrSession,
    ViInt32*    rangeCount
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_SEM_RANGE_COUNT, rangeCount),
                                                2, "Range Count");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function inserts a new range before or after the specified range. 
/// HIFN  The range numbers are updated accordingly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
/// HIPAR insertRange/This control inserts a new range before or after the specified range. 
/// HIPAR insertRange/The range numbers are updated accordingly.
ViStatus _VI_FUNC rsemi_SEMInsertNewRange(
    ViSession    instrSession,
    ViInt32    range,
    ViInt32    insertRange
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_INSERT_NEW_RANGE, insertRange),
                                                3, "Insert Range");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function deletes the specified range. The range numbers are 
/// HIFN  updated accordingly. The reference range cannot be deleted. A minimum 
/// HIFN  of three ranges is mandatory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects the range.
ViStatus _VI_FUNC rsemi_SEMDeleteRange(
    ViSession    instrSession,
    ViInt32    range
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 20), 2, "Range");
    sprintf (buffer, "R%ld", range);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_DELETE_RANGE, NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the power classes used in the spectrum emission 
/// HIFN  mask measurement. It is only possible to use power classes for which 
/// HIFN  limits are defined. Also, either only one power class at a time or all 
/// HIFN  power classes together can be selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerClassNumber/This control selects the number of power class to be modified.
/// HIPAR usedPowerClass/This control sets the power classes used in the spectrum emission mask 
/// HIPAR usedPowerClass/measurement. 
ViStatus _VI_FUNC rsemi_ConfigureSEMUsedPowerClasses(
    ViSession    instrSession,
    ViInt32    powerClassNumber,
    ViInt32    usedPowerClass
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, powerClassNumber, 1, 4), 2, "Power Class Number");
    
    sprintf (buffer, "PC%ld", powerClassNumber);
    
    if (usedPowerClass == RSEMI_VAL_ALL)
    {
        viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "PC1", 
                                                    RSEMI_ATTR_SEM_POWER_CLASS_USED, 
                                                    VI_FALSE)); 
        viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "PC2", 
                                                    RSEMI_ATTR_SEM_POWER_CLASS_USED, 
                                                    VI_FALSE)); 
        viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "PC3", 
                                                    RSEMI_ATTR_SEM_POWER_CLASS_USED, 
                                                    VI_FALSE)); 
        viCheckErr( rsemi_SetAttributeViBoolean (instrSession, "PC4", 
                                                    RSEMI_ATTR_SEM_POWER_CLASS_USED, 
                                                    VI_FALSE)); 
        
    }
    else
    {
        viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                    RSEMI_ATTR_SEM_POWER_CLASS_USED, 
                                                    (ViBoolean) usedPowerClass), 3, "Used Power Class");
        
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets the number of power classes to be defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR count/This control sets the number of power classes to be defined.
ViStatus _VI_FUNC rsemi_ConfigureSEMPowerClassCount(
    ViSession    instrSession,
    ViInt32    count
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_SEM_POWER_CLASS_COUNT, 
                                                count), 2, "Count");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the power class settings of the Spectrum 
/// HIFN  Emission Mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerClassNumber/This control selects the number of power class to be modified.
/// HIPAR limitState/This control defines which limits are evaluated in the measurement.
/// HIPAR lowerLevelLimit/This control sets the minimum lower level limit for one power class. 
/// HIPAR lowerLevelLimit/The limit always start at -200 dBm, i.e. the first lower limit can not 
/// HIPAR lowerLevelLimit/be set. If more than one power class is in use, the lower limit must 
/// HIPAR lowerLevelLimit/equal the upper limit of the previous power class.
/// HIPAR upperLevelLimit/This control sets the upper limit level for one power class. The limit 
/// HIPAR upperLevelLimit/always ends at 200 dBm, i.e. the upper limit of the last power class 
/// HIPAR upperLevelLimit/can not be set. If more than one power class is in use, the upper limit 
/// HIPAR upperLevelLimit/must equal the lower limit of the next power class.
ViStatus _VI_FUNC rsemi_ConfigureSEMPowerClassLimits(
    ViSession    instrSession,
    ViInt32    powerClassNumber,
    ViInt32    limitState,
    ViReal64    lowerLevelLimit,
    ViReal64    upperLevelLimit
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, powerClassNumber, 1, 4), 2, "Power Class Number");
    
    sprintf (buffer, "PC%ld", powerClassNumber);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_POWER_CLASS_LIMIT_STATE, 
                                                limitState), 3, "Limit State");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_POWER_CLASS_LOWER_LEVEL_LIMIT, 
                                                lowerLevelLimit), 4, "Lower Level Limit");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_POWER_CLASS_UPPER_LEVEL_LIMIT, 
                                                upperLevelLimit), 5, "Upper Level Limit");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function sets up to 4 power classes in one step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfLimits/This control sets the number of power class limits.
/// HIPAR limits/This control sets up to 4 power classes in one step.
ViStatus _VI_FUNC rsemi_ConfigureSEMPowerClassAllLimits(
    ViSession    instrSession,
    ViInt32    numberOfLimits,
    ViReal64    limits[]
)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      *pbuffer;
    ViChar      *p2buf;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    pbuffer = (ViChar*) malloc (150 * numberOfLimits + 30);

    p2buf = pbuffer + sprintf (pbuffer, "CALC:LIM:ESP:LIM ");
    
    for (i = 0; i < numberOfLimits; i++)
        p2buf += sprintf (p2buf, "%lf,", limits[i]);
    
    *--p2buf = '\0';
    
    viCheckErr (viPrintf (instrSession, "%s\n", pbuffer));
    
    free (pbuffer);
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function returns up to 4 power classes in one step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limits/This control returns up to 4 power classes in one step.
ViStatus _VI_FUNC rsemi_QuerySEMPowerClassAllLimits(
    ViSession    instrSession,
    ViReal64    limits[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      *p2buff;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_QueryViString (instrSession, 
                                        "CALC:LIM:ESP:LIM?", 
                                        RSEMI_IO_BUFFER_SIZE, 
                                        buffer));

    p2buff = strtok (buffer, ",");
    
    while (p2buff)
    {
        limits[i++] = atof (p2buff);
        p2buff = strtok (NULL, ",");
    }
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects a transducer set and turns that transducer set on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transducerSetName/This control sets the name of the transducer set. If the name
/// HIPAR transducerSetName/does not exist yet, the R&S  ESR creates a transducer set by
/// HIPAR transducerSetName/that name.
/// HIPAR transducerSetState/This control switches the selected transducer set on or off.
ViStatus _VI_FUNC rsemi_SelectTransducerSet (ViSession instrSession,
                                             ViString transducerSetName,
                                             ViBoolean transducerSetState)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_TSET_SELECT, transducerSetName),
                2, "Transducer Set Name");                

    viCheckParm ( rsemi_SetAttributeViBoolean (instrSession, "", 
                RSEMI_ATTR_TSET_STATE, transducerSetState),
                3, "Transducer Set State");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects a transducer set and turns that transducer set on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transducerSetName/This control sets the name of the transducer set. If the name
/// HIPAR transducerSetName/does not exist yet, the R&S  ESR creates a transducer set by
/// HIPAR transducerSetName/that name.
/// HIPAR comment/Defines a comment for a transducer set.
/// HIPAR unit/This control selects the unit of a transducer set.
/// HIPAR transducerBreak/This control turns a transducer break on and off.
/// HIPAR range/Selects the range.
/// HIPAR startFrequency/Defines the start frequency of the frequency range.
/// HIPAR stopFrequency/Defines the stop frequency of the frequency range.
/// HIPAR factors/String containing the name of the transducer factor.
ViStatus _VI_FUNC rsemi_ConfigureTransducerSet (ViSession instrSession,
                                                ViString transducerSetName,
                                                ViString comment, ViInt32 unit,
                                                ViBoolean transducerBreak, ViInt32 range,
                                                ViReal64 startFrequency,
                                                ViReal64 stopFrequency,
                                                ViString factors)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 10), 6, "Range");
    
    viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_TSET_SELECT, transducerSetName),
                2, "Transducer Set Name");
    viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_TSET_COMMENT, comment),
                3, "Comment");                    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                RSEMI_ATTR_TSET_UNIT, unit),
                4, "Unit");                
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
                RSEMI_ATTR_TSET_BREAK, transducerBreak),
                5, "Transducer Break");    
                    
    viCheckErr (viPrintf (instrSession, "SENS:CORR:TSET:RANG%ld %.12lG,%.12lG,%s\n", range,
                                                startFrequency, stopFrequency, factors));
    
    checkErr( rsemi_CheckStatus (instrSession)); 
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects a set of transducer factors used for a particular
/// HIFN  frequency range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the range.
/// HIPAR startFrequency/Defines the start frequency of the frequency range.
/// HIPAR stopFrequency/Defines the stop frequency of the frequency range.
/// HIPAR factors/String containing the name of the transducer factor.
ViStatus _VI_FUNC rsemi_ConfigureTransducerFactors (
    ViSession instrSession,
    ViInt32 range,
    ViReal64 startFrequency,
    ViReal64 stopFrequency,
    ViString factors
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 10), 2, "Range");
    
    viCheckErr (viPrintf (instrSession, "SENS:CORR:TSET:RANG%ld %.12lG,%.12lG,%s\n", range,
                                                startFrequency, stopFrequency, factors));
    
    checkErr( rsemi_CheckStatus (instrSession)); 
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function reads out the names of all transducer sets stored
/// HIFN  on the harddisk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfTransducerSets/This control returns number of names of all transducer sets.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog 
/// HIPAR bufferSize/list parameter.
/// HIPAR transducerSetsList/This control returns the names of all transducer sets stored on the 
/// HIPAR transducerSetsList/hard disk.
ViStatus _VI_FUNC rsemi_QueryTransducerSetCatalog (ViSession instrSession,
                                                   ViInt32 *numberOfTransducerSets,
                                                   ViInt32 bufferSize,
                                                   ViChar transducerSetsList[])
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      *buf=VI_NULL;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if (bufferSize < 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");         
    if ((bufferSize > 0) && (transducerSetsList == VI_NULL))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Transducer Set List");  
       
    viCheckErr (viPrintf (instrSession, ":CORR:TSET:CAT?\n"));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &buf, &retCnt));
    viCheckErr (rsemi_ParseCatalog(buf, bufferSize, transducerSetsList, numberOfTransducerSets));

    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);      
    return error;
}

/// HIFN  This function deletes a transducer set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transducerSetName/This control selects the transducer set name.
ViStatus _VI_FUNC rsemi_DeleteTransducerSet (ViSession instrSession,
                                             ViString transducerSetName)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_TSET_SELECT, transducerSetName),
                2, "Transducer Set Name");

    viCheckErr ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_TSET_DELETE, VI_NULL));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the active transducer set. Any empty
/// HIFN  string is returned, if no transducer set is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR activeTransducerSet/This control the active transducer set. Any empty string is
/// HIPAR activeTransducerSet/returned, if no transducer set is active.
ViStatus _VI_FUNC rsemi_QueryActiveTransducerSet (ViSession instrSession,
                                                  ViChar activeTransducerSet[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm (rsemi_GetAttributeViString (instrSession, "", RSEMI_ATTR_TSET_ACTIVE, 512,
                 activeTransducerSet), 2, "Active Transducer Set");
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function connects to a device with the corresponding network addresses.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR networkAddress/Defines the network address of the switch matrix.
ViStatus _VI_FUNC rsemi_ConnectingRFSwitch (
    ViSession instrSession,
    ViString networkAddress
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_RF_SWITCH_NETWORK_ADDRESS, networkAddress),
                2, "Network Address");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function program the RF switch any way you want, with one
/// HIFN  or more commands per transducer range, as long as the command
/// HIFN  you are using is supported by the RF switch you are using. For a
/// HIFN  comprehensive description of supported commands, refer to the
/// HIFN  documentation of the RF switch.
/// HIFN  
/// HIFN  If you are using more than one command, add all commands in one
/// HIFN  line, separated by a semicolon.
/// HIFN  
/// HIFN  Each sequence of commands is sent to the RF switch when a
/// HIFN  transducer break occurs. This is the case when the measurement
/// HIFN  is done for a certain transducer range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the range.
/// HIPAR SCPI/This control turns the use of remote commands that comply to the
/// HIPAR SCPI/SCPI standard on and off
/// HIPAR command/This control defines a remote command that is sent to the switch
/// HIPAR command/matrix before all other commands.
ViStatus _VI_FUNC rsemi_ControlRFSwitch (
    ViSession instrSession,
    ViInt32   range,
    ViBoolean SCPI, 
    ViString  command
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rsemi_SetAttributeViBoolean (instrSession, "", 
                RSEMI_ATTR_RF_SWITCH_SCPI, SCPI),
                2, "SCPI");

    if (range == 0) 
    {
        viCheckParm ( rsemi_SetAttributeViString (instrSession, "", 
                    RSEMI_ATTR_RF_SWITCH_COMMAND, command),
                    3, "Command");
    
        viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_RF_SWITCH_COMMAND_EXECUTE, ""));
    }
    else
    {
        sprintf (rep_cap, "TR%ld", range);
        
        viCheckParm ( rsemi_SetAttributeViString (instrSession, rep_cap, 
                    RSEMI_ATTR_RF_SWITCH_RANGE_COMMAND, command),
                    3, "Command");
    
        viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_RF_SWITCH_RANGE_COMMAND_EXECUTE, ""));
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines a name and a comment for a dataset that controls 
/// HIFN  a switch matrix.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR datasetName/Defines a name for a dataset that controls a switch matrix.
/// HIPAR comment/Defines a comment for a dataset that controls a switch matrix.
ViStatus _VI_FUNC rsemi_ConfigureRFSwitchDataset (
    ViSession instrSession,
    ViString datasetName,
    ViString comment
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_RF_SWITCH_NAME, datasetName),
                2, "Dataset Name");
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_RF_SWITCH_COMMENT, comment),
                3, "Comment");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects, stores, restores or deletes a dataset that 
/// HIFN  controls a switch matrix.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of dataset operation to be performed.
/// HIPAR filename/String containing the file name and path.
ViStatus _VI_FUNC rsemi_RFSwitchDatasetFileOperations (
    ViSession instrSession,
    ViInt32 operation,
    ViString filename
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, operation, 0, 3), 2, "Operation");

    switch(operation)
    {
        case RSEMI_VAL_OPER_SELECT:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_RF_SWITCH_SELECT,
                filename), 3, "Filename");
        break;
        case RSEMI_VAL_OPER_SAVE:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_RF_SWITCH_STORE,
                filename), 3, "Filename");
        break;
        case RSEMI_VAL_OPER_RECALL:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_RF_SWITCH_LOAD,
                filename), 3, "Filename");
        break;  
        case RSEMI_VAL_OPER_DELETE:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_RF_SWITCH_DELETE,
                filename), 3, "Filename");  
        break;  
    }       
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
    
}

/// HIFN  This function selects the RF input of the R&S ESW used for the measurement with 
/// HIFN  the switch matrix.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR rfInput/Selects the RF input of the R&S ESW used for the measurement with 
/// HIPAR rfInput/the switch matrix.
ViStatus _VI_FUNC rsemi_SelectRFSwitchInput (
    ViSession instrSession,
    ViInt32 rfInput
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm ( rsemi_SetAttributeViInt32 (instrSession, "", 
                RSEMI_ATTR_RF_SWITCH_INPUT, rfInput),
                2, "RF Input");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function  switches the selected transducer factor on or off. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR rfInput/This control selects the RF Input.
/// HIPAR transducerState/This command switches the selected transducer factor on or off.
ViStatus _VI_FUNC rsemi_SelectTransducerRFInputFactor (
    ViSession instrSession,
    ViInt32 rfInput,
    ViBoolean transducerState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (rep_cap, "RF%ld", rfInput);     
    
    viCheckParm ( rsemi_SetAttributeViBoolean (instrSession, rep_cap, 
                RSEMI_ATTR_TRAN_INPUT_STATE, transducerState),
                3, "Transducer State");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the active transducer factor. Any empty string 
/// HIFN  is returned, if no transducer factor is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR rfInput/This control selects the RF Input.
/// HIPAR activeTransducer/This control the active transducer factor. Any empty string is 
/// HIPAR activeTransducer/returned, if no transducer factor is active.
ViStatus _VI_FUNC rsemi_GetTransducerRFInputActive (
    ViSession instrSession,
    ViInt32 rfInput,
    ViChar activeTransducer[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (rep_cap, "RF%ld", rfInput);  
   
    viCheckParm (rsemi_GetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_TRAN_INPUT_ACT_NAME, 512,
                 activeTransducer), 3, "Active Transducer");
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches the selected transducer set on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR rfInput/This control selects the RF Input.
/// HIPAR transducerSetState/This control switches the selected transducer set on or off.
ViStatus _VI_FUNC rsemi_SelectTransducerSetRFInput (
    ViSession instrSession,
    ViInt32 rfInput,
    ViBoolean transducerSetState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (rep_cap, "RF%ld", rfInput);
    
    viCheckParm ( rsemi_SetAttributeViBoolean (instrSession, rep_cap, 
                RSEMI_ATTR_TSET_INPUT_STATE, transducerSetState),
                3, "Transducer Set State");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the active transducer set. Any empty 
/// HIFN  string is returned, if no transducer set is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR rfInput/This control selects the RF Input.
/// HIPAR activeTransducerSet/This control the active transducer set. Any empty string is
/// HIPAR activeTransducerSet/returned, if no transducer set is active.
ViStatus _VI_FUNC rsemi_QueryActiveTransducerSetRFInput (
    ViSession instrSession,
    ViInt32 rfInput,
    ViChar activeTransducerSet[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (rep_cap, "RF%ld", rfInput);  

    viCheckParm (rsemi_GetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_TSET_INPUT_ACTIVE, 512,
                 activeTransducerSet), 3, "Active Transducer Set");
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the decimal separator (decimal point or 
/// HIFN  comma) for floating-point numerals contained in the file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR decimalSeparator/This control sets the decimal separator for the ASCII data stored to 
/// HIPAR decimalSeparator/the file.
ViStatus _VI_FUNC rsemi_FileDecimalSeparator(
    ViSession    instrSession,
    ViInt32    decimalSeparator
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_FILE_DEC_SEPARATOR, 
                 decimalSeparator), 2, "Decimal Separator");
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  The MMEMory (mass memory) subsystem function provides commands which 
/// HIFN  allow for access to the storage media of the instrument and for storing 
/// HIFN  and loading various instrument settings.
/// HIFN     The File to be Printed/Plotted command stores the HCOPy outputs in a 
/// HIFN  file. The various drives can be addressed via the mass storage unit 
/// HIFN  specifier <msus> using the conventional DOS syntax. The internal hard 
/// HIFN  disk is addressed by "C:", the floppy-disk drive installed by "A:". The 
/// HIFN  file names <file_name> are indicated as string parameters with the 
/// HIFN  commands being enclosed in quotation marks. They correspond to the DOS 
/// HIFN  conventions.
/// HIFN     DOS file names consist of max. 8 ASCII characters and an extension of 
/// HIFN  up to three characters separated from the file name by a colon "." 
/// HIFN  Both, the colon and the extension are optional. The colon is not part 
/// HIFN  of the file name. DOS file names do not differ between uppercase and 
/// HIFN  lowercase notation. All letters and digits are permitted as well as the 
/// HIFN  special characters "_", "^", "$", "~", "!", "#", "%", "&", "-", 
/// HIFN  "{","}", "(", ")", "@" and "' ". Reserved file names are CLOCK$, CON, 
/// HIFN  AUX, COM1 to COM4, LPT1 to LPT3, NUL and PRN. The two characters "*" 
/// HIFN  and "?" have the function of so-called "wildcards", i.e., they are 
/// HIFN  variables for
/// HIFN     selection of several files. The question mark "?" replaces exactly one 
/// HIFN  character which may be any, the asterisk means any of the remaining 
/// HIFN  characters in the file name. "*.*" thus means all files in a directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR source/This control specifies the source of the operation to be performed. If 
/// HIPAR source/the operation requires a single parameter, this control specifies that 
/// HIPAR source/parameter.
/// HIPAR destination/This control specifies the destination of the operation to be 
/// HIPAR destination/performed.
ViStatus _VI_FUNC rsemi_FileManagerOperations(
    ViSession    instrSession,
    ViInt32    operation,
    ViString    source,
    ViString    destination
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, operation, 0, 8), 2, "Operation");
    if (!source) 
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Source");
    
    switch(operation){
        case RSEMI_VAL_FILE_NEW:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_MANAGER_MAKE_DIR,
                source));   
        break;
        case RSEMI_VAL_FILE_COPY:
            if (!destination) 
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Destination");
            viCheckErr (viPrintf (instrSession, ":MMEM:COPY '%s','%s'\n", source, destination));
        break;  
        case RSEMI_VAL_FILE_RENAME:
            if (!destination) 
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Destination");
            viCheckErr (viPrintf (instrSession, ":MMEM:MOVE '%s','%s'\n", source, destination));
        break;  
        case RSEMI_VAL_FILE_DELETE:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_MANAGER_DELETE, 
                 source));
        break;           
        case RSEMI_VAL_FILE_RDIR:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_MANAGER_DELETE_DIR, 
                 source));
        break;       
        case RSEMI_VAL_FILE_FORMAT:
        break;  
        case RSEMI_VAL_FILE_CDIR:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_MANAGER_EDIT_PATH, 
                 source));
        break;
        case RSEMI_VAL_FILE_CDISC:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_MANAGER_EDIT_PATH_DEVICE, 
                 source));
        break;
        case RSEMI_VAL_FILE_DELETE_IMM:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_MANAGER_DELETE_IMMEDIATE, 
                 source));
        break;
    }       
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function reads the indicated directory. According to DOS 
/// HIFN  convention, wild card characters can be entered in order to query eg. a 
/// HIFN  list of all file of a certain type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
/// HIPAR output/This indicator returns the files in the indicated directory as comma 
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus _VI_FUNC rsemi_FileDirectory(
    ViSession    instrSession,
    ViString    directory,
    ViInt32    bufferSize,
    ViChar    output[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = VI_NULL;
    ViUInt32    count = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!directory)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Directory");
    viCheckErr (viPrintf(instrSession, ":MMEM:CAT? '%s'\n", directory));

    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &count)); 
    checkErr( rsemi_CheckStatus (instrSession));
    //strncpy(output, pbuffer, (size_t) bufferSize);
    if (count>=(ViUInt32)bufferSize)
    {
        strncpy(output, pbuffer, (size_t) (bufferSize-1));
        output[bufferSize-1]='\0';
        error = (ViStatus) (count+1);
    } 
    else
    {
        strncpy(output, pbuffer, (size_t) count); 
        output[count]='\0';   
    }
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the directories and files in the given path. 
/// HIFN  According to DOS convention, wild card characters can be entered in 
/// HIFN  order to query eg. a list of all file of a certain type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
/// HIPAR output/This indicator returns the file/directory information in the indicated 
/// HIPAR output/directory as comma separated strings, in format: 
/// HIPAR output/<used_bytes_in_this_directory>,<free_bytes_on_this_disk>,
ViStatus _VI_FUNC rsemi_FileDirectoryPath(
    ViSession    instrSession,
    ViString    directory,
    ViInt32    bufferSize,
    ViChar    output[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = VI_NULL;
    ViUInt32    count = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!directory)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Directory");
    viCheckErr (viPrintf(instrSession, ":MMEM:CAT:LONG? '%s'\n", directory));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &count)); 
    checkErr( rsemi_CheckStatus (instrSession));
    
    if (count>=(ViUInt32)bufferSize)
    {
        strncpy(output, pbuffer, (size_t) (bufferSize-1));     
        output[bufferSize-1]='\0';
        error = (ViStatus) (count+1);
    } 
    else
    {
        strncpy(output, pbuffer, (size_t) count); 
        output[count]='\0';   
    }
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function allows to provide data set operation for Save/Recall 
/// HIFN  items of memory subsystem.
/// HIFN     
/// HIFN     The various drives can be addressed via the "mass storage unit 
/// HIFN  specifier" <msus> using the conventional DOS syntax. The internal hard 
/// HIFN  disk is addressed by "D:", the floppy disk drive by "A:".
/// HIFN     
/// HIFN     For reasons of compatibility with the FSE instruments, addressing the 
/// HIFN  hard disk by "C:" is also accepted. Since hard disk "C:" is reserved 
/// HIFN  for instrument software, all read and write operations are rerouted to 
/// HIFN  hard disk "D:" in normal operation (service level 0).
/// HIFN     
/// HIFN     The file names <file_name> are indicated as string parameters with the 
/// HIFN  commands being enclosed in quotation marks. They also comply with DOS 
/// HIFN  conventions. DOS file names consist of max. 8 ASCII characters and an 
/// HIFN  extension of up to three characters
/// HIFN     separated from the file name by a dot "." Both, the dot and the 
/// HIFN  extension are optional. The dot is not part of the file name. DOS file 
/// HIFN  names do not distinguish between uppercase and lowercase notation. All 
/// HIFN  letters and digits are permitted as well as the special characters "_", 
/// HIFN  "^", "$", "~", "!", "#", "%", "&", "-", "{", "}", "(", ")", "@" and 
/// HIFN  "`". Reserved file names are CLOCK$, CON, AUX, COM1 to COM4, LPT1 to 
/// HIFN  LPT3,
/// HIFN     NUL and PRN. The two characters "*" and "?" have the function of 
/// HIFN  so-called "wildcards", i.e., they are variables for
/// HIFN     selection of several files. The question mark "?" replaces exactly one 
/// HIFN  character, the asterisk means any
/// HIFN     of the remaining characters in the file name. "*.*" thus means all 
/// HIFN  files in a directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR path/This control specifies the source of the operation to be performed. 
ViStatus _VI_FUNC rsemi_DataSetFileOperations(
    ViSession    instrSession,
    ViInt32    operation,
    ViString    path
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_invalidViInt32Range (instrSession, operation, 0, 7), 2, "Operation");
    
    switch(operation){
        case RSEMI_VAL_MEM_SAVE:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_SAVE,
                path), 2, "Path");  
        break;
        case RSEMI_VAL_MEM_RECALL:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_RECALL,
                path), 2, "Path");  
        break;  
        case RSEMI_VAL_MEM_DELETE:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_DATA_CLEAR,
                path), 2, "Path");  
        break;  
        case RSEMI_VAL_MEM_STARTUP:
            viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_STARTUP_RECALL,
                path), 2, "Path");  
        break;           
    }       
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function stores the current device settings in a *.FSL.dfl file. 
/// HIFN  The file name used in the last saving process is automatically counted 
/// HIFN  up to the next unused name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_DataSetFileSaveNext(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_SAVE_NEXT,
                NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function defines a comment ( max. 60 characters) for a device 
/// HIFN  setting to be stored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR comment/This control specifies the file comment.
ViStatus _VI_FUNC rsemi_DataSetFileComment(
    ViSession    instrSession,
    ViString    comment
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_EDIT_COMM,
                comment), 2, "Comment");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}


/// HIFN  This function deletes all device settings in the selected directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory.
ViStatus _VI_FUNC rsemi_DataSetFileClearAll(
    ViSession    instrSession,
    ViString    directory
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_MANAGER_EDIT_PATH,
                directory), 2, "Directory");
    viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_DATA_CLEAR_ALL,
                VI_NULL));              
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}


/// HIFN  This function add/remove settings to/from the list of partial datasets 
/// HIFN  of a device setting to be stored/loaded. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR itemSelector/This control selects the items to save or recall.
/// HIPAR itemState/This control sets if selected item is active or not for to 
/// HIPAR itemState/save/recall.
ViStatus _VI_FUNC rsemi_DataSetFileSelectItems(
    ViSession    instrSession,
    ViInt32    itemSelector,
    ViBoolean    itemState
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch(itemSelector){
        case RSEMI_VAL_FILE_ITEMS_NONE:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_ITEMS_SAVE_RECAL_NONE,
                VI_NULL));  
        break;
        case RSEMI_VAL_FILE_ITEMS_DEFAULT:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_ITEMS_SAVE_RECAL_DEFAULT,
                VI_NULL));  
        break;
        case RSEMI_VAL_FILE_ITEMS_ALL:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FILE_ITEMS_SAVE_RECAL_ALL,
                VI_NULL));  
        break;
        case RSEMI_VAL_FILE_ITEMS_HWSETTINGS:
            viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_HWSETTINGS,
                itemState), 3, "Item State");   
        break;
        case RSEMI_VAL_FILE_ITEMS_ALLTRACES:
            viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAC,
                itemState), 3, "Item State");   
        break;
        case RSEMI_VAL_FILE_ITEMS_ALLLINES:
            viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_LINE,
                itemState), 3, "Item State");   
        break;
        case RSEMI_VAL_FILE_ITEMS_ALLTRAN:
            viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAN,
                itemState), 3, "Item State");   
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Item Selector");
    }   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  This function defines whether the data from the entire
/// HIFN  instrument or only from the current channel is stored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/This control selects the store mode.
ViStatus _VI_FUNC rsemi_DataSetStoreMode (
    ViSession instrSession,
    ViInt32 mode
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                        RSEMI_ATTR_DATA_SET_STORE_MODE,
                                        mode), 2, "Mode");
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function defines the settings file to be loaded when the 
/// HIFN  specified user-definable softkey is selected. The settings file must 
/// HIFN  exist, otherwise an error occurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR softkeyNumber/Number of the softkey in the "User" menu.
/// HIPAR filename/This parameter defines the path and file name of the settings to be 
/// HIPAR filename/loaded.
/// HIPAR label/This parameter defines the label of the user-definable softkey.
ViStatus _VI_FUNC rsemi_ConfigureUserSettingsSoftkey(
    ViSession    instrSession,
    ViInt32    softkeyNumber,
    ViString    filename,
    ViString    label
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, softkeyNumber, 1, 8), 2, "Softkey Number");

    viCheckErr (viPrintf (instrSession, "MMEM:USER%ld '%s','%s'\n", softkeyNumber, filename, label));
    
    viCheckErr (rsemi_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function maps a drive to a server or server folder of the 
/// HIFN  network. As a prerequisite in Microsoft networks, sharing of this 
/// HIFN  server or server folder must be enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR driveLetter/This control sets the drive letter.
/// HIPAR hostNameIPAddress/This control sets the host name or IP address and shared folder.
/// HIPAR useOptionalParameters/This control sets whether the user name, password and reconnect at 
/// HIPAR useOptionalParameters/logon controls are used.
/// HIPAR userName/This control sets the user name.
/// HIPAR password/This control sets the password.
/// HIPAR reconnectAtLogon/This control sets the whether tha mapped drive should be reconnected 
/// HIPAR reconnectAtLogon/at the logon time.
ViStatus _VI_FUNC rsemi_NetworkMapDrive(
    ViSession    instrSession,
    ViString    driveLetter,
    ViString    hostNameIPAddress,
    ViBoolean    useOptionalParameters,
    ViString    userName,
    ViString    password,
    ViBoolean    reconnectAtLogon
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "\0";
    ViChar      *p2buf = buffer;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    p2buf += sprintf (p2buf, "MMEM:NETW:MAP '%s','%s'", driveLetter, hostNameIPAddress); 
    
    if (useOptionalParameters == VI_TRUE)
    {
        p2buf += sprintf (p2buf, ",'%s','%s',%s", userName, password, switchArr[reconnectAtLogon]);
    }
    
    p2buf += sprintf (p2buf, "\n");
    
    viCheckErr (viPrintf (instrSession, buffer));
    
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function disconnects the selected drive.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR driveLetter/This control sets the drive letter to be disconnected.
ViStatus _VI_FUNC rsemi_NetworkDisconnectDrive(
    ViSession    instrSession,
    ViString    driveLetter
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViString (instrSession, "",
                                                RSEMI_ATTR_NETWORK_DISCONNECT_DRIVE,
                                                driveLetter), 2, "Drive Letter");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function lists all unused network drive names.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control defines the size of Unused Drives array.
/// HIPAR unusedDrives/This control lists all unused network drive names.
ViStatus _VI_FUNC rsemi_NetworkUnusedDrivesList(
    ViSession    instrSession,
    ViInt32    arraySize,
    ViChar    unusedDrives[]
)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_GetAttributeViString (instrSession, "",
                                                RSEMI_ATTR_UNUSED_NETWORK_DRIVES,
                                                arraySize, unusedDrives), 3, "Unused Drives");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function lists all mapped network drives.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR showFolderInfo/This control sets if the folder info is showed in the list.
/// HIPAR arraySize/This control defines the size of Used Drives array.
/// HIPAR usedDrives/This control lists all mapped network drives.
ViStatus _VI_FUNC rsemi_NetworkUsedDrivesList(
    ViSession    instrSession,
    ViBoolean    showFolderInfo,
    ViInt32    arraySize,
    ViChar    usedDrives[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar*     pbuffer = NULL;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (viPrintf (instrSession, "MMEM:NETW:USED? %s\n", showFolderInfo?"ON":"OFF"));
    viCheckErr( Rs_ReadDataUnknownLength(instrSession, &pbuffer, VI_NULL));
    
    strncpy(usedDrives, pbuffer, arraySize);
    usedDrives[arraySize-1] = '\0';
    
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures the input for servicing purposes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR input/Toggles between the RF input on the front panel and the internal 
/// HIPAR input/128-MHz reference signal in the selected measurement window.
ViStatus _VI_FUNC rsemi_ServiceConfigureInput(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    input
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    window;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_SERVICE_INP_SOURCE, input), 3, "Input");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function configures pulsed calibration signal. This function 
/// HIFN  takes effect only if the RF input has been set to the internal 
/// HIFN  reference signal using rsemi_ServiceConfigureInput.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR reserved/Reserved
/// HIPAR reserved2/reserved 
/// HIPAR state/between pulsed (On) and non-pulsed (Off).
/// HIPAR frequency/Selects the calibration frequency for the pulsed calibration signal in 
/// HIPAR frequency/the selected measurement window.
ViStatus _VI_FUNC rsemi_ServiceConfigurePulsedInput(
    ViSession    instrSession,
    ViInt32    reserved,
    ViBoolean    reserved2,
    ViReal64    frequency
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    reserved;
    reserved2;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer,
                RSEMI_ATTR_SERVICE_PULSE_CAL_FREQ, frequency), 4, "Frequency");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function switches the 28-V supply of the noise source at the rear 
/// HIFN  panel on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR noise/Switches the 28-V supply of the noise source at the rear panel on or 
/// HIPAR noise/off.
ViStatus _VI_FUNC rsemi_ServiceConfigureNoise(
    ViSession    instrSession,
    ViBoolean    noise
)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_SERVICE_NOISE_SOURCE, noise), 2, "Noise");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures RF input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/Selects the calibration frequency for the RF calibration signal
/// HIPAR RFSpectrum/Selects the bandwidth of the calibration signal.
ViStatus _VI_FUNC rsemi_ConfigureCalibrationFreqRF (
    ViSession instrSession,
    ViReal64 frequency,
    ViInt32 RFSpectrum
)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, NULL));
 
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                RSEMI_ATTR_SERVICE_PULSE_CAL_FREQ, frequency), 2, "Frequency");
   
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                RSEMI_ATTR_SERVICE_CAL_FREQ_RF_SPECTRUM, RFSpectrum), 3, "RF Spectrum");
   
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/// HIFN This function selects the distance of the peaks of the microwave calibration 
/// HIFN signal for calibration of the YIG filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR distance/Selects the distance of the peaks of the microwave calibration 
/// HIPAR distance/signal for calibration of the YIG filter.
ViStatus _VI_FUNC rsemi_ConfigureCalibrationFreqMW (
    ViSession instrSession,
    ViInt32 distance
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_SERVICE_INPUT_SOURCE, 
                                                            RSEMI_VAL_INPUT_MCAL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_SERVICE_CAL_FREQ_MW_DIST, 
                                                            distance), 2, "Distance");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function enables or disables automatic power reduction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR automaticPowerReduction/Enables or disables automatic power reduction
ViStatus _VI_FUNC rsemi_ConfigureCalAutoPowerReduction (
    ViSession instrSession,
    ViBoolean automaticPowerReduction
)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
                                 RSEMI_ATTR_SERVICE_CAL_AUTO_POWER_REDUCTION,
                                automaticPowerReduction), 2, "Automatic Power Reduction");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function specifies the Directory for Additionally Available Cal Kits and 
/// HIFN  Conn Types. All cal kit files in the special directory will be (re-)loaded 
/// HIFN  automatically every time the NWA application is started.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR additionalDirectory/Specifies the Directory for Additionally Available 
/// HIPAR additionalDirectory/Cal Kits and Conn Types.
ViStatus _VI_FUNC rsemi_ConfigureCalKitAdditionalDir (
    ViSession instrSession,
    ViString additionalDirectory
)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
                                 RSEMI_ATTR_SERVICE_CAL_KIT_ADDITIONAL_DIRECTORY,
                                additionalDirectory), 2, "Additional Directory");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/// HIFN  This function defines the frequency of the calibration signal. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Defines the frequency of the calibration signal.
ViStatus _VI_FUNC rsemi_ConfigureCalibrationFreqMicrowave (
    ViSession instrSession,
    ViReal64 frequency
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
       
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                               RSEMI_ATTR_SERVICE_CAL_FREQ_MICROWAVE,
                                               frequency), 2, "Frequency");    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the contents of the module info table. Table 
/// HIFN  lines are output as string data and are separated by commas.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of array passed to Result argument.
/// HIPAR hwInfo/This control returns the contents of the module info table. Table 
/// HIPAR hwInfo/lines are output as string data and are separated by commas.
ViStatus _VI_FUNC rsemi_ServiceHWInfo(
    ViSession    instrSession,
    ViInt32    arraySize,
    ViChar    hwInfo[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *pbuffer=VI_NULL;
    ViUInt32    ret_val;
    ViUInt32     length;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    //viCheckParm (rsemi_GetAttributeViString (instrSession, "", 
    //            RSEMI_ATTR_SERVICE_HW_INFO, arraySize, HWInfo), 2, "HW Info");
    viCheckErr (viPrintf (instrSession, "DIAG:SERV:HWIN?\n"));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, &ret_val));
    length=((ViUInt32)arraySize<=ret_val)?(ViUInt32)arraySize-1:ret_val;
    memcpy (hwInfo,pbuffer,length);
    hwInfo[length]='\0';
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the instrument firmware and option versions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of array passed to Version Info 
/// HIPAR arraySize/argument.
/// HIPAR versionInfo/This control returns the instrument firmware and option versions.
ViStatus _VI_FUNC rsemi_ServiceVersionInfo(
    ViSession    instrSession,
    ViInt32    arraySize,
    ViChar    versionInfo[]
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *pbuffer=VI_NULL;
    ViUInt32    ret_val;
    ViUInt32    length;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (viPrintf (instrSession, "DIAG:SERV:VERS?\n"));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, &ret_val));
    length=((ViUInt32)arraySize<=ret_val)?arraySize-1:(ViUInt32)ret_val;
    memcpy (versionInfo,pbuffer,length);
    versionInfo[length]='\0';
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This Function sets the Enable, NTransition, and PTransition bits
/// HIFN of the Main, Power, Limit, Left Margin, and Sync status         
/// HIFN questionable registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR enable/Document parameter here.
/// HIPAR PTransition/Document parameter here.
/// HIPAR NTransition/Document parameter here.
ViStatus _VI_FUNC rsemi_SetStatusRegisterValue (ViSession instrSession,
                ViInt32 registerOperation, ViInt32 questionableRegister, 
                ViInt32 enable,
                ViInt32 PTransition, ViInt32 NTransition)
{
    ViStatus    error   = VI_SUCCESS;
    ViReal64     discrete;
    ViChar       *p2string = NULL;
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, NULL));     
    
    switch (registerOperation) {
        case RSEMI_VAL_STAT_REG_PRESET:
             viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_PRESET_STATUS_REGISTERS, NULL));  
        break;
        case RSEMI_VAL_STAT_REG_QUESTIONABLE:
            do
            {
                discrete = rsemi_rngStatusRegisterQuestionable.rangeValues[i].discreteOrMinValue;
                p2string = rsemi_rngStatusRegisterQuestionable.rangeValues[i].cmdString;

                if ((i != 0) &&(p2string == RS_RANGE_TABLE_END_STRING))
                    viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Questionable Register");

                if (fabs(discrete - questionableRegister) <= 0.01) break;

                i++;

            } while (i != 0);
            switch (questionableRegister){
                case RSEMI_VAL_STAT_REG_MAIN:
                    viCheckErr (viPrintf (instrSession, ":STAT:QUES:ENAB %ld;PTR %ld"
                        ";NTR %ld\n", enable, PTransition, NTransition));
                break;
                default:
                    viCheckErr (viPrintf (instrSession, ":STAT:QUES:%s:ENAB %ld;PTR %ld"
                        ";NTR %ld\n", p2string, enable, PTransition,
                        NTransition));
            }
        break;
        case RSEMI_VAL_STAT_REG_OPERATION:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, enable, 0, 65535), 4, "Enable");
            viCheckParm (rsemi_invalidViInt32Range (instrSession, PTransition, 0, 65535), 5, "PTransition");
            viCheckParm (rsemi_invalidViInt32Range (instrSession, NTransition, 0, 65535), 6, "NTransition");

            viCheckErr (viPrintf (instrSession, ":STAT:OPER:ENAB %ld;PTR %ld"
                ";NTR %ld\n", enable, PTransition, NTransition));
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");     
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function contains the commands for the querying status     */
/// HIFN reporting system. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerOperation/Specifies the operation to perform.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR registerPart/Sets the register part.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsemi_GetStatusRegisterValue (ViSession instrSession,
                                                   ViInt32 registerOperation,
                                                   ViInt32 questionableRegister,
                                                   ViInt32 registerPart,
                                                   ViInt32 *registerValue)
{
    ViStatus    error   = VI_SUCCESS;
    ViReal64     discrete;
    ViChar       *p2string = NULL;
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, NULL));     
    viCheckParm (rsemi_invalidViInt32Range (instrSession, registerPart, 0, 1), 4, "Register Part");        
        
    switch (registerOperation) {
        case RSEMI_VAL_STAT_REG_QUESTIONABLE:
            do
            {
                discrete = rsemi_rngStatusRegisterQuestionable.rangeValues[i].discreteOrMinValue;
                p2string = rsemi_rngStatusRegisterQuestionable.rangeValues[i].cmdString;

                if ((i!=0) && p2string == RS_RANGE_TABLE_END_STRING)
                    viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Questionable Register");

                if (fabs(discrete - questionableRegister) <= 0.01) break;

                i++;

            } while (i != 0);
            switch (questionableRegister){
                case RSEMI_VAL_STAT_REG_MAIN:
                    viCheckErr( viQueryf (instrSession, ":STAT:QUES:%s?\n", "%ld",
                    rsemi_rngStatusRegisterPart.rangeValues[registerPart].cmdString,registerValue));
                break;
                default:
                    viCheckErr( viQueryf (instrSession, ":STAT:QUES:%s:%s?\n", "%ld",
                    p2string, rsemi_rngStatusRegisterPart.rangeValues[registerPart].cmdString,registerValue));
            }
            
        break;
        case RSEMI_VAL_STAT_REG_OPERATION:
            viCheckErr( viQueryf (instrSession, ":STAT:OPER:%s?\n", "%ld",
                    rsemi_rngStatusRegisterPart.rangeValues[registerPart].cmdString,registerValue));
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");     
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;    
}

/// HIFN  This function selects the unit for relative demodulation results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR resultUnit/Selects the unit for relative demodulation results.
ViStatus _VI_FUNC rsemi_ConfigureADEMResultUnit (
    ViSession instrSession,
    ViInt32 resultUnit
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
       
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_ANALOG_DEMOD_RESULT_UNIT,
                                               resultUnit), 2, "Result Unit");    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This control defines the number of sub blocks in the SEM measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR count/Number of sub blocks in the SEM measurement.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockCount (
    ViSession instrSession,
    ViInt32 count
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SEM_SUBBLOCK_COUNT,
                                               count), 2, "Count");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the RF attenuation setting for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR centerFrequency/This control  defines the center frequency of the selected sub
/// HIPAR centerFrequency/block in a Multi-SEM measurement.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockCenterFrequency (
    ViSession instrSession,
    ViInt32 subBlock,
    ViReal64 centerFrequency
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer,
                                               RSEMI_ATTR_SEM_SUBBLOCK_CENTER_FREQUENCY,
                                               centerFrequency), 3, "Center Frequency");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the subblock reference range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR powerReferenceType/Sets the power reference type.
/// HIPAR txBandwidth/Defines the bandwidth used for measuring the channel power (reference range).
/// HIPAR rrcFilterState/Activates or deactivates the use of an RRC filter.
/// HIPAR rrcFilterAlpha/Sets the alpha value of the RRC filter.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockReferenceRange (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 powerReferenceType,
    ViReal64 txBandwidth,
    ViBoolean rrcFilterState,
    ViReal64 rrcFilterAlpha
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer,
                                               RSEMI_ATTR_SEM_SUBBLOCK_POWER_REF_TYPE, powerReferenceType),
                                               3, "Power Reference Type");
    
    if (powerReferenceType == RSEMI_VAL_SEM_REF_TYPE_CPOW)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                   RSEMI_ATTR_SEM_SUBBLOCK_BANDWIDTH, txBandwidth),
                                                   4, "TX Bandwidth");
    
        viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                   RSEMI_ATTR_SEM_SUBBLOCK_RRC_FILTER_STATE, rrcFilterState),
                                                   5, "RRC Filter State");
    
        if (rrcFilterState == VI_TRUE)
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer,
                                                       RSEMI_ATTR_SEM_SUBBLOCK_RRC_FILTER_ALPHA_VALUE, rrcFilterAlpha),
                                                       6, "RRC Filter Alpha");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the current position (number) of the reference range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR referenceRangePosition/This control returns the current position (number) of the
/// HIPAR referenceRangePosition/reference range.
ViStatus _VI_FUNC rsemi_QuerySEMSubblockReferenceRangePosition (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 *referenceRangePosition
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);
    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, buffer,
                                               RSEMI_ATTR_SEM_SUBBLOCK_REF_RANGE_POSITION, referenceRangePosition),
                                               3, "Reference Range Position");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function activates Fast SEM mode to accelerate spectrum emission mask measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR fastSEM/Activates Fast SEM mode to accelerate spectrum emission mask measurements.
ViStatus _VI_FUNC rsemi_ConfigureFastSEMSubblock (
    ViSession instrSession,
    ViInt32 subBlock, 
    ViBoolean fastSEM
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer,
                                               RSEMI_ATTR_SEM_SUBBLOCK_FAST_SEM, fastSEM),
                                               3, "Fast SEM");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the specified XML file under C:\r_s\instr\sem_std. 
/// HIFN  If the file is stored in a subdirectory, include the relative path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR presetStandard/Selects the specified XML file.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockPresetStandard (
    ViSession instrSession,
    ViInt32 subBlock,
    ViString presetStandard
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, buffer,
                                               RSEMI_ATTR_SEM_SUBBLOCK_PRESET_STANDARD, presetStandard),
                                               3, "Preset Standard");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function copies the XML files from the C:\R_S\instr\sem_backup folder to 
/// HIFN  the C:\R_S\instr\sem_std folder. Files of the same name are overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
ViStatus _VI_FUNC rsemi_RestoreSEMSubblockStandardFiles (
    ViSession instrSession,
    ViInt32 subBlock
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                                               RSEMI_ATTR_SEM_SUBBLOCK_RESTORE_STANDARD, NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function stores the current settings as presettings in the specified XML 
/// HIFN  file under C:\r_s\instr\sem_backup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR settingsFileName/Specifies XML file that stores the current settings as presettings.
ViStatus _VI_FUNC rsemi_StoreSEMSubblockPresetSettings (
    ViSession instrSession,
    ViInt32 subBlock,
    ViString settingsFileName
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, buffer,
                                               RSEMI_ATTR_SEM_SUBBLOCK_STORE_PRESET, settingsFileName),
                                               3, "Settings File Name");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the start and stop frequency for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR rangeStart/Sets the start frequency for the specified range.
/// HIPAR rangeStop/Sets the stop frequency for the specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeStartStop (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViReal64 rangeStart,
    ViReal64 rangeStop
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_FREQ_START, rangeStart),
                                                4, "Range Start");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_FREQ_STOP, rangeStop),
                                                5, "Range Stop");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the filter type for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR filterType/
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeFilterType (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViInt32 filterType
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_FILTER_TYPE, filterType),
                                                4, "Filter Type");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the resolution and video bandwidth values for the 
/// HIFN  specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR resolutionBandwidth/Sets the RBW value for the specified range.
/// HIPAR videoBandwidth/Sets the VBW value for the specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeBandwidth (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViReal64 resolutionBandwidth,
    ViReal64 videoBandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_BANDWIDTH, resolutionBandwidth),
                                                4, "Resolution Bandwidth");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_VBW, videoBandwidth),
                                                5, "Video Bandwidth");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the sweep time values for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR sweepTimeMode/Activates or deactivates the automatic sweep time setting for
/// HIPAR sweepTimeMode/the specified range.
/// HIPAR sweepTime/Sets the sweep time for the specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeSweepTime (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViBoolean sweepTimeMode,
    ViReal64 sweepTime
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_SWEEP_TIME_AUTO, sweepTimeMode),
                                                4, "Sweep Time Mode");
    
    if (sweepTimeMode == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                    RSEMI_ATTR_SEM_SUBBLOCK_RANGE_SWEEP_TIME, sweepTime),
                                                    5, "Sweep Time");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the reference level for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR referenceLevel/sets the reference level for the specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeReferenceLevel (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViReal64 referenceLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_REF_LEVEL, referenceLevel),
                                                4, "Reference Level");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the RF attenuation setting for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR rfAttenuationMode/activates or deactivates the automatic RF attenuation setting
/// HIPAR rfAttenuationMode/for the specified range.
/// HIPAR rfAttenuator/Sets the attenuation for the specified range.
/// HIPAR preamplifiier/Switches the preamplifier on or off for the specified range.
/// HIPAR preamplifiierLevel/Selects the gain for a SEM range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeRFAttenuation (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViBoolean rfAttenuationMode,
    ViReal64 rfAttenuator,
    ViBoolean preamplifiier,
    ViInt32 preamplifiierLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_INPUT_ATTENUATION_AUTO, rfAttenuationMode),
                                                4, "RF Attenuation Mode");
    if (rfAttenuationMode == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                    RSEMI_ATTR_SEM_SUBBLOCK_RANGE_INPUT_ATTENUATION, rfAttenuator),
                                                    5, "RF Attenuator");
    }
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_PREAMPLIFIER, preamplifiier),
                                                6, "Preamplifiier");
    if (preamplifiier == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                    RSEMI_ATTR_SEM_SUBBLOCK_RANGE_PREAMPLIFIER_LEVEL, preamplifiierLevel),
                                                    7, "Preamplifiier Level");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures a transducer for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR transducerFactor/Sets a transducer for the specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeTransducerFactor (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViString transducerFactor
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range); 
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_TRANSDUCER, transducerFactor),
                                                4, "Transducer Factor");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines the function used to calculate the limit line for the n-th 
/// HIFN  power class for overlapping ranges in Multi-SEM measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR multiLimitCalc/Defines the function used to calculate the limit line for the
/// HIPAR multiLimitCalc/n-th power class for overlapping ranges in Multi-SEM measurements.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeMultiLimitCalc (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViInt32 multiLimitCalc
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_MULTILIMT_CALC, multiLimitCalc),
                                                4, "Multi Limit Calc");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines the minimum number of sweep points for the range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR minimumSweepPoints/Defines the minimum number of sweep points for the range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeMinimumSweepPoints (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViInt32 minimumSweepPoints
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                               RSEMI_ATTR_SEM_SUBBLOCK_RANGE_MIN_SWEEP_POINTS, minimumSweepPoints),
                                               4, "Minimum Sweep Points");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines the minimum number of sweep points for the range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR source/Selects the source.
/// HIPAR limitCheckType/Sets the type of limit check for all ranges.
/// HIPAR absoluteLimitStart/Sets an absolute limit value at the start frequency of the
/// HIPAR absoluteLimitStart/specified range.
/// HIPAR absoluteLimitStop/Sets an absolute limit value at the stop frequency of the
/// HIPAR absoluteLimitStop/specified range.
/// HIPAR relativeLimitStart/Sets a relative limit value at the start frequency of the
/// HIPAR relativeLimitStart/specified range.
/// HIPAR relativeLimitStop/Sets a relative limit value at the stop frequency of the
/// HIPAR relativeLimitStop/specified range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeLimits (
    ViSession instrSession,
    ViInt32 subBlock, 
    ViInt32 range,
    ViInt32 source,
    ViInt32 limitCheckType,
    ViReal64 absoluteLimitStart,
    ViReal64 absoluteLimitStop,
    ViReal64 relativeLimitStart,
    ViReal64 relativeLimitStop
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, source, 1, 4), 4, "Source");
    
    sprintf (buffer, "ESB%ld,R%ld,PC%ld", subBlock, range, source);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_LIMIT_CHECK_TYPE, limitCheckType),
                                                5, "Limit Check Type");
    
    switch (limitCheckType)
    {
        case RSEMI_VAL_SEM_LIM_CHECK_TYPE_ABS:
        {
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_START_FREQ_ABS_LIMIT, absoluteLimitStart),
                                                6, "Absolute Limit Start");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_STOP_FREQ_ABS_LIMIT, absoluteLimitStop),
                                                7, "Absolute Limit Stop");
            break;
        }
        case RSEMI_VAL_SEM_LIM_CHECK_TYPE_REL:
        {
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_START_FREQ_REL_LIMIT, relativeLimitStart),
                                                8, "Relative Limit Start");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_STOP_FREQ_REL_LIMIT, relativeLimitStop),
                                                9, "Relative Limit Stop");
            break;
        }
        case RSEMI_VAL_SEM_LIM_CHECK_TYPE_AND:
        case RSEMI_VAL_SEM_LIM_CHECK_TYPE_OR:
        {
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_START_FREQ_ABS_LIMIT, absoluteLimitStart),
                                                5, "Absolute Limit Start");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_STOP_FREQ_ABS_LIMIT, absoluteLimitStop),
                                                6, "Absolute Limit Stop");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_START_FREQ_REL_LIMIT, relativeLimitStart),
                                                7, "Relative Limit Start");
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_STOP_FREQ_REL_LIMIT, relativeLimitStop),
                                                8, "Relative Limit Stop");
            break;
        }
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Type");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines a relative and an absolute limit level. In this case, 
/// HIFN  the maximum of the two values is used as the limit level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR source/Selects the source.
/// HIPAR function/Enables the use of a function when defining the relative limit
/// HIPAR function/for a SEM range.
/// HIPAR relativeLevel/Sets a relative limit value at the start frequency of the
/// HIPAR relativeLevel/specified range.
/// HIPAR absoluteLevel/Defines an absolute limit for the MAX function of the relative
/// HIPAR absoluteLevel/limit for a SEM range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeRelativeLimitStartFunction (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViInt32 source,
    ViInt32 function,
    ViReal64 relativeLevel,
    ViReal64 absoluteLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, source, 1, 4), 4, "Source");
    
    sprintf (buffer, "ESB%ld,R%ld,PC%ld", subBlock, range, source);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_START_FREQ_REL_LIMIT_FUNC, function),
                                                5, "Function");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_START_FREQ_REL_LIMIT, relativeLevel),
                                                6, "Relative Level");    
    
    if (function == RSEMI_VAL_SEM_LIM_FUNC_MAX)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_START_FREQ_REL_LIMIT_ABS_MAX, absoluteLevel),
                                                7, "Absolute Level");        
    }
            
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines a relative and an absolute limit level. In this case, 
/// HIFN  the maximum of the two values is used as the limit level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR source/Selects the source.
/// HIPAR function/Enables the use of a function when defining the relative limit
/// HIPAR function/for a SEM range.
/// HIPAR relativeLevel/sets a relative limit value at the stop frequency of the
/// HIPAR relativeLevel/specified range.
/// HIPAR absoluteLevel/Defines an absolute limit for the MAX function of the relative
/// HIPAR absoluteLevel/limit for a SEM range.
ViStatus _VI_FUNC rsemi_ConfigureSEMSubblockRangeRelativeLimitStopFunction (
    ViSession instrSession,
    ViInt32 subBlock,
    ViInt32 range,
    ViInt32 source,
    ViInt32 function,
    ViReal64 relativeLevel,
    ViReal64 absoluteLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, source, 1, 4), 4, "Source");
    
    sprintf (buffer, "ESB%ld,R%ld,PC%ld", subBlock, range, source);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_STOP_FREQ_REL_LIMIT_FUNC, function),
                                                5, "Function");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_STOP_FREQ_REL_LIMIT, relativeLevel),
                                                6, "Relative Level");    
    
    if (function == RSEMI_VAL_SEM_LIM_FUNC_MAX)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_STOP_FREQ_REL_LIMIT_ABS_MAX, absoluteLevel),
                                                7, "Absolute Level");        
    }
            
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the number of defined ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR rangeCount/Returns the number of defined ranges.
ViStatus _VI_FUNC rsemi_GetSEMSubblockRangeCount (
    ViSession instrSession,
    ViInt32 subBlock, 
    ViInt32 *rangeCount
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);    
    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_RANGE_COUNT, rangeCount),
                                                3, "Range Count");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function inserts a new range before or after the specified range. 
/// HIFN  The range numbers are updated accordingly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
/// HIPAR insertRange/Inserts a new range before or after the specified range.
ViStatus _VI_FUNC rsemi_SEMSubblockInsertNewRange (
    ViSession instrSession,
    ViInt32 subBlock, 
    ViInt32 range,
    ViInt32 insertRange
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_INSERT_NEW_RANGE, insertRange),
                                                4, "Insert Range");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function deletes the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block.
/// HIPAR range/Selects the range.
ViStatus _VI_FUNC rsemi_SEMSubblockDeleteRange (
    ViSession instrSession,
    ViInt32 subBlock, 
    ViInt32 range
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, range, 1, 30), 3, "Range");
    
    sprintf (buffer, "ESB%ld,R%ld", subBlock, range); 
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_DELETE_RANGE, NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the band category and RF bandwidth for
/// HIFN  the MSR SEM measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block 
/// HIPAR bandCategory/This attribute defines the band category for MSR measurements
/// HIPAR RFBandwidth/This control defines the RF bandwidth of the base station for
/// HIPAR RFBandwidth/MSR measurements
/// HIPAR GSMEdgePresent/This control defines whether a GSM/Edge carrier is located at
/// HIPAR GSMEdgePresent/the edge of the specified RF bandwidth. In this case, the
/// HIPAR GSMEdgePresent/specification demands specific limits for the SEM ranges.
/// HIPAR LTEPresent/This control defines whether an LTE FDD 1.4 MHz or 3 MHz carrier
/// HIPAR LTEPresent/is located at the edge  of the specified RF bandwidth. In this
/// HIPAR LTEPresent/case, the specification demands specific limits for the SEM ranges.
ViStatus _VI_FUNC rsemi_ConfigureMSRSettings (
    ViSession instrSession,
    ViInt32 subBlock, 
    ViInt32 bandCategory,
    ViReal64 RFBandwidth,
    ViBoolean GSMEdgePresent,
    ViBoolean LTEPresent
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer,
                                               RSEMI_ATTR_SEM_SUBBLOCK_MSR_BAND_CATEGORY,
                                               bandCategory), 3, "Band Category");
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer,
                RSEMI_ATTR_SEM_SUBBLOCK_MSR_RF_BANDWIDTH, RFBandwidth), 4, "RF Bandwidth");    
    
    if(bandCategory == RSEMI_VAL_SEM_MSR_BAND_CATEGORY_2)
    {
        viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                     RSEMI_ATTR_SEM_SUBBLOCK_MSR_GSM_PRESENT,
                                                     GSMEdgePresent), 5, "GSM/Edge Present");        
        
        viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer,
                                                     RSEMI_ATTR_SEM_SUBBLOCK_MSR_LTE_PRESENT,
                                                     LTEPresent), 6, "LTE Present");        
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Configures the SEM sweep list according to the specified MSR settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/Selects the sub block
ViStatus _VI_FUNC rsemi_MSRApplyToSEM (
    ViSession instrSession, 
    ViInt32 subBlock
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, subBlock, 1, 3), 2, "Sub Block");
    
    sprintf (buffer, "ESB%ld", subBlock);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_SEM_SUBBLOCK_MSR_APPLY, NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the signal path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR iQMode/Defines the input signal path used.
/// HIPAR inputConfiguration/This attribute toggles the baseband inputs between balanced and
/// HIPAR inputConfiguration/unbalanced.
ViStatus _VI_FUNC rsemi_ConfigureAnalogBasebandInputParameters (
    ViSession instrSession,
    ViInt32 iQMode,
    ViBoolean inputConfiguration
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_BB_INPUT_SIGNAL_PATH, iQMode),
                2, "I/Q Mode");
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_BB_INPUT_BALANCED, inputConfiguration),
                3, "Input Configuration");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  This function configures the fullscale level. The fullscale level defines 
/// HIFN  the maximum power you can input at the Baseband Input connector without 
/// HIFN  clipping the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mode/This control defines whether the fullscale level (i.e. the maximum input 
/// HIPAR mode/power on the Baseband Input connector) is defined automatically according 
/// HIPAR mode/to the reference level, or manually.
/// HIPAR level/This control defines the peak voltage at the Baseband Input connector if 
/// HIPAR level/the fullscale level is set to manual mode (Mode control is set to Off).
ViStatus _VI_FUNC rsemi_ConfigureAnalogBasebandFullscaleLevel (
    ViSession instrSession,
    ViBoolean mode,
    ViReal64 level
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
                                                 RSEMI_ATTR_BB_FULLSCALE_MODE, 
                                                 mode), 2, "Mode");  

    if (mode == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_BB_FULLSCALE_LEVEL,
                                                level), 3, "Level"); 
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN This function activates realtime mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or rsemi_InitWithOptions function.
ViStatus _VI_FUNC rsemi_RealtimeSpectrumAnalysisMode (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_REALTIME_MODE, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function activates IQ mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or rsemi_InitWithOptions function.
ViStatus _VI_FUNC rsemi_IQAnalyzerMode (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_RECEIVER_IQ_MODE, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the frequency range of the spectrum analyzer 
/// HIFN  using the center frequency and the frequency span.  If span corresponds 
/// HIFN  to zero hertz, then the spectrum analyzer is in time-domain. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR centerFrequency/The center frequency of the frequency sweep.
/// HIPAR span/The frequency span of the frequency sweep.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramFrequencyCenterSpan(
    ViSession    instrSession,
    ViReal64    centerFrequency,
    ViReal64    span
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_SPEM_CENTER_FREQUENCY,
                                                centerFrequency), 2, "Center Frequency");
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                 RSEMI_ATTR_SPEM_SPAN,
                                                 span), 3, "Span");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the frequency span in the analyzer mode to its 
/// HIFN  maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramFrequencyFullSpan(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "",
                                               RSEMI_ATTR_SPEM_SPAN_FULL, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the frequency offset attribute of the 
/// HIFN  spectrum analyzer. This function affects the setting of the spectrum 
/// HIFN  analyzer's absolute frequencies, such as start, stop, center, and 
/// HIFN  marker. It does not affect values, which are the difference of 
/// HIFN  frequencies, such as span and delta marker. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequencyOffset/Frequency offset.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramFrequencyOffset(
    ViSession    instrSession,
    ViReal64    frequencyOffset
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_SPEM_FREQUENCY_OFFSET,
                                                frequencyOffset), 2, "Frequency Offset");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Defines the center frequency step size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR stepSize/Sets the center frequency step size.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramFrequencyCenterStepSize(
    ViSession    instrSession,
    ViReal64    stepSize
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_FREQUENCY_CENTER_STEP,
                                                stepSize), 2, "Step Size");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function couples the step width of the center frequency to span 
/// HIFN  (span >0) or to the resolution bandwidth (span = 0) or cancels the 
/// HIFN  couplings and sets the coupling factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR coupling/Couples the step width of the center frequency to span (span >0) or to 
/// HIPAR coupling/the resolution bandwidth (span = 0) or cancels the couplings.
/// HIPAR couplingFactor/Sets the coupling factor.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramFrequencyCouplingFactor(
    ViSession    instrSession,
    ViInt32    coupling,
    ViInt32    couplingFactor
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SPEM_FREQUENCY_CENTER_LINK,
                                               coupling), 2, "Coupling");
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SPEM_FREQUENCY_CENTER_LINK_FACTOR,
                                               couplingFactor), 3, "Coupling Factor");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function starts a continuous sweep measurement or stops it and 
/// HIFN  switches to single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sweepModeContinuous/This control toggles the sweep.
/// HIPAR numberOfSweeps/The number of sweeps to take.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramAcquisition(
    ViSession    instrSession,
    ViBoolean    sweepModeContinuous,
    ViInt32    numberOfSweeps
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", 0);  
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SWEEP_MODE_CONTINUOUS,
                                                sweepModeContinuous), 3, "Sweep Mode Continuous");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_NUMBER_OF_SWEEPS,
                                                numberOfSweeps), 4, "Number Of Sweeps");                                                

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the sweep time values of the spectrum 
/// HIFN  analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sweepTimeAuto/This control sets automatic sweep time. This automatically sets the 
/// HIPAR sweepTimeAuto/sweep time to 32 ms.
/// HIPAR sweepTime/The length of time to complete one sweep.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramSweepTime(
    ViSession    instrSession,
    ViBoolean    sweepTimeAuto,
    ViReal64    sweepTime
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_SPEM_SWEEP_TIME_AUTO,
                                                 sweepTimeAuto), 2, "Sweep Time Auto");

    if (sweepTimeAuto == VI_FALSE)
    {
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                    RSEMI_ATTR_SPEM_SWEEP_TIME,
                                                    sweepTime), 3, "Sweep Time");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resolutionBandwidthAuto/Turns resolution bandwidth auto coupling on/off.
/// HIPAR resolutionBandwidth/Specifies the measurement resolution bandwidth.
/// HIPAR resolutionBandwidthRatio/Specifies the ratio resolution bandwidth (Hz) / span (Hz). The ratio 
/// HIPAR resolutionBandwidthRatio/to be entered is reciprocal to the ratio span/RBW used in manual 
/// HIPAR resolutionBandwidthRatio/control.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramBandwidth(
    ViSession    instrSession,
    ViBoolean    resolutionBandwidthAuto,
    ViReal64    resolutionBandwidth,
    ViReal64    reserved
)
{
    ViStatus    error = VI_SUCCESS;
    reserved;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    /* Function not available
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH_AUTO,
                                                 resolutionBandwidthAuto), 2, "Resolution Bandwidth Auto");
    if (resolutionBandwidthAuto == VI_FALSE)
    {
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                    RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH,
                                                    resolutionBandwidth), 3, "Resolution Bandwidth");
    }
     */
    resolutionBandwidthAuto;
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH,
                                                resolutionBandwidth), 3, "Resolution Bandwidth");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function specifies the ratio resolution bandwidth (Hz) / span (Hz). The ratio
/// HIFN  to be entered is reciprocal to the ratio span/RBW used in manual control
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resolutionBandwidthRatio/Specifies the ratio resolution bandwidth (Hz) / span (Hz). The ratio 
/// HIPAR resolutionBandwidthRatio/to be entered is reciprocal to the ratio span/RBW used in manual 
/// HIPAR resolutionBandwidthRatio/control.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramBandwidthRatio (
    ViSession instrSession,
    ViReal64 resolutionBandwidthRatio
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH_RATIO,
                                                resolutionBandwidthRatio), 2, "Resolution Bandwidth Ratio");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the minimum or maximum ratio resolution bandwidth (Hz) / span 
/// HIFN  (Hz). The ratio to be entered is reciprocal to the ratio span/RBW used 
/// HIFN  in manual control.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH_RATIO_MIN_MAX
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]BANDwidth|BWIDth[:RESolution]:RATio
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR ratio/Sets the minimum or maximum ratio resolution bandwidth (Hz) / span 
/// HIPAR ratio/(Hz). The ratio to be entered is reciprocal to the ratio span/RBW used 
/// HIPAR ratio/in manual control.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramBandwidthRatioMinMax(
    ViSession    instrSession,
    ViInt32    ratio
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH_RATIO_MIN_MAX,
                                                ratio), 2, "Ratio"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}


/// HIFN  This function Queries minimal and maximal ratio resolution bandwidth 
/// HIFN  (Hz) / span (Hz).
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH_RATIO_GET_MIN
/// HIFN     RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH_RATIO_GET_MAX
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]BANDwidth|BWIDth[:RESolution]:RATio
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR minimum/Queries the minimal ratio resolution bandwidth (Hz) / span (Hz).
/// HIPAR maximum/Queries the maximal ratio resolution bandwidth (Hz) / span (Hz).
ViStatus _VI_FUNC rsemi_QuerySpectrogramBandwidthRatioMinMax(
    ViSession    instrSession,
    ViReal64*    minimum,
    ViReal64*    maximum
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH_RATIO_GET_MIN,
                                                minimum), 2, "minimum"); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_SPEM_RESOLUTION_BANDWIDTH_RATIO_GET_MAX,
                                                maximum), 3, "maximum");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function configures Spectogram display parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR color/This control defines the frame color setting.
/// HIPAR reserved/Reserved
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDisplayParameters (ViSession instrSession,
                                                               ViInt32 window, 
                                                               ViInt32 color,
                                                               ViInt32 reserved)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    sprintf (buffer, "Win%ld", window); 

    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_SPEM_COLOR, color),
                                            3, "Color");
    reserved;
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures Spectogram color parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR shape/This control defines the shape and focus of the color curve for the 
/// HIPAR shape/spectrogram result.
/// HIPAR colorStyle/This control sets the color scheme for the spectrogram. 
/// HIPAR colorLower/This control sets the lower percentage boundary of the spectrogram.
/// HIPAR colorUpper/This control sets the upper percentage boundary of the spectrogram.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramColors (ViSession instrSession,
                                                    ViReal64 shape,
                                                    ViInt32 colorStyle,
                                                    ViReal64 colorLower,
                                                    ViReal64 colorUpper)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", RSEMI_ATTR_SPEM_SHAPE, shape),
                                            2, "Shape");
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_SPEM_COLOR_STYLE, colorStyle),
                                            3, "Color Style");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", RSEMI_ATTR_SPEM_COLOR_LOW, colorLower),
                                             4, "Color Lower");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", RSEMI_ATTR_SPEM_COLOR_UPPER, colorUpper),
                                             5, "Color Upper");
    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function sets the color settings for the spectrogram result 
/// HIFN  display to its default state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDefaultColors (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_SPEM_COLOR_DEFAULT, NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures Spectogram measurement parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR historyDepth/This control sets the number of frames to be stored in the history 
/// HIPAR historyDepth/buffer.
/// HIPAR timeStamp/Toggles the Spectrogram Measurement option.
/// HIPAR frameCount/This control sets the number of frames recorded in a single sweep 
/// HIPAR frameCount/measurement.
/// HIPAR continueFrame/Determines whether the results of the last measurement are deleted 
/// HIPAR continueFrame/before starting a new measurement in single sweep mode.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMeasurementParameters (ViSession instrSession,
                                                                   ViInt32 window,
                                                                   ViInt32 historyDepth,
                                                                   ViBoolean timeStamp,
                                                                   ViInt32 frameCount,
                                                                   ViBoolean continueFrame)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 

    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_SPEM_HDEP, historyDepth),
                                            3, "History Depth");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_SPEM_TIME_STAMP_STATE, timeStamp),
                                            4, "Time Stamp");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_SPEM_FRAME_COUNT, frameCount),
                                            5, "Frame Count");
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_SPEM_CONTINUOUS, continueFrame),
                                            6, "Continue Frame");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the frame if no measurement is running. During a 
/// HIFN  measurement, the value is fixed to 0 and this command is available only 
/// HIFN  if no measurement is running or after a single sweep has ended.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR frame_number/This control selects the frame if no measurement is running. During a 
/// HIPAR frame_number/measurement, the value is fixed to 0 and this command is available only 
/// HIPAR frame_number/if no measurement is running or after a single sweep has ended.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramSelectedFrame (ViSession instrSession,
                                                           ViInt32 window, ViInt32 frame_number)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_SPEM_FRAME_SELECT, frame_number),
                                            3, "Frame number");
    
Error:
     Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function selects the minimum/maximum frame if no measurement is 
/// HIFN  running. During a measurement, the value is fixed to 0 and this command 
/// HIFN  is available only if no measurement is running or after a single sweep 
/// HIFN  has ended.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minMax/Selects minimal/maximal frame.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramSelectedFrameMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    minMax
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SPEM_FRAME_SELECT_MIN_MAX,
                                                minMax), 3, "Min Max"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}


/// HIFN  This function queries the minumum and maximum frame that can be 
/// HIFN  selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR minimum/This control queries the minimum frame that can be selected.
/// HIPAR maximum/This control queries the maximum frame that can be selected.
ViStatus _VI_FUNC rsemi_QuerySpectrogramSelectedFrameMinMax (ViSession instrSession,
                                                             ViInt32 window,
                                                             ViInt32 *minimum,
                                                             ViInt32 *maximum)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_GetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SPECTROGRAM_FRAME_SELECT_GET_MIN,
                                                minimum), 3, "minimum"); 
    
    viCheckParm (rsemi_GetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SPECTROGRAM_FRAME_SELECT_GET_MAX,
                                                maximum), 4, "maximum");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  This function deletes the spectrogram result display and the history 
/// HIFN  buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_SpectrogramClear (ViSession instrSession, ViInt32 window)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 

    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_SPEM_CLEAR, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the type of FFT window that you want to use in 
/// HIFN  realtime mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR FFTWindowType/Selects the type of FFT window that you want to use in realtime mode.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramFFTWindow (ViSession instrSession,
                                                       ViInt32 FFTWindowType)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
        RSEMI_ATTR_SPEM_FFT_WINDOW, FFTWindowType), 3, "FFT Window Type");
Error:
     Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function configures the reference level
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referenceLevel/The calibrated vertical position of the captured data used as a 
/// HIPAR referenceLevel/reference for amplitude measurements. This is typically set to a value 
/// HIPAR referenceLevel/slightly higher than the highest expected signal level. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramReferenceLevel(
    ViSession    instrSession,
    ViReal64    referenceLevel
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_SPEM_REFERENCE_LEVEL,
                                                referenceLevel), 2, "Reference Level");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the offset value to the reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referenceLevelOffset/Specifies the offset value to the reference level.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramReferenceLevelOffset(
    ViSession    instrSession,
    ViReal64    referenceLevelOffset
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_SPEM_REFERENCE_LEVEL_OFFSET,
                                                referenceLevelOffset), 2, "Reference Level Offset");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Selects the unit for measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerUnit/Selects the unit for power in the selected measurement window.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramUnit(
    ViSession    instrSession,
    ViInt32    powerUnit
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "Win0",
                                               RSEMI_ATTR_POWER_UNIT,
                                               powerUnit), 2, "Power Unit");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Selects the nominal input impedance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR inputImpedance/Selects the nominal input impedance.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramInputImpedance(
    ViSession    instrSession,
    ViInt32    inputImpedance
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsemi_SetAttributeViInt32 (instrSession, "",
        RSEMI_ATTR_INPUT_IMPEDANCE, inputImpedance), 2, "Input Impedance");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Toggles the RF input of the R&S ESR between AC and DC coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR inputCoupling/Toggles the RF input of the R&S ESR between AC and DC coupling.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramInputCoupling(
    ViSession    instrSession,
    ViInt32    inputCoupling
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_RF_INPUT_COUPLING,
                                               inputCoupling), 2, "Input Coupling");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the input attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoAttenuation/Enables or disables the automatic configuration of the attenuation 
/// HIPAR autoAttenuation/level.
/// HIPAR attenuation/Sets the attenuation level at the RF input.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramRFAttenuation(
    ViSession    instrSession,
    ViBoolean    autoAttenuation,
    ViReal64    attenuation
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_AUTO_RANGE,
                                               autoAttenuation), 2, "Auto Attenuation");
    if (autoAttenuation == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                               RSEMI_ATTR_RF_ATTENUATION_LEVEL,
                                               attenuation), 3, "Attenuation");
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function turns the 20 dB preamplifier on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR preamplifier/Enables and disables the 20 dB preamplifier for the instrument. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramPreamplifier(
    ViSession    instrSession,
    ViBoolean    preamplifier
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_PREAMPLIFIER_STATE,
                                               preamplifier), 2, "Preamplifier");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function turns noise cancellation on and off.
/// HIFN     If noise cancellation is on, the R&S  ESR performs a reference 
/// HIFN  measurement to determine its inherent noise and subtracts the result 
/// HIFN  from the channel power measurement result (first active trace only).
/// HIFN     The inherent noise of the instrument depends on the selected center 
/// HIFN  frequency, resolution bandwidth and level setting. Therefore, the 
/// HIFN  correction function is disabled whenever one of these parameters is 
/// HIFN  changed. A corresponding message is displayed on the screen. Noise 
/// HIFN  correction must be turned on again manually after the change.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noiseCorrection/Turns noise cancellation on and off.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramNoiseCorrection(
    ViSession    instrSession,
    ViInt32    noiseCorrection
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SPEM_NOISE_CORRECTION,
                                               noiseCorrection), 2, "Noise Correction");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Selects the scaling type of the y-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Sets the trace.
/// HIPAR scalingType/Selects the scaling of the y-axis.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDisplayVerticalScaling(
    ViSession    instrSession,
    ViInt32    trace,
    ViInt32    scalingType
)
{
    ViStatus    error = VI_SUCCESS;
    trace;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_AMPLITUDE_SCALING_TYPE,
                                               scalingType), 3, "Scaling Type");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the scale type of the Y axis (absolute or 
/// HIFN  relative) in the selected measurement window.
/// HIFN  When SYSTem:DISPlay is set to OFF, this command has no immediate 
/// HIFN  effect on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxisGridMode/Defines the scale type of the Y axis (absolute or relative) in the 
/// HIPAR yAxisGridMode/selected measurement window.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDisplayVerticalGridMode(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    yAxisGridMode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSEMI_ATTR_SPEM_DISPLAY_VERTICAL_GRID_MODE,
                                               yAxisGridMode), 3, "Y Axis Grid Mode");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the display range of the Y axis (level axis) in 
/// HIFN  the selected measurement window with logarithmic scaling 
/// HIFN  (DISP:TRAC:Y:SPAC LOG).
/// HIFN  For linear scaling, (DISP:TRAC:Y:SPAC LIN | PERC) the display range is 
/// HIFN  fixed and cannot be modified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR range/Defines the display range of the Y axis (level axis) in the selected 
/// HIPAR range/measurement window with logarithmic scaling (DISP:TRAC:Y:SPAC LOG).
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDisplayLogRange(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    range
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_SPEM_DISPLAY_LOG_RANGE,
                                                range), 3, "Range");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the display zooming parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR zoomState/This control activates and deactivates the zoom mode.
/// HIPAR startFrequency/Defines the start frequency of the zoom area.
/// HIPAR startTime/Defines the start time of the zoom area.
/// HIPAR stopFrequency/Defines the stop frequency of the zoom area.
/// HIPAR stopTime/Defines the stop time of the zoom area.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDisplayZoom(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    zoomState,
    ViReal64    startFrequency,
    ViReal64    startTime,
    ViReal64    stopFrequency,
    ViReal64    stopTime
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";     
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  

    viCheckParm ( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                RSEMI_ATTR_SPEM_DISPLAY_ZOOM_STATE, zoomState),
                3, "Zoom State");
    
    if (zoomState == VI_TRUE) 
    {
        viCheckErr( viPrintf (instrSession, "DISP:WIND%ld:ZOOM:AREA %.12lG,%.12lG,%.12lG,%.12lG\n", window, startFrequency, startTime, stopFrequency, stopTime));
        
        checkErr( rsemi_CheckStatus (instrSession)); 
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*
 *------------------- Layout ------------------------------------------------*
 *===========================================================================*/
/// HIFN This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR direction/Direction the window is added relative to the window set with "Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/Returns the name of the new window.
ViStatus _VI_FUNC rsemi_AddSpectrogramWindow (
    ViSession instrSession,
    ViInt32 stringSize,
    ViString windowName,
    ViInt32 direction,
    ViInt32 windowType,
    ViChar returnedWindowName[]
)
{
    ViStatus    error   = VI_SUCCESS;
    size_t      length;
    ViChar*     pbuffer = NULL;
    ViChar*     p2buf = NULL;
    ViReal64     discrete;
    ViChar       *p2direction = NULL;
    ViChar       *p2type = NULL;   
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    do
    {
        discrete = rsemi_rngLayoutDirection.rangeValues[i].discreteOrMinValue;
        p2direction = rsemi_rngLayoutDirection.rangeValues[i].cmdString;

        if (p2direction == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Direction");

        if (fabs(discrete - direction) <= 0.01) break;

        i++;

    } while (i != 0);
    i=0;
    do
    {
        discrete = rsemi_rngSpectrogramLayoutQueryWindowType.rangeValues[i].discreteOrMinValue;
        p2type = rsemi_rngSpectrogramLayoutQueryWindowType.rangeValues[i].cmdString;

        if (p2type == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Window Type");

        if (fabs(discrete - windowType) <= 0.01) break;

        i++;

    } while (i != 0);        
    
    viCheckErr (viPrintf (instrSession, "LAY:ADD? '%s',%s,%s", windowName, p2direction, p2type));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, NULL));
    
    if (returnedWindowName != NULL)
    {
        size_t len = strstr (pbuffer+1,"'") - pbuffer - 1;
        length = (size_t) stringSize - 1 <= len ? (size_t) stringSize-1 : len;
        p2buf = strtok (pbuffer, "'");
        strncpy (returnedWindowName, p2buf, length);
    }
 
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    if (pbuffer != NULL) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function removes a window from the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
ViStatus _VI_FUNC rsemi_RemoveSpectrogramWindow (
    ViSession instrSession,
    ViString windowName
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_SPEM_LAYOUT_WINDOW_REMOVE, windowName),
                2, "Window Name");                

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsemi_ReplaceSpectrogramWindow (
    ViSession instrSession,
    ViString windowName,
    ViInt32 windowType
)
{
    ViStatus    error   = VI_SUCCESS;
    ViReal64     discrete;
    ViChar       *p2type = NULL;   
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    do
    {
        discrete = rsemi_rngSpectrogramLayoutQueryWindowType.rangeValues[i].discreteOrMinValue;
        p2type = rsemi_rngSpectrogramLayoutQueryWindowType.rangeValues[i].cmdString;

        if (p2type == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Window Type");

        if (fabs(discrete - windowType) <= 0.01) break;

        i++;

    } while (i != 0);
        
    viCheckErr (viPrintf (instrSession, "LAY:REPL '%s',%s", windowName, p2type));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function queries the name and index of all active windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR bufferSize/This control specifies the size of the buffer.
/// HIPAR results/Returns the name and index of all active windows.
ViStatus _VI_FUNC rsemi_QuerySpectrogramWindowsList (
    ViSession instrSession,
    ViInt32 bufferSize,
    ViChar results[]
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL)); 
    
    viCheckParm (rsemi_GetAttributeViString (instrSession, "", RSEMI_ATTR_SPEM_LAYOUT_WINDOW_LIST, 
                                                                bufferSize, results), 3, "Results");                


Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function queries the index of a particular display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR index/Index number of the window.
ViStatus _VI_FUNC rsemi_QuerySpectrogramWindowIndex (
    ViSession instrSession,
    ViString windowName,
    ViInt32 *index
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *buffer = NULL;
    ViChar      *pbuff = NULL; 
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (viPrintf (instrSession, "LAY:IDEN? '%s'\n", windowName));
    
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &buffer, NULL));
    
    pbuff = strtok (buffer, ",\n");
    *index = atol(pbuff);
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function queries the name of a particular display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR index/Index number of the window.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to. 
ViStatus _VI_FUNC rsemi_QuerySpectrogramWindowName (
    ViSession instrSession,
    ViInt32 stringSize,
    ViInt32 index,
    ViChar windowName[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SPEM_LAYOUT_QUERY_WINDOW_NAME,
                                                 "Window",
                                                 index - 1,
                                                 10,
                                                 rep_cap));
    viCheckParm (rsemi_GetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_SPEM_LAYOUT_QUERY_WINDOW_NAME, 
                                                                stringSize, windowName), 3, "Window Name");
    
                                             
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function changes the position of a splitter
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR firstWindowName/String containing the number or name of the first window the splitter controls.
/// HIPAR secondWindowName/String containing the number or name of the second window the splitter controls. 
/// HIPAR position/New vertical or horizontal position of the splitter in percent of the screen area.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramSplitterPosition (
    ViSession instrSession,
    ViString firstWindowName,
    ViString secondWindowName,
    ViInt32 position
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      cmd [100] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
        
    sprintf (cmd, "LAY:SPL '%s','%s',%ld", firstWindowName, secondWindowName, position);
    
    viCheckErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  Defines the type of display and the evaluation of the traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/This control sets the trace.
/// HIPAR mode/This control defines the type of display and the evaluation of the 
/// HIPAR mode/trace.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramTrace(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViInt32    mode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");
    
    sprintf (buffer, "Win%ld,TR%ld", window, trace); 
    
    switch (mode)
    {
        case RSEMI_VAL_TRACE_MODE_BLANK:
            viCheckErr ( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                        RSEMI_ATTR_SPEM_TRACE_STATE,
                                                        VI_FALSE));
        break;
        default:
            viCheckErr ( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                        RSEMI_ATTR_SPEM_TRACE_STATE,
                                                        VI_TRUE));
            
            viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SPEM_TRACE_MODE,
                                                mode), 4, "Mode"); 
        break;
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Selects the detector for the data acquisition in the selected trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Sets the trace.
/// HIPAR detector/Selects the detector for the data acquisition in the selected trace.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramTraceDetector(
    ViSession    instrSession,
    ViInt32    trace,
    ViInt32    detector
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";  
     
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "TR%ld", trace);  
    viCheckParm(rsemi_SetAttributeViInt32 (instrSession, buffer,
        RSEMI_ATTR_TRACE_DETECTOR_SELECTION, detector), 3, "Detector");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function specifies whether or not the traces in Min Hold, Max 
/// HIFN  Hold and Average mode are reset after a parameter is changed.
/// HIFN     Usually the measurement must be restarted after a parameter change, 
/// HIFN  before an evaluation of the measurement results is performed (e.g. with 
/// HIFN  a marker). In cases in which a change causes a compulsory new 
/// HIFN  measurement, the trace is automatically reset in order to prevent 
/// HIFN  erroneous measurements of previous measurement results (e.g. when the 
/// HIFN  span changes). For applications in which this behavior is not desired, 
/// HIFN  this mechanism can be switched off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/This control sets the trace.
/// HIPAR resetatChange/Specifies whether or not the traces with peak or minimum value 
/// HIPAR resetatChange/detection are reset after specific parameter changes.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramResetBehavior(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViBoolean    resetatChange
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");

    sprintf (rep_cap, "Win%ld,TR%ld", window, trace);

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSEMI_ATTR_SPEM_TRACE_RESET_BEHAVIOR,
                                                 resetatChange), 4, "Reset At Change");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function copies one trace array to another trace array.  Any data 
/// HIFN  in the destination trace is over written. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR destinationTrace/Selects the destination trace.
/// HIPAR sourceTrace/Selects the source trace.
ViStatus _VI_FUNC rsemi_CopySpectrogramTrace(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    destinationTrace,
    ViInt32    sourceTrace
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, destinationTrace, 1, 6), 3, "Destination Trace");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, sourceTrace, 1, 6), 4, "Source Trace");        
    
    viCheckErr (viPrintf (instrSession, "TRAC%ld:COPY TRACE%ld,TRACE%ld\n", window, destinationTrace, sourceTrace));
    
    checkErr (rsemi_CheckStatus (instrSession)); 

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function stores the selected trace in the specified window in a 
/// HIFN  file with ASCII format. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/The file name includes indication of the path and the drive name. 
/// HIPAR fileName/Indication of the path complies with DOS conventions.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR decimalSeparator/Defines which decimal separator (decimal point or comma) is to be used 
/// HIPAR decimalSeparator/for outputting measurement data to the file in ASCII format.
ViStatus _VI_FUNC rsemi_StoreTraceToFile(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    trace,
    ViInt32    decimalSeparator,
    ViString    fileName
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");
    if (fileName == VI_NULL) 
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "File Name");

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SPEM_FILE_DEC_SEPARATOR,
                                               decimalSeparator), 4, "Decimal Separator");
    
    viCheckErr (viPrintf (instrSession,"MMEM:STOR%ld:TRAC %ld,'%s'\n", window, trace, fileName));
        
    checkErr (rsemi_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Enables and disables the delta markers on the specified trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control sets the marker.
/// HIPAR markerEnabled/Enables and disables the markers.
/// HIPAR trace/Sets the trace.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViBoolean    markerEnabled,
    ViInt32    trace
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_ENABLED,
                                               markerEnabled), 4, "Marker Enabled");
    
    if (markerEnabled == VI_TRUE)
    {
       viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_TRACE,
                                               trace), 5, "Trace");    
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Moves the active marker to the specified horizontal position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control sets the marker.
/// HIPAR position/Sets the position of the marker on a particular coordinate on the 
/// HIPAR position/x-axis.
ViStatus _VI_FUNC rsemi_SpectrogramMoveMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64    position
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm(rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_POSITION,
                                               position), 4, "Position");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Queries the measured value of the vertical marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control sets the marker.
/// HIPAR markerAmplitude/Returns the measured value of the vertical marker position.
ViStatus _VI_FUNC rsemi_QuerySpectrogramMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64*    markerAmplitude
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_MARKER_AMPLITUDE, 
                markerAmplitude), 4, "Marker Amplitude");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function turns off all the markers in selected measurement 
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_SpectrogramDisableAllMarkers(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_AOFF, VI_NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function makes the selected marker frequency to be the center 
/// HIFN  frequency or sets the reference level to the power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR instrumentSetting/Specifies the instrument setting to be set from the marker position.
ViStatus _VI_FUNC rsemi_SpectrogramSetInstrumentFromMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViInt32    instrumentSetting
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        

    
    sprintf (buffer, "Win%ld,M%ld", window, marker);  
    
    switch(instrumentSetting)
    {
        case RSEMI_VAL_INSTRUMENT_SETTING_FREQUENCY_CENTER:
            viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                RSEMI_ATTR_MARKER_TO_CENTER, VI_NULL));
        break;
        case RSEMI_VAL_INSTRUMENT_SETTING_REFERENCE_LEVEL:
               viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                RSEMI_ATTR_SPEM_MARKER_TO_REFERENCE, VI_NULL));
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Instrument Setting");
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the range to be zoomed around marker 1 in the 
/// HIFN  selected measurement window. Marker 1 is activated first, if necessary.
/// HIFN  The subsequent frequency sweep is stopped at the marker position and 
/// HIFN  the frequency of the signal is counted. This frequency becomes the new 
/// HIFN  center frequency, and the zoomed span is set. In order to recognize the 
/// HIFN  end of the operation the synchronization to the sweep end should be 
/// HIFN  activated. This is only possible in single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerZoom/Defines the range to be zoomed around marker 1 in the selected 
/// HIPAR markerZoom/measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerZoom(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64    markerZoom,
    ViInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    checkErr (rsemi_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rsemi_SetOPCTimeout (instrSession, timeout));
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_ZOOM,
                                               markerZoom), 4, "Marker Zoom");
    
Error:
    if (old_timeout >= 0)    
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the step size of the rotary knob for marker or 
/// HIFN  delta marker value changes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR stepSize/Defines the step size. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerStepSize(
    ViSession    instrSession,
    ViInt32    stepSize
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SPEM_MARKER_STEP_SIZE,
                                               stepSize), 2, "Step Size");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the frame number of the specified marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR markerNumber/Selects the marker to configure.
/// HIPAR frameNumber/This control defines the frame number of the specified marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerFrameNumber (ViSession instrSession,
                                                               ViInt32 window, ViInt32 markerNumber,
                                                               ViInt32 frameNumber)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, markerNumber, 1, 4), 3, "Marker Number");
    
    sprintf (buffer, "Win%ld,M%ld", window, markerNumber);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SPEM_MARK_FRAME_NUMBER,
                                                frameNumber), 4, "Frame Number");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines the search area of the specified marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR markerNumber/Selects the marker to configure.
/// HIPAR searchArea/This control sets teh search area.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerSearchArea (ViSession instrSession,
                                                              ViInt32 window, ViInt32 markerNumber,
                                                              ViInt32 searchArea)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, markerNumber, 1, 4), 3, "Marker Number");
    
    sprintf (buffer, "Win%ld,M%ld", window, markerNumber);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_AREA,
                                                searchArea), 4, "Search Area");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Enables and disables marker search limits, and sets the sides of the 
/// HIFN  limits of the marker search range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control sets the marker.
/// HIPAR searchLimits/Enables and disables the marker search limit.
/// HIPAR searchLimitLeft/Sets the left limit of the marker search range.
/// HIPAR searchLimitRight/Sets the right limit of the marker search range.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerSearchLimits(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViBoolean    searchLimits,
    ViReal64    searchLimitLeft,
    ViReal64    searchLimitRight
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);  
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_SEARCH_LIMITS_STATE,
                                               searchLimits), 4, "Search Limits");
    if (searchLimits == VI_TRUE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_SEARCH_LIMITS_RIGHT,
                                               searchLimitRight), 6, "Search Limit Right");
        
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_SEARCH_LIMITS_LEFT,
                                               searchLimitLeft), 5, "Search Limit Left");
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the limits of the search range for markers 
/// HIFN  and delta markers to the zoom area. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR searchLimitsZoom/Sets the limits of the search range for markers and delta markers to 
/// HIPAR searchLimitsZoom/the zoom area.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerSearchLimitsZoom(
    ViSession    instrSession,
    ViBoolean    searchLimitsZoom
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_SPEM_MARKER_SEARCH_LIMITS_ZOOM,
                                                 searchLimitsZoom), 2, "Search Limits Zoom");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function positions the specified marker on the specified place.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR markerNumber/Selects the marker.
/// HIPAR searchMode/This control positions the marker to the desired position in the trace 
/// HIPAR searchMode/memory.
ViStatus _VI_FUNC rsemi_SpectrogramMarkerSearch (ViSession instrSession,
                                                 ViInt32 window, 
                                                 ViInt32 markerNumber,
                                                 ViInt32 searchMode)
{
    ViStatus    error = VI_SUCCESS;
     ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MARKER_SEARCH_PEAK,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_MARKER_SEARCH_PEAK,
                                                 "Marker",
                                                 markerNumber - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker Number");     
    
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    switch (searchMode){
        case RSEMI_VAL_SPECM_MARK_MAX_PEAK:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_MARKER_SEARCH_PEAK, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MAX_NEXT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_MARKER_SEARCH_PEAK_NEXT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MAX_LEFT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_MARKER_SEARCH_PEAK_LEFT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MAX_RIGHT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_MARKER_SEARCH_PEAK_RIGHT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MIN_PEAK:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_MARKER_SEARCH_MIN, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MIN_NEXT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_MARKER_SEARCH_MIN_NEXT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MIN_LEFT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_MARKER_SEARCH_MIN_LEFT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MIN_RIGHT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_MARKER_SEARCH_MIN_RIGHT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MAX_PEAK:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_Y_MAXIMUM, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MAX_NEXT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_Y_MAXIMUM_NEXT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MAX_ABOVE:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_Y_MAXIMUM_NEXT_ABOVE, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MAX_BELOW:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_Y_MAXIMUM_NEXT_BELOW, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MIN_PEAK:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_Y_MINIMUM, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MIN_NEXT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_Y_MINIMUM_NEXT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MIN_ABOVE:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_Y_MINIMUM_NEXT_ABOVE, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MIN_BELOW:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_Y_MINIMUM_NEXT_BELOW, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_XY_MAX:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_XY_MAXIMUM, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_XY_MIN:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_XY_MINIMUM, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_TRIGGER:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_MARKER_SEARCH_Y_TRIGGER, NULL));
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Search Mode");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures an automatic peak search action
/// HIFN  for Marker at the end of each particular sweep. This function may be 
/// HIFN  used during adjustments of a device under test to keep track of the 
/// HIFN  actual peak marker position and level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
/// HIPAR autoPeak/Switches the peak auto search state. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerSearchPeakAuto(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViInt32    markerSearch,
    ViBoolean    autoPeak
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[25] = "";
    ViAttr    attr[2] = {RSEMI_ATTR_SPEM_MARKER_SEARCH_PEAK_AUTO,
                       RSEMI_ATTR_SPEM_MARKER_SEARCH_MIN_AUTO};

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, markerSearch, RSEMI_VAL_MARKER_SEARCH_HIGHEST, RSEMI_VAL_MARKER_SEARCH_MINIMUM), 4, "Marker Search");
    
    sprintf (rep_cap, "Win%ld,M%ld", window, marker);  

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                                 attr[markerSearch - 1],
                                                 autoPeak), 5, "Auto Peak");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures state of local oscillator supression.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR localOscillatorSuppression/Switches the local oscillator suppression for peak search on or off. 
/// HIPAR localOscillatorSuppression/This setting is valid for all markers and delta markers in all 
/// HIPAR localOscillatorSuppression/measurement windows. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerSearchLocalOscillator(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    localOscillatorSuppression
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSEMI_ATTR_SPEM_MARKER_LOCAL_OSCILLATOR_SUPPRESSION,
                                                 localOscillatorSuppression), 3, "Local Oscillator Suppression");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the marker peak excursion The marker peak 
/// HIFN  excursion specifies the minimum amplitude variation that can be 
/// HIFN  recognized as a peak or minimum by the marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR peakExcursion/Sets the minimum amplitude variation of the signal in dB that the 
/// HIPAR peakExcursion/marker can identify as a peak. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerSearchPeakExcursion(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    peakExcursion
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_SPEM_MARKER_PEAK_EXCURSION,
                                                peakExcursion), 3, "Peak Excursion");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures marker threshold values. The marker threshold 
/// HIFN  specifies a lower bound for ALL marker search functions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR thresholdState/Switches on or off the threshold line in the selected measurement 
/// HIPAR thresholdState/window.
/// HIPAR markerThreshold/Sets the threshold value for the maximum/minimum search of markers 
/// HIPAR markerThreshold/with marker search functions MAX PEAK, NEXT PEAK, etc in the selected 
/// HIPAR markerThreshold/measurement window.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerSearchThreshold(
    ViSession    instrSession,
    ViBoolean    thresholdState,
    ViReal64    markerThreshold
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_SPEM_MARKER_THRESHOLD_STATE,
                                                 thresholdState), 2, "Threshold State");

    if (thresholdState == VI_TRUE)
    {
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                    RSEMI_ATTR_SPEM_MARKER_THRESHOLD,
                                                    markerThreshold), 3, "Marker Threshold");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function toggles the display of the marker table. With automatic 
/// HIFN  display, the table is displayed if 2 or more markers are active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR markerTable/This control toggles the display of the marker table. With automatic 
/// HIPAR markerTable/display, the table is displayed if 2 or more markers are active.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerTable(
    ViSession    instrSession,
    ViInt32    markerTable
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SPEM_MARKER_TABLE,
                                               markerTable), 2, "Marker Table");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function enables the active marker on the specified trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR state/Specifies whether the specified marker is enabled.
/// HIPAR trace/Selects trace number the marker is positioned on.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViBoolean    state,
    ViInt32    trace
)
{
    return rsemi_ConfigureDeltaMarker(instrSession, window, deltaMarker, state, trace);    
}

/// HIFN  This function configures the delta marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR mode/Switches between relative and absolute frequency input of the delta 
/// HIPAR mode/marker (or time with span = 0). It affects all delta markers 
/// HIPAR mode/independent of the measurement window.. 
/// HIPAR position/Sets the position of selected delta marker in the indicated 
/// HIPAR position/measurement window. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerPosition(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViInt32    mode,
    ViReal64    position
)
{
    return rsemi_ConfigureDeltaMarkerPosition(instrSession, window, deltaMarker, mode, position);
}

/// HIFN  This function returns the horizontal position and the marker amplitude 
/// HIFN  level of the selected delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR mode/Switches between absolute and relative position value.
/// HIPAR position/Returns the absolute or relative position of selected delta marker.
/// HIPAR amplitude/Returns the measured value of the selected delta marker in the 
/// HIPAR amplitude/indicated measurement window. The corresponding delta marker will be 
/// HIPAR amplitude/activated, if necessary. The output is always a relative value referred 
/// HIPAR amplitude/to marker 1 or to the reference position (reference fixed active). 
ViStatus _VI_FUNC rsemi_QuerySpectrogramDeltaMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViInt32    mode,
    ViReal64*    position,
    ViReal64*    amplitude
)
{
    return rsemi_QueryDeltaMarker(instrSession, window, deltaMarker, mode, position, amplitude);
}

/// HIFN  This function turns off all the delta markers in selected measurement 
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_SpectrogramDisableAllDeltaMarkers(
    ViSession    instrSession,
    ViInt32    window
)
{
    return rsemi_DisableAllDeltaMarker(instrSession, window);
}

/// HIFN  This function defines the frame number of the specified delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarkerNumber/Selects the delta marker to configure.
/// HIPAR frameNumber/This control defines the frame number of the specified delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerFrameNumber (ViSession instrSession,
                                                                    ViInt32 window, 
                                                                    ViInt32 deltaMarkerNumber,
                                                                    ViInt32 frameNumber)
{
    ViStatus    error = VI_SUCCESS;
       ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SPEM_DMARKER_FRAME_NUMBER,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SPEM_DMARKER_FRAME_NUMBER,
                                                 "Marker",
                                                 deltaMarkerNumber - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker Number");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DMARKER_FRAME_NUMBER,
                                                frameNumber), 4, "Frame Number");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines the search area of the specified delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/Selects the marker to configure.
/// HIPAR searchArea/This control sets teh search area.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerSearchArea (ViSession instrSession,
                                                                   ViInt32 window,
                                                                   ViInt32 deltaMarker,
                                                                   ViInt32 searchArea)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_AREA,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_AREA,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_AREA,
                                                searchArea), 4, "Search Area");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function sets the persistence spectrum parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/This control selects the trace number.
/// HIPAR enablePersistence/This control switches persistence in the persistence spectrum on and 
/// HIPAR enablePersistence/off.
/// HIPAR duration/This control sets the duration of the persistence.
/// HIPAR granularity/This control sets the persistence granularity.
ViStatus _VI_FUNC rsemi_ConfigurePersistenceSpectrumParameters (ViSession instrSession,
                                                                ViInt32 window,
                                                                ViInt32 trace,
                                                                ViBoolean enablePersistence,
                                                                ViReal64 duration,
                                                                ViReal64 granularity)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld,TR%ld", window, trace);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                 RSEMI_ATTR_PSPEM_STATE, 
                                                 enablePersistence), 3, "Enable Persistence");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_PSPEM_DURATION,
                                                duration), 4, "Duration");
    
     sprintf (buffer, "Win%ld", window); 
     
     viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_PSPEM_GRANULARITY,
                                                granularity), 5, "Granularity");
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function sets the persistence spectrum display parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR shape/This control defines the shape and focus of the color curve for the 
/// HIPAR shape/persistence spectrum result display.
/// HIPAR style/This control sets the color scheme for the persistence spectrum.
/// HIPAR colorLower/This controls sets the lower percentage boundary of the persistence 
/// HIPAR colorLower/spectrum.
/// HIPAR colorUpper/This controls sets the upper percentage boundary of the persistence 
/// HIPAR colorUpper/spectrum.
ViStatus _VI_FUNC rsemi_ConfigurePersistenceSpectrumDisplayParameters (ViSession instrSession,
                                                                       ViInt32 window,
                                                                       ViReal64 shape,
                                                                       ViInt32 style,
                                                                       ViReal64 colorLower,
                                                                       ViReal64 colorUpper)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
     
    viCheckParm(rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_PSPEM_SHAPE,
                                                shape), 3, "Shape");
    
    viCheckParm(rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_PSPEM_COLOR_STYLE,
                                                style), 4, "Style");
        
    viCheckParm(rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_PSPEM_COLOR_LOWER,
                                                colorLower), 5, "Color Lower"); 
                                             
    
    viCheckParm(rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_PSPEM_COLOR_UPPER,
                                                colorUpper), 6, "Color Upper"); 
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function sets the color settings for the persistence
/// HIFN spectrum result display to its default state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or rsemi_InitWithOptions function.
ViStatus _VI_FUNC rsemi_ConfigurePersistenceSpectrumDefaultColors (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr(rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_PSPEM_COLOR_DEFAULT, NULL));  
                                                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function reduces the range of the color map of the persistence 
/// HIFN  spectrum if there are no hits at the start or end of the value range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR truncate/This control reduces the range of the color map of the persistence 
/// HIPAR truncate/spectrum if there are no hits at the start or end of the value range.
ViStatus _VI_FUNC rsemi_ConfigurePersistenceSpectrumTruncate (ViSession instrSession,
                                                              ViInt32 window,
                                                              ViBoolean truncate)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm(rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_PSPEM_TRUN,
                                                truncate), 3, "Truncate"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function sets the persistence spectrum trace style.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/This control selects the trace number.
/// HIPAR symbol/This control sets the display style of the persistence spectrum.
ViStatus _VI_FUNC rsemi_ConfigurePersistenceSpectrumTraceStyle (ViSession instrSession,
                                                                ViInt32 window,
                                                                ViInt32 trace,
                                                                ViInt32 symbol)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld,TR%ld", window, trace);
    
    viCheckParm(rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_PSPEM_TRACE_STYLE,
                                                symbol), 4, "Symbol");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function sets the persistence spectrum maxhold trace parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/This control selects the trace number.
/// HIPAR enableMaxhold/This control switches the maxhold trace in the persistence spectrum on 
/// HIPAR enableMaxhold/and off.
/// HIPAR intensity/The control defines the color intensity of the maxhold persistence 
/// HIPAR intensity/spectrum.
ViStatus _VI_FUNC rsemi_ConfigurePersistenceSpectrumMaxholdIntensity (ViSession instrSession,
                                                                     ViInt32 window, 
                                                                     ViInt32 trace,
                                                                     ViBoolean enableMaxhold,
                                                                     ViInt32 intensity)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld,TR%ld", window, trace);
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, 
        RSEMI_ATTR_PSPEM_MAXH_TRACE, enableMaxhold), 4, "Enable Maxhold");
    
    if (enableMaxhold == VI_TRUE)
    {
        viCheckParm(rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_PSPEM_MAXH_INTENS,
                                                intensity), 5, "Intensity");
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN This function resets the maxhold trace in the persistence
/// HIFN spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or rsemi_InitWithOptions function.
ViStatus _VI_FUNC rsemi_ConfigurePersistenceSpectrumMaxholdReset (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_PSPEM_MAXH_RESET, NULL));    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN This function defines if the delta marker is positioned on the
/// HIFN persistence trace or the maxhold trace. .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects delta marker.
/// HIPAR markerPostion/This control defines if the marker is positioned on the
/// HIPAR markerPostion/persistence trace or the maxhold trace.
ViStatus _VI_FUNC rsemi_ConfigurePersistenceSpectrumMarkerPosition (ViSession instrSession,
                                                                    ViInt32 window,
                                                                    ViInt32 marker,
                                                                    ViInt32 markerPostion)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker Number");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
        RSEMI_ATTR_PSPEM_MARKER_POSITION, markerPostion), 4, "Marker Postion"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);  
    return error;
}

/// HIFN This function queries the z-axis of the indicated marker in the
/// HIFN persistence spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR zAxis/queries the z-axis of the indicated marker in the persistence
/// HIPAR zAxis/spectrum result display.
ViStatus _VI_FUNC rsemi_QueryPersistenceMarkerZAxis (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViInt32 marker,
                                                     ViReal64 *zAxis)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker Number");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_QUERY_PSPEM_MARKER_Z_AXIS, zAxis), 4, "Z-Axis"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);  
    return error;
}

/// HIFN This function defines if the delta marker is positioned on the
/// HIFN persistence trace or the maxhold trace. .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects delta marker.
/// HIPAR deltaMarkerPostion/This control defines if the delta marker is positioned on the
/// HIPAR deltaMarkerPostion/persistence trace or the maxhold trace.
ViStatus _VI_FUNC rsemi_ConfigurePersistenceSpectrumDeltaMarkerPosition (ViSession instrSession,
                                                                         ViInt32 window,
                                                                         ViInt32 marker,
                                                                         ViInt32 deltaMarkerPostion)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker Number");
    
    sprintf (buffer, "Win%ld,DM%ld", window, marker);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
        RSEMI_ATTR_PSPEM_DELTA_MARKER_POSITION, deltaMarkerPostion), 4, "Delta Marker Postion"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);  
    return error;
}

/// HIFN  This function queries the z-axis of the indicated delta marker in the 
/// HIFN  persistence spectrum result display. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR zAxis/This control queries the z-axis of the indicated delta marker in the 
/// HIPAR zAxis/persistence spectrum result display. 
ViStatus _VI_FUNC rsemi_QueryPersistenceDeltaMarkerZAxis (ViSession instrSession,
                                                          ViInt32 window,
                                                          ViInt32 deltaMarker,
                                                          ViReal64 *zAxis)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, deltaMarker, 1, 16), 3, "Marker Number");
    
    sprintf (buffer, "Win%ld,DM%ld", window, deltaMarker);
    
    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_QUERY_PSPEM_DELTA_Z_AXIS, zAxis), 4, "Z-Axis"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);  
    return error;
}

/// HIFN  This function positions the specified delta marker on the specified 
/// HIFN  place.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/Selects the marker.
/// HIPAR searchMode/This control positions the delta marker to the desired position in the 
/// HIPAR searchMode/trace memory.
ViStatus _VI_FUNC rsemi_SpectrogramDeltaMarkerSearch (ViSession instrSession,
                                                      ViInt32 window,
                                                      ViInt32 deltaMarker,
                                                      ViInt32 searchMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_PEAK,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_PEAK,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");     
    
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    switch (searchMode){
        case RSEMI_VAL_SPECM_MARK_MAX_PEAK:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_PEAK, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MAX_NEXT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_PEAK_NEXT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MAX_LEFT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_PEAK_NEXT_LEFT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MAX_RIGHT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_PEAK_NEXT_RIGHT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MIN_PEAK:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_MIN, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MIN_NEXT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_MIN_NEXT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MIN_LEFT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_MIN_NEXT_LEFT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_MIN_RIGHT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_MIN_NEXT_RIGHT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MAX_PEAK:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_Y_MAXIMUM, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MAX_NEXT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_Y_MAXIMUM_NEXT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MAX_ABOVE:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_Y_MAXIMUM_NEXT_ABOVE, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MAX_BELOW:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_Y_MAXIMUM_NEXT_BELOW, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MIN_PEAK:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_Y_MINIMUM, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MIN_NEXT:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_Y_MINIMUM_NEXT, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MIN_ABOVE:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_Y_MINIMUM_NEXT_ABOVE, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_MIN_BELOW:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_Y_MINIMUM_NEXT_BELOW, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_XY_MAX:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_XY_MAXIMUM, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_XY_MIN:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_XY_MINIMUM, NULL));
        break;
        case RSEMI_VAL_SPECM_MARK_Y_TRIGGER:
            viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                                                RSEMI_ATTR_SPEM_DELTA_MARKER_SEARCH_Y_TRIGGER, NULL));
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Search Mode");
    }
    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function links delta marker 1 to marker 1. If you change the 
/// HIFN  horizontal position of the marker, so does the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR state/Specifies the marker and delta marker link state.
ViStatus _VI_FUNC rsemi_SpectrogramLinkMarkerAndDeltaMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, deltaMarker, 1, 16), 3, "Delta Marker");
    
    sprintf (buffer, "Win%ld,M%ld", window, deltaMarker);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_LINK,
                                               state), 4, "State");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the reference marker for the selected delta 
/// HIFN  marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR marker/Defines the reference marker for the selected delta marker. 
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramReferenceMarker(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SPEM_DMARKER_REFERENCE_MARKER,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SPEM_DMARKER_REFERENCE_MARKER,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                               RSEMI_ATTR_SPEM_DMARKER_REFERENCE_MARKER,
                                               marker), 4, "Marker");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function specifies the trigger source that causes the spectrum 
/// HIFN  analyzer to leave the Wait-for-Trigger state. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerSource/Specifies the source of the trigger signal that causes the analyzer to 
/// HIPAR triggerSource/leave the Wait-For-Trigger state.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramTriggerSource(
    ViSession    instrSession,
    ViInt32    triggerSource
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SPEM_TRIGGER_SOURCE,
                                               triggerSource), 2, "Trigger Source");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function specifies the delay and polarity for triggering.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerDelay/Sets the length of the trigger delay.
/// HIPAR triggerPolarity/Specifies the polarity for triggering.
/// HIPAR preTriggerTime/Defines the length of the pretrigger.
/// HIPAR postTriggerTime/Defines the length of the posttrigger.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramTrigger(
    ViSession    instrSession,
    ViReal64    triggerDelay,
    ViInt32    triggerPolarity,
    ViReal64    preTriggerTime,
    ViReal64    postTriggerTime
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_SPEM_TRIGGER_DELAY,
                                                triggerDelay), 2, "Trigger Delay");
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                               RSEMI_ATTR_SPEM_TRIGGER_SLOPE,
                                               triggerPolarity), 3, "Trigger Polarity");
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_SPEM_PRE_TRIGGER_TIME,
                                                preTriggerTime), 4, "Pre Trigger Time");
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                                RSEMI_ATTR_SPEM_POST_TRIGGER_TIME,
                                                postTriggerTime), 5, "Post Trigger Time");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the time stamp of the frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR frame/Defimes the frame for time stamp.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR part1/Returns array of part 1 od time stamp of frame(s). If All frames is 
/// HIPAR part1/selected, all frames sorted in a descending order, starting with the 
/// HIPAR part1/current frame is returned.
/// HIPAR part2/Returns array of part 2 od time stamp of frame(s). If All frames is 
/// HIPAR part2/selected, all frames sorted in a descending order, starting with the 
/// HIPAR part2/current frame is returned.
/// HIPAR part3/Returns array of part 3 od time stamp of frame(s). If All frames is 
/// HIPAR part3/selected, all frames sorted in a descending order, starting with the 
/// HIPAR part3/current frame is returned.
/// HIPAR part4/Returns array of part 4 od time stamp of frame(s). If All frames is 
/// HIPAR part4/selected, all frames sorted in a descending order, starting with the 
/// HIPAR part4/current frame is returned.
/// HIPAR returnedValues/Returns the number of time stamps returned from the instrument.
ViStatus _VI_FUNC rsemi_GetSpectrogramTimeStamp (ViSession instrSession, ViInt32 window,
                                              ViInt32 frame, ViInt32 arraySize,
                                              ViInt32 part1[], ViInt32 part2[],
                                              ViInt32 part3[], ViInt32 part4[],
                                              ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;  
    ViChar*     pbuffer = NULL;
    ViInt32     i;
    ViChar*     pstring_value;
    ViChar      buffer [RSEMI_IO_BUFFER_SIZE] = "";     
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    checkErr (rsemi_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSEMI_IO_BUFFER_SIZE, buffer));
    
    if (strstr (buffer, "K55") == NULL)
        checkErr (RS_ERROR_INSTRUMENT_OPTION); 
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, frame, RSEMI_VAL_SPECM_TSTAMP_CURR, RSEMI_VAL_SPECM_TSTAMP_ALL), 3, "Frame");
    
    viCheckErr (viPrintf (instrSession, ":CALC%ld:SPEC:TST:DATA? %s\n", window, (frame==0)?"CURR":"ALL"));
    viCheckErr( Rs_ReadDataUnknownLength(instrSession, &pbuffer, VI_NULL));

    pstring_value = strtok (pbuffer, ",");              
    i = 0;
    while (pstring_value)
    {
        if (i < arraySize){
            part1[i] = atol (pstring_value);
            pstring_value = strtok (NULL, ",");
            part2[i] = atol (pstring_value);
            pstring_value = strtok (NULL, ",");
            part3[i] = atol (pstring_value);
            pstring_value = strtok (NULL, ",");
            part4[i] = atol (pstring_value);
            pstring_value = strtok (NULL, ",");
        }   
        else
        {
             pstring_value = strtok (NULL, ",");
             pstring_value = strtok (NULL, ",");
             pstring_value = strtok (NULL, ",");
             pstring_value = strtok (NULL, ",");
        }
        i++;
    };
    if (returnedValues) *returnedValues = i;
    
    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Returns the trace data for the corresponding trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR resultsType/Selects the type of requested results.
/// HIPAR arraySize/Sets the number if the returned results.
/// HIPAR traceData/Returns the trace data that consists of a list of power levels that 
/// HIPAR traceData/have been measured.
/// HIPAR noofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsemi_ReadTraceData(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    timeout,
    ViInt32    resultsType,
    ViInt32    arraySize,
    ViReal64    traceData[],
    ViInt32*    noofValues
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_Initiate (instrSession, window, timeout));
    
    viCheckErr (rsemi_FetchTraceData (instrSession, window, resultsType, arraySize, traceData, noofValues));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the frame results. The frames are sorted 
/// HIFN  according to their time stamp in descending order (starting with the 
/// HIFN  current frame).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR results/Returns an array of frame results. The frames are sorted according to 
/// HIPAR results/their time stamp in descending order (starting with the current frame).
/// HIPAR returnedValues/Returns number of value returned from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter Array Size.
ViStatus _VI_FUNC rsemi_ReadSpectrogramTraceData (ViSession instrSession, ViInt32 window,
                                           ViInt32 timeout, ViInt32 arraySize, 
                                           ViReal64 results[], ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckErr (rsemi_Initiate (instrSession, window, timeout));
    
    viCheckErr (rsemi_FetchSpectrogramTraceData (instrSession, window, arraySize, results, returnedValues));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the results of the persistence spectrum result 
/// HIFN  display. The function returns 801*600 percentages, one for each pixel 
/// HIFN  in the histogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR results/Returns an array of frame results. The frames are sorted according to 
/// HIPAR results/their time stamp in descending order (starting with the current frame).
/// HIPAR returnedValues/Returns number of value returned from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter Array Size.
ViStatus _VI_FUNC rsemi_ReadPersistenceSpectrumTraceData (ViSession instrSession,
                                                          ViInt32 window,
                                                          ViInt32 timeout,
                                                          ViInt32 arraySize,
                                                          ViReal64 results[],
                                                          ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckErr (rsemi_Initiate (instrSession, window, timeout));
    
    viCheckErr (rsemi_FetchPersistenceSpectrumTraceData (instrSession, window, arraySize, results, returnedValues));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the results of the maxhold trace in the 
/// HIFN  persistence spectrum result display. The function returns 801*600 
/// HIFN  percentages, one for each pixel in the maxhold trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR results/Returns an array of frame results. The frames are sorted according to 
/// HIPAR results/their time stamp in descending order (starting with the current frame).
/// HIPAR returnedValues/Returns number of value returned from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter Array Size.
ViStatus _VI_FUNC rsemi_ReadMaxholdTraceData (ViSession instrSession,
                                              ViInt32 window,
                                              ViInt32 timeout,
                                              ViInt32 arraySize,
                                              ViReal64 results[],
                                              ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckErr (rsemi_Initiate (instrSession, window, timeout));
    
    viCheckErr (rsemi_FetchMaxholdTraceData (instrSession, window, arraySize, results, returnedValues));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Returns the trace data for the corresponding trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultsType/Selects the type of requested results.
/// HIPAR arraySize/Sets the number if the returned results.
/// HIPAR traceData/Returns the trace data that consists of a list of power levels that 
/// HIPAR traceData/have been measured.
/// HIPAR noofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsemi_FetchTraceData(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    resultsType,
    ViInt32    arraySize,
    ViReal64    traceData[],
    ViInt32*    noofValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsemi_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSEMI_IO_BUFFER_SIZE, buffer));
    
    if (strstr (buffer, "K55") == NULL)
        checkErr (RS_ERROR_INSTRUMENT_OPTION); 
        
    viCheckParm (rsemi_invalidViInt32Range (instrSession, resultsType, RSEMI_VAL_TRACE1, RSEMI_VAL_TRACE6), 2, "Results Type");
    sprintf (buffer, "TRACE%ld", resultsType);
    
    viCheckErr (rsemi_dataReadTrace (instrSession, window, buffer, arraySize, traceData, noofValues));
      
    checkErr(rsemi_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function returns the frame results. The frames are sorted 
/// HIFN  according to their time stamp in descending order (starting with the 
/// HIFN  current frame).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR results/Returns an array of frame results. The frames are sorted according to 
/// HIPAR results/their time stamp in descending order (starting with the current frame).
/// HIPAR returnedValues/Returns number of value returned from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter Array Size.
ViStatus _VI_FUNC rsemi_FetchSpectrogramTraceData(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    arraySize,
    ViReal64    results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE]="";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    checkErr (rsemi_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSEMI_IO_BUFFER_SIZE, buffer));
    
    if (strstr (buffer, "K55") == NULL)
        checkErr (RS_ERROR_INSTRUMENT_OPTION); 
        
    sprintf (buffer, "SPEC");
    
    viCheckErr (rsemi_dataReadTrace (instrSession, window, buffer, arraySize, results, returnedValues));

    checkErr( rsemi_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the results of the persistence spectrum result 
/// HIFN  display. The function returns 801*600 percentages, one for each pixel 
/// HIFN  in the histogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR results/Returns an array of frame results. The frames are sorted according to 
/// HIPAR results/their time stamp in descending order (starting with the current frame).
/// HIPAR returnedValues/Returns number of value returned from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter Array Size.
ViStatus _VI_FUNC rsemi_FetchPersistenceSpectrumTraceData(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    arraySize,
    ViReal64    results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE]="";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    checkErr (rsemi_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSEMI_IO_BUFFER_SIZE, buffer));
    
    if (strstr (buffer, "K55") == NULL)
        checkErr (RS_ERROR_INSTRUMENT_OPTION); 
        
    sprintf (buffer, "PSP");
    
    viCheckErr (rsemi_dataReadTrace (instrSession, window, buffer, arraySize, results, returnedValues));

    checkErr( rsemi_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the results of the maxhold trace in the 
/// HIFN  persistence spectrum result display. The function returns 801*600 
/// HIFN  percentages, one for each pixel in the maxhold trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR results/Returns an array of frame results. The frames are sorted according to 
/// HIPAR results/their time stamp in descending order (starting with the current frame).
/// HIPAR returnedValues/Returns number of value returned from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter Array Size.
ViStatus _VI_FUNC rsemi_FetchMaxholdTraceData(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    arraySize,
    ViReal64    results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE]="";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    checkErr (rsemi_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSEMI_IO_BUFFER_SIZE, buffer));
    
    if (strstr (buffer, "K55") == NULL)
        checkErr (RS_ERROR_INSTRUMENT_OPTION); 
        
    sprintf (buffer, "HMAX");
    
    viCheckErr (rsemi_dataReadTrace (instrSession, window, buffer, arraySize, results, returnedValues));

    checkErr( rsemi_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the scaling of the x-axis in the spectrogram. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR frame/Selects the spectrogram frame.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR xAxis/This control returns the array of the x-axies values in the 
/// HIPAR xAxis/spectrogram.
/// HIPAR returnedValues/Returns number of value returned from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter Array Size.
ViStatus _VI_FUNC rsemi_GetSpectrogramXAxisScaling (ViSession instrSession,
                                                    ViInt32 window,  ViInt32 frame, ViInt32 arraySize, 
                                                    ViReal64 xAxis[], ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE]="";
    ViChar      *pbuffer;
    ViChar      *pbuff;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    checkErr (rsemi_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSEMI_IO_BUFFER_SIZE, buffer));
    
    if (strstr (buffer, "K55") == NULL)
        checkErr (RS_ERROR_INSTRUMENT_OPTION); 
        
    viCheckParm (rsemi_invalidViInt32Range (instrSession, frame, RSEMI_VAL_SPECM_FRAME_CURRENT, RSEMI_VAL_SPECM_FRAME_ALL), 3, "Frame"); 
    if (arraySize<=0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Array Size"); 
    if ((arraySize>0) && (xAxis == NULL))    
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "X-Axis");
    
    viCheckErr( viPrintf (instrSession, "CALC%ld:SGR:X:DATA? %s\n", window, 
        (frame == RSEMI_VAL_SPECM_FRAME_CURRENT) ? "CURR" : "ALL"));
    
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL));
    
    pbuff = strtok (pbuffer, ",");
    
    while((pbuff != NULL) && (i < arraySize))
    {
        xAxis[i++] = atof (pbuff);
        pbuff = strtok (NULL, ",");
    }   
    
    if (returnedValues) *returnedValues = i;
    
    checkErr (rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function stores the spectrogram data in ASCII format to a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file, where spectrogram data will be 
/// HIPAR fileName/stored.
ViStatus _VI_FUNC rsemi_StoreSpectrogramToFile (ViSession instrSession,
                                                   ViString fileName)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViString (instrSession, "", 
        RSEMI_ATTR_SPEM_STORE_TO_FILE, fileName), 2, "File Name");  

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function initiates an acquisition. After calling this function, 
/// HIFN  the instrument leaves the idle state and waits for a trigger.
/// HIFN     
/// HIFN     This function does not check the instrument status. Typically, the 
/// HIFN  end-user calls this function only in a sequence of calls to other 
/// HIFN  low-level driver functions. The sequence performs one operation. The 
/// HIFN  end-user uses the low-level functions to optimize one or more aspects 
/// HIFN  of interaction with the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_Initiate(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    timeout
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout));  

    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_SPEM_INIT, NULL));

Error:
    if (old_timeout > 0)
        viCheckErr( rsemi_SetOPCTimeout (instrSession, old_timeout));
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function continues a stopped measurement at the current position 
/// HIFN  in single sweep mode. The function is useful especially for trace 
/// HIFN  functions MAXHold, MINHold and AVERage if the previous results are not 
/// HIFN  to be cleared with Sweep Count > 0 or Average Count > 0 on restarting 
/// HIFN  the measurement (INIT:IMMediate resets the previous results on 
/// HIFN  restarting the measurement).
/// HIFN     The single-sweep mode is automatically switched on. Synchronization to 
/// HIFN  the end of the indicated number of measurements can then be performed 
/// HIFN  with the command *OPC, *OPC? or *WAI. In the continuous-sweep mode, 
/// HIFN  synchronization to the sweep end is not possible since the overall 
/// HIFN  measurement never ends.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_Continue(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_SPEM_CONTINUE_MEASUREMENT, NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function turns continuous triggering on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trigger/Turns continuous triggering on and off.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramContinuousTrigger (ViSession instrSession,
                                                                  ViInt32 trigger)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
        RSEMI_ATTR_SPEM_CONTINUOUS_TRIGGER, trigger), 2, "Trigger");

Error:
     Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function configures time domain trigger as source and level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeDomainTriggerLevel/Sets the trigger level for the time domain trigger.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramTimeDomainTrigger (ViSession instrSession,
                                                                  ViReal64 timeDomainTriggerLevel)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", 
        RSEMI_ATTR_SPEM_TIME_DOMAIN_TRIGGER, VI_NULL));
    
     viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
         RSEMI_ATTR_SPEM_TIME_DOMAIN_TRIGGER_LEVEL, timeDomainTriggerLevel), 2, "Time Domain Trigger Level");
    
Error:
     Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function selects the result display in realtime mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultDisplay/This control selets the result display in realtime mode.
ViStatus _VI_FUNC rsemi_ConfigureResultDisplay (ViSession instrSession,
                                                               ViInt32 window, ViInt32 resultDisplay)
{
    
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm(rsemi_SetAttributeViInt32(instrSession, buffer, RSEMI_ATTR_RESULT_DISPLAY, resultDisplay), 
                        3, "Result Display");    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the frequency mask trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMaskTriggerSource (ViSession instrSession,
                                                             ViInt32 window)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 

    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
        RSEMI_ATTR_SPEM_MASK_TRIGGER_SOURCE, NULL));

    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the frequency mask trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMask (ViSession instrSession,
                                                ViInt32 window,
                                                ViString name,
                                                ViString comment)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm(rsemi_SetAttributeViString(instrSession, buffer, RSEMI_ATTR_SPEM_MASK_TRIGGER_NAME, name), 
                        3, "Name"); 
    
    viCheckParm(rsemi_SetAttributeViString(instrSession, buffer, RSEMI_ATTR_SPEM_MASK_TRIGGER_COMMENT, comment), 
                        4, "Comment");    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function sets the directory the instrument stores frequency masks in.
/// HIRET Returns the status code of this operation.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR directory/This control sets the directory to store the frequency mask file in.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMaskDirectory (ViSession instrSession,
                                                         ViInt32 window, ViString directory)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm(rsemi_SetAttributeViString(instrSession, buffer, RSEMI_ATTR_SPEM_MASK_TRIGGER_DIRECTORY, directory), 
                        3, "Directory"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function deletes a frequency mask trigger mask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR name/This control deletes a frequency mask trigger mask.
ViStatus _VI_FUNC rsemi_DeleteFrequencyMask (ViSession instrSession,
                                             ViInt32 window, ViString name)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm(rsemi_SetAttributeViString(instrSession, buffer, RSEMI_ATTR_SPEM_MASK_TRIGGER_DELETE, name), 
                        3, "Name"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);  
    return error;
}

/// HIFN  This function deletes selected frequency mask trigger mask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_DeleteSelectedFrequencyMask(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr (rsemi_SetAttributeViString(instrSession, buffer, 
                                           RSEMI_ATTR_SPEM_MASK_TRIGGER_DELETE_SELECTED, VI_NULL)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);  
    return error;    
}

/// HIFN  This function sets the condition that activates the frequency mask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR condition/This control sets the condition that activates the frequency mask.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMaskCondition (ViSession instrSession,
                                                         ViInt32 window, ViInt32 condition)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm(rsemi_SetAttributeViInt32(instrSession, buffer, RSEMI_ATTR_SPEM_MASK_TRIGGER_CONDITION, condition), 
                        3, "Condition"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);  
    return error;
}

/// HIFN  This function configures the frequency mask shape.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR mode/This control defines the scalling of the level axis for the frequency 
/// HIPAR mode/mask trigger masks.
/// HIPAR lower/This control activates and defines the shape of the selected lower 
/// HIPAR lower/frequency mask trigger mask.
/// HIPAR upper/This control activates and defines the shape of the selected  upper 
/// HIPAR upper/frequency mask trigger mask. 
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMaskShape (ViSession instrSession,
                                                     ViInt32 window, ViInt32 arraySize, 
                                                     ViInt32 mode,
                                                     ViReal64 lower[],
                                                     ViReal64 upper[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm(rsemi_SetAttributeViInt32(instrSession, buffer, RSEMI_ATTR_SPEM_MASK_TRIGGER_MODE, mode), 
                        3, "Mode"); 
    
    p2buf = buffer + sprintf (buffer, "CALC:MASK:LOW ");
    
    for (i = 0; i < arraySize * 2; i++)
        p2buf += sprintf (p2buf, "%.12lG,", lower[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    checkErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) strlen (buffer), NULL));
    
    checkErr( rsemi_CheckStatus (instrSession));
    
    p2buf = buffer + sprintf (buffer, "CALC:MASK:UPP ");
    
    for (i = 0; i < arraySize * 2; i++)
        p2buf += sprintf (p2buf, "%.12lG,", upper[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    checkErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) strlen (buffer), NULL));

    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
    
}

/// HIFN  This function configures the frequency mask shape.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR mode/This control defines the scalling of the level axis for the frequency 
/// HIPAR mode/mask trigger masks.
/// HIPAR shape/This control selects which shape should be defined, lower or upper.
/// HIPAR data/This control activates and defines the shape of the selected  lower/upper 
/// HIPAR data/frequency mask trigger mask. 
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMaskShapeLowerOrUpper (ViSession instrSession,
                                                                 ViInt32 window,
                                                                 ViInt32 arraySize,
                                                                 ViInt32 mode,
                                                                 ViInt32 shape,
                                                                 ViReal64 data[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm(rsemi_SetAttributeViInt32(instrSession, buffer, RSEMI_ATTR_SPEM_MASK_TRIGGER_MODE, 
                                          mode), 3, "Mode"); 
    
    switch (shape)
    {
        case RSEMI_VAL_LOWER:
            p2buf = buffer + sprintf (buffer, "CALC:MASK:LOW ");  
        break;
        case RSEMI_VAL_UPPER:
            p2buf = buffer + sprintf (buffer, "CALC:MASK:UPP "); 
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Shape");     
    }
    
    for (i = 0; i < arraySize * 2; i++)
        p2buf += sprintf (p2buf, "%.12lG,", data[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    checkErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) strlen (buffer), NULL));
    
    checkErr( rsemi_CheckStatus (instrSession)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function automatically defines the shape of the frequency
/// HIFN mask trigger mask according to the spectrum that is currently
/// HIFN measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or rsemi_InitWithOptions function.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMaskAuto (ViSession instrSession,
                                                    ViInt32 window)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    checkErr(rsemi_SetAttributeViString(instrSession, buffer, RSEMI_ATTR_SPEM_MASK_UPP_AUTO, NULL)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function sets the frequency mask shift. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR type/This control selects the upper and the lower mask.
/// HIPAR frequency/This control shifts the selected frequency mask trigger by a specific 
/// HIPAR frequency/distance. Positive values move the mask to the right, negative values 
/// HIPAR frequency/shift the mask to the left.
/// HIPAR amplitude/This control shifts the selected amplitude mask trigger by a specific 
/// HIPAR amplitude/distance. Positive values move the mask upwards, negative values shift 
/// HIPAR amplitude/the mask downwards. 
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMaskShift (ViSession instrSession,
                                                     ViInt32 window, ViInt32 type,
                                                     ViReal64 frequency,
                                                     ViReal64 amplitude)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
     switch (type)
     {
        case RSEMI_VAL_LOWER:
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SPEM_MASK_SHIFT_LOW_X, 
                frequency), 4, "Frequency");
            
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SPEM_MASK_SHIFT_LOW_Y, 
                amplitude), 5, "Amplitude"); 
        break;
        case RSEMI_VAL_UPPER:
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SPEM_MASK_SHIFT_UPP_X, 
                frequency), 4, "Frequency");
            
            viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_SPEM_MASK_SHIFT_UPP_Y, 
                amplitude), 5, "Amplitude"); 
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");     
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function turns the lower or upper frequency mask on and off.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     
/// HIFN     (1) Before making any changes to a frequency mask, you have to select 
/// HIFN  one by name with attribute RSEMI_ATTR_SPEM_MASK_TRIGGER_NAME.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR type/This control selects the upper and the lower mask.
/// HIPAR frequencyMask/This control turns the lower or upper frequency mask on and off.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMaskState(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    type,
    ViBoolean    frequencyMask
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
     switch (type)
     {
        case RSEMI_VAL_LOWER:
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_SPEM_MASK_LOWER_STATE, 
                frequencyMask), 4, "Frequency Mask");
        break;
        case RSEMI_VAL_UPPER:
            viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_SPEM_MASK_UPPER_STATE, 
                frequencyMask), 4, "Frequency Mask");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");     
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the frequency mask span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR span/This control sets the frequency mask span.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMaskSpan(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    span
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 0, 4), 2, "Window");

    sprintf (rep_cap, "Win%ld", window);  

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSEMI_ATTR_SPEM_MASK_SPAN,
                                                span), 3, "Span");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function activates receiver mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or rsemi_InitWithOptions function.
ViStatus _VI_FUNC rsemi_ReceiverMode (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_RECEIVER_MODE, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Selects the measurement performed in a Receiver channel during measurement sequences.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or rsemi_InitWithOptions function.
/// HIPAR measurement/Selects the measurement performed in a Receiver channel during measurement sequences.    
ViStatus _VI_FUNC rsemi_ReceiverMeasurementSequences (ViSession instrSession,
                                                     ViInt32 measurement)

{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_RECEIVER_MEASUREMENT_SEQUENCES, measurement),
                                               2, "Measurement");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Selects the nominal input impedance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR inputImpedance/Selects the nominal input impedance.
ViStatus _VI_FUNC rsemi_ConfigureInputImpedance (ViSession instrSession,
                                                 ViInt32 inputImpedance)
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));


    viCheckParm(rsemi_SetAttributeViInt32 (instrSession, "",
        RSEMI_ATTR_INPUT_IMPEDANCE, inputImpedance), 2, "Input Impedance");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Selects the detector for the scan.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Sets the trace.
/// HIPAR detector/Selects the detector for the scan.
ViStatus _VI_FUNC rsemi_ConfigureTraceDetectorSelection (ViSession instrSession,
                                                         ViInt32 trace,
                                                         ViInt32 detector)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";  
     
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "TR%ld", trace);  
    viCheckParm(rsemi_SetAttributeViInt32 (instrSession, buffer,
        RSEMI_ATTR_TRACE_DETECTOR_SELECTION, detector), 3, "Detector");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Configures the acquisition attributes. Single sweep, continuous sweep 
/// HIFN  and sweep count can be set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR sweepModeContinuous/Turns on/off continuous sweeping.
/// HIPAR numberOfSweeps/Sets the number of sweeps to take.
ViStatus _VI_FUNC rsemi_ConfigureAcquisition (ViSession instrSession,
                                              ViInt32 window,
                                              ViBoolean sweepModeContinuous,
                                              ViInt32 numberOfSweeps)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_SWEEP_MODE_CONTINUOUS,
                                                sweepModeContinuous), 3, "Sweep Mode Continuous");
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_NUMBER_OF_SWEEPS,
                                                numberOfSweeps), 4, "Number Of Sweeps");                                                

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Queries the number of scans that have started in a single sweep that 
/// HIFN  consists of more than one scan.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sweepCount/Returns the number of scans that have started in a single sweep that 
/// HIPAR sweepCount/consists of more than one scan.
ViStatus _VI_FUNC rsemi_QueryStartedSweepNumber (ViSession instrSession,
                                                 ViInt32 *sweepCount)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_GetAttributeViInt32 (instrSession, "", 
                                                RSEMI_ATTR_QUERY_STARTED_SWEEP_NUMBER,
                                                sweepCount), 2, "Sweep Count"); 
                                                   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;  
}

/// HIFN  Defines the start frequency for scans in the frequency domain, and 
/// HIFN  also defines the stop frequency for scans in the time domain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR startFrequency/Sets the start frequency for scans in the frequency domain.
/// HIPAR stopFrequency/Sets the stop frequency for scans in the time domain.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyStartStop (ViSession instrSession,
                                                     ViReal64 startFrequency,
                                                     ViReal64 stopFrequency)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64  (instrSession, "", 
                                                RSEMI_ATTR_START_FREQUENCY,
                                                startFrequency), 2, "Start Frequency");
    
    viCheckParm( rsemi_SetAttributeViReal64  (instrSession, "", 
                                                RSEMI_ATTR_STOP_FREQUENCY,
                                                stopFrequency), 3, "Stop Frequency"); 
                                                   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error; 
}

/// HIFN  Defines the receiver frequency for measurements in the frequency or 
/// HIFN  time domain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR receiverFrequency/Sets the receiver frequency for measurements in the frequency or time 
/// HIPAR receiverFrequency/domain.
ViStatus _VI_FUNC rsemi_ConfigureReceiverFrequency (ViSession instrSession,
                                                      ViReal64 receiverFrequency)

{
   
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64  (instrSession, "", 
                                                RSEMI_ATTR_RECEIVER_FREQUENCY,
                                                receiverFrequency), 2, "Receiver Frequency");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;  
}

/// HIFN  Defines the center frequency step size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR stepSize/Sets the center frequency step size.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyCenterStepSize (ViSession instrSession,
                                                          ViReal64 stepSize)

{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64  (instrSession, "", 
                                                RSEMI_ATTR_FREQUENCY_CENTER_STEP,
                                                stepSize), 2, "Step Size");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;  
}

/// HIFN  This function couples or decouples the center frequency step
/// HIFN  size to the span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR stepSizeAuto/Couples or decouples the center frequency step size to the span. 
ViStatus _VI_FUNC rsemi_ConfigureFrequencyCenterStepSizeAuto (ViSession instrSession,
                                                              ViBoolean stepSizeAuto)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", 
            RSEMI_ATTR_FREQUENCY_CENTER_STEP_AUTO, stepSizeAuto), 2, "Step Size Auto");

    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function couples the step width of the center frequency to
/// HIFN  span (span >0) or to the resolution bandwidth (span = 0) or
/// HIFN  cancels the couplings and sets the coupling factor. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/Couples the step width of the center frequency to span (span >0)
/// HIPAR coupling/or to the resolution bandwidth (span = 0) or cancels the couplings. 
/// HIPAR couplingFactor/Defines a step size factor if the center frequency step size is
/// HIPAR couplingFactor/coupled to the span or the resolution bandwidth.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyCenterCouplingFactor (ViSession instrSession,
                                                                ViInt32 coupling,
                                                                ViInt32 couplingFactor)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_FREQUENCY_CENTER_STEPSIZE,
                                              coupling), 2, "Coupling");
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_FREQUENCY_CENTER_STEPSIZE_FACTOR, 
                                            couplingFactor), 3, "Coupling Factor");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function defines the stepsize by which the frequency is
/// HIFN  increased or decreased.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Defines the stepsize by which the frequency is increased or decreased.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyCenterStepSizeType (ViSession instrSession,
                                                              ViInt32 type)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_FREQUENCY_CENTER_STEPSIZE_TYPE,
                                              type), 2, "Type");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Selects the frequency mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Selects the frequency mode.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyMode (ViSession instrSession,
                                                ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32  (instrSession, "", 
                                                RSEMI_ATTR_FREQUENCY_MODE,
                                                mode), 2, "Mode");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;   
}

/// HIFN  Defines the measurement time for fixed frequency scans.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR scanTime/Defines the measurement time for fixed frequency scans.
ViStatus _VI_FUNC rsemi_ConfigureScanTime (ViSession instrSession,
                                           ViReal64 scanTime)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_SCAN_TIME,
                                                scanTime), 2, "Scan Time");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Selects the mode for time domain scans.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Selects the mode for time domain scans.
ViStatus _VI_FUNC rsemi_ConfigureTimeDomainMode (ViSession instrSession,
                                                 ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsemi_SetAttributeViInt32(instrSession, "",
        RSEMI_ATTR_TIME_DOMAIN_MODE, mode), 2, "Time Domain Mode");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function turns configures signal-tracking. 
/// HIFN     
/// HIFN     With signal track activated, the maximum signal is determined after 
/// HIFN  each frequency sweep and the center frequency is set to the frequency 
/// HIFN  of this signal. Thus with drifting signals the center frequency follows 
/// HIFN  the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR signalTrackEnabled/Switches the signal-track function on or off for the selected 
/// HIPAR signalTrackEnabled/measurement window. 
/// HIPAR signalTrackBandwidth/Defines the bandwidth around the center frequency within which the 
/// HIPAR signalTrackBandwidth/largest signal is searched. 
/// HIPAR signalTrackThreshold/Defines the threshold above which the largest signal is searched for 
/// HIPAR signalTrackThreshold/in the selected measurement window. 
/// HIPAR trace/Defines the trace on which the largest signal is searched for in the 
/// HIPAR trace/selected measurement window. 
ViStatus _VI_FUNC rsemi_ConfigureFrequencySignalTrack (ViSession instrSession,
                                                       ViInt32 window,
                                                       ViBoolean signalTrackEnabled,
                                                       ViReal64 signalTrackBandwidth,
                                                       ViReal64 signalTrackThreshold,
                                                       ViInt32 trace)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    buffer[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_RECEIVER_SIGNAL_TRACK_ENABLED, 
                signalTrackEnabled), 3, "Signal Track Enabled");
    if (signalTrackEnabled == VI_TRUE)
    {                
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_RECEIVER_SIGNAL_TRACK_BWID, 
                    signalTrackBandwidth), 4, "Signal Track Bandwidth");
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_RECEIVER_SIGNAL_TRACK_THRESHOLD, 
                    signalTrackThreshold), 5, "Signal Track Threshold");
        viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_RECEIVER_SIGNAL_TRACK_TRACE, 
                    trace), 6, "Trace");
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the minimum or maximum bandwidth around the center frequency 
/// HIFN  within which the largest signal is searched. It is only available in 
/// HIFN  the frequency domain (span > 0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minMax/This control defines minimum/maximum bandwidth around the center 
/// HIPAR minMax/frequency within which the largest signal is searched. It is only 
/// HIPAR minMax/available in the frequency domain (span > 0).
ViStatus _VI_FUNC rsemi_ConfigureFrequencySignalTrackBandwidthMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    minMax
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_RECEIVER_SIGNAL_TRACK_BWID_MIN_MAX,
                                                minMax), 3, "Min Max"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;     
}


/// HIFN  This function queries minimal and maximal signal track bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minimum/Queries the minimum signal track bandwidth.
/// HIPAR maximum/Queries the maximum signal track bandwidth.
ViStatus _VI_FUNC rsemi_QueryFrequencySignalTrackBandwidthMinMax(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64*    minimum,
    ViReal64*    maximum
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);  
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_RECEIVER_SIGNAL_TRACK_BWID_GET_MIN,
                                                minimum), 3, "minimum"); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_RECEIVER_SIGNAL_TRACK_BWID_GET_MAX,
                                                maximum), 4, "maximum");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;     
}

/// HIFN  Selects the unit for measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerUnit/Selects the unit for power in the selected measurement window.
ViStatus _VI_FUNC rsemi_ConfigureAmplitudeLevelUnit (ViSession instrSession,
                                                     ViInt32 powerUnit)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32  (instrSession, "Win0", 
                                                RSEMI_ATTR_POWER_UNIT,
                                                powerUnit), 2, "Power Unit");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the minimum level displayed on the vertical diagram axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Sets the trace.
/// HIPAR level/Sets the minimum level displayed on the vertical diagram axis.
ViStatus _VI_FUNC rsemi_ConfigureAmplitudeMinimumLevel (ViSession instrSession,
                                                        ViInt32 window,
                                                        ViInt32 trace,
                                                        ViReal64 level)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    window;
    trace;
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_AMP_MINIMUM_LEVEL,
                                               level), 4, "Level");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Selects the scaling type of the y-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Sets the trace.
/// HIPAR scalingType/Selects the scaling of the y-axis.
ViStatus _VI_FUNC rsemi_ConfigureAmplitudeScalingType (ViSession instrSession,
                                                       ViInt32 window,
                                                       ViInt32 trace,
                                                       ViInt32 scalingType)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    window;
    trace;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                               RSEMI_ATTR_AMP_SCALING_TYPE,
                                               scalingType), 4, "Scaling Type");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Selects the signal source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/Selects the signal source.
ViStatus _VI_FUNC rsemi_ConfigureRFInput (ViSession instrSession,
                                          ViInt32 source)
{
    
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_RF_INPUT,
                                               source), 2, "Source");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Toggles the RF input of the R&S ESR between AC and DC coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR RFInputCoupling/Toggles the RF input of the R&S ESR between AC and DC coupling.
ViStatus _VI_FUNC rsemi_ConfigureRFInputCoupling (ViSession instrSession,
                                                  ViInt32 RFInputCoupling)
{
    
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_RF_INPUT_COUPLING,
                                               RFInputCoupling), 2, "RF Input Coupling");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  This function couples or decouples the IF span to the measurement bandwidth 
/// HIFN  used for the bargraph.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR spanCoupled/Couples or decouples the IF span to the measurement bandwidth
/// HIPAR spanCoupled/used for the bargraph.
ViStatus _VI_FUNC rsemi_ConfigureIFSpanCoupled (
    ViSession instrSession,
    ViInt32 window,
    ViBoolean spanCoupled
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_IF_ANALYSIS_SPAN_COUPLED,
                                                spanCoupled), 3, "Span Coupled");                                                 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Defines the resolution bandwidth, and couples or decouples the 
/// HIFN  resolution bandwidth to the selected frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resolutionBandwidthAuto/Enables or disables the automatic bandwidth resolution.
/// HIPAR resolutionBandwidth/Selects the resolution bandwidth.
ViStatus _VI_FUNC rsemi_ConfigureResolutionBandwidth (ViSession instrSession,
                                                      ViBoolean resolutionBandwidthAuto,
                                                      ViReal64 resolutionBandwidth)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean  (instrSession, "", 
                                               RSEMI_ATTR_RESOLUTION_BANDWIDTH_AUTO,
                                               resolutionBandwidthAuto), 2, "Resolution Bandwidth Auto");
    
   if (!resolutionBandwidthAuto)
       
    {
       viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                               RSEMI_ATTR_RESOLUTION_BANDWIDTH,
                                               resolutionBandwidth), 3, "Resolution Bandwidth");
    }        
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;    
}

/// HIFN  Selects the type of resolution filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterType/Selects the type of resolution filter.
ViStatus _VI_FUNC rsemi_ConfigureResolutionBandwidthFilterType (ViSession instrSession,
                                                                ViInt32 filterType)
{
 
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_RESOLUTION_BANDWIDTH_FILTER_TYPE,
                                               filterType), 2, "Filter Type");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
    
}

/// HIFN  Selects the frequency step mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR spacing/Selects the frequency step mode.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanStepMode (ViSession instrSession,
                                                        ViInt32 spacing)
{
    
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_FREQUENCY_SCAN_STEP_MODE,
                                               spacing), 2, "Spacing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the start frequency and the stop frequency of a particular 
/// HIFN  scan range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR startFrequency/Defines the start frequency of a particular scan range.
/// HIPAR stopFrequency/Defines the stop frequency of a particular scan range.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanRangeStartStop (ViSession instrSession,
                                                              ViInt32 range,
                                                              ViReal64 startFrequency,
                                                              ViReal64 stopFrequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_RANGE_START,
                                               startFrequency), 3, "Start Frequency");
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_RANGE_STOP,
                                               stopFrequency), 4, "Stop Frequency");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the frequency step size within a particular scan range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR frequency/Defines the frequency step size within a particular scan range.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanStepSize (ViSession instrSession,
                                                        ViInt32 range,
                                                        ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range);
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_STEP_SIZE,
                                               frequency), 3, "Frequency");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the number of scan ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR rangesCount/Defines the number of scan ranges.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanRangeCount (ViSession instrSession,
                                                          ViInt32 range,
                                                          ViInt32 rangesCount)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_RANGE_COUNT,
                                               rangesCount), 3, "Ranges Count");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the gain level of the preamplifier applied in the selected scan range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR preamplifierGain/Defines the gain level of the preamplifier applied in the
/// HIPAR preamplifierGain/selected scan range.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanRangePreamplifierGain (
    ViSession instrSession,
    ViInt32 range,
    ViInt32 preamplifierGain
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_RANGE_PREAMPLIFIER_GAIN,
                                               preamplifierGain), 3, "Preamplifier Gain");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  This function configures the frequency scan IF attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR attenuationAuto/State of the IF Gain Auto Mode  On/ OFF  (Rec mode) for scan table.
/// HIPAR attenuation/This attribute defines the IF Gain Value (rec mode)  for scan table.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanIFAttenuation (
    ViSession instrSession,
    ViInt32 range,
    ViBoolean attenuationAuto,
    ViReal64 attenuation
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_SCAN_IF_AUTO_RANGING_STATE,
                                                attenuationAuto), 3, "Attenuation Auto");                                                 
    if (attenuationAuto == VI_FALSE)
    {
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_SCAN_IF_ATTENUATION,
                                                attenuation), 4, "Attenuation");                                                
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches on the auto mode for the LNA Preamp on/off for scan table. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR state/Auto Mode for the LNA Preamp on/ OFF for scan table.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanLNAPreamplifierState (
    ViSession instrSession,
    ViInt32 range,
    ViBoolean state
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_SCAN_RANGE_LNA_PREAMPLIFIER,
                                                state), 3, "State");                                                 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures state for the LNA Preamp for scan table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR LNAPreamplifierAuto/State for the LNA Preamp for scan table.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanLNAPreamplifier (
    ViSession instrSession,
    ViInt32 range,
    ViBoolean LNAPreamplifierAuto
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range); 
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_SCAN_RANGE_LNA_PREAMPLIFIER_AUTO,
                                                LNAPreamplifierAuto), 3, "LNA Preamplifier Auto");                                                 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Selects the scale of the frequency axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Sets the trace.
/// HIPAR scale/Selects the scale of the frequency axis.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanAxisScale (ViSession instrSession,
                                                         ViInt32 window,
                                                         ViInt32 trace,
                                                         ViInt32 scale)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    window;
    trace;
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQ_SCAN_AXIS_SCALE,
                                               scale), 4, "Scale");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the measurement time for a particular scan range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR time/Sets the measurement bandwidth for a particular scan range.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanMeasurementTime (ViSession instrSession,
                                                               ViInt32 range,
                                                               ViReal64 time)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_MEASUREMENT_TIME,
                                               time), 3, "Time");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the measurement bandwidth for a particular scan range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR bandwidth/Sets the measurement bandwidth for a particular scan range.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanBandwidth (ViSession instrSession,
                                                     ViInt32 range,
                                                     ViReal64 bandwidth)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_BANDWIDTH,
                                               bandwidth), 3, "Bandwidth");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Sets maximum/minimum measurement time.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_FREQUENCY_SCAN_MEASUREMENT_TIME_MIN_MAX
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]SCAN<range>:TIME
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR minMax/Set maximum/minimum measurement time.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanMeasurementTimeMinMax(
    ViSession    instrSession,
    ViInt32    range,
    ViInt32    minMax
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range); 
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_SCAN_MEASUREMENT_TIME_MIN_MAX,
                                                minMax), 3, "Min Max"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}


/// HIFN  This function queries minimal and maximal measurement time.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_FREQUENCY_SCAN_MEASUREMENT_TIME_GET_MIN
/// HIFN     RSEMI_ATTR_FREQUENCY_SCAN_MEASUREMENT_TIME_GET_MAX
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]SCAN<range>:TIME
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR minimum/Queries the minimum measurement time.
/// HIPAR maximum/Queries the maximum measurement time.
ViStatus _VI_FUNC rsemi_QueryFrequencyScanMeasurementTimeMinMax(
    ViSession    instrSession,
    ViInt32    range,
    ViReal64*    minimum,
    ViReal64*    maximum
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_SCAN_MEASUREMENT_TIME_GET_MIN,
                                                minimum), 3, "minimum"); 
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, 
                                                RSEMI_ATTR_FREQUENCY_SCAN_MEASUREMENT_TIME_GET_MAX,
                                                maximum), 4, "maximum");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN  Defines the attenuation level for a particular scan range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR attenuation/Defines the attenuation level for a particular scan range.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanRFAttenuation (ViSession instrSession,
                                                         ViInt32 range,
                                                         ViReal64 attenuation)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_RF_ATTENUATION,
                                               attenuation), 3, "Attenuation");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Enables and disables auto ranging in a particular scan range, and 
/// HIFN  include the preamplifier in the autoranging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR autoRanging/Enables and disables auto ranging in a particular scan range.
/// HIPAR autoPreamplifier/Includes the preamplifier in the autoranging function of the selected 
/// HIPAR autoPreamplifier/receiver scan range.
/// HIPAR preamplifier/Enables and disables the preamplifier in particular scan range.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanAutoRangingState (ViSession instrSession,
                                                            ViInt32 range,
                                                            ViBoolean autoRanging,
                                                            ViBoolean autoPreamplifier,
                                                            ViBoolean preamplifier)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_AUTO_RANGING_STATE,
                                               autoRanging), 3, "Auto Ranging");
    
    if (autoRanging == VI_TRUE)
    {    
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_AUTO_PREAMPLIFIER_STATE,
                                               autoPreamplifier), 4, "Auto Preamplifier");
    
    }
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_RANGE_PREAMPLIFIER,
                                               preamplifier), 5, "Preamplifier");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Selects the signal source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the scan range.
/// HIPAR source/Selects the signal source.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyScanRangeSignalSource (ViSession instrSession,
                                                             ViInt32 range,
                                                             ViInt32 source)
{   
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "R%ld", range);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                               RSEMI_ATTR_FREQUENCY_SCAN_RANGE_SIGNAL_SOURCE,
                                               source), 3, "Source");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the attenuation level at the RF input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR attenuation/Sets the attenuation level at the RF input.
ViStatus _VI_FUNC rsemi_ConfigureRFAttenuationLevel (ViSession instrSession,
                                                     ViReal64 attenuation)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
                                               RSEMI_ATTR_RF_ATTENUATION_LEVEL,
                                               attenuation), 3, "Attenuation");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Turns the availability of attenuation levels of 10 dB or less on and 
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR attenuationProtection/Enables and disables the attenuation level of 10 dB.
ViStatus _VI_FUNC rsemi_ConfigureRFAttenuationProtection (ViSession instrSession,
                                                          ViBoolean attenuationProtection)
{

    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_RF_ATTENUATION_PROTECTION,
                                               attenuationProtection), 3, "Attenuation Protection");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  WEnables and disables the automatic selection of the preamplifier, and 
/// HIFN  enables the 20 dB preamplifier in case the automatic preamplifier is 
/// HIFN  turned off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR preamplifierAuto/Enables and disables the automatic selection of the preamplifier.
/// HIPAR preamplifier/Enables and disables the 20 dB preamplifier for the instrument. 
/// HIPAR autoRange/Enables or disables the automatic configuration of the attenuation 
/// HIPAR autoRange/level.
ViStatus _VI_FUNC rsemi_ConfigurePreamplifier (ViSession instrSession,
                                               ViBoolean autoRange,
                                               ViBoolean preamplifierAuto,
                                               ViBoolean preamplifier)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
     viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_AUTO_RANGE,
                                               autoRange), 2, "Auto Range");
    
    if (autoRange)
    {
        viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_AUTO_PREAMPLIFIER,
                                               preamplifierAuto), 3, "Preamplifier Auto");
    }
    if (!preamplifierAuto)
    {
        viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_PREAMPLIFIER_STATE,
                                               preamplifier), 4, "Preamplifier");
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN Queries the number of peaks found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR size/Number of peaks found in the peak search..
ViStatus _VI_FUNC rsemi_QueryFinalMeasurementPeakListSize (ViSession instrSession,
                                                           ViInt32 *size)
{
    ViStatus    error = VI_SUCCESS;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsemi_GetAttributeViInt32(instrSession, "",
        RSEMI_ATTR_FINAL_MEASUREMENT_NUMBER_OF_PEAKS, size), 2, "Size");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the information for all peaks found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Defines size of arrays in arguments 'Trace Number', 'Frequency', 'Level' and 'Delta Limit'.
/// HIPAR detectorBufferSize/Defines the size of buffer in argument 'Detector'.
/// HIPAR commentBufferSize/Defines the size of buffer in argument 'Comment'.
/// HIPAR traceNumber/Returns array of trace numbers, one for each peak in peak list.
/// HIPAR detector/Comma separated list of detectors, one for each peak in the peak list.
/// HIPAR frequency/Returns array of frequencies, one for each peak in peak list.
/// HIPAR level/Returns array of levels, one for each peak in peak list.
/// HIPAR deltaLimit/Returns array of delta limits, one for each peak in peak list. If no limit line was used, 0 is returned.
/// HIPAR comment/Comma separated list of comments, one for each peak in the peak list.
ViStatus _VI_FUNC rsemi_QueryFinalMeasurementPeakListData (ViSession instrSession,
                                                           ViInt32 arraySize,
                                                           ViInt32 detectorBufferSize,
                                                           ViInt32 commentBufferSize,
                                                           ViInt32 traceNumber[],
                                                           ViChar detector[],
                                                           ViReal64 frequency[],
                                                           ViReal64 level[],
                                                           ViReal64 deltaLimit[],
                                                           ViChar comment[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[] = "CALC:FME:PLIS:ALL?\n";
    ViChar*  pbuf = NULL;
    ViChar*  p2buf = NULL;
    ViChar*  input_data_needle = NULL;
    ViInt32  peak_count = 0;
    ViInt32  count_to_copy = 0;
    ViInt32  i = 0;
    ViInt32  skip_further_detectors = 0;
    ViInt32  skip_further_comments = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsemi_GetAttributeViInt32 (instrSession, "", RSEMI_ATTR_PEAK_LIST_NUMBER_OF_PEAKS, &peak_count));
    count_to_copy = (peak_count > arraySize) ? arraySize : peak_count;
    
    checkErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &pbuf, NULL));
    checkErr( rsemi_CheckStatus (instrSession));
    
    /* Zero both strings to be able to count length of string already stored in them */
    memset (detector, 0, detectorBufferSize);
    memset (comment, 0, commentBufferSize);
    
    /* Response comes in format:
    <TraceNumber>,<Detector>,<Frequency>,<Level>,<DeltaLimit>,<Comment>
    'TRACE1,Quasi-Peak,4.5e+07,53.99,0,yahoo,
    TRACE2,Average,4.5e+07,53.07,0,,
    TRACE3,RMS,4.5e+07,50.89,0,,
    TRACE4,Quasi-Peak,4.5e+07,53.99,0,,' */
    
    /* Get rid of first ' */
    p2buf = strtok (pbuf, "'");
    
    for (i = 0; i < count_to_copy; i++)
    {
        /* TraceNumber */
        if (i == 0)
            input_data_needle = strtok (p2buf, ",");
        else
            input_data_needle = strtok (NULL, ",");
            
        sscanf (input_data_needle, "TRACE%ld", &traceNumber[i]);
        
        /* Detector */
        input_data_needle = strtok (NULL, ",");
        /* One zero at the end is needed to terminate the character, one character is comma in between values */
        if ((skip_further_detectors == 0) && ((strlen (input_data_needle) + strlen (detector)) < ((size_t)detectorBufferSize - 2)))
        {
            if (i > 0) strcat (detector, ",");
            strcat (detector, input_data_needle);
        }
        else
            skip_further_detectors = 1;
        
        /* Frequency */
        input_data_needle = strtok (NULL, ",");
        sscanf (input_data_needle, "%lf", &frequency[i]);
        
        /* Level */
        input_data_needle = strtok (NULL, ",");
        sscanf (input_data_needle, "%lf", &level[i]);
        
        /* DeltaLimit */
        input_data_needle = strtok (NULL, ",");
        sscanf (input_data_needle, "%lf", &deltaLimit[i]);
        
        /* Comment */
        /* Empty comment, strtok skips ",," to next token */
		if (*(input_data_needle+ 1+strlen (input_data_needle)) != '\0')
		{
	        if ((*(input_data_needle+ 1+strlen (input_data_needle))) != ',')
	        {
	            input_data_needle = strtok (NULL, ",");
	            if ((skip_further_comments == 0) && ((strlen (input_data_needle) + strlen (comment)) < ((size_t)commentBufferSize - 2)))
	            {
	                if (i > 0) strcat (comment, ",");
	                strcat (comment, input_data_needle);
	            }
	            else
	                skip_further_comments = 1;
	        }
	        else
	        {
	            if (i > 0) strcat (comment, ",");
	        }
		}
		else
		{
			if (i > 0) strcat (comment, ",");
		}
    }
    
Error:
    if (pbuf) free (pbuf);
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Assigns a comment to a peak found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR comment/Comment string
ViStatus _VI_FUNC rsemi_ConfigureFinalMeasurementPeakComment (ViSession instrSession,
                                                              ViInt32 peak,
                                                              ViString comment)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
	viCheckParm(rsemi_SetAttributeViString(instrSession, rep_cap,
        RSEMI_ATTR_FINAL_MEASUREMENT_PEAK_COMMENT, comment), 3, "Comment");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the frequency of a peak found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR frequency/Queries the frequency of a peak found in the peak search.
ViStatus _VI_FUNC rsemi_QueryFinalMeasurementPeakFrequency (ViSession instrSession,
                                                            ViInt32 peak,
                                                            ViReal64 *frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViReal64(instrSession, rep_cap,
        RSEMI_ATTR_FINAL_MEASUREMENT_PEAK_FREQUENCY, frequency), 3, "Frequency");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the level of a peak found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR level/Queries the level of a peak found in the peak search.
ViStatus _VI_FUNC rsemi_QueryFinalMeasurementPeakLevel (ViSession instrSession,
                                                        ViInt32 peak,
                                                        ViReal64 *level)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViReal64(instrSession, rep_cap,
        RSEMI_ATTR_FINAL_MEASUREMENT_PEAK_LEVEL, level), 3, "Level");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the detector with which a peak was found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR detectorBufferSize/Defines the size of buffer in argument 'Detector'.
/// HIPAR detector/Queries the detector with which a peak was found in the peak search.
ViStatus _VI_FUNC rsemi_QueryFinalMeasurementPeakDetector (ViSession instrSession,
                                                           ViInt32 peak,
                                                           ViInt32 detectorBufferSize,
                                                           ViChar detector[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViString(instrSession, rep_cap,
        RSEMI_ATTR_FINAL_MEASUREMENT_PEAK_DETECTOR, detectorBufferSize, detector), 3, "Detector");    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the distance of a peak to the nearest limit line in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR deltaLimit/Queries the distance of a peak to the nearest limit line in the peak search..
ViStatus _VI_FUNC rsemi_QueryFinalMeasurementPeakDeltaLimit (ViSession instrSession,
                                                             ViInt32 peak,
                                                             ViReal64 *deltaLimit)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViReal64(instrSession, rep_cap,
        RSEMI_ATTR_FINAL_MEASUREMENT_PEAK_DELTA, deltaLimit), 3, "Delta Limit");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the trace that a peak found in the peak search is located on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR traceNumber/Queries the trace that a peak found in the peak search is located on.
ViStatus _VI_FUNC rsemi_QueryFinalMeasurementPeakTrace (ViSession instrSession,
                                                        ViInt32 peak,
                                                        ViInt32 *traceNumber)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViInt32(instrSession, rep_cap,
        RSEMI_ATTR_FINAL_MEASUREMENT_PEAK_TRACE, traceNumber), 3, "Trace Number");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the information for a peak found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR detectorBufferSize/Defines the size of buffer in argument 'Detector'.
/// HIPAR commentBufferSize/Defines the size of buffer in argument 'Comment'.
/// HIPAR traceNumber/Trace number
/// HIPAR detector/Detector
/// HIPAR frequency/Frequency
/// HIPAR level/Peak level
/// HIPAR deltaLimit/Delta limit. If no limit line was used, 0 is returned.
/// HIPAR comment/Comment
ViStatus _VI_FUNC rsemi_QueryFinalMeasurementPeakData (ViSession instrSession,
                                                       ViInt32 peak,
                                                       ViInt32 detectorBufferSize,
                                                       ViInt32 commentBufferSize,
                                                       ViInt32 *traceNumber,
                                                       ViChar detector[],
                                                       ViReal64 *frequency,
                                                       ViReal64 *level,
                                                       ViReal64 *deltaLimit,
                                                       ViChar comment[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[100] = "";
    ViChar*  pbuf = NULL;
    ViChar*  p2buf = NULL;
    ViChar*  input_data_needle = NULL;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rsemi_invalidViInt32Range (instrSession, peak, 1, 3000), 2, "Peak");

	sprintf (cmd, "CALC:FME:PLIS%ld?\n", peak);
	checkErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &pbuf, NULL));
    checkErr( rsemi_CheckStatus (instrSession));
    
    /* Zero both strings to be able to count length of string already stored in them */
    memset (detector, 0, detectorBufferSize);
    memset (comment, 0, commentBufferSize);
    
    /* Response comes in format:
    <TraceNumber>,<Detector>,<Frequency>,<Level>,<DeltaLimit>,<Comment>
    'TRACE1,Quasi-Peak,4.5e+07,53.99,0,yahoo' */
    
    /* Get rid of first ' */
    p2buf = strtok (pbuf, "'");
    
    /* TraceNumber */
    input_data_needle = strtok (p2buf, ",");
    sscanf (input_data_needle, "TRACE%ld", traceNumber);
    
    /* Detector */
    input_data_needle = strtok (NULL, ",");
    /* One zero at the end is needed to terminate the character, one character is comma in between values */
	strncpy (detector, input_data_needle, detectorBufferSize - 1);

	/* Frequency */
    input_data_needle = strtok (NULL, ",");
    sscanf (input_data_needle, "%lf", frequency);
    
    /* Level */
    input_data_needle = strtok (NULL, ",");
    sscanf (input_data_needle, "%lf", level);
    
    /* DeltaLimit */
    input_data_needle = strtok (NULL, ",");
    sscanf (input_data_needle, "%lf", deltaLimit);
    
    /* Comment */
    /* Empty comment, strtok skips ",," to next token */
	if (*(input_data_needle+ 1+strlen (input_data_needle)) != '\0')
	{
        if ((*(input_data_needle+ 1+strlen (input_data_needle))) != ',')
	    {
	        input_data_needle = strtok (NULL, ",");
			strncpy (comment, input_data_needle, commentBufferSize);
	    }
	}
    
Error:
    if (pbuf) free (pbuf);
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Enables and disables the 150 kHz high pass filter of the ENV216 
/// HIFN  network for the premeasurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR 150kHzHighpassFilter/Enables and disables the 150 kHz high pass filter of the ENV216 
/// HIPAR 150kHzHighpassFilter/network for the premeasurement.
ViStatus _VI_FUNC rsemi_ConfigurePrescanningVnetworkHighpass (ViSession instrSession,
                                                              ViBoolean _150_kHzHighPassFilter)
{
   
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_PRESCANNING_LISN_V_NETWORK_HIGH_PASS_FILTER,
                                               _150_kHzHighPassFilter), 2, "150 kHz High pass Filter");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Selects the phase of the network used for the premeasurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR phases/Selects the phase of the network used for the premeasurement.
ViStatus _VI_FUNC rsemi_ConfigurePrescanningVnetworkPhase (ViSession instrSession,
                                                           ViInt32 phases)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_PRESCANNING_LISN_V_NETWORK_PHASE,
                                               phases), 2, "Phases");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Enables and disables the automatic control of the network used for the 
/// HIFN  premeasurement, and also selects the type of network in use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR networkType/Selects the type of network in use.
ViStatus _VI_FUNC rsemi_ConfigurePrescanningVnetworkType (ViSession instrSession,
                                                          ViInt32 networkType)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, "", 
                                               RSEMI_ATTR_PRESCANNING_LISN_V_NETWORK_TYPE,
                                               networkType), 2, "Network Type");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Enables the active marker on the specified trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerEnabled/Specifies whether the specified delta marker is enabled. 
/// HIPAR trace/Selects the trace for Detector definition.
ViStatus _VI_FUNC rsemi_ConfigureMarker (ViSession instrSession,
                                         ViInt32 window, ViInt32 marker,
                                         ViBoolean markerEnabled, ViInt32 trace)
{
    
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_ENABLED,
                                               markerEnabled), 4, "Marker Enabled");
    
    if (markerEnabled == VI_TRUE)
    {
       viCheckParm( rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_TRACE,
                                               trace), 5, "Trace");    
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Couples or decouples the receiver frequency to the current marker 
/// HIFN  frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerTracking/Enables and disables coupling or decoupling the receiver frequency to 
/// HIPAR markerTracking/the current marker frequency.
ViStatus _VI_FUNC rsemi_ConfigureMarkerFrequencyCoupling (ViSession instrSession,
                                                          ViInt32 window,
                                                          ViInt32 marker,
                                                          ViBoolean markerTracking)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_FREQUENCY_COUPLING,
                                               markerTracking), 4, "Marker Tracking");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;    
    
}

/// HIFN  Maches the selected marker frequency to the center frequency or 
/// HIFN  frequency step size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR instrumentSetting/Specifies the instrument setting to be set from the marker position.
ViStatus _VI_FUNC rsemi_ConfigureReceiverFromMarker (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViInt32 marker,
                                                     ViInt32 instrumentSetting)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        

    
    sprintf (buffer, "Win%ld,M%ld", window, marker);  
    
    switch(instrumentSetting)
    {
        case RSEMI_VAL_RECEIVER_FREQUENCY_CENTER:
            viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                RSEMI_ATTR_MARKER_TO_CENTER, VI_NULL));
        break;
        
        case RSEMI_VAL_RECEIVER_FREQUENCY_STEP:
               viCheckErr( rsemi_SetAttributeViString (instrSession, buffer,
                RSEMI_ATTR_MARKER_TO_STEP, VI_NULL));
               
        break;       
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the range to be zoomed around the selected marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR range/Defines the range to be zoomed around selected marker.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_ConfigureMarkerZoom (ViSession instrSession,
                                             ViInt32 window, ViInt32 marker,
                                             ViReal64 range, ViInt32 timeout)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout = -1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    if (timeout < 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Timeout");
    
    checkErr (rsemi_GetOPCTimeout (instrSession, &old_timeout));
    checkErr (rsemi_SetOPCTimeout (instrSession, timeout));
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_ZOOM,
                                               range), 4, "Range");
    
Error:
    if (old_timeout >= 0)    
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Turns the marker info field in the diagram on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR infoFieldEnabled/Enables and disables the marker info field in the diagram on and off.
ViStatus _VI_FUNC rsemi_ConfigureMarkerInfoFieldState (ViSession instrSession,
                                                       ViInt32 window,
                                                       ViBoolean infoFieldEnabled)
{
    
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    
    viCheckParm(rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_MARKER_INFO_FIELD_ENABLED,
                                               infoFieldEnabled), 3, "Info Field Enabled");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Couples or decouples the marker frequency to the scan range settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR settingsCoupled/Couples or decouples the marker frequency to the scan range settings
ViStatus _VI_FUNC rsemi_ConfigureMarkerCoupling (ViSession instrSession,
                                                 ViInt32 window, ViInt32 marker,
                                                 ViBoolean settingsCoupled)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_COUPLING_ENABLED,
                                               settingsCoupled), 4, "Settings Coupled");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Selects the measurement configuration to be applied when you
/// HIFN  couple receiver settings to scan range settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR useLastScan/Selects the measurement configuration to be applied when you
/// HIPAR useLastScan/couple receiver settings to scan range settings.
ViStatus _VI_FUNC rsemi_ConfigureMarkerCouplingUseLastScan (
    ViSession instrSession,
    ViInt32 window,
    ViInt32 marker,
    ViBoolean useLastScan
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_USE_LAST_SCAN_ENABLED,
                                               useLastScan), 4, "Use Last Scan");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Enables and disables marker search limits, and sets the sides of the 
/// HIFN  limits of the marker search range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control sets the marker.
/// HIPAR searchLimits/Enables and disables the marker search limit.
/// HIPAR searchLimitLeft/Sets the left limit of the marker search range.
/// HIPAR searchLimitRight/Sets the right limit of the marker search range.
ViStatus _VI_FUNC rsemi_ConfigureMarkerSearchLimits (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViInt32 marker,
                                                     ViBoolean searchLimits,
                                                     ViReal64 searchLimitLeft,
                                                     ViReal64 searchLimitRight)
{
    
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        

    
    sprintf (buffer, "Win%ld,M%ld", window, marker);  
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_SEARCH_LIMITS_STATE,
                                               searchLimits), 4, "Search Limits");
    if (searchLimits == VI_TRUE)
    {
        
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_SEARCH_LIMITS_RIGHT,
                                               searchLimitRight), 6, "Search Limit Right");
        
        viCheckParm( rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_SEARCH_LIMITS_LEFT,
                                               searchLimitLeft), 5, "Search Limit Left");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Sets the Positions of the marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control sets the marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
ViStatus _VI_FUNC rsemi_ConfigureMarkerSearch (ViSession instrSession,
                                               ViInt32 window, ViInt32 marker,
                                               ViInt32 markerSearch)
{
   ViStatus    error   = VI_SUCCESS;
   ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");        
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    switch (markerSearch)
    {
            case RSEMI_VAL_MARKER_SEARCH_HIGHEST:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_PEAK, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_PEAK_NEXT, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_PEAK_LEFT, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_MARKER_SEARCH_PEAK_RIGHT, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_MINIMUM:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_SEARCH_MIN, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_SEARCH_MIN_NEXT, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_SEARCH_MIN_LEFT, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_SEARCH_MIN_RIGHT, VI_NULL));
            break;

            default: 
                viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Parameter to set");
            break;
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Moves the active marker to the specified horizontal position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control sets the marker.
/// HIPAR position/Sets the position of the marker on a particular coordinate on the 
/// HIPAR position/x-axis.
ViStatus _VI_FUNC rsemi_MoveMarker (ViSession instrSession, ViInt32 window,
                                    ViInt32 marker, ViReal64 position)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm(rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                               RSEMI_ATTR_MARKER_POSITION,
                                               position), 4, "Position");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Turns off all the markers in the selected measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_DisableAllMarkers (ViSession instrSession,
                                           ViInt32 window)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                RSEMI_ATTR_MARKER_AOFF, VI_NULL));
    
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error; 
}

/// HIFN  Queries the measured value of the vertical marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control sets the marker.
/// HIPAR markerAmplitude/Returns the measured value of the vertical marker position.
ViStatus _VI_FUNC rsemi_QueryMarker (ViSession instrSession, ViInt32 window,
                                     ViInt32 marker, ViReal64 *markerAmplitude)
{
 
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm (rsemi_GetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_MARKER_AMPLITUDE, 
                markerAmplitude), 4, "Marker Amplitude");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Setting defines the vertical marker position in the persistence 
/// HIFN  spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control sets the marker.
/// HIPAR markerAmplitude/Setting defines the vertical marker position in the persistence 
/// HIPAR markerAmplitude/spectrum result display.
ViStatus _VI_FUNC rsemi_ConfigureMarkerAmplitude(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64    markerAmplitude
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, marker, 1, 16), 3, "Marker");
    
    sprintf (buffer, "Win%ld,M%ld", window, marker);
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_MARKER_AMPLITUDE, 
                markerAmplitude), 4, "Marker Amplitude");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;    
}

/// HIFN  Enables and disables the delta markers on the specified trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control sets the delta marker.
/// HIPAR state/Enables and disables the delta markers.
/// HIPAR trace/Sets the trace.
ViStatus _VI_FUNC rsemi_ConfigureDeltaMarker (ViSession instrSession,
                                              ViInt32 window,
                                              ViInt32 deltaMarker,
                                              ViBoolean state, ViInt32 trace)
{
    
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_STATE,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_STATE,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm(rsemi_SetAttributeViBoolean (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_STATE, state),
                                                4, "State");
    if (state == VI_TRUE) 
    {        
    viCheckParm(rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                            RSEMI_ATTR_DMARKER_TRACE, trace),
                                            5, "Trace");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;    
}

/// HIFN  Disables all the delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_DisableAllDeltaMarker (ViSession instrSession,
                                               ViInt32 window)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_AOFF,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap));

    viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                RSEMI_ATTR_DMARKER_AOFF, VI_NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error; 
    
}

/// HIFN  Enables and disables the linking between delta markers and markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control sets the delta marker.
/// HIPAR state/Enables and disables linking between delta markers and markers.
ViStatus _VI_FUNC rsemi_LinkMarkerAndDeltaMarker (ViSession instrSession,
                                                  ViInt32 window,
                                                  ViInt32 deltaMarker,
                                                  ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_LINK,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_LINK,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, rep_cap, 
                                               RSEMI_ATTR_DMARKER_LINK,
                                               state), 4, "State");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Sets the position of the delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control sets the delta marker.
/// HIPAR deltaMarkerSearch/Specifies delta marker search type.
ViStatus _VI_FUNC rsemi_DeltaMarkerSearch (ViSession instrSession,
                                           ViInt32 window, ViInt32 deltaMarker,
                                           ViInt32 deltaMarkerSearch)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_PEAK,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_PEAK,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    switch (deltaMarkerSearch)
    {
            case RSEMI_VAL_MARKER_SEARCH_HIGHEST:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_DMARKER_PEAK, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_DMARKER_PEAK_NEXT, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_DMARKER_PEAK_NEXT_LEFT, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_DMARKER_PEAK_NEXT_RIGHT, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_MINIMUM:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_DMARKER_MIN, 
                                     VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                RSEMI_ATTR_DMARKER_MIN_NEXT, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                RSEMI_ATTR_DMARKER_MIN_NEXT_LEFT, VI_NULL));
            break;
            case RSEMI_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
                viCheckErr( rsemi_SetAttributeViString (instrSession, rep_cap, 
                RSEMI_ATTR_DMARKER_MIN_NEXT_RIGHT, VI_NULL));
            break;

            default: 
                viCheckParm ( RS_ERROR_INVALID_PARAMETER, 4, "Delta Marker Search");
            break;
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Configures the delta marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control sets the delta marker.
/// HIPAR deltaMarkerMode/Selects the delta marker mode.
/// HIPAR position/Sets the position of the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureDeltaMarkerPosition (ViSession instrSession,
                                                      ViInt32 window,
                                                      ViInt32 deltaMarker,
                                                      ViInt32 deltaMarkerMode,
                                                      ViReal64 position)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_MODE,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_MODE,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm( rsemi_SetAttributeViInt32 (instrSession, rep_cap, 
                                               RSEMI_ATTR_DMARKER_MODE, deltaMarkerMode),
                                               4, "Delta Marker Mode");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_POSITION, position),
                                                5, "Position");
                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  Returns the relative horizontal position and the marker amplitude 
/// HIFN  level of the selected delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR mode/Switches between absolute and relative position value.
/// HIPAR position/Returns the relative position of selected delta marker relative to 
/// HIPAR position/marker 1 or reference position.
/// HIPAR amplitude/Queries the measured value of a delta marker. The corresponding delta 
/// HIPAR amplitude/marker is activated, if necessary. The output is always a relative 
/// HIPAR amplitude/value referred to marker 1 or to the reference position (reference 
/// HIPAR amplitude/fixed active).
ViStatus _VI_FUNC rsemi_QueryDeltaMarker (ViSession instrSession,
                                          ViInt32 window, ViInt32 deltaMarker,
                                          ViInt32 mode, ViReal64 *position,
                                          ViReal64 *amplitude)
{
   
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_REL_POSITION,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_DMARKER_REL_POSITION,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    switch (mode)
    {
        case RSEMI_VAL_REL:
            viCheckParm( rsemi_GetAttributeViReal64 (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_REL_POSITION, position),
                                                5, "Position");   
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Mode"); 
    }        
    
    viCheckParm( rsemi_GetAttributeViReal64 (instrSession, rep_cap, 
                                                RSEMI_ATTR_DMARKER_AMPLITUDE, amplitude),
                                                6, "Amplitude");
                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Switches the numeric indication of the maximum value of the bargraph.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bargraphMaxhold/This control switches on the numeric indication of the maximum value 
/// HIPAR bargraphMaxhold/of the bargraph.
ViStatus _VI_FUNC rsemi_ConfigureDisplayBargraphMaxhold (ViSession instrSession,
                                                         ViBoolean bargraphMaxhold)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsemi_SetAttributeViBoolean (instrSession, "", 
                                               RSEMI_ATTR_BARGRAPH_MAXHOLD_STATE,
                                               bargraphMaxhold), 2, "Bargraph Maxhold");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error; 
}

/// HIFN  This function resets the maxhold value of the numeric indication of 
/// HIFN  the bargraph measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_ResetDisplayBargraphMaxhold (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr(rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_BARGRAPH_MAXHOLD_RESET, NULL));    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Queries the minimum and maximum or maximum level of the bargraph.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR levelType/Selects the level type.
/// HIPAR bargraphLevel/Returns the minimum  or the maximum level of the bargraph.
ViStatus _VI_FUNC rsemi_QueryDisplayBargraphLevel (ViSession instrSession,
                                                   ViInt32 levelType,
                                                   ViReal64 *bargraphLevel)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (levelType){
        case RSEMI_VAL_LOW:
            viCheckParm(rsemi_GetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_BARGRAPH_LEVEL_MINIMUM, bargraphLevel),
                                                3, "Bargraph Level");
            break;
        case RSEMI_VAL_UPP:
             viCheckParm(rsemi_GetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_BARGRAPH_LEVEL_MAXIMUM, bargraphLevel),
                                                3, "Bargraph Level");
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Level Type");  
    }
                    
    
    Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Selects the detector for the bargraph measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR detector/Selects the detector for the bargraph measurement.
/// HIPAR arraySize/Sets the number of detectors to be enabled.
ViStatus _VI_FUNC rsemi_ConfigureDisplayBargraphDetectorSelection (ViSession instrSession,
                                                                   ViInt32 arraySize, ViInt32 detector[])
{
 
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, arraySize, 1, 4), 2, "Array Size");
    
    pCmd += sprintf (pCmd, "DET:REC ");
   
    for ( i = 0; i < arraySize; i++ )
    {
        pCmd += sprintf(pCmd, "%s,", DetectorTypeArr[detector[i]]);
    }

    *pCmd = '\0';
    *--pCmd = '\n';
               
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
       
    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);  
    
    return error; 
}

/// HIFN  Defines the measurement time for bargraph measurements and scans.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR time/Selects the measurement time for bargraph measurements and scans.
ViStatus _VI_FUNC rsemi_ConfigureDisplayBargraphMeasurementTime (ViSession instrSession,
                                                                 ViReal64 time)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm(rsemi_SetAttributeViReal64 (instrSession, "", 
                                                RSEMI_ATTR_BARGRAPH_MEASUREMENT_TIME, time),
                                                2, "Time");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the type of display and the evaluation of the traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/This control sets the trace.
/// HIPAR mode/This control defines the type of display and the evaluation of the 
/// HIPAR mode/trace.
ViStatus _VI_FUNC rsemi_ConfigureDisplayBargraphTraceMode (ViSession instrSession,
                                                           ViInt32 window,
                                                           ViInt32 trace,
                                                           ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 3, "Trace");        
    
    sprintf (buffer, "Win%ld,TR%ld",window, trace); 
    
    viCheckParm(rsemi_SetAttributeViInt32 (instrSession, buffer, 
                                                RSEMI_ATTR_BARGRAPH_TRACE_MODE, mode),
                                                4, "Mode");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Couples or decouples the bargraph detector and scan trace detector.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_BARGRAPH_COUPLING
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DISPlay:BARGraph:TCOupling
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR coupling/Couples or decouples the bargraph detector and scan trace detector.
ViStatus _VI_FUNC rsemi_ConfigureDisplayBargraphCoupling(
    ViSession    instrSession,
    ViBoolean    coupling
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                              RSEMI_ATTR_BARGRAPH_COUPLING,
                                              coupling), 2, "Coupling");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Enables and disables time line. Time lines are only available with SPAN = 0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/Selects the display line.
/// HIPAR state/Enables and disables time line.
ViStatus _VI_FUNC rsemi_ConfigureTimeLineState (ViSession instrSession,
                                                ViInt32 window, ViInt32 line,
                                                ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");        
    
    sprintf (buffer, "Win%ld,TL%ld",window, line); 
    
    viCheckParm(rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                             RSEMI_ATTR_TIME_LINE_STATE, state),
                                             4, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Defines the position of a time line. Time lines are only available with SPAN = 0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/Selects the display line.
/// HIPAR position/Defines the position of a time line.
ViStatus _VI_FUNC rsemi_ConfigureTimeLinePosition (ViSession instrSession,
                                                   ViInt32 window, ViInt32 line,
                                                   ViReal64 position)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");        
    
    sprintf (buffer, "Win%ld,TL%ld",window, line); 
    
    viCheckParm(rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                             RSEMI_ATTR_TIME_LINE_POSITION, position),
                                             4, "Position");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Enables and disables display line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/Selects the display line.
/// HIPAR state/Enables and disables display line.
ViStatus _VI_FUNC rsemi_ConfigureDisplayLineState (ViSession instrSession,
                                                   ViInt32 window, ViInt32 line,
                                                   ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");        
    
    sprintf (buffer, "Win%ld,DL%ld",window, line); 
    
    viCheckParm(rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                             RSEMI_ATTR_DISPLAY_LINE_STATE, state),
                                             4, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the (vertical) position of a display line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/Selects the display line.
/// HIPAR position/Defines the position of Display Line.
ViStatus _VI_FUNC rsemi_ConfigureDisplayLinePosition (ViSession instrSession,
                                                      ViInt32 window,
                                                      ViInt32 line,
                                                      ViReal64 position)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");        
    
    sprintf (buffer, "Win%ld,DL%ld",window, line); 
    
    viCheckParm(rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                             RSEMI_ATTR_DISPLAY_LINE_POSITION, position),
                                             4, "Position");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Enables and disables line frequency line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/Selects the frequency line.
/// HIPAR state/Enables and disables the frequency line.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyLineState (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViInt32 line,
                                                     ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");        
    
    sprintf (buffer, "Win%ld,FL%ld",window, line); 
    
    viCheckParm(rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                             RSEMI_ATTR_FREQUENCY_LINE_STATE, state),
                                             4, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Defines the position of a frequency line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR line/Selects the frequency line.
/// HIPAR position/Defines the position of Frequency Line.
ViStatus _VI_FUNC rsemi_ConfigureFrequencyLinePosition (ViSession instrSession,
                                                        ViInt32 window,
                                                        ViInt32 line,
                                                        ViReal64 position)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, line, 1, 2), 3, "Line");        
    
    sprintf (buffer, "Win%ld,FL%ld",window, line); 
    
    viCheckParm(rsemi_SetAttributeViReal64 (instrSession, buffer, 
                                             RSEMI_ATTR_FREQUENCY_LINE, position),
                                             4, "Position");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  This function creates new limit or selects existing one and configures 
/// HIFN  basic parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects limit line.
/// HIPAR name/This control sets the limit line name.
/// HIPAR domain/This control defines the frequency or time domain for the x axis 
/// HIPAR domain/values.
/// HIPAR comment/This control sets the limit line comment.
/// HIPAR assign_toTrace/Assigns a limit line to a trace in the indicated measurement window.
/// HIPAR deleteExistingLine/Removes specified limit line before new is created.
ViStatus _VI_FUNC rsemi_CreateLimitLine (ViSession instrSession,
                                         ViInt32 window, ViInt32 limit,
                                         ViString name, ViInt32 domain,
                                         ViString comment,
                                         ViInt32 assign_toTrace,
                                         ViBoolean deleteExistingLine)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      error_message[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 3, "Limit");
    
    if (strlen (name) > 8)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Name");
    }

    if (strlen (comment) > 40)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Comment");
    }

    sprintf (buffer, "LL%ld", limit);
    
    if (deleteExistingLine == VI_TRUE)
    {
        error = rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_LIMIT_DELETE,"");
    
        do
        {
            (void) rsemi_error_query (instrSession, &error, error_message);
        } while (error != 0);
    }
    
    viCheckParm (rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_LIMIT_NAME, name), 4, "Name");
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_LIMIT_CONTROL_DOMAIN,domain), 5, "Domain");
    viCheckParm (rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_LIMIT_COMMENT,comment), 6, "Comment");
    
    sprintf (buffer, "Win%ld,LL%ld", window, limit);
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_LIMIT_ASSIGN_TRACE, assign_toTrace), 7, "Assign to Trace");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  Queries the names of all active limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control defines the size of array in argument 'Active Limit 
/// HIPAR arraySize/Lines'.
/// HIPAR activeLimitLines/Returns the names of all active limit lines.
/// HIPAR returnedStringLength/Returns the length of string returned from the instrument.
ViStatus _VI_FUNC rsemi_QueryActiveLimitLines (ViSession instrSession,
                                               ViInt32 arraySize,
                                               ViChar activeLimitLines[],
                                               ViInt32 *returnedStringLength)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer,
                *pbuf;
    ViInt32    retCnt;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!activeLimitLines)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Active Limit Lines");
    
    viCheckErr( viPrintf (instrSession, ":CALC:LIM:ACT?\n")); 
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, VI_NULL)); 
    retCnt = (ViUInt32) (strrchr (pbuffer, '\'') - pbuffer);
    if (returnedStringLength) 
    {
        *returnedStringLength = retCnt;
    }   
    
    retCnt=(arraySize>retCnt)?retCnt:arraySize;
    pbuf=pbuffer;
    pbuf++;
    
    strncpy(activeLimitLines, pbuf, retCnt-1);
    activeLimitLines[retCnt-1]='\0';
    
    if (pbuffer) free(pbuffer);
    
    checkErr( rsemi_CheckStatus (instrSession)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function copies specified limit line into another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit.
/// HIPAR copyTo/This control selects limit.
ViStatus _VI_FUNC rsemi_CopyLimitLine (ViSession instrSession, ViInt32 limit,
                                       ViInt32 copyTo)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 2, "Limit");

    sprintf (buffer, "LL%ld", limit);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_LIMIT_COPY, copyTo), 3, "Copy To");


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Defines the unit of a limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects the limit line.
/// HIPAR unit/Selects the unit of the selected limit line.
ViStatus _VI_FUNC rsemi_ConfigureLimitLinesUnit (ViSession instrSession,
                                                 ViInt32 limit, ViInt32 unit)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 2, "Limit");

    sprintf (buffer, "LL%ld", limit);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_LIMIT_UNIT, unit), 3, "Unit");


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Selects the scaling mode for selected axis of the selected limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects the limit line.
/// HIPAR type/Selects the axis for which the interpolation should be set.
/// HIPAR limitLineScaling/Selects the limit line scaling.
ViStatus _VI_FUNC rsemi_ConfigureLimitLineScaling (ViSession instrSession,
                                                   ViInt32 window, ViInt32 limit,
                                                   ViInt32 type,
                                                   ViInt32 limitLineScaling)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_lim[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_SCALING_X,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win));
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_SCALING_X,
                                                 "LimitLine",
                                                 limit - 1,
                                                 10,
                                                 rep_cap_lim));
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_lim);

    switch (type){
        case  RSEMI_VAL_LIMIT_CONT:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_SCALING_X, limitLineScaling), 3, "Limit Line Scaling");
        break;
        case  RSEMI_VAL_LIMIT_LOW:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_SCALING_LOWER, limitLineScaling), 3, "Limit Line Scaling");
        break;
        case  RSEMI_VAL_LIMIT_UPP:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_SCALING_UPPER, limitLineScaling), 3, "Limit Line Scaling");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");  
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}        



/// HIFN  Defines the selected axis values of the upper or lower limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit line. 
/// HIPAR type/An upper line will be used as the maximum allowable value when 
/// HIPAR type/comparing with the data. A lower limit line defines the minimum value.
/// HIPAR count/Specifies number of values in X Axis and Amplitude arrays.
/// HIPAR xAxis/Specifies the frequency or time value for a limit point.
/// HIPAR amplitude/Specifies the amplitude value for the current limit point.
ViStatus _VI_FUNC rsemi_DefineLimitLine (ViSession instrSession,
                                         ViInt32 window, ViInt32 limit,
                                         ViInt32 type, ViInt32 count,
                                         ViReal64 xAxis[], ViReal64 amplitude[])
{

    ViStatus    error   = VI_SUCCESS;
    ViChar      *pbuffer= VI_NULL;
    ViChar      *p2buf;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 3, "Limit");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, type, 1, 2), 4, "Type");
    if (count < 1)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Count");
    }
    if (xAxis == NULL)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "X Axis");
    }
    if (amplitude == NULL)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Amplitude");
    }

    viCheckAlloc (pbuffer = (ViChar*) malloc ((size_t)(count * 20 + 200)));
    
    p2buf = pbuffer + sprintf (pbuffer, "CALC%ld:LIM%ld:CONT ", window, limit);
    
    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%le,", xAxis[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    viCheckErr (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), VI_NULL));

    p2buf = pbuffer + sprintf (pbuffer, "CALC%ld:LIM%ld:%s ", window, limit, limTypeArr[type]);
    
    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%le,", amplitude[i]);
    
    *p2buf = '\0';
    *--p2buf = '\n';
    
    viCheckErr (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), VI_NULL));

    checkErr( rsemi_CheckStatus (instrSession));

Error:
    if (pbuffer) free (pbuffer);
    
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
    
}

/// HIFN  Defines an area around a upper and lower limit line where limit check 
/// HIFN  violations are still tolerated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/Selects the limit line.
/// HIPAR type/Selects the limit line type.
/// HIPAR margin/Defines an area around an upper and lower limit line where limit check 
/// HIPAR margin/violations are still tolerated.
ViStatus _VI_FUNC rsemi_ConfigureLimitLineMargin (ViSession instrSession,
                                                  ViInt32 limit, ViInt32 type,
                                                  ViReal64 margin)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_MARGIN_LOWER,
                                                 "LimitLine",
                                                 limit - 1,
                                                 10,
                                                 rep_cap));
    switch (type){
        case RSEMI_VAL_LIMIT_LOW:
                viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_MARGIN_LOWER, margin), 4, "Margin");
        break;
        case RSEMI_VAL_LIMIT_UPP:
                viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_MARGIN_UPPER, margin), 4, "Margin");
        break;        
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");      
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  Defines an offset for a complete lower and upper limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects the limit line.
/// HIPAR type/Selects the axis for which the interpolation should be set.
/// HIPAR offset/Defines an offset for the X axis value of the selected relative limit 
/// HIPAR offset/line in the frequency or time domain.
ViStatus _VI_FUNC rsemi_ConfigureLimitLineOffset (ViSession instrSession,
                                                   ViInt32 limit, ViInt32 type,
                                                   ViReal64 offset)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_OFFSET_X,
                                                 "LimitLine",
                                                 limit - 1,
                                                 10,
                                                 rep_cap));
     
     switch (type){
        case  RSEMI_VAL_LIMIT_CONT:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_OFFSET_X, offset), 4, "Offset");
        break;
        case  RSEMI_VAL_LIMIT_LOW:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_OFFSET_LOWER, offset), 4, "Offset");
        break;
        case  RSEMI_VAL_LIMIT_UPP:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_OFFSET_UPPER, offset), 4, "Offset");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");  
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Moves a complete limit line horizontally and vertically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects the limit line.
/// HIPAR type/Selects the axis for which the interpolation should be set.
/// HIPAR distance/Moves a limit line by the indicated value in x and y direction.
ViStatus _VI_FUNC rsemi_ConfigureLimitLineShift (ViSession instrSession,
                                                 ViInt32 limit,ViInt32 type,
                                                 ViReal64 distance)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_SHIFT_X,
                                                 "LimitLine",
                                                 limit - 1,
                                                 10,
                                                 rep_cap));
     
     switch (type){
        case  RSEMI_VAL_LIMIT_CONT:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_SHIFT_X, distance), 4, "Distance"); 
        break;
        case  RSEMI_VAL_LIMIT_LOW:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_SHIFT_LOWER, distance), 4, "Distance"); 
        break;
        case  RSEMI_VAL_LIMIT_UPP:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_SHIFT_UPPER, distance), 4, "Distance"); 
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");  
    }


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Selects linear or logarithmic interpolation for the calculation of 
/// HIFN  limit lines from frequency points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects the limit line.
/// HIPAR type/Selects the axis for which the interpolation should be set.
/// HIPAR interpolationMode/Selects linear or logarithmic interpolation for the calculation of 
/// HIPAR interpolationMode/limit lines from frequency points.
ViStatus _VI_FUNC rsemi_ConfigureLimitLineInterpolation (ViSession instrSession,
                                                         ViInt32 limit,
                                                         ViInt32 type,
                                                         ViInt32 interpolationMode)
{
    
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_SPACING_X,
                                                 "LimitLine",
                                                 limit - 1,
                                                 10,
                                                 rep_cap));
     
     switch (type){
        case  RSEMI_VAL_LIMIT_CONT:
            viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_SPACING_X, interpolationMode), 4, "Interpolation Mode");
        break;
        case  RSEMI_VAL_LIMIT_LOW:
            viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_SPACING_LOWER, interpolationMode), 4, "Interpolation Mode");
        break;
        case  RSEMI_VAL_LIMIT_UPP:
            viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_SPACING_UPPER, interpolationMode), 4, "Interpolation Mode");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");  
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Enables and disables the upper and lower limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects the limit line.
/// HIPAR type/This control selects the limit type.
/// HIPAR state/Enables and disables the indicated limit line.
ViStatus _VI_FUNC rsemi_ConfigureLimitLineVisibility (ViSession instrSession,
                                                      ViInt32 limit,
                                                      ViInt32 type,
                                                      ViBoolean state)
{
    ViStatus    error   = VI_SUCCESS;
       ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_VISIBILITY_LOWER,
                                                 "LimitLine",
                                                 limit - 1,
                                                 10,
                                                 rep_cap));
     
     switch (type){
        case  RSEMI_VAL_LIMIT_LOW:
            viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_VISIBILITY_LOWER, state), 4, "State");
        break;
        case  RSEMI_VAL_LIMIT_CONT:
            viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_VISIBILITY_UPPER, state), 4, "State");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");  
    }
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  Defines a threshold for relative limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects the limit line.
/// HIPAR type/Selects the limit line type.
/// HIPAR threshold/Defines a threshold for relative limit lines.
ViStatus _VI_FUNC rsemi_ConfigureLimitLineThreshold (ViSession instrSession,
                                                     ViInt32 limit,
                                                     ViInt32 type,
                                                     ViReal64 threshold)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_LIMIT_LINE_THRESHOLD_LOWER,
                                                 "LimitLine",
                                                 limit - 1,
                                                 10,
                                                 rep_cap));
     
     switch (type){
        case  RSEMI_VAL_LIMIT_LOW:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_THRESHOLD_LOWER, threshold), 4, "Threshold");
        break;
        case  RSEMI_VAL_LIMIT_UPP:
            viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap, RSEMI_ATTR_LIMIT_LINE_THRESHOLD_UPPER, threshold), 4, "Threshold");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");  
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Enables and disables the limit line check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects the limit line.
/// HIPAR state/Enables and disables the limit check.
ViStatus _VI_FUNC rsemi_ConfigureLimitLineCheck (ViSession instrSession,
                                                 ViInt32 limit, ViBoolean state)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 2, "Limit");
    
    sprintf (buffer, "LL%ld", limit);

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, buffer, RSEMI_ATTR_LIMIT_LINE_CHECK_ENABLED, state), 3, "State");


Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Deletes the result of the current limit check for all limit lines in 
/// HIFN  all windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_ClearLimitLineCheckResult (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "",
                                                  RSEMI_ATTR_CLEAR_LIMIT_LINE_CHECK_RESULT,
                                                  VI_NULL));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Queries the result of a limit check for the selected limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects the limit line.
/// HIPAR result/Returns the result of the limit check.
ViStatus _VI_FUNC rsemi_QueryLimitLineCheckResult (ViSession instrSession,
                                                   ViInt32 limit,
                                                   ViInt32 *result)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, limit, 1, 8), 2, "Limit");
    
    sprintf (buffer, "LL%ld", limit);

    viCheckParm (rsemi_GetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_QUERY_LIMIT_LINE_CHECK_RESULT, result), 3, "Result");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*
 *------------------- Layout ------------------------------------------------*
 *===========================================================================*/
/// HIFN This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR direction/Direction the window is added relative to the window set with "Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/Returns the name of the new window.
ViStatus _VI_FUNC rsemi_AddReceiverWindow (
    ViSession instrSession,
    ViInt32 stringSize,
    ViString windowName,
    ViInt32 direction, 
    ViInt32 windowType,
    ViChar returnedWindowName[]
)
{
    ViStatus    error   = VI_SUCCESS;
    size_t      length;
    ViChar*     pbuffer = NULL;
    ViChar*     p2buf = NULL;
    ViReal64     discrete;
    ViChar       *p2direction = NULL;
    ViChar       *p2type = NULL;   
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    do
    {
        discrete = rsemi_rngLayoutDirection.rangeValues[i].discreteOrMinValue;
        p2direction = rsemi_rngLayoutDirection.rangeValues[i].cmdString;

        if (p2direction == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Direction");

        if (fabs(discrete - direction) <= 0.01) break;

        i++;

    } while (i != 0);
    i=0;
    do
    {
        discrete = rsemi_rngReceiverLayoutQueryWindowType.rangeValues[i].discreteOrMinValue;
        p2type = rsemi_rngReceiverLayoutQueryWindowType.rangeValues[i].cmdString;

        if (p2type == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Window Type");

        if (fabs(discrete - windowType) <= 0.01) break;

        i++;

    } while (i != 0);        
    
    viCheckErr (viPrintf (instrSession, "LAY:ADD? '%s',%s,%s", windowName, p2direction, p2type));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &pbuffer, NULL));
    
    if (returnedWindowName != NULL)
    {
        size_t len = strstr (pbuffer+1,"'") - pbuffer - 1;
        length = (size_t) stringSize - 1 <= len ? (size_t) stringSize-1 : len;
        p2buf = strtok (pbuffer, "'");
        strncpy (returnedWindowName, p2buf, length);
    }
 
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    if (pbuffer != NULL) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function removes a window from the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
ViStatus _VI_FUNC rsemi_RemoveReceiverWindow (
    ViSession instrSession,
    ViString windowName
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    
    viCheckParm ( rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_RECEIVER_LAYOUT_WINDOW_REMOVE, windowName),
                2, "Window Name");                

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsemi_ReplaceReceiverWindow (
    ViSession instrSession,
    ViString windowName,
    ViInt32 windowType
)
{
    ViStatus    error   = VI_SUCCESS;
    ViReal64     discrete;
    ViChar       *p2type = NULL;   
    ViInt32     i=0;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    do
    {
        discrete = rsemi_rngReceiverLayoutQueryWindowType.rangeValues[i].discreteOrMinValue;
        p2type = rsemi_rngReceiverLayoutQueryWindowType.rangeValues[i].cmdString;

        if (p2type == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Window Type");

        if (fabs(discrete - windowType) <= 0.01) break;

        i++;

    } while (i != 0);
        
    viCheckErr (viPrintf (instrSession, "LAY:REPL '%s',%s", windowName, p2type));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function queries the name and index of all active windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR bufferSize/This control specifies the size of the buffer.
/// HIPAR results/Returns the name and index of all active windows.
ViStatus _VI_FUNC rsemi_QueryReceiverWindowsList (
    ViSession instrSession,
    ViInt32 bufferSize,
    ViChar results[]
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL)); 
    
    viCheckParm (rsemi_GetAttributeViString (instrSession, "", RSEMI_ATTR_RECEIVER_LAYOUT_WINDOW_LIST, 
                                                                bufferSize, results), 3, "Results");                


Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function queries the index of a particular display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR index/Index number of the window.
ViStatus _VI_FUNC rsemi_QueryReceiverWindowIndex (
    ViSession instrSession,
    ViString windowName,
    ViInt32 *index
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      *buffer = NULL;
    ViChar      *pbuff = NULL; 
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (viPrintf (instrSession, "LAY:IDEN? '%s'\n", windowName));
    
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &buffer, NULL));
    
    pbuff = strtok (buffer, ",\n");
    *index = atol(pbuff);
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function queries the name of a particular display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR index/Index number of the window.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to. 
ViStatus _VI_FUNC rsemi_QueryReceiverWindowName (
    ViSession instrSession,
    ViInt32 stringSize,
    ViInt32 index,
    ViChar windowName[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_RECEIVER_LAYOUT_QUERY_WINDOW_NAME,
                                                 "Window",
                                                 index - 1,
                                                 10,
                                                 rep_cap));
    viCheckParm (rsemi_GetAttributeViString (instrSession, rep_cap, RSEMI_ATTR_RECEIVER_LAYOUT_QUERY_WINDOW_NAME, 
                                                                stringSize, windowName), 3, "Window Name");
    
                                             
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function changes the position of a splitter
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init.
/// HIPAR firstWindowName/String containing the number or name of the first window the splitter controls.
/// HIPAR secondWindowName/String containing the number or name of the second window the splitter controls. 
/// HIPAR position/New vertical or horizontal position of the splitter in percent of the screen area.
ViStatus _VI_FUNC rsemi_ConfigureReceiverSplitterPosition (
    ViSession instrSession,
    ViString firstWindowName,
    ViString secondWindowName,
    ViInt32 position
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      cmd [100] = "";

    checkErr( Rs_LockSession (instrSession, NULL));
    
        
    sprintf (cmd, "LAY:SPL '%s','%s',%ld", firstWindowName, secondWindowName, position);
    
    viCheckErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  Turns analog demodulation at the receiver frequency on and off and 
/// HIFN  selects the type of analog demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR demodulation/Selects the type of analog demodulation.
ViStatus _VI_FUNC rsemi_ConfigureAnalogDemodulationState (ViSession instrSession,
                                                          ViInt32 demodulation)
{
    ViStatus    error   = VI_SUCCESS;
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_ANALOG_DEMODULATION_STATE,
                                            demodulation), 2, "Demodulation");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  Enables selective demodulation at the marker position, and defines the 
/// HIFN  threshold for the selective demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR squelchState/Enables the input of a level threshold below which the audible AF is 
/// HIPAR squelchState/cut off.
/// HIPAR threshold/Sets the threshold for selective demodulation.
ViStatus _VI_FUNC rsemi_ConfigureAnalogDemodulationSquelching (ViSession instrSession,
                                                               ViBoolean squelchState,
                                                               ViReal64 threshold)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", 
        RSEMI_ATTR_ANALOG_DEMODULATION_SQUELCH_ENABLED, squelchState), 2, "Squelch State");
    
    viCheckParm( rsemi_SetAttributeViReal64 (instrSession, "", 
        RSEMI_ATTR_ANALOG_DEMODULATION_SQUELCH_LEVEL, threshold), 3, "Threshold");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This command turns the spectrogram on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR spectrogramState/Turns the spectrogram on and off.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramState (
    ViSession instrSession,
    ViInt32 window,
    ViBoolean spectrogramState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, buffer, 
                                               RSEMI_ATTR_SGRAM_STATE,
                                               spectrogramState), 3, "Spectrogram State");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Resets the Spectrogram result display and clears the history buffer.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_CLEAR
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:SGRam:CLEar[:IMMediate]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramClear(
    ViSession    instrSession,
    ViInt32    window
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [15] = "";
                               
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_CLEAR,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap));

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_CLEAR, NULL));
    

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Selects a specific frame for further analysis.
/// HIFN     
/// HIFN     Note(s): 
/// HIFN     Available only if no measurement is running or after a single sweep 
/// HIFN  has ended.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_SELECT_FRAME
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:SGRam:FRAMe:SELect
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR frame/Selects a specific frame for further analysis.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramSelectFrame(
    ViSession    instrSession,
    ViInt32    window,
    ViReal64    frame
)
{
    ViStatus    error = VI_SUCCESS;
      ViChar      rep_cap [15] = "";
                               
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_SELECT_FRAME,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_SGRAM_SELECT_FRAME,
                                             frame), 3, "Frame");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Sets the number of frames to be stored in the R&S ESR's memory.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_FRAMES_IN_MEMORY
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:SGRam:FRAMe:SELect
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR frames/Sets the number of frames to be stored in the R&S ESR's memory.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramFramesInMemory(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    frames
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [15] = "";
                               
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_FRAMES_IN_MEMORY,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_FRAMES_IN_MEMORY,
                                            frames), 3, "Frames");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Selects the spectrogram layout.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SGRAM_LAYOUT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:SGRam:LAYout
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR color/Selects the spectrogram layout.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramLayout (
    ViSession instrSession,
    ViInt32 window,
    ViInt32 layout
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [15] = "";
                               
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_LAYOUT,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_LAYOUT,
                                            layout), 3, "Layout");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Selects the trace the spectrogram is based on.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_TRACE
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate:SGRam:TRACe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects the trace the spectrogram is based on.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramTraceNew(
    ViSession    instrSession,
    ViInt32    trace
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                            RSEMI_ATTR_SPECTROGRAM_TRACE,
                                            trace), 2, "Trace");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Activates and deactivates the time stamp.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_TIME_STAMP
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:SGRam:TSTamp[:STATe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR timeStamp/Activates and deactivates the time stamp.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramTimeStamp(
    ViSession    instrSession,
    ViInt32    window,
    ViBoolean    timeStamp
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [15] = "";
                               
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_TIME_STAMP,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, rep_cap,
                                              RSEMI_ATTR_SGRAM_TIME_STAMP,
                                              timeStamp), 3, "Time Stamp");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Activates and deactivates the time stamp.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_COLOR_DEFAULT
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DISPlay:WINDow:SGRam:COLor:DEFault
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramColorDefault(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "",
                                            RSEMI_ATTR_SPECTROGRAM_COLOR_DEFAULT, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Selects the color scheme of the spectrogram result display.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_COLOR
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:SGRam:COLor
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR color/Selects the color scheme of the spectrogram result display.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramColor(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    color
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [15] = "";
                               
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_COLOR,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_COLOR,
                                            color), 3, "Color");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Sets the color scheme for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_COLOR_STYLE
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DISPlay:WINDow:SGRam:COLor[:STYLe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR style/Sets the color scheme for the spectrogram.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramColorStyle(
    ViSession    instrSession,
    ViInt32    style
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                            RSEMI_ATTR_SPECTROGRAM_COLOR_STYLE,
                                            style), 2, "Style");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Defines the shape and focus of the color curve for the spectrogram 
/// HIFN  result display.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_COLOR_SHAPE
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DISPlay:WINDow:SGRam:COLor:SHAPe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR shape/Defines the shape and focus of the color curve for the spectrogram 
/// HIPAR shape/result display.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramColorShape(
    ViSession    instrSession,
    ViReal64    shape
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                             RSEMI_ATTR_SPECTROGRAM_COLOR_SHAPE,
                                             shape), 2, "Shape");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Sets the lower percentage boundary of the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_COLOR_LOWER
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DISPlay:WINDow:SGRam:COLor:LOWer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR lower/Sets the lower percentage boundary of the spectrogram.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramColorLower(
    ViSession    instrSession,
    ViInt32    lower
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                            RSEMI_ATTR_SPECTROGRAM_COLOR_LOWER,
                                            lower), 2, "Lower");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Sets the upper percentage boundary of the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_COLOR_UPPER
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DISPlay:WINDow:SGRam:COLor:UPPer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR upper/Sets the upper percentage boundary of the spectrogram.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramColorUpper(
    ViSession    instrSession,
    ViInt32    upper
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "",
                                            RSEMI_ATTR_SPECTROGRAM_COLOR_UPPER,
                                            upper), 2, "Upper");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the marker on a particular frame.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_FRAME
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:FRAMe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
/// HIPAR frame/Positions the marker on a particular frame.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerFrame(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64    frame
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_FRAME,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_FRAME,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_SGRAM_MARKER_FRAME,
                                             frame), 4, "Frame");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the marker on a particular frame.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_FRAME_TIME
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:FRAMe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
/// HIPAR frameTime/Positions the marker on a particular frame.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerFrameTime(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViReal64    frameTime
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_FRAME_TIME,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_FRAME_TIME,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_SGRAM_MARKER_FRAME_TIME,
                                             frameTime), 4, "Frame Time");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Defines the marker search area.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_SEARCH_AREA
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:SARea
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
/// HIPAR searchArea/Defines the marker search area.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerSearchAreaSGR(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker,
    ViInt32    searchArea
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_SEARCH_AREA,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_SEARCH_AREA,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_SEARCH_AREA,
                                            searchArea), 4, "Search Area");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the current peak level of the 
/// HIFN  spectrogram. The possible position depends on the marker search area.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_MAXIMUM_LEVEL
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:XY:MAXimum[:PEAK]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerMaximumLevel(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_MAXIMUM_LEVEL,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_MAXIMUM_LEVEL,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_MAXIMUM_LEVEL, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the current minimum level of the 
/// HIFN  spectrogram. The possible position depends on the marker search area.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_MINIMUM_LEVEL
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:XY:MINimum[:PEAK]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerMinimumLevel(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_MINIMUM_LEVEL,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_MINIMUM_LEVEL,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_MINIMUM_LEVEL, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the next peak level of the vertical 
/// HIFN  axis. The search includes only frames above the current marker 
/// HIFN  position. It does not change the horizontal position of the marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_Y_NEXT_PEAK_ABOVE
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:Y:MAXimum:ABOVe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerYNextPeakAbove(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_PEAK_ABOVE,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_PEAK_ABOVE,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_PEAK_ABOVE, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the next peak level of the vertical 
/// HIFN  axis. The search includes only frames below the current marker 
/// HIFN  position. It does not change the horizontal position of the marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_Y_NEXT_PEAK_BELOW
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:Y:MAXimum:BELow
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerYNextPeakBelow(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_PEAK_BELOW,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_PEAK_BELOW,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_PEAK_BELOW, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the next peak level of the vertical 
/// HIFN  axis. The search includes frames above and below the current marker 
/// HIFN  position. It does not change the horizontal position of the marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_Y_NEXT_PEAK
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:Y:MAXimum:NEXT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerYNextPeak(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_PEAK,
                                                 "Window",                   
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_PEAK,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_PEAK, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the current peak level of the 
/// HIFN  vertical axis. The search is performed over all frames. It does not 
/// HIFN  change the horizontal position of the marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_Y_PEAK
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:Y:MAXimum[:PEAK]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerYPeak(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_PEAK,
                                                 "Window",                   
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_PEAK,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_Y_PEAK, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the next minimum level of the 
/// HIFN  vertical axis. The search includes only frames above the current marker 
/// HIFN  position. It does not change the horizontal position of the marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_Y_NEXT_MINIMUM_ABOVE
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:Y:MINimum:ABOVe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerYNextMinimumAbove(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,               
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_MINIMUM_ABOVE,
                                                 "Window",                   
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_MINIMUM_ABOVE,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_MINIMUM_ABOVE, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the next minimum level of the 
/// HIFN  vertical axis. The search includes only frames below the current marker 
/// HIFN  position. It does not change the horizontal position of the marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_Y_NEXT_MINIMUM_BELOW
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:Y:MINimum:BELow
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerYNextMinimumBelow(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,               
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_MINIMUM_BELOW,
                                                 "Window",                   
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_MINIMUM_BELOW,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);
    
    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_MINIMUM_BELOW, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the next minimum level of the 
/// HIFN  vertical axis. The search includes frames above and below the current 
/// HIFN  marker position. It does not change the horizontal position of the 
/// HIFN  marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_Y_NEXT_MINIMUM
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:Y:MINimum:NEXT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerYNextMinimum(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,               
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_MINIMUM,
                                                 "Window",                   
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_MINIMUM,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_Y_NEXT_MINIMUM, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified marker on the current minimum level of the 
/// HIFN  vertical axis. The search is performed over all frames. It does not 
/// HIFN  change the horizontal position of the marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_MARKER_Y_MINIMUM
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:MARKer<m>:SGRam:Y:MINimum[:PEAK]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR marker/Selects the marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramMarkerYMinimum(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    marker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,               
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_MINIMUM,
                                                 "Window",                   
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_MARKER_Y_MINIMUM,
                                                 "Marker",
                                                 marker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_MARKER_Y_MINIMUM, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the delta marker on a particular frame.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_FRAME
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:FRAMe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
/// HIPAR frame/Positions the delta marker on a particular frame.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerFrame(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViReal64    frame
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_FRAME,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_FRAME,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_SGRAM_DELTA_MARKER_FRAME,
                                             frame), 4, "Frame");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the delta marker on a particular frame.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_FRAME_TIME
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:FRAMe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
/// HIPAR frameTime/Positions the delta marker on a particular frame.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerFrameTime(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViReal64    frameTime
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_FRAME_TIME,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_FRAME_TIME,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, rep_cap,
                                             RSEMI_ATTR_SGRAM_DELTA_MARKER_FRAME_TIME,
                                             frameTime), 4, "Frame Time");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Defines the delta marker search area.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_SEARCH_AREA
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:FRAMe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
/// HIPAR searchArea/Defines the delta marker search area.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerSearchAreaSGR(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker,
    ViInt32    searchArea
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_SEARCH_AREA,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_SEARCH_AREA,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_SEARCH_AREA,
                                            searchArea), 4, "Search Area");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the current peak level of the 
/// HIFN  spectrogram. The possible position depends on the delta marker search 
/// HIFN  area.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_MAXIMUM_LEVEL
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:XY:MAXimum[:PEAK]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerMaximumLevel(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_MAXIMUM_LEVEL,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_MAXIMUM_LEVEL,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    
    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_MAXIMUM_LEVEL, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the current minimum level of 
/// HIFN  the spectrogram. The possible position depends on the delta marker 
/// HIFN  search area.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_MINIMUM_LEVEL
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:XY:MINimum[:PEAK]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerMinimumLevel(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_MINIMUM_LEVEL,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_MINIMUM_LEVEL,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_MINIMUM_LEVEL, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the next peak level of the 
/// HIFN  vertical axis. The search includes only frames above the current delta 
/// HIFN  marker position. It does not change the horizontal position of the 
/// HIFN  delta marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_Y_NEXT_PEAK_ABOVE
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:Y:MAXimum:ABOVe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerYNextPeakAbove(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_PEAK_ABOVE,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_PEAK_ABOVE,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_PEAK_ABOVE, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the next peak level of the 
/// HIFN  vertical axis. The search includes only frames below the current delta 
/// HIFN  marker position. It does not change the horizontal position of the 
/// HIFN  delta marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_Y_NEXT_PEAK_BELOW
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:Y:MAXimum:BELow
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerYNextPeakBelow(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_PEAK_BELOW,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_PEAK_BELOW,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_PEAK_BELOW, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the next peak level of the 
/// HIFN  vertical axis. The search includes frames above and below the current 
/// HIFN  delta marker position. It does not change the horizontal position of 
/// HIFN  the delta marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_Y_NEXT_PEAK
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:Y:MAXimum:NEXT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerYNextPeak(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_PEAK,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_PEAK,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_PEAK, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the current peak level of the 
/// HIFN  vertical axis. The search is performed over all frames. It does not 
/// HIFN  change the horizontal position of the delta marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_Y_PEAK
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:Y:MAXimum[:PEAK]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerYPeak(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_PEAK,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_PEAK,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_PEAK, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the next minimum level of the 
/// HIFN  vertical axis. The search includes only frames above the current marker 
/// HIFN  position. It does not change the horizontal position of the marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_Y_NEXT_MINIMUM_ABOVE
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:Y:MINimum:ABOVe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerYNextMinimumAbove(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_MINIMUM_ABOVE,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_MINIMUM_ABOVE,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_MINIMUM_ABOVE, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the next minimum level of the 
/// HIFN  vertical axis. The search includes only frames below the current delta 
/// HIFN  marker position. It does not change the horizontal position of the 
/// HIFN  delta marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_Y_NEXT_MINIMUM_BELOW
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:Y:MINimum:BELow
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerYNextMinimumBelow(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_MINIMUM_BELOW,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_MINIMUM_BELOW,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_MINIMUM_BELOW, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the next minimum level of the 
/// HIFN  vertical axis. The search includes frames above and below the current 
/// HIFN  delta marker position. It does not change the horizontal position of 
/// HIFN  the delta marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_Y_NEXT_MINIMUM
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:Y:MINimum:NEXT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerYNextMinimum(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_MINIMUM,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_MINIMUM,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_NEXT_MINIMUM, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Positions the specified delta marker on the current minimum level of 
/// HIFN  the vertical axis. The search is performed over all frames. It does not 
/// HIFN  change the horizontal position of the delta marker.
/// HIFN     
/// HIFN     Note: This function is available for the spectrogram.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSEMI_ATTR_SPECTROGRAM_DELTA_MARKER_Y_MINIMUM
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DELTamarker<m>:SGRam:Y:MINimum[:PEAK]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR deltaMarker/Selects the delta marker.
ViStatus _VI_FUNC rsemi_ConfigureSpectrogramDeltaMarkerYMinimum(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    deltaMarker
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
    ViChar      rep_cap_win[10] = "";
    ViChar      rep_cap_mark[10] = "";
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
       viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_MINIMUM,
                                                 "Window",
                                                 window,
                                                 10,
                                                 rep_cap_win), 2, "Window");
    viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_MINIMUM,
                                                 "Marker",
                                                 deltaMarker - 1,
                                                 10,
                                                 rep_cap_mark), 3, "Delta Marker");
    sprintf (rep_cap, "%s,%s", rep_cap_win, rep_cap_mark);

    viCheckErr (rsemi_SetAttributeViString (instrSession, rep_cap,
                                            RSEMI_ATTR_SGRAM_DELTA_MARKER_Y_MINIMUM, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the time offset between the trigger event
/// HIFN  and the start of the measurement (data capturing).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerOffset/Defines the time offset between the trigger event and the start
/// HIPAR triggerOffset/of the measurement (data capturing).
ViStatus _VI_FUNC rsemi_ConfigureReceiverTriggerOffset (
    ViSession instrSession,
    ViReal64 triggerOffset
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                             RSEMI_ATTR_RECEIVER_TRIGGER_OFFSET,
                                             triggerOffset), 2, "Trigger Offset");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the IF power trigger parameters. This is
/// HIFN  applicable when the trigger source is set to IF Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerLevel/Sets the level of the IF power trigger source.
/// HIPAR holdoff/This command defines the holding time before the next trigger event.
/// HIPAR hysteresis/This command defines the trigger hysteresis, which is only
/// HIPAR hysteresis/available for "IF Power" trigger sources.
ViStatus _VI_FUNC rsemi_ConfigureReceiverIFPowerTrigger (
    ViSession instrSession,
    ViReal64 triggerLevel,
    ViReal64 holdoff,
    ViReal64 hysteresis
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                             RSEMI_ATTR_RECEIVER_TRIGGER_IFP_LEVEL,
                                             triggerLevel), 2, "Trigger Level");
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                             RSEMI_ATTR_RECEIVER_TRIGGER_IFP_HOLDOFF,
                                             holdoff), 3, "Holdoff");
    
    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                             RSEMI_ATTR_RECEIVER_TRIGGER_IFP_HYSTERESIS,
                                             hysteresis), 4, "Hysteresis");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function specifies sets the level of the RF power trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerLevel/Sets the level of the RF power trigger source.
ViStatus _VI_FUNC rsemi_ConfigureReceiverRFPowerTrigger (
    ViSession instrSession,
    ViReal64 triggerLevel
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                             RSEMI_ATTR_RECEIVER_TRIGGER_RFP_LEVEL,
                                             triggerLevel), 2, "Trigger Level");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This attribute sets the time interval for the time trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeInterval/This control sets the time interval for the time trigger source.
ViStatus _VI_FUNC rsemi_ConfigureReceiverTimeTrigger (
    ViSession instrSession,
    ViReal64 timeInterval
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "",
                                             RSEMI_ATTR_RECEIVER_TRIGGER_TIME_INTERVAL,
                                             timeInterval), 2, "Time Interval");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the trace data for the corresponding trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR resultsType/Selects the type of requested results.
/// HIPAR arraySize/Sets the number if the returned results.
/// HIPAR traceData/Returns the trace data that consists of a list of power levels that 
/// HIPAR traceData/have been measured.
/// HIPAR no_ofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsemi_ReadReceiverTraceData (ViSession instrSession,
                                                 ViInt32 timeout,
                                                 ViInt32 resultsType,
                                                 ViInt32 arraySize,
                                                 ViReal64 traceData[],
                                                 ViInt32 *no_ofValues)
{
 
    ViStatus    error = VI_SUCCESS;
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr(rsemi_InitiateScan (instrSession, 2, timeout));
    
    checkErr(rsemi_FetchReceiverTraceData (instrSession, resultsType, arraySize, traceData, no_ofValues));           
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Returns the results of a final measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR finalMeasurement/This control sets the number of the final measurement.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR frequency/Returns an array of frequency list in Hz.
/// HIPAR level/Returns an array of level list in dBm.
/// HIPAR deltaLevel/Returns an array of delta to limit line values in dB. A delta limit of 
/// HIPAR deltaLevel/+200dB indicates no limit check is active.
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to limit 
/// HIPAR returnedValues/line) read (available to return) from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter No of Values.
ViStatus _VI_FUNC rsemi_ReadReceiverFinalMeasurement (ViSession instrSession,
                                                      ViInt32 timeout,
                                                      ViInt32 finalMeasurement,
                                                      ViInt32 noOfValues,
                                                      ViReal64 frequency[],
                                                      ViReal64 level[],
                                                      ViReal64 deltaLevel[],
                                                      ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr(rsemi_InitiateFinalMeasurementTest (instrSession, 2, timeout));
    
    checkErr(rsemi_FetchReceiverFinalMeasurement (instrSession, finalMeasurement, noOfValues, frequency, level, deltaLevel, returnedValues));           
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Returns the results of the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR peakList/This control sets the number of the peak list.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR frequency/Returns an array of frequency list in Hz.
/// HIPAR level/Returns an array of level list in dBm.
/// HIPAR deltaLevel/Returns an array of delta to limit line values in dB. A delta limit of 
/// HIPAR deltaLevel/+200dB indicates no limit check is active.
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to limit 
/// HIPAR returnedValues/line) read (available to return) from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter No of Values.
ViStatus _VI_FUNC rsemi_ReadReceiverPeakList (ViSession instrSession,
                                              ViInt32 timeout, ViInt32 peakList,
                                              ViInt32 noOfValues,
                                              ViReal64 frequency[],
                                              ViReal64 level[],
                                              ViReal64 deltaLevel[],
                                              ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr(rsemi_InitiateEMITest (instrSession, 2, timeout));
    
    checkErr(rsemi_FetchReceiverPeakList (instrSession, peakList, noOfValues, frequency, level, deltaLevel, returnedValues));           
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  Queries the time stamp of the frames.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Receiver
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:SGRam:TSTamp:DATA? <Mode>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/Selects the measurement window.
/// HIPAR mode/Selects the query mode.
/// HIPAR timeStamp/Queries the time stamp of the frames.
ViStatus _VI_FUNC rsemi_ReadSpectrogramTimeStampData(
    ViSession    instrSession,
    ViInt32    window,
    ViInt32    mode,
    ViInt32 arraySize,
    ViChar    timeStamp[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViUInt32    return_count;
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 4), 2, "Window");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, mode, 0, 1), 3, "Mode");
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (Rs_ClearBeforeRead(instrSession));

    sprintf (buffer, "CALC%ld:SGR:TST:DATA? %s\n", window, timeStampModeArr[mode]);

    viCheckErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32)strlen (buffer), VI_NULL));
    
    viCheckErr (viRead (instrSession, (ViBuf) timeStamp, arraySize, &return_count));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Defines the peak excursion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR peakExcursion/Defines the peak excursion value.
ViStatus _VI_FUNC rsemi_ConfigurePeakSearchExcursion (ViSession instrSession,
                                                      ViInt32 window,
                                                      ViReal64 peakExcursion)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
                                                                                             
    sprintf (buffer, "Win%ld", window);

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, buffer, RSEMI_ATTR_PEAK_SEARCH_EXCURSION,
                                             peakExcursion), 3, "Peak Excursion");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;  
}

/// HIFN  Defines a margin for the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR margin/Defines a margin for the peak search.
ViStatus _VI_FUNC rsemi_ConfigurePeakSearchMargin (ViSession instrSession,
                                                   ViReal64 margin)
{
 
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", RSEMI_ATTR_PEAK_SEARCH_MARGIN,
                                             margin), 2, "Margin");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  It manually adds a particular frequency to the peak list, regardless 
/// HIFN  if the level threshold conditions have been fulfilled for that 
/// HIFN  frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/Sets the added frequency value to the peak list, regardless if the 
/// HIPAR frequency/level threshold conditions have been fulfilled for that frequency.
ViStatus _VI_FUNC rsemi_AddPeakSearchFrequency (ViSession instrSession,
                                                              ViReal64 frequency)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", RSEMI_ATTR_PEAK_SEARCH_FREQUENCY_ADD,
                                             frequency), 2, "Frequency");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Selects the way how to creates a peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR method/Selects the way the R&S ESR creates a peak list.
/// HIPAR methodValue/In case where peaks search mode is selected, this control sets the 
/// HIPAR methodValue/number of peaks during the peak search.
ViStatus _VI_FUNC rsemi_ConfigurePeakSearchListCreationMethod (ViSession instrSession,
                                                               ViInt32 method,
                                                               ViInt32 methodValue)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_PEAK_SEARCH_LIST_METHOD,
                                             method), 2, "Method");
    
    switch (method)
    {
      case RSEMI_VAL_METHOD_SUBRANGE:
            viCheckParm(rsemi_SetAttributeViInt32 (instrSession, "", 
                                                   RSEMI_ATTR_PEAK_SEARCH_LIST_VALUE, methodValue),
                                                   3, "Subranges");   
      break;
      
      case RSEMI_VAL_METHOD_PEAK:
            viCheckParm(rsemi_SetAttributeViInt32 (instrSession, "", 
                                                   RSEMI_ATTR_PEAK_SEARCH_LIST_VALUE, methodValue),
                                                   3, "Peaks");   
      break;    
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Defines the number of peaks to be found in each subrange.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR peakNumber/Sets the number of peaks.
ViStatus _VI_FUNC rsemi_ConfigurePeakSearchNumber (ViSession instrSession,
                                                   ViInt32 window,
                                                   ViInt32 peakNumber)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    
    sprintf (buffer, "Win%ld", window);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_PEAK_SEARCH_NUMBER,
                                             peakNumber), 3, "Peak Number");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Deletes the contents of the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_ClearPeakSearchList (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "",
                                                  RSEMI_ATTR_CLEAR_PEAK_SEARCH_LIST,
                                                  VI_NULL));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Turns the peak labels in the diagram on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Sets the trace.
/// HIPAR symbol/Selects the peak label.
ViStatus _VI_FUNC rsemi_ConfigurePeakListDisplay (ViSession instrSession,
                                                  ViInt32 trace, ViInt32 symbol)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 2, "Trace");
    
    sprintf (buffer, "TR%ld", trace);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_PEAK_LIST_DISPLAY,
                                             symbol), 3, "Symbol");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Exports the contents of a peak list to a file in ASCII format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Exports the contents of a peak list to a file in ASCII format.
ViStatus _VI_FUNC rsemi_ConfigurePeakListMeasurementFile (ViSession instrSession,
                                                          ViString fileName)
{
   ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rsemi_SetAttributeViString (instrSession, "",
                                                  RSEMI_ATTR_PEAK_LIST_FILE_NAME,
                                                  fileName), 2, "File Name");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the number of peaks found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR size/Number of peaks found in the peak search..
ViStatus _VI_FUNC rsemi_QueryPeakListSize (ViSession instrSession, ViInt32 *size)
{
    ViStatus    error = VI_SUCCESS;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsemi_GetAttributeViInt32(instrSession, "",
        RSEMI_ATTR_PEAK_LIST_NUMBER_OF_PEAKS, size), 2, "Size");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the information for all peaks found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Defines size of arrays in arguments 'Trace Number', 'Frequency', 'Level' and 'Delta Limit'.
/// HIPAR detectorBufferSize/Defines the size of buffer in argument 'Detector'.
/// HIPAR commentBufferSize/Defines the size of buffer in argument 'Comment'.
/// HIPAR traceNumber/Returns array of trace numbers, one for each peak in peak list.
/// HIPAR detector/Comma separated list of detectors, one for each peak in the peak list.
/// HIPAR frequency/Returns array of frequencies, one for each peak in peak list.
/// HIPAR level/Returns array of levels, one for each peak in peak list.
/// HIPAR deltaLimit/Returns array of delta limits, one for each peak in peak list. If no limit line was used, 0 is returned.
/// HIPAR comment/Comma separated list of comments, one for each peak in the peak list.
ViStatus _VI_FUNC rsemi_QueryPeakListData (ViSession instrSession,
                                           ViInt32 arraySize,
                                           ViInt32 detectorBufferSize,
                                           ViInt32 commentBufferSize,
                                           ViInt32 traceNumber[],
                                           ViChar detector[],
                                           ViReal64 frequency[],
                                           ViReal64 level[],
                                           ViReal64 deltaLimit[],
                                           ViChar comment[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[] = "CALC:PSE:PLIS:ALL?\n";
    ViChar*  pbuf = NULL;
    ViChar*  p2buf = NULL;
    ViChar*  input_data_needle = NULL;
    ViInt32  peak_count = 0;
    ViInt32  count_to_copy = 0;
    ViInt32  i = 0;
    ViInt32  skip_further_detectors = 0;
    ViInt32  skip_further_comments = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsemi_GetAttributeViInt32 (instrSession, "", RSEMI_ATTR_PEAK_LIST_NUMBER_OF_PEAKS, &peak_count));
    count_to_copy = (peak_count > arraySize) ? arraySize : peak_count;
    
    checkErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &pbuf, NULL));
    checkErr( rsemi_CheckStatus (instrSession));
    
    /* Zero both strings to be able to count length of string already stored in them */
    memset (detector, 0, detectorBufferSize);
    memset (comment, 0, commentBufferSize);
    
    /* Response comes in format:
    <TraceNumber>,<Detector>,<Frequency>,<Level>,<DeltaLimit>,<Comment>
    'TRACE1,Quasi-Peak,4.5e+07,53.99,0,yahoo,
    TRACE2,Average,4.5e+07,53.07,0,,
    TRACE3,RMS,4.5e+07,50.89,0,,
    TRACE4,Quasi-Peak,4.5e+07,53.99,0,,' */
    
    /* Get rid of first ' */
    p2buf = strtok (pbuf, "'");
    
    for (i = 0; i < count_to_copy; i++)
    {
        /* TraceNumber */
        if (i == 0)
            input_data_needle = strtok (p2buf, ",");
        else
            input_data_needle = strtok (NULL, ",");
            
        sscanf (input_data_needle, "TRACE%ld", &traceNumber[i]);
        
        /* Detector */
        input_data_needle = strtok (NULL, ",");
        /* One zero at the end is needed to terminate the character, one character is comma in between values */
		if ((skip_further_detectors == 0) && ((strlen(input_data_needle) + strlen(detector)) < ((size_t)detectorBufferSize - 2)))
        {
            if (i > 0) strcat (detector, ",");
            strcat (detector, input_data_needle);
        }
        else
            skip_further_detectors = 1;
        
        /* Frequency */
        input_data_needle = strtok (NULL, ",");
        sscanf (input_data_needle, "%lf", &frequency[i]);
        
        /* Level */
        input_data_needle = strtok (NULL, ",");
        sscanf (input_data_needle, "%lf", &level[i]);
        
        /* DeltaLimit */
        input_data_needle = strtok (NULL, ",");
        sscanf (input_data_needle, "%lf", &deltaLimit[i]);
        
        /* Comment */
        /* Empty comment, strtok skips ",," to next token */
		if (*(input_data_needle+ 1+strlen (input_data_needle)) != '\0')
		{
	        if ((*(input_data_needle+ 1+strlen (input_data_needle))) != ',')
	        {
	            input_data_needle = strtok (NULL, ",");
				if ((skip_further_comments == 0) && ((strlen(input_data_needle) + strlen(comment)) < ((size_t)commentBufferSize - 2)))
	            {
	                if (i > 0) strcat (comment, ",");
	                strcat (comment, input_data_needle);
	            }
	            else
	                skip_further_comments = 1;
	        }
	        else
	        {
	            if (i > 0) strcat (comment, ",");
	        }
		}
		else
		{
			if (i > 0) strcat (comment, ",");
		}
    }
    
Error:
    if (pbuf) free (pbuf);
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Assigns a comment to a peak found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR comment/Comment string
ViStatus _VI_FUNC rsemi_ConfigurePeakListPeakComment (ViSession instrSession,
                                                      ViInt32 peak,
                                                      ViString comment)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
	viCheckParm(rsemi_SetAttributeViString(instrSession, rep_cap,
        RSEMI_ATTR_PEAK_LIST_PEAK_COMMENT, comment), 3, "Comment");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the frequency of a peak found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR frequency/Queries the frequency of a peak found in the peak search.
ViStatus _VI_FUNC rsemi_QueryPeakListPeakFrequency (ViSession instrSession,
                                                    ViInt32 peak,
                                                    ViReal64 *frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViReal64(instrSession, rep_cap,
        RSEMI_ATTR_PEAK_LIST_PEAK_FREQUENCY, frequency), 3, "Frequency");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the level of a peak found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR level/Queries the level of a peak found in the peak search.
ViStatus _VI_FUNC rsemi_QueryPeakListPeakLevel (ViSession instrSession,
                                                ViInt32 peak, ViReal64 *level)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViReal64(instrSession, rep_cap,
        RSEMI_ATTR_PEAK_LIST_PEAK_LEVEL, level), 3, "Level");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the detector with which a peak was found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR detectorBufferSize/Defines the size of buffer in argument 'Detector'.
/// HIPAR detector/Queries the detector with which a peak was found in the peak search.
ViStatus _VI_FUNC rsemi_QueryPeakListPeakDetector (ViSession instrSession,
                                                   ViInt32 peak,
                                                   ViInt32 detectorBufferSize,
                                                   ViChar detector[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViString(instrSession, rep_cap,
        RSEMI_ATTR_PEAK_LIST_PEAK_DETECTOR, detectorBufferSize, detector), 3, "Detector");    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the distance of a peak to the nearest limit line in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR deltaLimit/Queries the distance of a peak to the nearest limit line in the peak search..
ViStatus _VI_FUNC rsemi_QueryPeakListPeakDeltaLimit (ViSession instrSession,
                                                     ViInt32 peak,
                                                     ViReal64 *deltaLimit)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViReal64(instrSession, rep_cap,
        RSEMI_ATTR_PEAK_LIST_PEAK_DELTA, deltaLimit), 3, "Delta Limit");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the trace that a peak found in the peak search is located on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR traceNumber/Queries the trace that a peak found in the peak search is located on.
ViStatus _VI_FUNC rsemi_QueryPeakListPeakTrace (ViSession instrSession,
                                                ViInt32 peak,
                                                ViInt32 *traceNumber)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_GetAttributeViInt32(instrSession, rep_cap,
        RSEMI_ATTR_PEAK_LIST_PEAK_TRACE, traceNumber), 3, "Trace Number");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Queries the information for a peak found in the peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
/// HIPAR detectorBufferSize/Defines the size of buffer in argument 'Detector'.
/// HIPAR commentBufferSize/Defines the size of buffer in argument 'Comment'.
/// HIPAR traceNumber/Trace number
/// HIPAR detector/Detector
/// HIPAR frequency/Frequency
/// HIPAR level/Peak level
/// HIPAR deltaLimit/Delta limit. If no limit line was used, 0 is returned.
/// HIPAR comment/Comment
ViStatus _VI_FUNC rsemi_QueryPeakListPeakData (ViSession instrSession,
                                               ViInt32 peak,
                                               ViInt32 detectorBufferSize,
                                               ViInt32 commentBufferSize,
                                               ViInt32 *traceNumber,
                                               ViChar detector[],
                                               ViReal64 *frequency,
                                               ViReal64 *level,
                                               ViReal64 *deltaLimit,
                                               ViChar comment[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[100] = "";
    ViChar*  pbuf = NULL;
    ViChar*  p2buf = NULL;
    ViChar*  input_data_needle = NULL;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rsemi_invalidViInt32Range (instrSession, peak, 1, 3000), 2, "Peak");

	sprintf (cmd, "CALC:PSE:PLIS%ld?\n", peak);
	checkErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &pbuf, NULL));
    checkErr( rsemi_CheckStatus (instrSession));
    
    /* Zero both strings to be able to count length of string already stored in them */
    memset (detector, 0, detectorBufferSize);
    memset (comment, 0, commentBufferSize);
    
    /* Response comes in format:
    <TraceNumber>,<Detector>,<Frequency>,<Level>,<DeltaLimit>,<Comment>
    'TRACE1,Quasi-Peak,4.5e+07,53.99,0,yahoo' */
    
    /* Get rid of first ' */
    p2buf = strtok (pbuf, "'");
    
    /* TraceNumber */
    input_data_needle = strtok (p2buf, ",");
    sscanf (input_data_needle, "TRACE%ld", traceNumber);
    
    /* Detector */
    input_data_needle = strtok (NULL, ",");
    /* One zero at the end is needed to terminate the character, one character is comma in between values */
	strncpy (detector, input_data_needle, detectorBufferSize - 1);

	/* Frequency */
    input_data_needle = strtok (NULL, ",");
    sscanf (input_data_needle, "%lf", frequency);
    
    /* Level */
    input_data_needle = strtok (NULL, ",");
    sscanf (input_data_needle, "%lf", level);
    
    /* DeltaLimit */
    input_data_needle = strtok (NULL, ",");
    sscanf (input_data_needle, "%lf", deltaLimit);
    
    /* Comment */
    /* Empty comment, strtok skips ",," to next token */
	if (*(input_data_needle+ 1+strlen (input_data_needle)) != '\0')
	{
        if ((*(input_data_needle+ 1+strlen (input_data_needle))) != ',')
	    {
	        input_data_needle = strtok (NULL, ",");
			strncpy (comment, input_data_needle, commentBufferSize);
	    }
	}
    
Error:
    if (pbuf) free (pbuf);
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN Deletes a peak from the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Selects peak.
ViStatus _VI_FUNC rsemi_DeletePeakListPeak (ViSession instrSession,
                                            ViInt32 peak)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsemi_CreatePeakRepCap (instrSession, peak, rep_cap));
    
    viCheckParm(rsemi_SetAttributeViString(instrSession, rep_cap,
        RSEMI_ATTR_PEAK_LIST_PEAK_DELETE, VI_NULL), 3, "Peak List Peak Delete");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Selects the detector for the final measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/This control sets the trace.
/// HIPAR detector/Selects the detector for the scan.
ViStatus _VI_FUNC rsemi_ConfigureFinalMeasurementDetector (ViSession instrSession,
                                                           ViInt32 trace,
                                                           ViInt32 detector)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, trace, 1, 6), 2, "Trace");
    
    sprintf (buffer, "TR%ld", trace);

    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, buffer, RSEMI_ATTR_FINAL_MEASUREMENT_DETECTOR,
                                             detector), 3, "Detector");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Turns automatic final measurements on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR automaticMeasurement/Turns automatic final measurements on and off.
ViStatus _VI_FUNC rsemi_ConfigureFinalMeasurementAutoState (ViSession instrSession,
                                                            ViBoolean automaticMeasurement)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "", RSEMI_ATTR_FINAL_MEASUREMENT_AUTO,
                                             automaticMeasurement), 2, "Automatic Measurement");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  Defines the time that each frequency in the peak list is measured 
/// HIFN  during the final measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR time/Sets the time that each frequency in the peak list is measured during 
/// HIPAR time/the final measurement.
ViStatus _VI_FUNC rsemi_ConfigureFinalMeasurementPeakFrequencyTime (ViSession instrSession,
                                                                    ViReal64 time)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", RSEMI_ATTR_FINAL_MEASUREMENT_TIME,
                                             time), 2, "Time");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  Exports the contents of the final measurement peak list to a file in 
/// HIFN  ASCII format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Exports the contents of the final measurement peak list to a file in 
/// HIPAR fileName/ASCII format.
ViStatus _VI_FUNC rsemi_ConfigureFinalMeasurementPeakListFile (ViSession instrSession,
                                                               ViString fileName)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_FINAL_MEASUREMENT_FILE_NAME,
                                             fileName), 2, "File Name");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Selects the phase of the network used for the final measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Sets the number of phases to be set.
/// HIPAR phases/Selects the phase of the network used for the final measurement.
ViStatus _VI_FUNC rsemi_ConfigureFinalMeasurementVnetworkPhase (ViSession instrSession,
                                                                ViInt32 arraySize, ViInt32 phases[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, arraySize, 1, 4), 2, "Array Size");
    
    pCmd += sprintf (pCmd, "FME:LISN:PHAS ");
   
    for ( i = 0; i < arraySize; i++ )
    {
        pCmd += sprintf(pCmd, "%s,", PhaseArr[phases[i]]);
    }

    *pCmd = '\0';
    *--pCmd = '\n';
               
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
       
    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);  
    
    return error;
}

/// HIFN  This function defines the maximum number of measurement points
/// HIFN  which are transferred in one block after the query command
/// HIFN  TRACE? SCAN. The total amount of bytes which is transferred
/// HIFN  depends on the number of active traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfPoints/This control defines the maximum number of measurement points
/// HIPAR numberOfPoints/which are transferred in one block after the query command
/// HIPAR numberOfPoints/TRACE? SCAN. The total amount of bytes which is transferred
/// HIPAR numberOfPoints/depends on the number of active traces.
ViStatus _VI_FUNC rsemi_ConfigureScanPoints (ViSession instrSession,
                                             ViInt32 numberOfPoints)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", RSEMI_ATTR_SCAN_POINTS,
                 numberOfPoints), 2, "Number Of Points");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function searches the peaks in spectrum emission data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_PeakSearch (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rsemi_SetAttributeViString (instrSession, "Win1", RSEMI_ATTR_PEAK_SEARCH,
                 NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Initiates an automated test sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_InitiateEMITest (ViSession instrSession,
                                         ViInt32 window, ViInt32 timeout)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    if (timeout < 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Timeout"); 
        
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout));    
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_EMI_TEST_INITIATE, NULL));
    
Error:
    if (old_timeout > 0) 
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Initiates a final measurement based on the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_InitiateFinalMeasurementTest (ViSession instrSession,
                                                      ViInt32 window, ViInt32 timeout)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
        
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout));    
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_EMI_FINAL_MEASUREMENT_INITIATE, NULL));
    
Error:
    if (old_timeout > 0) 
        rsemi_SetOPCTimeout (instrSession, old_timeout);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the trace data for the corresponding trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resultsType/Selects the type of requested results.
/// HIPAR arraySize/Sets the number if the returned results.
/// HIPAR traceData/Returns the trace data that consists of a list of power levels that 
/// HIPAR traceData/have been measured.
/// HIPAR no_ofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsemi_FetchReceiverTraceData (ViSession instrSession,
                                                  ViInt32 resultsType,
                                                  ViInt32 arraySize,
                                                  ViReal64 traceData[],
                                                  ViInt32 *no_ofValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, resultsType, RSEMI_VAL_TRACE1, RSEMI_VAL_TRACE6), 2, "Results Type");

    sprintf (buffer, "TRACE%ld", resultsType);
    
    viCheckErr (rsemi_dataReadTrace (instrSession, 0, buffer, arraySize, 
                    traceData, no_ofValues));
      
    checkErr(rsemi_CheckStatus (instrSession));           
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error; 
}

/// HIFN  Returns the results of a final measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR finalMeasurement/This control sets the number of the final measurement.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR frequency/Returns an array of frequency list in Hz.
/// HIPAR level/Returns an array of level list in dBm.
/// HIPAR deltaLevel/Returns an array of delta to limit line values in dB. A delta limit of 
/// HIPAR deltaLevel/+200dB indicates no limit check is active.
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to limit 
/// HIPAR returnedValues/line) read (available to return) from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter No of Values.
ViStatus _VI_FUNC rsemi_FetchReceiverFinalMeasurement (ViSession instrSession,
                                                       ViInt32 finalMeasurement,
                                                       ViInt32 noOfValues,
                                                       ViReal64 frequency[],
                                                       ViReal64 level[],
                                                       ViReal64 deltaLevel[],
                                                       ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     arrayLength;
    ViReal64    *data = VI_NULL;
    ViInt32     retVal;
    ViInt32     i;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    arrayLength = 3 * noOfValues;
    data=malloc (sizeof (ViReal64) * arrayLength);
    
    sprintf (buffer, "FINal%ld", finalMeasurement);

    viCheckErr (rsemi_dataReadTrace (instrSession, 0, buffer, arrayLength, data, &retVal));

    if (returnedValues)
        *returnedValues = retVal / 3; /* Three result arrays */

    retVal = (retVal > arrayLength) ? arrayLength : retVal;

    retVal /= 3;

    for (i = 0; i < retVal; i++)
    {
        frequency[i] = data[i*3];
        level[i] = data[i*3+1];
        deltaLevel[i] = data[i*3+2];
    }


    checkErr (rsemi_CheckStatus (instrSession));

Error:
    if (data) free(data);
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Returns the results of the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peakList/This control sets the number of the peak list.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR frequency/Returns an array of frequency list in Hz.
/// HIPAR level/Returns an array of level list in dBm.
/// HIPAR deltaLevel/Returns an array of delta to limit line values in dB. A delta limit of 
/// HIPAR deltaLevel/+200dB indicates no limit check is active.
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to limit 
/// HIPAR returnedValues/line) read (available to return) from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter No of Values.
ViStatus _VI_FUNC rsemi_FetchReceiverPeakList (ViSession instrSession,
                                               ViInt32 peakList,
                                               ViInt32 noOfValues,
                                               ViReal64 frequency[],
                                               ViReal64 level[],
                                               ViReal64 deltaLevel[],
                                               ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     arrayLength;
    ViReal64    *data = VI_NULL;
    ViInt32     retVal;
    ViInt32     i;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    arrayLength = 3 * noOfValues;
    data = malloc (sizeof (ViReal64) * arrayLength);
    
    sprintf (buffer, "PLISt%ld", peakList);

    viCheckErr (rsemi_dataReadTrace (instrSession, 0, buffer, arrayLength, data, &retVal));

    if (returnedValues)
        *returnedValues = retVal / 3; /* Three result arrays */

    retVal = (retVal > arrayLength) ? arrayLength : retVal;

    retVal /= 3;

    for (i = 0; i < retVal; i++)
    {
        frequency[i] = data[i*3];
        level[i] = data[i*3+1];
        deltaLevel[i] = data[i*3+2];
    }


    checkErr (rsemi_CheckStatus (instrSession));

Error:
    if (data) free(data);
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the values of the maximum peak of all active 
/// HIFN  detectors from successive measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Pass the number of elements in the arrays you specify as the array 
/// HIPAR arraySize/output parameters.
/// HIPAR peakLevel/Returns the levels of the maximum peak of all active detectors from 
/// HIPAR peakLevel/successive measurements.
/// HIPAR peakFrequency/Returns the frequencies of the maximum peak of all active detectors 
/// HIPAR peakFrequency/from successive measurements.
/// HIPAR numberOfResults/Returns the number of the transmitted measurement results of a trace.
ViStatus _VI_FUNC rsemi_FetchReceiverPeakResults (ViSession instrSession,
                                                  ViInt32 arraySize,
                                                  ViReal64 peakLevel[],
                                                  ViReal64 peakFrequency[],
                                                  ViInt32 *numberOfResults)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViUInt32    return_count;
    ViInt32     i = 0;
    ViChar      *p2buf;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (Rs_ClearBeforeRead(instrSession));

    sprintf (buffer, ":FORM ASC;:TRAC? PHOL\n");

    viCheckErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32)strlen (buffer), VI_NULL));
    
    viCheckErr (viRead (instrSession, (ViBuf) buffer, RSEMI_IO_BUFFER_SIZE, &return_count));
    
    buffer[return_count] = '\0';

    p2buf = strtok (buffer, ",");

    while (p2buf && i < arraySize)
    {
        peakLevel[i] = atof (p2buf);
        p2buf = strtok (VI_NULL, ",");
        
        peakFrequency[i++] = atof (p2buf);
        p2buf = strtok (VI_NULL, ",");
    }
    
    *numberOfResults = i;
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the result of the scan measurement. The
/// HIFN  number of transmitted measurement results depends on the scan
/// HIFN  settings. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Pass the number of elements in the arrays you specify as the
/// HIPAR arraySize/array output parameters.
/// HIPAR traceStatus/Returns the trace status information (4 byte).
/// HIPAR numberOfResults/Returns the number of the transmitted measurement results of a trace.
/// HIPAR statusData/Returns the status information per measurement result during
/// HIPAR statusData/scan measurements.
/// HIPAR trace1/Returns an array of measurement results  of trace 1 if trace 1 is active.
/// HIPAR trace2/Returns an array of measurement results  of trace 2 if trace 2 is active.
/// HIPAR trace3/Returns an array of measurement results  of trace 3 if trace 3 is active.
ViStatus _VI_FUNC rsemi_FetchReceiverScanMeasurement (ViSession instrSession,
                                                      ViInt32 arraySize,
                                                      ViInt32 *traceStatus,
                                                      ViInt32 *numberOfResults,
                                                      ViChar statusData[],
                                                      ViReal64 trace1[],
                                                      ViReal64 trace2[],
                                                      ViReal64 trace3[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE];
    ViUInt32    return_count;

    ViInt32     lenLength   = 0,
                T1          = 0,
                T2          = 0,
                T3          = 0,
                T4          = 0,
                i           = 0,
                points      = 0;
    ViReal32    *temp;
    ViInt32     *p2data;
    ViChar      *exBuf;
    ViUInt16    remote_interface = VI_INTF_GPIB;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (arraySize < 1)
        return VI_ERROR_PARAMETER2;
    if (!traceStatus)
        return VI_ERROR_PARAMETER3;
    if (!numberOfResults)
        return VI_ERROR_PARAMETER4;
    if (!statusData)
        return VI_ERROR_PARAMETER5;

    /* Initial settings. */
    *numberOfResults = 0;
    statusData[0] = '\0';
    *traceStatus = 0;
    /* Temporary setings. */
    exBuf = buffer;

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &remote_interface));

    viCheckErr (Rs_ClearBeforeRead(instrSession));

    sprintf (buffer, ":FORM REAL,32;:TRAC? SCAN\n");
    /* Returns the result of the scan measurement

    Note:

    Structure of data:
    - 4 byte: trace status:
     bit 0 to 9 subscan
     bit 10: last block of subscan
     Bit 11: last block of last subscan of scan
     Bit 12: last of all blocks (for multiple scans after the last
    scan)
    - 4 bytes:number n of the transmitted measurement results of a
    trace
    - 4 byte:trace1 active (0/1)
    - 4 byte:trace2 active (0/1)
    - 4 byte:trace3 active (0/1)
    - 4 byte:reserved
    - n*4 bytes:measurement results of trace 1 if trace 1 is active
    - n*4 bytes:measurement results of trace 2 if trace 2 is active
    - n*4 bytes:measurement results of trace 3 if trace 3 is active
    - n*1 byte:status information per measurement result:
      bit 0: underrange trace1
      bit 1: underrange trace2
      bit 2: underrange trace3
      bit 4: overrange trace1 to trace4

    Where n is number of results. */

    switch (remote_interface)
        {
        case VI_INTF_GPIB: default: /* GPIB, RSIB Interface. */

            viCheckErr( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL));
            if ((error = rsemi_CheckBeforeRead (instrSession, 60000)) < 0)
                {
                viPrintf (instrSession, "FORM ASCII\n");
                return error;
                }
            
            if ((error = viRead (instrSession, (ViBuf) buffer, 2, &return_count)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return error;
            }
            buffer[return_count] = '\0';

            /* Get header info. */
            if (sscanf (buffer, "#%1d", &lenLength) != 1)
            {
                viPrintf (instrSession, "FORM ASCII");
                return RS_ERROR_UNEXPECTED_RESPONSE;
            }

            if ((error = viRead (instrSession, (ViBuf) buffer, lenLength, &return_count)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return error;
            }
            buffer[return_count] = '\0';

            if (sscanf (buffer, "%ld", numberOfResults) != 1)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return RS_ERROR_UNEXPECTED_RESPONSE;
            }

            /* Allocate memory space */
            if ((exBuf = malloc (*numberOfResults + 1)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return VI_ERROR_ALLOC;
            }
            if ((error = viRead (instrSession, (ViBuf) exBuf, *numberOfResults + 1, &return_count)) < 0)
                {
                viPrintf (instrSession, "FORM ASCII\n");
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return error;
                }

        break;
        case VI_INTF_ASRL: /* Serial Support. */

            viCheckErr( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL< 0));
            if ((error = rsemi_CheckBeforeRead (instrSession, 60000)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return error;
            }
            
            /* Switch to Off using of termination character. */
            if ((error = viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return error;
            }
            
            if ((error =  viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return error;
            }

            /* Parse binary data header. */
            if ((error = viRead (instrSession, (ViBuf) buffer, 2, VI_NULL)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                return error;
                }

            if (sscanf (buffer, "#%1d", &lenLength) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                return RS_ERROR_UNEXPECTED_RESPONSE;
                }

            /* Get number of points. */
            if ((error = viRead (instrSession, (ViBuf) buffer, lenLength, VI_NULL)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                return error;
                }

            if (sscanf (buffer, "%ld", numberOfResults) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                return RS_ERROR_UNEXPECTED_RESPONSE;
                }

            /* Allocate memory space */
            if ((exBuf = malloc (*numberOfResults + 2)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return VI_ERROR_ALLOC;
            }
            /* Read exact number of bytes equal to number of points. */
            if ((error = viRead (instrSession, (ViBuf) exBuf, *numberOfResults + 2, &return_count)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return error;
                }

            /* Switch to On using of termination character. */
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);

        break;
    }

    /* Get data info. */
    p2data = (ViInt32*)(exBuf);

    /* Binary data byte order (MOTOROLA or INTEL). */
    rsemi_SWAP_32((unsigned char*) exBuf, 2);

    *traceStatus = (ViInt32)*p2data++;
    *numberOfResults = (ViInt32)*p2data++;
    T1 = (ViInt32)*p2data++;
    T2 = (ViInt32)*p2data++;
    T3 = (ViInt32)*p2data++;

    points = (arraySize > *numberOfResults) ? *numberOfResults : arraySize;

    /* Get trace data. */
    if (T1 || T2 || T3) {

        /* Trace 1 */
        if (T1)
        {
            /* Binary data byte order (MOTOROLA or INTEL). */
            rsemi_SWAP_32((unsigned char*) exBuf+24, *numberOfResults);

            temp = (ViReal32*)(exBuf+24);

            for (i = 0; i < points; i++)
                *trace1++ = (ViReal64)*temp++;
        }
        else
            trace1 = memset (trace1, 0, 8*(points));
        /* Trace 2 */
        if (T2)
        {
            /* Binary data byte order (MOTOROLA or INTEL). */
            rsemi_SWAP_32((unsigned char*) exBuf+24+4*(*numberOfResults)*(T1), *numberOfResults);

            temp = (ViReal32*)(exBuf+24+4*(*numberOfResults)*(T1));

            for (i = 0; i < points; i++)
                *trace2++ = (ViReal64)*temp++;
        }
        else
            trace2 = memset (trace2, 0, 8*(points));
        /* Trace 3 */
        if (T3)
        {
            /* Binary data byte order (MOTOROLA or INTEL). */
            rsemi_SWAP_32((unsigned char*) exBuf+24+4*(*numberOfResults)*(T1+T2), *numberOfResults);

            temp = (ViReal32*)(exBuf+24+4*(*numberOfResults)*(T1+T2));

            for (i = 0; i < points; i++)
                *trace3++ = (ViReal64)*temp++;
        }
        else
            trace3 = memset (trace3, 0, 8*(points));
    }
    else
    {
        trace1 = memset (trace1, 0, 8*(points));
        trace2 = memset (trace2, 0, 8*(points));
        trace3 = memset (trace3, 0, 8*(points));
    }

    /* Remove header and remaining data - move pointer to status data. */
    statusData = memmove (statusData, exBuf+24+4*(*numberOfResults)*(T1+T2+T3+T4), points);

    statusData[points] = '\0';

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    if ((error = viPrintf (instrSession, "FORM ASCII")) < 0)
        return error;

    if ((error = rsemi_CheckStatus (instrSession)) < 0)
        return error;
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns 1 byte status information for each measurement 
/// HIFN  result.
/// HIFN     bit 2: overrange trace 1 to trace 3
/// HIFN     Available for scan measurements. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR statusData/Returns the status information per measurement result during scan 
/// HIPAR statusData/measurements.
ViStatus _VI_FUNC rsemi_FetchReceiverScanMeasurementStatus (ViSession instrSession,
                                                            ViChar statusData[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE];
    ViUInt32    return_count;

    ViInt32     lenLength   = 0,
                noofPoints  = 0;
    ViChar      *exBuf;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (!statusData)
        return VI_ERROR_PARAMETER2;

    /* Initial settings. */
    statusData[0] = '\0';
    /* Temporary setings. */
    exBuf = buffer;

    checkErr (Rs_ClearBeforeRead(instrSession));

    sprintf (buffer, ":FORM REAL,32;:TRAC? STAT\n");
    /* Returns the result of the scan measurement status

    1 byte status information per measurement result is transmitted: 
 
    Bit 2: Overrange trace 1 to trace 6 */

    checkErr( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL));
    if ((error = rsemi_CheckBeforeRead (instrSession, 60000)) < 0)
    {
        viPrintf (instrSession, "FORM ASCII\n");
        return error;
    }

    if ((error = viRead (instrSession, (ViBuf) buffer, 2, &return_count)) < 0)
    {
        viPrintf (instrSession, "FORM ASCII\n");
        return error;
    }
    buffer[return_count] = '\0';

    /* Get header info. */
    if (sscanf (buffer, "#%1d", &lenLength) != 1)
    {
        viPrintf (instrSession, "FORM ASCII");
        return RS_ERROR_UNEXPECTED_RESPONSE;
    }

    if ((error = viRead (instrSession, (ViBuf) buffer, lenLength, &return_count)) < 0)
    {
        viPrintf (instrSession, "FORM ASCII\n");
        return error;
    }
    buffer[return_count] = '\0';

    if (sscanf (buffer, "%ld", &noofPoints) != 1)
    {
        viPrintf (instrSession, "FORM ASCII\n");
        return RS_ERROR_UNEXPECTED_RESPONSE;
    }

    /* Allocate memory space */
    if ((exBuf = malloc (noofPoints + 1)) == NULL)
    {
        viPrintf (instrSession, "FORM ASCII\n");
        return VI_ERROR_ALLOC;
    }
    if ((error = viRead (instrSession, (ViBuf) exBuf, noofPoints + 1, &return_count)) < 0)
    {
        viPrintf (instrSession, "FORM ASCII\n");
        /* If not NULL pointer, then deallocate memory. */
        if (exBuf) free (exBuf);
        return error;
    }

    /* Binary data byte order (MOTOROLA or INTEL). */
    rsemi_SWAP_32((ViPBuf) exBuf, noofPoints);

    statusData = memmove (statusData, exBuf, noofPoints);

    statusData[noofPoints] = '\0';

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    if ((error = viPrintf (instrSession, "FORM ASCII")) < 0)
        return error;

    if ((error = rsemi_CheckStatus (instrSession)) < 0)
        return error;

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the results for all detectors that were active 
/// HIFN  during a single sweep measurement. The results are sorted by detector 
/// HIFN  type: Max Peak, Min Peak, Quasipeak, Average, RMS.
/// HIFN     Inactive detectors are not included. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Sets the number if the returned results.
/// HIPAR detectors/Returns the list of detectors.
/// HIPAR noOfValues/Returns the number of detectors.
ViStatus _VI_FUNC rsemi_FetchReceiverDetector (ViSession instrSession,
                                               ViInt32 arraySize,
                                               ViReal64 detectors[],
                                               ViInt32 *noOfValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViUInt32    return_count;
    ViInt32     i = 0;
    ViChar      *p2buf;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (Rs_ClearBeforeRead(instrSession));

    sprintf (buffer, ":FORM ASC;:TRAC? SING\n");

    viCheckErr( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL));
    
    viCheckErr( viRead (instrSession, (ViBuf) buffer, RSEMI_IO_BUFFER_SIZE, &return_count));
    
    buffer[return_count] = '\0';

    p2buf = strtok (buffer, ",");

    while (p2buf && (arraySize > i))
    {
        sscanf (p2buf, "%Le", &detectors[i++]);
        p2buf = strtok (VI_NULL, ",");
    }
    
    *noOfValues = i;
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Initiates a new measurement.
/// HIFN     
/// HIFN     Note(s):
/// HIFN     
/// HIFN     (1) In single sweep mode, you can synchronize to the end of the 
/// HIFN  measurement with *OPC, *OPC? or *WAI. In continuous sweep mode, 
/// HIFN  synchronization to the end of the measurement is not possible. Thus, it 
/// HIFN  is not recommended that you use continuous sweep mode in remote 
/// HIFN  control, as results like trace data or markers are only valid after a 
/// HIFN  single sweep end synchronization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_InitiateScan (ViSession instrSession, ViInt32 window, ViInt32 timeout)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, window, 1, 2), 2, "Window");
    
    sprintf (buffer, "Win%ld", window);
    
    if (timeout == 0)
    {
        viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_INITIATE_WITHOUT_OPC, NULL));
    }
    else
    {
        viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
        viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout));  
    
        viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, RSEMI_ATTR_INITIATE, NULL));
    }
Error:
    if (old_timeout > 0)
        viCheckErr( rsemi_SetOPCTimeout (instrSession, old_timeout));  
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Interrupts a running scan measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsemi_HoldScan (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_HOLD, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error; 
}

/// HIFN  Resumes a scan that was interrupted by a transducer stop at the 
/// HIFN  current reciever frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ContinueScanAtHold (ViSession instrSession,
                                            ViInt32 window)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "Win%ld", window);
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, buffer, 
                                                RSEMI_ATTR_CONTINUE_MEASUREMENT, VI_NULL));   

    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Resumes a scan that was interrupted by a transducer stop at the 
/// HIFN  current reciever frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsemi_ContinueScanAtReceiverFrequency (ViSession instrSession,
                                                         ViInt32 window)
{
    return rsemi_InitiateScan (instrSession, window, 0);
}

/// HIFN  Stops the measurement and resets the trigger system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_StopScan (ViSession instrSession, ViInt32 timeout)
{

    ViStatus    error   = VI_SUCCESS;
    ViInt32     old_timeout=-1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr( rsemi_GetOPCTimeout (instrSession, &old_timeout));  
    viCheckErr( rsemi_SetOPCTimeout (instrSession, timeout));  
    
    viCheckErr( rsemi_SetAttributeViString (instrSession, "", RSEMI_ATTR_ABORT, 
                VI_NULL));
    
Error:
    if (old_timeout > 0)
        viCheckErr( rsemi_SetOPCTimeout (instrSession, old_timeout));
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rsemi_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/
/* ViInt32 */
ViStatus _VI_FUNC rsemi_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rsemi_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rsemi_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rsemi_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rsemi_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: rsemi_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rsemi_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/* ViReal64 */
ViStatus _VI_FUNC rsemi_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/* ViString */
ViStatus _VI_FUNC rsemi_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rsemi_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/* ViSession */
ViStatus _VI_FUNC rsemi_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}


/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rsemi_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rsemi_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[3*RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, 3*RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize > 3*RS_MAX_MESSAGE_BUF_SIZE) ?
        3*RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: rsemi_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rsemi_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rsemi_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rsemi_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rsemi_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rsemi_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rsemi_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function resets the instrument to a known state and sends 
/// HIFN  initialization commands to the instrument. The initialization commands 
/// HIFN  set instrument settings such as Headers Off, Short Command form, and 
/// HIFN  Data Transfer Binary to the state necessary for the operation of the 
/// HIFN  instrument driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rsemi_DefaultInstrSetup (instrSession));
    checkErr( rsemi_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);

    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rsemi_ConfigureErrorChecking (ViSession instrSession,
                                                ViBoolean optionChecking,
                                                ViBoolean rangeChecking,
                                                ViBoolean statusChecking)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     p2value         = 0;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    sessionProperties -> optionChecking = optionChecking;
    p2value = (ViInt32) statusChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, &p2value));
    p2value = (ViInt32) rangeChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_RANGE_CHECK, 0, &p2value));
    
Error:
    return error;
}

/// HIFN  This function runs the instrument's self test routine and returns the 
/// HIFN  test result(s). 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR testResult/This control contains the value returned from the instrument self 
/// HIPAR testResult/test.  Zero means success.  For any other code, see the device's 
/// HIPAR testResult/operator's manual.
/// HIPAR testMessage/This control contains the string returned from the self test. See the 
/// HIPAR testMessage/device's operation manual for an explanation of the string's contents.
ViStatus _VI_FUNC rsemi_self_test (ViSession instrSession, ViInt16 *testResult,
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*TST?;*OPC\n"));
        viCheckErr( rsemi_WaitForOPC (instrSession, 600000));  
        viCheckErr( viScanf (instrSession, "%hd", testResult));
        if (*testResult == 0)
            strcpy (testMessage, "Self-Test Passed.");
        else
            strcpy (testMessage, "Self-Test Failed.");
        }
    else
        {
        /* Simulate Self Test */
        *testResult = 0;
        strcpy (testMessage, "No error.");
        }

    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function reads the results of the selftest out of the instrument. 
/// HIFN  The lines of the result table are output as string data separated by 
/// HIFN  commas.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control specifies the size of the buffer.
/// HIPAR results/Returns contains the results of the selftest out of the instrument. 
/// HIPAR results/The lines of the restlt table are output as string data separated by 
/// HIPAR results/commas.
ViStatus _VI_FUNC rsemi_SelfTestResults (ViSession instrSession,
                                            ViInt32 bufferSize,
                                            ViChar results[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[] = "DIAG:SERV:STES:RES?\n";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (viRead (instrSession, (ViBuf) results, (ViUInt32) bufferSize, NULL));

    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/// HIFN  This function returns the revision numbers of the instrument driver 
/// HIFN  and instrument firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of 
/// HIPAR driverRev/a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a 
/// HIPAR instrRev/string.
ViStatus _VI_FUNC rsemi_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSEMI_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSEMI_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is 
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the agesa_init or 
/// HIPAR instrSession/agesa_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Version string.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the 
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rsemi_SystemVersion (ViSession instrSession,
                                          ViInt32 length, 
                                          ViChar systemVersion[])

{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rsemi_GetAttributeViString (instrSession, "", 
                                                RSEMI_ATTR_SYSTEM_VERSION, length,
                                                systemVersion),
                                                2, "System Version");
    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/// HIFN  This function reads an error code and a message from the instrument's 
/// HIFN  error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error 
/// HIPAR errMessage/message queue.
ViStatus _VI_FUNC rsemi_error_query (ViSession instrSession, ViInt32 *errCode,
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViInt32     oldCode = 0;
    ViChar      oldMessage[256] = "";
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
    {
        p2buf = (char *)malloc (1);

        *p2buf='\0';
            
        do 
        {
            viCheckErr (rsemi_GetAttributeViString( instrSession, "", RSEMI_ATTR_SYSTEM_ERROR, RSEMI_IO_BUFFER_SIZE, buffer));
            if  (sscanf (buffer, "%ld,%*[^0-9a-zA-Z]%256[^\"]", &oldCode, oldMessage) != 2)
            {
                viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);
            }
            else
            {
                if (oldCode != 0)
                {
                    *errCode = oldCode;
                    
                    p2buf = (char *)realloc (p2buf, strlen(p2buf) + strlen (oldMessage) + 2);
        
                    strcat (p2buf, oldMessage);
        
                    strcat (p2buf, "\n");
                    
                    i++;
                }
                else if (i == 0)
                {
                    *errCode = oldCode;
    
                    strncpy (errMessage, oldMessage, 1024);
                }
            }

        } while (oldCode != 0);
        
        if (i > 0)
        {
            p2buf[strlen (p2buf) - 1] = '\0'; 
            
            strncpy (errMessage, p2buf, 1024);
        }
    }
    
    else
    {
        /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
    }

Error:
    if (p2buf) free (p2buf);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function reads all system messages and returns a list of
/// HIFN  comma separated strings. Each string corresponds to an entry in
/// HIFN  the table SYSTEM MESSAGES.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the size of Errors string.
/// HIPAR errors/Returns the error message string read from the instrument's
/// HIPAR errors/error message queue.
ViStatus _VI_FUNC rsemi_ErrorList (ViSession instrSession,
                                   ViInt32 bufferSize, ViChar errors[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (bufferSize < 0)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Buffer Size");
    if (errors == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Errors");

    viCheckErr (rsemi_GetAttributeViString( instrSession, "", RSEMI_ATTR_SYST_ERR_LIST, 
        bufferSize, errors));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function converts a status code returned by an instrument driver 
/// HIFN  function into a user-readable string. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument 
/// HIPAR errorCode/driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the 
/// HIPAR errorMessage/status code you specify.
ViStatus _VI_FUNC rsemi_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSEMI_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function deletes all entries in the table SYSTEM MESSAGES.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_ClearInstrumentErrors (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_SYST_ERR_CLEAR, VI_NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function deletes the error message buffer for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_ClearRemoteErrors (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_SYST_ERR_CLEAR_REMOTE, VI_NULL));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function displays the error display during remote operation
/// HIFN  on and off. If activated, the instrument displays a message box
/// HIFN  at the bottom of the screen that contains the most recent type
/// HIFN  of error and the command that caused the error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR displayRemoteErrors/This control displays the error display during remote operation on and 
/// HIPAR displayRemoteErrors/off. If activated, the R&S FSW displays a message box at the bottom of 
/// HIPAR displayRemoteErrors/the screen that contains the most recent type of error and the command 
/// HIPAR displayRemoteErrors/that caused the error.
ViStatus _VI_FUNC rsemi_DisplayRemoteErrors (
    ViSession instrSession,
    ViBoolean displayRemoteErrors
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_SetAttributeViBoolean (instrSession, "",
                                                 RSEMI_ATTR_SYST_DISPLAY_REMOTE_ERRORS,
                                                 displayRemoteErrors), 2, "Display Remote Errors");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;}

/// HIFN  This function initiates an instrument reset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_Preset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsemi_SetAttributeViString (instrSession, "", 
                RSEMI_ATTR_SYSTEM_PRESET, VI_NULL));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This control selects the type of settings that are restored when you 
/// HIFN  preset the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mode/This control selects the type of settings that are restored when you 
/// HIPAR mode/preset the instrument.
ViStatus _VI_FUNC rsemi_PresetMode(
    ViSession    instrSession,
    ViInt32    mode
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                RSEMI_ATTR_SYSTEM_PRESET_MODE, mode), 2, "Mode");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function selects the averaging during calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR averaging/Selects the bandwidth of the calibration signal.
ViStatus _VI_FUNC rsemi_ConfigureCalibrationAutoAveraging (
    ViSession instrSession,
    ViInt32 averaging
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
    viCheckParm (rsemi_SetAttributeViInt32 (instrSession, "", 
                                RSEMI_ATTR_CALIBRATION_AUTO_AVERAGING, averaging), 2, "Averaging");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN This function sets the OPC timeout. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  
/// HIPAR OPCtimeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_SetTimeout (ViSession instrSession,
                                       ViUInt32 timeout)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_SetOPCTimeout (instrSession, timeout));
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}


/// HIFN This function returns the OPC timeout.    
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  
/// HIPAR timeout/This control returns the timeout value.
ViStatus _VI_FUNC rsemi_GetTimeout (ViSession instrSession,
                                       ViUInt32 *timeout)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr (rsemi_GetOPCTimeout (instrSession, (ViInt32*) timeout));
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rsemi_SetVISATimeout(ViSession instrSession,
                                  ViUInt32 VISATimeout)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, VISATimeout ));
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rsemi_GetVISATimeout(ViSession instrSession,
                                  ViUInt32 *VISATimeout)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, VISATimeout ));
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_ClearStatus (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsemi_WriteInstrData (instrSession, "*CLS\n"));
    
    checkErr( rsemi_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rsemi_IDQueryResponse (ViSession instrSession,
                                             ViChar IDQueryResponse[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsemi_QueryViString (instrSession, "*IDN?", 256, IDQueryResponse));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_ProcessAllPreviousCommands (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsemi_WriteInstrData (instrSession, "*WAI\n"));
    
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rsemi_QueryOPC (ViSession instrSession, ViInt32 *OPC)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsemi_QueryViInt32 (instrSession, "*OPC?", OPC));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function sets fast sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR fastSweepMode/Fast sweep mode
ViStatus _VI_FUNC rsemi_SetFastSweepMode (ViSession instrSession,
                                             ViInt32 fastSweepMode)
{
	ViStatus	error = VI_SUCCESS;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
	viCheckParm(rsemi_invalidViInt32Range (instrSession, fastSweepMode, RSEMI_VAL_FAST_SWEEP_NORMAL, RSEMI_VAL_FAST_SWEEP_FAST), 2, "Fast Sweep Mode");
	
	sessionProperties->fastSweepInstrument = fastSweepMode;
Error:
	return error;
}

/// HIFN  This function retrieves and then clears the error information for the 
/// HIFN  session or the current execution thread. One exception exists: If the 
/// HIFN  BufferSize parameter is 0, the function does not clear the error 
/// HIFN  information. By passing 0 for the buffer size, the caller can ascertain 
/// HIFN  the buffer size required to get the entire error description string and 
/// HIFN  then call the function again with a sufficiently large buffer.
/// HIFN     
/// HIFN     If the user specifies a valid session for the instrSession 
/// HIFN  parameter, Get Error retrieves and then clears the error information 
/// HIFN  for the session. If the user passes VI_NULL for the instrSession 
/// HIFN  parameter, this function retrieves and then clears the error 
/// HIFN  information for the current execution thread.  If the instrSession 
/// HIFN  parameter is an invalid session, the function does nothing and returns 
/// HIFN  an error. Normally, the error information describes the first error 
/// HIFN  that occurred since the user last called rsemi_GetError or 
/// HIFN  rsemi_ClearError.
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If 
/// HIPAR description/there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsemi_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE + 256]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
        {
        if (errorCode == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
            }
        checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }

    else
        {
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
        }

    if (description == VI_NULL)
        {
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsemi_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsemi_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN  This function clears the error code and error description for the 
/// HIFN  session. If the user specifies a valid session for the instrSession 
/// HIFN  parameter, this function clears the error information for the session. 
/// HIFN  If the user passes VI_NULL for the Vi parameter, this function clears 
/// HIFN  the error information for the current execution thread. If the Vi 
/// HIFN  parameter is an invalid session, the function does nothing and returns 
/// HIFN  an error.
/// HIFN     The function clears the error code by setting it to VI_SUCCESS.  If 
/// HIFN  the error description string is non-NULL, the function de-allocates the 
/// HIFN  error description string and sets the address to VI_NULL.
/// HIFN     
/// HIFN     Maintaining the error information separately for each thread is useful 
/// HIFN  if the user does not have a session handle to pass to the 
/// HIFN  rsemi_GetError function, which occurs when a call to rsemi_init or 
/// HIFN  rsemi_InitWithOptions fails.
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_ClearError (ViSession instrSession)
{
    return Rs_ClearErrorInfo (instrSession);
}

/// HIFN This function is used to read data from the instrument and write 
/// HIFN it to a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument
/// HIPAR source/to control computer.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from instrument to control computer applies.
ViStatus _VI_FUNC rsemi_ReadToFileFromInstrument (ViSession instrSession,
                                                     ViString source,
                                                     ViString destination)
{
    ViStatus    error = VI_SUCCESS;
    //ViInt32     count = 0;
    ViUInt32    ret_count = 0;
    ViUInt64    length = 0;
    ViChar      buffer[RSEMI_IO_BUFFER_SIZE] = "";
    ViUInt16 intfc = VI_INTF_GPIB;     
	ViChar      *pbuffer = NULL;
	int i;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
	viCheckParm (!source, 2, "Source");
	viCheckParm (!destination, 3, "Destination");    
    
	checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));
    switch (intfc)
    {
        case VI_INTF_ASRL: /* Serial intfc. */
            viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        break;
    }
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
    viCheckErr (viPrintf(instrSession, ":MMEMory:DATA? '%s'\n", source));
    checkErr (viRead (instrSession, (ViBuf) buffer, 1, NULL)); 

	
	if (buffer[0] == '#')
	{   /* Binary data */
		/* Disable termination character */
		checkErr(viSetAttribute(instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

		/* Parse header */
		checkErr(viRead(instrSession, (ViBuf)buffer, 1, NULL));

		buffer[1] = '\0';
		if (buffer[0] == '(')
		{
			for (i = 0; i < 20; i++)
			{
				checkErr(viRead(instrSession, (ViBuf)buffer + i, 1, NULL));

				if (buffer[i] == ')')
				{
					buffer[i] = '\0';
					sscanf(buffer, "%lld", &length);
					break;
				}
			}
			if (i == 20)
			{
				//couldn't find the ')' character
				checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
			}
		}
		else if (buffer[0] == '0')
		{
			length = 0;
		}
		else
		{
			sscanf(buffer, "%lld", &length);
			if (length > 9 || length < 1)
			{
				//expected number 1 to 9
				checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
			}

			checkErr(viRead(instrSession, (ViBuf)buffer, (ViUInt32) length, NULL));
			buffer[length] = '\0';
			sscanf(buffer, "%lld", &length);
		}

		if (length > 0)
		{
			viCheckErr(Rs_ReadToFile2(instrSession, destination, length, RS_VAL_TRUNCATE));
			/* Flush garbage */
			viScanf(instrSession, "%*[^\r\n]");
			if (error > 0) error = VI_SUCCESS;
		}
		else
		{
			/* Unlimited size of data */
			checkErr(Rs_ReadDataUnknownLength(instrSession, &pbuffer, &ret_count));
		}
    }
    else
    {
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);         
    }
	
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);        
    checkErr( rsemi_CheckStatus (instrSession));
  
Error:
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    if (pbuffer) free(pbuffer);
    switch (intfc)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }  
    (void) Rs_UnlockSession(instrSession, NULL);
    
    return error;
}

/// HIFN This function is used to read data from the host computer and 
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
 ViStatus _VI_FUNC rsemi_WriteFromFileToInstrument (ViSession instrSession,
                                                      ViString source,
                                                      ViString destination)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    digits = 0;
    FILE        *sourceFile = NULL;
    ViUInt64    fileSize = 0;
#if defined (_CVI_)
	ViUInt32      fileSize32 = 0;
#endif
    
    checkErr( Rs_LockSession (instrSession, NULL));
    viCheckParm (!source, 2, "Source");
    viCheckParm (!destination, 3, "Destination");  
        
    /* Open source file from host controller */
    if ((sourceFile = fopen (source, "rb")) == NULL)
        viCheckErr(RS_ERROR_CANNOT_OPEN_FILE);

    /* Get size of file */
    (void) fseek (sourceFile, 0, SEEK_END);
#if defined (_CVI_)
    GetFileSize (source, &fileSize32);
	fileSize = (ViUInt64) fileSize32;
#elif defined(_MSC_VER)
    fileSize = (ViUInt64) _ftelli64 (sourceFile);
#else
    fileSize = (ViUInt64) ftell (sourceFile);
#endif
    (void) fseek (sourceFile, 0, SEEK_SET);

    /* Create command header */
    digits = (ViUInt32) ((int)floor(log10((double)fileSize))) + 1;
    if (digits < 10)
	{
		viCheckErr (viPrintf (instrSession, ":MMEM:DATA '%s',#%ld%lld", destination, digits, fileSize));
	}
	else
	{
		viCheckErr (viPrintf (instrSession, ":MMEM:DATA '%s',#(%lld)", destination, fileSize));
	}
    
    viCheckErr (Rs_WriteFromFile2(instrSession, source, fileSize, 0));
        
    checkErr( rsemi_CheckStatus (instrSession));
 
 Error:
    if (sourceFile) (void) fclose (sourceFile);
    
    (void) Rs_UnlockSession(instrSession, NULL);
    
    return error;
}

/// HIFN  This function writes a user-specified string to the instrument.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function bypasses attribute state caching. Therefore, when you 
/// HIFN  call this function, the cached values for all attributes will be 
/// HIFN  invalidated.
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsemi_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

/// HIFN  This function reads data from the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.  
/// HIPAR rdBuf/After this function executes, this parameter contains the data that 
/// HIPAR rdBuf/was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and 
/// HIPAR bytesRead/stored in the Read Buffer.
ViStatus _VI_FUNC rsemi_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN  This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value
ViStatus _VI_FUNC rsemi_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsemi_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value
ViStatus _VI_FUNC rsemi_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsemi_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value
ViStatus _VI_FUNC rsemi_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsemi_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsemi_init or 
/// HIPAR instrSession/rsemi_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the  
/// HIPAR bufferSize/Value parameter.  
/// HIPAR value/Returns the ViBoolean value
ViStatus _VI_FUNC rsemi_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsemi_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN  This function performs the following operations:
/// HIFN     
/// HIFN     - Closes the instrument I/O session.
/// HIFN     
/// HIFN     - Destroys the instrument driver session and all of its attributes.
/// HIFN     
/// HIFN     - Deallocates any memory resources the driver uses.
/// HIFN     
/// HIFN     Notes:
/// HIFN     
/// HIFN     (1) You must unlock the session before calling RSEMI_close.
/// HIFN     
/// HIFN     (2) After calling RSEMI_close, you cannot use the instrument driver 
/// HIFN  again until you call RSEMI_init or RSEMI_InitWithOptions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsemi_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsemi_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}


/*****************************************************************************
 * Function: rsemi_RsClose
 * Purpose:  This function performs all of the drivers clean-up operations
 *           except for closing the RS session.  This function is called by
 *           rsemi_close or by an RS class driver.
 *
 *           Note:  This function must close the I/O session and set
 *           RS_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC rsemi_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

Error:
    
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        viWrite (instrSession, (ViBuf)"&GTL\n", 5, NULL);
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/************************** DEPRECATED *****************************************/
/// HIFN  This function sets the Enable, NTransition, and PTransition bits     
/// HIFN  of the status questionable and operating registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR registerOperation/Specifies the operation to perform.
/// HIPAR questionableRegister/This control selects the registers that will be configured. 
/// HIPAR enable/This control sets the enable bits of the status register.  
/// HIPAR ptransition/This control sets the PTransition bits (edge detectors) of the status 
/// HIPAR ptransition/register.  
/// HIPAR ntransition/This control sets the NTransition bits (edge detectors) of the status 
/// HIPAR ntransition/register.  
ViStatus _VI_FUNC rsemi_SetStatusRegister(
    ViSession    instrSession,
    ViInt32    registerOperation,
    ViInt32    questionableRegister,
    ViInt32    enable,
    ViInt32    ptransition,
    ViInt32    ntransition
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   rep_cap_reg[25] = "";
    ViChar   rep_cap[75] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch (registerOperation) 
    {
        case 0:
            viCheckErr (rsemi_SetAttributeViString (instrSession, "",
                                               RSEMI_ATTR_PRESET_STATUS_REGISTERS, NULL));
        break;
        case 1:
            viCheckParm (rsemi_invalidViInt32Range (instrSession, questionableRegister, 1, 15), 2, "Questionable Register");
    
            viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                 RSEMI_ATTR_QUESTIONABLE_REGISTER,
                                                 "QRegister",
                                                 questionableRegister - 1,
                                                 25,
                                                 rep_cap_reg), 2, "Questionable Register");

            strcpy (rep_cap, rep_cap_reg);
            strcat (rep_cap, ",");
            strcat (rep_cap, "RegEnable");

            viCheckParm(rsemi_SetAttributeViInt32(instrSession, rep_cap,
                RSEMI_ATTR_QUESTIONABLE_REGISTER, enable), 3, "Enable");
    
            strcpy (rep_cap, rep_cap_reg);
            strcat (rep_cap, ",");
            strcat (rep_cap, "RegNTR");

            viCheckParm(rsemi_SetAttributeViInt32(instrSession, rep_cap,
                RSEMI_ATTR_QUESTIONABLE_REGISTER, ntransition), 5, "NTR Transition");

            strcpy (rep_cap, rep_cap_reg);
            strcat (rep_cap, ",");
            strcat (rep_cap, "RegPTR");

            viCheckParm(rsemi_SetAttributeViInt32(instrSession, rep_cap,
                RSEMI_ATTR_QUESTIONABLE_REGISTER, ptransition), 4, "PTR Transition");
    
        break;
        case 2:
            viCheckParm(rsemi_SetAttributeViInt32(instrSession, "RegEnable",
                RSEMI_ATTR_OPERATION_REGISTER, enable), 3, "Enable");
    
            viCheckParm(rsemi_SetAttributeViInt32(instrSession, "RegNTR",
                RSEMI_ATTR_OPERATION_REGISTER, ntransition), 5, "NTR Transition");

            viCheckParm(rsemi_SetAttributeViInt32(instrSession, "RegPTR",
                RSEMI_ATTR_OPERATION_REGISTER, ptransition), 4, "PTR Transition");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;  
}

/// HIFN  This function contains the commands for the querying status reporting 
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSEMI_init or 
/// HIPAR instrSession/RSEMI_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR statusRegisterQuery/This control selects the registers that will be queried. 
/// HIPAR registerPart/Selects register part to be queried.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsemi_GetStatusRegister(
    ViSession    instrSession,
    ViInt32    statusRegisterQuery,
    ViInt32    registerPart,
    ViInt32*    registerValue
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   rep_cap_reg[25] = "";
    ViChar   rep_cap_part[25] = "";
    ViChar   rep_cap[75] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsemi_invalidViInt32Range (instrSession, statusRegisterQuery, 0, 15), 2, "Status Register Query");
    viCheckParm (rsemi_invalidViInt32Range (instrSession, registerPart, 3, 4), 3, "Register Part");
    
    if (statusRegisterQuery == 0)
    {
        viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                     RSEMI_ATTR_OPERATION_REGISTER,
                                                     "RegisterPart",
                                                     registerPart,
                                                     25,
                                                     rep_cap_part), 3, "Register Part");

        viCheckParm(rsemi_GetAttributeViInt32(instrSession, rep_cap_part,
            RSEMI_ATTR_OPERATION_REGISTER, registerValue), 4, "Register Value");
    }
    else
    {
        viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                     RSEMI_ATTR_QUESTIONABLE_REGISTER,
                                                     "QRegister",
                                                     statusRegisterQuery - 1,
                                                     25,
                                                     rep_cap_reg), 2, "Questionable Register");
        viCheckParm (rsemi_GetAttributeRepCapName (instrSession,
                                                     RSEMI_ATTR_QUESTIONABLE_REGISTER,
                                                     "RegisterPart",
                                                     registerPart,
                                                     25,
                                                     rep_cap_part), 3, "Register Part");

        strcpy (rep_cap, rep_cap_reg);
        strcat (rep_cap, ",");
        strcat (rep_cap, rep_cap_part);

        viCheckParm(rsemi_GetAttributeViInt32(instrSession, rep_cap,
            RSEMI_ATTR_QUESTIONABLE_REGISTER, registerValue), 4, "Register Value");
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;   
}

/// HIFN  This function configures the power channel weighting filters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterType/This control selects the type of the filter to be modified.
/// HIPAR filterState/This control sets the state of the selected filter.
/// HIPAR filterValue/This control sets the weighting filter alpha value.
ViStatus _VI_FUNC rsemi_ConfigurePowerChannelWeightingFilters(
    ViSession    instrSession,
    ViInt32    filterType,
    ViBoolean    filterState,
    ViReal64    filterValue
)
{
    ViStatus    error = VI_SUCCESS;
    ViAttr      attr_state[4] = {RSEMI_ATTR_SEM_WEIGHT_FILTER_ALL_STATE,
                                 RSEMI_ATTR_SEM_WEIGHT_FILTER_ACH_STATE,
                                 RSEMI_ATTR_SEM_WEIGHT_FILTER_ALT_STATE,
                                 RSEMI_ATTR_SEM_WEIGHT_FILTER_TX_CHANNEL_STATE};
    ViAttr      attr_val[4] = {RSEMI_ATTR_SEM_WEIGHT_FILTER_ALL_ALPHA,
                               RSEMI_ATTR_SEM_WEIGHT_FILTER_ACH_ALPHA,
                               RSEMI_ATTR_SEM_WEIGHT_FILTER_ALT_ALPHA,
                               RSEMI_ATTR_SEM_WEIGHT_FILTER_TX_CHANNEL_ALPHA};
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsemi_invalidViInt32Range (instrSession, filterType, RSEMI_VAL_WFILTER_ALL, RSEMI_VAL_WFILTER_TX), 2, "Filter Type");
    
    viCheckParm( rsemi_SetAttributeViBoolean (instrSession, "", attr_state[filterType], filterState),
                                                 3, "Filter State");
    
    if (filterState == VI_TRUE)
    {
        viCheckParm (rsemi_SetAttributeViReal64 (instrSession, "", attr_val[filterType], filterValue),
                                                    4, "Filter Value");
    }
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function reads out the names of all limit lines stored on the 
/// HIFN  hard disk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofLimitLines/This control returns number of names of all limit lines.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog 
/// HIPAR bufferSize/list parameter.
/// HIPAR limitLinesList/This control returns the names of all limit lines stored on the hard 
/// HIPAR limitLinesList/disk.
ViStatus _VI_FUNC rsemi_GetLimitLineCatalog(
    ViSession    instrSession,
    ViInt32*    numberofLimitLines,
    ViInt32    bufferSize,
    ViChar    limitLinesList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      *buf=VI_NULL;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if (bufferSize < 0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");         
    if ((bufferSize > 0) && (limitLinesList == VI_NULL))
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Limit Lines List");  
       
    viCheckErr (viPrintf (instrSession, "CALC:LIM:CAT?\n"));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &buf, &retCnt));
    viCheckErr (rsemi_ParseLimitLineCatalog(buf, bufferSize, limitLinesList, numberofLimitLines));

    checkErr( rsemi_CheckStatus (instrSession));
    
Error:
    if (buf) free(buf);
    (void) Rs_UnlockSession(instrSession, VI_NULL);      
    return error;
}
